Package: modelSelection
Type: Package
Version: 1.0.7
Date: 2026-05-16
Title: High-Dimensional Model Selection
Authors@R: c(person(given = "David", family = "Rossell", role = c("aut", "cre"), email = "rosselldavid@gmail.com"), person(given = c("John", "D."), family = "Cook", role = "ctb"), person(given = "Donatello", family = "Telesca", role = "aut"), person(given = "P.", family = "Roebuck", role = "ctb"), person(given = "Oriol", family = "Abril", role = "aut"), person(given = "Miquel", family = "Torrens", role = "aut"), person(given = "Peter", family = "Mueller", role = "ctb"), person(given = "William", family = "Hallahan", role = "ctb"))
Maintainer: David Rossell <rosselldavid@gmail.com>
Depends: R (>= 2.14.0), methods
Suggests: parallel, testthat, patrick
biocViews:
Imports: Rcpp (>= 0.12.16), dplyr, glmnet, intervals, L0Learn, Matrix,
        mclust, mgcv, mvtnorm, ncvreg, pracma, sparseMatrixStats,
        survival
LinkingTo: Rcpp, RcppArmadillo, RcppEigen
Description: Model selection and averaging for regression, generalized linear models, generalized additive models, graphical models and mixtures, focusing on Bayesian model selection and information criteria (Bayesian information criterion etc.). See Rossell (2025) <doi:10.5281/zenodo.17119597> (see the URL field below for its URL) for a hands-on book describing the methods, examples and suggested citations if you use the package.
License: GPL (>= 2)
URL: https://github.com/davidrusi/modelSelection,
        https://github.com/davidrusi/modelSelection-book
BugReports: https://github.com/davidrusi/modelSelection/issues
LazyLoad: yes
Collate: AllClasses.R AllGenerics.R alapl.R bms_ortho.R cil.R cox.R
        derivatives_nlps.R distribs.R dmom.R eBayes.R gam.R ggm.R
        greedyGLM.R infocriteria.R initParameters.R localnulltest.R
        marginalLikelihood.R msPriorSpec.R modelsearch.R
        modelSelection.R modelSelectionGLM.R mombf.R normaliwish.R
        normmix.R postMode.R rmom.R RcppExports.R testfunction.R
RoxygenNote: 7.3.2
NeedsCompilation: yes
Packaged: 2026-05-16 06:00:53 UTC; u138097
Author: David Rossell [aut, cre],
  John D. Cook [ctb],
  Donatello Telesca [aut],
  P. Roebuck [ctb],
  Oriol Abril [aut],
  Miquel Torrens [aut],
  Peter Mueller [ctb],
  William Hallahan [ctb]
Repository: CRAN
Date/Publication: 2026-05-16 14:50:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-05-16 17:10:36 UTC; unix
Archs: modelSelection.so.dSYM

Package: modelSelection
Type: Package
Version: 1.0.7
Date: 2026-05-16
Title: High-Dimensional Model Selection
Authors@R: c(person(given = "David", family = "Rossell", role = c("aut", "cre"), email = "rosselldavid@gmail.com"), person(given = c("John", "D."), family = "Cook", role = "ctb"), person(given = "Donatello", family = "Telesca", role = "aut"), person(given = "P.", family = "Roebuck", role = "ctb"), person(given = "Oriol", family = "Abril", role = "aut"), person(given = "Miquel", family = "Torrens", role = "aut"), person(given = "Peter", family = "Mueller", role = "ctb"), person(given = "William", family = "Hallahan", role = "ctb"))
Maintainer: David Rossell <rosselldavid@gmail.com>
Depends: R (>= 2.14.0), methods
Suggests: parallel, testthat, patrick
biocViews:
Imports: Rcpp (>= 0.12.16), dplyr, glmnet, intervals, L0Learn, Matrix,
        mclust, mgcv, mvtnorm, ncvreg, pracma, sparseMatrixStats,
        survival
LinkingTo: Rcpp, RcppArmadillo, RcppEigen
Description: Model selection and averaging for regression, generalized linear models, generalized additive models, graphical models and mixtures, focusing on Bayesian model selection and information criteria (Bayesian information criterion etc.). See Rossell (2025) <doi:10.5281/zenodo.17119597> (see the URL field below for its URL) for a hands-on book describing the methods, examples and suggested citations if you use the package.
License: GPL (>= 2)
URL: https://github.com/davidrusi/modelSelection,
        https://github.com/davidrusi/modelSelection-book
BugReports: https://github.com/davidrusi/modelSelection/issues
LazyLoad: yes
Collate: AllClasses.R AllGenerics.R alapl.R bms_ortho.R cil.R cox.R
        derivatives_nlps.R distribs.R dmom.R eBayes.R gam.R ggm.R
        greedyGLM.R infocriteria.R initParameters.R localnulltest.R
        marginalLikelihood.R msPriorSpec.R modelsearch.R
        modelSelection.R modelSelectionGLM.R mombf.R normaliwish.R
        normmix.R postMode.R rmom.R RcppExports.R testfunction.R
RoxygenNote: 7.3.2
NeedsCompilation: yes
Packaged: 2026-05-16 06:00:53 UTC; u138097
Author: David Rossell [aut, cre],
  John D. Cook [ctb],
  Donatello Telesca [aut],
  P. Roebuck [ctb],
  Oriol Abril [aut],
  Miquel Torrens [aut],
  Peter Mueller [ctb],
  William Hallahan [ctb]
Repository: CRAN
Date/Publication: 2026-05-16 14:50:02 UTC

###
### NAMESPACE
###

import(methods, mgcv, stats)
importFrom(L0Learn, L0Learn.fit)
importFrom(mclust, mclustBIC)
importFrom("utils", "combn")
importFrom(Rcpp, evalCpp)
 
importFrom("graphics", "abline", "lines", "legend")
importFrom("grDevices", "devAskNewPage")

useDynLib(modelSelection, .registration=TRUE)


# Prior distributions on models

export(modelbbprior, modelbinomprior, modelcomplexprior, modelunifprior)
export(bbPrior, binomPrior, unifPrior)

export(aic, bic, ic)


# Prior distributions on parameters

export(momprior, groupmomprior, emomprior, groupemomprior, imomprior, groupimomprior) # Non-local priors
export(normalidprior, zellnerprior, groupzellnerprior, icarplusprior) #Local priors
export(igprior, exponentialprior)  #Priors on dispersion parameters

export(eprod)


# Main model selection functions

export(modelSelection, modelSelection_eBayes, modelSelectionGGM, cil)
export(localnulltest, localnulltest_fda, localnulltest_givenknots, localnulltest_fda_givenknots)
export(bestAIC, bestBIC, bestEBIC, bestIC, bestEBIC, bestAIC_fast, bestBIC_fast, bestEBIC_fast, bestIC_fast)
export(bfnormmix)


# Methods for output of main model selection functions

S3method(coef,icfit)
S3method(coef,mixturebf)
S3method(coef,msfit)
S3method(coef,msfit_ggm)
S3method(coef,cilfit)
S3method(coef,localtest)
export(coefByModel)

S3method(confint, icfit)

export(icov)

export(plotprior)
S3method(plot,msfit)
S3method(plot,msfit_ggm)

export(postProb, postSamples)

S3method(predict, icfit)
S3method(predict, msfit)
S3method(predict, localtest)

S3method(summary, icfit)


# Calculation of marginal (or integrated) likelihoods

export(marginalLikelihood)
export(marginalNIW)


# Densities, quantiles, and random number generation

export(dalapl, ddir, demom, dimom, dmom, dmomigmarg, demomigmarg, diwish,dpostNIW)
export(palapl, pemom, pemomigmarg, pimom, pmom, pmomigmarg)
export(qimom, qmom)
export(ralapl, rnlp, rpostNIW)

export(priorp2g)

# mombf

Model Selection with Bayesian Methods and Information Criteria

## Installation

``` r
# Install mombf from CRAN
install.packages("mombf")

# from GitHub:
# install.packages("devtools")
devtools::install_github("davidrusi/mombf")
```

## Quick start

The main Bayesian model selection (BMS) function is `modelSelection`. For information criteria
consider `bestBIC`, `bestEBIC`, `bestAIC`, `bestIC`. 
Bayesian model averaging (BMA) is also available for some models,
mainly linear and generalized linear models.
Local variable selection is implemented in `localnulltest` and `localnulltest_fda`.
Details are in [`mombf`'s vignette](https://CRAN.R-project.org/package=mombf/vignettes/mombf.pdf),
here we illustrate quickly how to get posterior model probabilities,
marginal posterior inclusion probabilities, BMA point estimates and posterior
intervals for the regression coefficients and predicted outcomes.

```r
library(mombf)
set.seed(1234)
x <- matrix(rnorm(100*3),nrow=100,ncol=3)
theta <- matrix(c(1,1,0),ncol=1)
y <- x %*% theta + rnorm(100)

priorCoef <- momprior(tau=0.348)  # Default MOM prior on parameters
priorModel <- modelbbprior(1,1)   # Beta-Binomial prior for model space
fit1 <- modelSelection(y ~ x[,1]+x[,2]+x[,3], priorCoef=priorCoef, priorModel=priorModel)
# Output
# Enumerating models...
# Computing posterior probabilities................ Done.
```

from here, we can also get the posterior model probabilities:

```r
postProb(fit1)
# Output
#    modelid family           pp
# 7      2,3 normal 9.854873e-01
# 8    2,3,4 normal 7.597369e-03
# 15   1,2,3 normal 6.771575e-03
# 16 1,2,3,4 normal 1.437990e-04
# 3        3 normal 3.240602e-17
# 5        2 normal 7.292230e-18
# 4      3,4 normal 2.150174e-19
# 11     1,3 normal 9.892869e-20
# 6      2,4 normal 5.615517e-20
# 13     1,2 normal 2.226164e-20
# 12   1,3,4 normal 1.477780e-21
# 14   1,2,4 normal 3.859388e-22
# 1          normal 2.409908e-25
# 2        4 normal 1.300748e-27
# 9        1 normal 2.757778e-28
# 10     1,4 normal 3.971521e-30
```

also the BMA estimates, 95% intervals, marginal posterior probability

```r
coef(fit1)
# Output
#              estimate        2.5%      97.5%      margpp
# (Intercept) 0.007230966 -0.02624289 0.04085951 0.006915374
# x[, 1]      1.134700387  0.93487948 1.33599873 1.000000000
# x[, 2]      1.135810652  0.94075622 1.33621298 1.000000000
# x[, 3]      0.000263446  0.00000000 0.00000000 0.007741168
# phi         1.100749637  0.83969879 1.44198567 1.000000000
```

and BMA predictions for y, 95% intervals

```r
ypred <- predict(fit1)
head(ypred)
# Output
#         mean       2.5%       97.5%
# 1 -0.8936883 -1.1165154 -0.67003262
# 2 -0.2162846 -0.3509188 -0.08331286
# 3  1.3152329  1.0673711  1.56348261
# 4 -3.2299241 -3.6826696 -2.77728625
# 5 -0.4431820 -0.6501280 -0.23919345
# 6  0.7727824  0.6348189  0.90977798
cor(y, ypred[,1])
# Output
#           [,1]
# [1,] 0.8468436
```

## Bug report

Please submit bug reports to the [issue tracker](https://github.com/davidrusi/mombf/issues).

##########################################################################################
##
## BAYESIAN MODEL SELECTION & AVERAGING UNDER DIAGONAL & BLOCK-DIAGONAL X'X
##
##########################################################################################


postModeOrtho <- function(y, x, priorCoef=momprior(tau=0.348), priorModel=modelbbprior(1,1),priorVar=igprior(0.01,0.01), bma=FALSE, includeModels, maxvars=100) {
    #Find posterior mode for linear regression under orthogonal X'X
    # - y: outcome
    # - x: predictors
    # - priorCoef: prior on coefficients
    # - priorModel: prior on model space
    # - priorVar: prior on residual variance
    # - bma: set to TRUE to obtain Bayesian model averaging parameter estimates
    # - includeModels: list of models that should always be included when computing posterior model probabilities. Each element should be a vector z where x[,z] selects the required variables
    # - integrateMethod: if exactpp=TRUE this is the numerical integration method to obtain logpy. Set to 'CSR' to compound Simpson's rule based on an adaptive grid defined by enumerating highly probable models, to 'AQ' to use adaptive quadrature as implemented in R function 'integrate'
    # - maxvars: the search is restricted to models with up to maxvars variables (note: posterior model prob and BMA are valid regardless of maxvars)
    # Output
    # -
    integrateMethod <- 'AQ'; exactpp <- TRUE #exactpp=FALSE means we only find posterior mode (for Zellner's prior, for pMOM this doesn't work)
    n <- length(y); p <- ncol(x)
    if (priorModel@priorDistr=='binomial' & ('p' %in% names(priorModel@priorPars))) {
        rho <- priorModel@priorPars[['p']]
        if (length(rho)>1) stop("postModeOrtho not implemented for vector p in modelbinomprior")
        priorModel <- function(nvar) nvar*log(rho) + (p-nvar)*log(1-rho)
        #priorModel <- function(nvar) dbinom(nvar, size=ncol(x),prob=rho,log=TRUE)
    } else if (priorModel@priorDistr=='binomial' & !('p' %in% names(priorModel@priorPars))) {
        alpha=priorModel@priorPars['alpha.p']; beta=priorModel@priorPars['beta.p']
        priorModel <- function(nvar) lbeta(nvar + alpha, p - nvar + beta) - lbeta(alpha, beta)
        rho <- alpha/(alpha+beta)
    } else if (priorModel@priorDistr=='uniform') {
        priorModel <- function(nvar) -p*log(2)
        rho <- 1/2
    } else if (priorModel@priorDistr=='complexity') {
        priorc= priorModel@priorPars['c']
        priornorm= log(1-1/p^(priorc*(p+1))) - log(1-1/p^priorc)
        priorModel <- function(nvar) lbeta(nvar + 1, p - nvar + 1) - (priorc*nvar)*log(p) - priornorm
        lseq= 1:(p+1)
        rho= sapply(lseq, function(l) -(priorc*l)*log(p) + lchoose(p-1,l-1) - lchoose(p,l) - priornorm )
        rho= sum(exp(rho[!is.nan(rho)]))
    } else { stop("Prior on model space not recognized. Use modelbbprior(), modelunifprior() or modelbinomprior()") }
    if (priorCoef@priorDistr == 'zellner') {
        g <- as.double(priorCoef@priorPars['tau'])
    } else if (priorCoef@priorDistr == 'pMOM') {
        tau <- as.double(priorCoef@priorPars['tau'])
        g <- n * tau
    } else {
        stop("priorCoef must be pMOM or Zellner. Use momprior() or zellnerprior()")
    }
    if (priorVar@priorDistr != 'invgamma') stop('priorVar must be an inverse gamma. Use igprior()')
    a.phi <- as.double(priorVar@priorPars['alpha'])
    l.phi <- as.double(priorVar@priorPars['lambda'])
    if (exactpp & !(integrateMethod %in% c('CSR','AQ'))) {
        stop("integrateMethod should be 'CSR' for compound Simpson's rule or 'AQ' for adaptive quadrature")
    }
    #Pre-compute useful quantities
    sumy2 <- l.phi + sum(y^2)
    apos <- a.phi+n
    shrinkage <- g/(1+g)
    Xty <- t(x) %*% matrix(y,ncol=1); XtX <- colSums(x^2)
    u <- Xty^2 / XtX
    #Avoid numerical overflow (also cases where X'X is not diagonal)
    sumu <- sum(u)
    if (sumu > sumy2-l.phi) u <- u * ((sumy2-l.phi)/sumu)
    #Consider sequence of models
    o <- order(u,decreasing=TRUE)
    uord <- u[o]
    cumsumu <- cumsum(uord)
    #gamma <- rep(FALSE,ncol(x))
    modelid <- character(length(o)+1)
    modelusum <- c(0,cumsumu)
    iseq <- 0:length(o)
    lpos <- sumy2 - shrinkage * c(0,cumsumu)
    pp <- priorModel(iseq) - 0.5*iseq*log(1+g) - 0.5*apos * log(lpos)
    for (i in 1:length(o)) { modelid[i+1] <- paste((o[1:i])[order(o[1:i])],collapse=',') }
    #Compute lpos, modelid, modelusum for includeModels
    if (!missing(includeModels)) {
        iextra= modelusumextra= double(length(includeModels))
        modelidextra= character(length(includeModels))
        for (i in 1:length(includeModels)) {
            if (is.numeric(includeModels[[i]])) {
                modelidextra[i]= paste(includeModels[[i]][order(includeModels[[i]])],collapse=',')
                iextra[i]= length(includeModels[[i]])
            } else if (is.logical(includeModels[[i]])) {
                modelidextra[i]= paste(which(includeModels[[i]]),collapse=',')
                iextra[i]= sum(includeModels[[i]])
            } else { stop("includeModels must be a list, each element either a logical or numeric vector indicating the variables to be included") }
            modelusumextra[i]= sum(u[includeModels[[i]]])
        }
        sel= !(modelidextra %in% modelid)
        if (any(sel)) {
            modelid= c(modelid,modelidextra[sel])
            modelusum= c(modelusum,modelusumextra[sel])
            lposextra= sumy2 - shrinkage * modelusumextra[sel]
            lpos= c(lpos, lposextra)
            pp= c(pp, priorModel(iextra[sel]) - 0.5*iextra[sel]*log(1+g) - 0.5*apos * log(lposextra[sel]))
        }
    }
    maxpp <- max(pp)
    variableids <- strsplit(as.character(modelid),split=',')
    nvars <- sapply(variableids,length)
    sel <- nvars<=maxvars; modelid <- modelid[sel]; variableids <- variableids[sel]; nvars <- nvars[sel]
    if (any(!sel)) { modelusum <- modelusum[1:(maxvars+1)]; lpos <- lpos[1:(maxvars+1)] }
    if (exactpp) {
      qnull <- 1/qgamma(.001,shape=.5*apos[1],.5*lpos[1])
      qfull <- 1/qgamma(.999,shape=.5*apos[length(apos)],.5*lpos[length(lpos)])
      phiseq <- c(qnull,.5*unique(lpos) / (.5*apos + 1),qfull)
      #Refine grid for phi by adding further promising models
      goodmodelsizes <- which((maxpp - pp[-1]) < log(1000))
      if (length(goodmodelsizes)>0) {
          if (length(goodmodelsizes)!=1 | goodmodelsizes[1]!=ncol(x)) {  #skip if only 1 model with ncol(x) variables remains
              #goodmodelsizes <- 1:(goodmodelsizes[length(goodmodelsizes)])
              goodmodelsizes <- goodmodelsizes[goodmodelsizes <= min(maxvars,ncol(x)-1)]
              phiseqextra <- modelidextra <- modelusumextra <- ppextra <- vector("list",length(goodmodelsizes))
              for (i in 1:length(goodmodelsizes)) {
                  nvars <- goodmodelsizes[i]
                  maxmodels <- lchoose(ncol(x),nvars)
                  if (maxmodels>0) {
                      if (nvars == 1) {
                          idx <- list(2,3,4,5,6) #idx <- list(2,3,4)
                          idx <- idx[idx<=ncol(x)]
                      } else {
                          if (nvars>2) idx <- 1:(nvars-2) else idx <- integer(0)
                          if (nvars+3<=ncol(x)) {
                              idx <- list(c(idx,nvars-1,nvars+1), c(idx,nvars-1,nvars+2), c(idx,nvars,nvars+1), c(idx,nvars-1,nvars+3), c(idx,nvars,nvars+2))
                          } else if (nvars+2<=ncol(x)) {
                              idx <- list(c(idx,nvars-1,nvars+1), c(idx,nvars-1,nvars+2), c(idx,nvars,nvars+1))
                          } else {
                              idx <- list(c(idx,nvars-1,nvars+1), c(idx,nvars,nvars+1))
                          }
                      }
                      idx <- idx[1:min(length(idx),ifelse(maxmodels>=log(6),5,exp(maxmodels)-1))]  #if <5 extra models available, just take those
                      modelidextra[[i]] <- sapply(idx,function(z) paste(o[z][order(o[z])],collapse=','))
                      modelusumextra[[i]] <- sapply(idx, function(z) sum(uord[z]))
                      lposextra <- sumy2 - shrinkage * modelusumextra[[i]]
                      phiseqextra[[i]] <- .5* lposextra / (.5*apos + 1)
                      ppextra[[i]] <- priorModel(nvars) - 0.5*nvars*log(1+g) - 0.5*apos * log(lposextra)
                  }
              }
              phiseq <- c(phiseq,unlist(phiseqextra))
              pp <- c(pp,unlist(ppextra))
              modelid <- c(modelid,unlist(modelidextra))
              modelusum <- c(modelusum,unlist(modelusumextra))
              #variableidsextra <- strsplit(as.character(modelidextra),split=',')
              variableidsextra <- strsplit(unlist(modelidextra),split=',')
              variableids <- c(variableids,variableidsextra)
              nvars <- c(nvars,sapply(variableidsextra,length))
          }
      }
      #Evaluate marginal of phi at selected grid points
      phiseq <- phiseq[order(phiseq,decreasing=TRUE)]
      if (priorCoef@priorDistr == 'zellner') {
          phiseqpost <- jointPhiyZellnerOrtho(phiseq, sumy2, apos, shrinkage, u, g, rho, logscale=TRUE)
      } else {  #pMOM
          phiseqpost <- jointPhiyMOMOrtho(phiseq, sumy2, apos, shrinkage, u, g, rho, logscale=TRUE)
      }
      #Avoid phi values associated to 0 posterior density
      zeropost <- match(-Inf,phiseqpost)
      if (!is.na(zeropost)) {
          zerophi <- phiseq[zeropost]
          phiseq <- phiseq[1:max(1,zeropost-1)]
          phiseqpost <- phiseqpost[1:max(1,zeropost-1)]
      }
      #Refine grid where target increases > tolf % its max value
      maxphipost <- max(phiseqpost)
      tolf <- 0.01
      sel <- which(abs(diff(exp(phiseqpost-maxphipost))) > tolf)
      while (length(sel)>0) {
          phiseqnew <- unlist(lapply(sel, function(z) seq(phiseq[z],phiseq[z+1],length=5)))
          if (priorCoef@priorDistr == 'zellner') {
              phiseqpostnew <- jointPhiyZellnerOrtho(phiseqnew, sumy2, apos, shrinkage, u, g, rho, logscale=TRUE)
          } else {  #pMOM
              phiseqpostnew <- jointPhiyMOMOrtho(phiseqnew, sumy2, apos, shrinkage, u, g, rho, logscale=TRUE)
          }
          maxphipost <- max(maxphipost, max(phiseqpostnew))
          phiseq <- c(phiseq,phiseqnew)
          phiseqpost <- c(phiseqpost,phiseqpostnew)
          ophiseq <- order(phiseq,decreasing=TRUE)
          phiseq <- phiseq[ophiseq]; phiseqpost <- phiseqpost[ophiseq]
          sel <- which(abs(diff(exp(phiseqpost-maxphipost))) > tolf)
      }
      #Compute marginal p(y)
      phiseqpost <- exp(phiseqpost-maxphipost)
      if (integrateMethod=='CSR') {
          myintegral <- int.simpson2(phiseq,phiseqpost,equi=FALSE,method="CSR")
      } else {
          if (priorCoef@priorDistr=='zellner') {
            f2int <- function(phi) { exp(jointPhiyZellnerOrtho(phi,sumy2=sumy2,apos=apos,shrinkage=shrinkage,u=u,g=g,rho=rho,logscale=TRUE) - maxphipost) }
          } else {
            f2int <- function(phi) { exp(jointPhiyMOMOrtho(phi,sumy2=sumy2,apos=apos,shrinkage=shrinkage,u=u,g=g,rho=rho,logscale=TRUE) - maxphipost) }
          }
          myintegral <- integrate(f2int,phiseq[length(phiseq)],Inf)$value
      }
      phiseqpost <- phiseqpost / myintegral  # p(phi|y)
      logpy <- log(myintegral) + maxphipost  #log p(y)
      #Format
      phi <- data.frame(phi=phiseq,phipostprob=phiseqpost)
      phi <- phi[order(phi[,1],decreasing=TRUE),]
      #Exact posterior probabilities
      modelidx <- sapply(strsplit(modelid,split=','),as.numeric)
      nvars <- sapply(modelidx,length)
      if (priorCoef@priorDistr=='zellner') {
          #Old version. It was numerically unstable
          #pp <- priorModel(nvars) - 0.5*nvars*log(g+1) - logpy + lgamma(0.5*apos) - 0.5*apos * log((sumy2 - shrinkage * modelusum)/2)
          #pp <- exp(pp)
          pp <- double(length(modelid))
          w <- phi[,2]/sum(phi[,2])
          for (i in 1:length(modelid)) {
              ppcond <- sapply(phi[,1],jointppZellnerKnownOrtho,sel=modelidx[[i]],u=u,g=g,shrinkage=shrinkage,rho=rho,logscale=FALSE)
              pp[i] <- sum(ppcond * w)
          }
      } else {
          pp <- double(length(modelid))
          w <- phi[,2]/sum(phi[,2])
          for (i in 1:length(modelid)) {
              ppcond <- sapply(phi[,1],jointppMomKnownOrtho,sel=modelidx[[i]],u=u,g=g,shrinkage=shrinkage,rho=rho,logscale=FALSE)
              pp[i] <- sum(ppcond * w)
          }
      }
    } else {
      phi <- NA
      logpy <- NA
      pp <- exp(pp - maxpp)
      pp <- pp/sum(pp)
    }
    #Remove unnecessary grid points
    dd <- abs(diff(phi[,2]))/max(phi[,2])
    ddsum <- 0; sel <- rep(TRUE,nrow(phi))
    for (i in 2:(nrow(phi)-1)) {
        ddsum <- ddsum + dd[i-1]
        if (ddsum < 0.0001) { sel[i] <- FALSE } else { ddsum <- 0 }
    }
    phi <- phi[sel,]
    #
    models <- data.frame(modelid=modelid,pp=pp)
    models <- models[order(pp,decreasing=TRUE),]
    if (sum(models$pp)>1) {
        warning("Posterior model probabilities are unstable, this often signals that t(x) %*% x is not diagonal")
        models$pp <- models$pp/sum(models$pp)
    }
    ans <- list(models=models, phi=phi, logpy=logpy)
    if (bma) {
        margpp <- double(p)
        postphigrid <- phi[,2]/sum(phi[,2])
        if (priorCoef@priorDistr=='zellner') {
            m <- shrinkage * Xty / XtX
            for (i in 1:length(u)) {
                margppcond <- margppZellnerKnownOrtho(phi=phi[,1],u=u[i],g=g,shrinkage=shrinkage,rho=rho,logscale=FALSE)
                margpp[i] <- sum(margppcond * postphigrid)
                #margpp[i] <- int.simpson2(phi[,1], margppcond * phi[,2], equi=FALSE, method="CSR")
            }
            pm <- m * margpp
        } else {
            m <- shrinkage * Xty / XtX
            pm <- double(p)
            for (i in 1:length(u)) {
                mi <- m[i] * (1 + 2 / (1+shrinkage*u[i]/phi[,1]))
                margppcond <- margppMomKnownOrtho(phi=phi[,1],u=u[i],g=g,shrinkage=shrinkage,rho=rho,logscale=FALSE)
                margpp[i] <- sum(margppcond * postphigrid)
                pm[i] <- sum(mi * margppcond * postphigrid)
                #margpp[i] <- int.simpson2(phi[,1], margppcond * phi[,2], equi=FALSE, method="CSR")
                #pm[i] <- int.simpson2(phi[,1], mi * margppcond * phi[,2], equi=FALSE, method="CSR")
            }
        }
        ans$bma <- data.frame(margpp=margpp,coef=pm)
    }
    return(ans)
}




postModeBlockDiag <- function(y, x, blocks, priorCoef=zellnerprior(tau=nrow(x)), priorModel=modelbinomprior(p=1/ncol(x)), priorVar=igprior(0.01,0.01), bma=FALSE, maxvars=100, momcoef) {
    #Find posterior mode for linear regression under block-diagonal X'X
    # - y: outcome
    # - x: predictors
    # - blocks: factor or integer of length ncol(x) indicating the block that each column in x belongs to
    # - priorCoef: prior on coefficients
    # - priorModel: prior on model space
    # - priorVar: prior on residual variance
    # - bma: set to TRUE to obtain Bayesian model averaging parameter estimates
    # - integrateMethod: if exactpp=TRUE this is the numerical integration method to obtain logpy. Set to 'CSR' to compound Simpson's rule based on an adaptive grid defined by enumerating highly probable models, to 'AQ' to use adaptive quadrature as implemented in R function 'integrate'
    # - maxvars: the search is restricted to models with up to maxvars variables (note: posterior model prob and BMA are valid regardless of maxvars)
    # - momcoef: optional argument containing pre-computed coefficients needed to obtain the marginal likelihood under the pMOM prior. A first call to postModeBlockDiag returns these coefficients, thus this argument is useful to speed up successive calls.
    # Output
    # -
    integrateMethod <- 'AQ'; exactpp <- TRUE #exactpp=FALSE means we only find posterior mode (for Zellner's prior, for pMOM this doesn't work)
    n <- length(y); p <- ncol(x)
    if (length(blocks) != p) stop('length(blocks) should be equal to ncol(x)')
    blocks <- as.integer(factor(blocks))
    nblocks <- max(blocks)
    blocksize <- table(blocks)
    nn <- 1:ncol(x)
    varidx <- lapply(as.integer(names(blocksize)), function(k) nn[blocks==k])
    if (priorModel@priorDistr=='binomial' & ('p' %in% names(priorModel@priorPars))) {
        rho <- priorModel@priorPars[['p']]
        if (length(rho)>1) stop("postModeBlockDiag not implemented for vector p in modelbinomprior")
        if (any(rho<0) | any(rho>1)) stop("Specified prior inclusion probability outside [0,1]. Check priorModel")
        priorModel <- function(nvar) nvar*log(rho) + (p-nvar)*log(1-rho)
        priorModelBlock <- function(nvar,blocksize) nvar*log(rho) + (blocksize-nvar)*log(1-rho)
    } else if (priorModel@priorDistr=='binomial' & !('p' %in% names(priorModel@priorPars))) {
        stop("Beta-Binomial prior not currently implemented")
        #alpha=priorModel@priorPars['alpha.p']; beta=priorModel@priorPars['beta.p']
        #priorModel <- function(nvar) lbeta(nvar + alpha, p - nvar + beta) - lbeta(alpha, beta)
    } else if (priorModel@priorDistr=='uniform') {
        rho <- 0.5
        priorModel <- function(nvar) rep(-p*log(2),length(nvar))
        priorModelBlock <- function(nvar,blocksize) rep(-blocksize*log(2),length(nvar))
    } else { stop("Prior on model space not recognized. Use modelbbprior(), modelunifprior() or modelbinomprior()") }
    if (priorCoef@priorDistr == 'zellner') {
        g <- as.double(priorCoef@priorPars['tau'])
    } else if (priorCoef@priorDistr == 'pMOM') {
        tau <- as.double(priorCoef@priorPars['tau'])
        g <- n * tau
    } else {
        stop("priorCoef must be pMOM or Zellner. Use momprior() or zellnerprior()")
    }
    if (priorVar@priorDistr != 'invgamma') stop('priorVar must be an inverse gamma. Use igprior()')
    a.phi <- as.double(priorVar@priorPars['alpha'])
    l.phi <- as.double(priorVar@priorPars['lambda'])
    if (exactpp & !(integrateMethod %in% c('CSR','AQ'))) {
        stop("integrateMethod should be 'CSR' for compound Simpson's rule or 'AQ' for adaptive quadrature")
    }
    #Pre-compute useful quantities
    sumy2 <- l.phi + sum(y^2)
    apos <- a.phi+n
    shrinkage <- g/(1+g)
    Xty <- t(x) %*% matrix(y,ncol=1)
    XtX <- lapply(1:nblocks,function(i) { sel <- which(blocks==i); t(x[,sel]) %*% x[,sel] })
    #Compute u-scores, least squares estimates and covariances within blocks
    ubyblocks <- blockuscores(blocks, blocksize=blocksize, y=y, Xty=Xty, XtX=XtX)
    u <- ubyblocks$u; umv <- ubyblocks$umv; ubest <- ubyblocks$ubest
    #Avoid overflow in cases where X'X not exactly block-diagonal
    sumumax <- sum(sapply(u, function(z) z$u[1]))
    if (sumumax > (sumy2-l.phi)) {
        u <- lapply(u, function(z) { z$u <- z$u * ((sumy2-l.phi)/sumumax); return(z) } )
        ubest <- lapply(ubest, function(z) { z$u <- z$u * ((sumy2-l.phi)/sumumax); return(z) } )
        for (i in 1:length(umv)) { umv[[i]] <- lapply(umv[[i]], function(z) { z$u <- z$u * ((sumy2-l.phi)/sumumax); return(z) } ) }
    }
    #Enumerate models
    models <- coolblock(ubest,g=g,priorModelBlock=priorModelBlock,varidx=varidx,maxvars=maxvars)
    models <- rbind(data.frame(nvars=0,u=0,modelid='',phi=Inf,m=NA,block=NA,u.lower=0,u.upper=0,stringsAsFactors=FALSE),models)
    lpos.upper <- sumy2 - shrinkage * models$u.upper
    pp.upper <- priorModel(models$nvars) - 0.5*models$nvars*log(1+g) - 0.5*apos * log(lpos.upper)
    mis <- is.na(models$u) & (pp.upper>max(pp.upper[!is.na(models$u)],na.rm=TRUE))
    if (any(mis)) {
        enumsizes <- models$nvar[mis & (pp.upper>max(pp.upper[!mis]))]
        for (i in 1:length(enumsizes)) {
            #Enumerate all models candidate to be best of variable enumsizes[i]
            candidateModels <- enumblock(enumsizes[i], blocksize=blocksize)
            #Find u-scores
            candidateU <- matrix(0, nrow=nrow(candidateModels), ncol=ncol(candidateModels))
            for (k in 1:ncol(candidateModels)) {
                sel <- candidateModels[,k]>0
                candidateU[sel,k] <- ubest[[k]][candidateModels[sel,k],'u']
            }
            candidateU <- rowSums(candidateU)
            sel <- which.max(candidateU)
            models$u[enumsizes[i]+1] <- models$u.lower[enumsizes[i]+1] <- models$u.upper[enumsizes[i]+1] <- candidateU[sel]
            #Figure out model id
            modelsel <- character(ncol(candidateModels))
            for (k in 1:ncol(candidateModels)) {
                if (candidateModels[sel,k]>0) {
                    modelsel[k] <- paste(varidx[[k]][as.integer(strsplit(ubest[[k]][candidateModels[sel,k],'modelid'], split=',')[[1]])],collapse=',')
                }
            }
            models$modelid[enumsizes[i]+1] <- paste(modelsel[modelsel!=""],collapse=',')
            pp.upper[enumsizes[i]+1] <- priorModel(enumsizes[i]) - 0.5*enumsizes[i]*log(1+g) - 0.5*apos * log(sumy2 - shrinkage * candidateU[sel])
        }
    }
    #Marginal posterior of phi
    phiseq <- models$phi[!is.na(models$phi)][-1]
    qnull <- 1/qgamma(.001,shape=.5*apos,.5*sumy2)
    lfull <- max(c(sumy2 - shrinkage * max(models$u,na.rm=TRUE),l.phi))
    qfull <- 1/qgamma(.999,shape=.5*apos,.5*lfull)
    if (qnull > phiseq[1]) phiseq <- c(qnull,phiseq)
    if (qfull < phiseq[length(phiseq)]) phiseq <- c(phiseq,qfull)
    #Evaluate marginal of phi at selected grid points
    if (priorCoef@priorDistr == 'zellner') {
        phiseqpost <- jointPhiyZellnerBlockDiag(phiseq,sumy2=sumy2,apos=apos,a.phi=a.phi,l.phi=l.phi,shrinkage=shrinkage,u=u,g=g,priorModelBlock=priorModelBlock,blocksize=blocksize,logscale=TRUE)
        maxphipost <- max(phiseqpost)
        while (phiseqpost[length(phiseqpost)]==maxphipost) {
            phiseq <- c(phiseq,phiseq[length(phiseq)]/2)
            phiseqpost <- c(phiseqpost,jointPhiyZellnerBlockDiag(phiseq[length(phiseq)],sumy2=sumy2,apos=apos,a.phi=a.phi,l.phi=l.phi,shrinkage=shrinkage,u=u,g=g,priorModelBlock=priorModelBlock,blocksize=blocksize,logscale=TRUE))
            maxphipost <- max(c(maxphipost,phiseqpost[length(phiseqpost)]))
        }
    } else {  #pMOM
        if (missing(momcoef)) {
            momcoef <- vector("list",length(umv))
            for (k in 1:length(momcoef)) momcoef[[k]] <- lapply(umv[[k]], momPolyCoef, shrinkage=shrinkage, g=g)
        }
        phiseqpost <- jointPhiyMOMBlockDiag(phiseq,sumy2=sumy2,apos=apos,a.phi=a.phi,l.phi=l.phi,shrinkage=shrinkage,u=u,umv=umv,momcoef=momcoef,g=g,priorModelBlock=priorModelBlock,blocksize=blocksize)
        maxphipost <- max(phiseqpost)
        while (phiseqpost[length(phiseqpost)]==maxphipost) {
            phiseq <- c(phiseq,phiseq[length(phiseq)]/2)
            phiseqpost <- c(phiseqpost,jointPhiyMOMBlockDiag(phiseq[length(phiseq)],sumy2=sumy2,apos=apos,a.phi=a.phi,l.phi=l.phi,shrinkage=shrinkage,u=u,umv=umv,momcoef=momcoef,g=g,priorModelBlock=priorModelBlock,blocksize=blocksize,logscale=TRUE))
            maxphipost <- max(c(maxphipost,phiseqpost[length(phiseqpost)]))
        }
    }
    #Refine grid where target increases > tolf % its max value
    tolf <- 0.01
    sel <- which(abs(diff(exp(phiseqpost-maxphipost))) > tolf)
    itrefine <- 0
    while ((length(sel)>0) && (itrefine<10)) {
        phiseqnew <- unlist(lapply(sel, function(z) { ans= seq(phiseq[z],phiseq[z+1],length=5); ans[c(-1,-length(ans))] } ))
        if (priorCoef@priorDistr == 'zellner') {
            phiseqpostnew <- jointPhiyZellnerBlockDiag(phiseqnew, sumy2, apos, a.phi, l.phi, shrinkage, u, g, priorModelBlock, blocksize, logscale=TRUE)
        } else {  #pMOM
            phiseqpostnew <- jointPhiyMOMBlockDiag(phiseqnew, sumy2, apos, a.phi, l.phi, shrinkage, u, umv, momcoef, g, priorModelBlock, blocksize, logscale=TRUE)
        }
        maxphipost <- max(maxphipost, max(phiseqpostnew))
        phiseq <- c(phiseq,phiseqnew)
        phiseqpost <- c(phiseqpost,phiseqpostnew)
        ophiseq <- order(phiseq,decreasing=TRUE)
        phiseq <- phiseq[ophiseq]; phiseqpost <- phiseqpost[ophiseq]
        sel <- which(abs(diff(exp(phiseqpost-maxphipost))) > tolf)
        itrefine <- itrefine+1
    }
    #Compute marginal p(y)
    zeropost <- which(is.infinite(phiseqpost))
    if (length(zeropost)>1) { phiseq <- phiseq[-zeropost[-1]]; phiseqpost <- phiseqpost[-zeropost[-1]] }
    phiseqpost <- exp(phiseqpost-maxphipost)
    if (integrateMethod=='CSR') {
        myintegral <- int.simpson2(phiseq[phiseqpost!=0],phiseqpost[phiseqpost!=0],equi=FALSE,method="CSR")
    } else {
        if (priorCoef@priorDistr=='zellner') {
            f2int <- function(phi) { exp(jointPhiyZellnerBlockDiag(phi,sumy2=sumy2,apos=apos,a.phi=a.phi,l.phi=l.phi,shrinkage=shrinkage,u=u,g=g,priorModelBlock=priorModelBlock,blocksize=blocksize,logscale=TRUE) - maxphipost) }
        } else {
            f2int <- function(phi) { exp(jointPhiyMOMBlockDiag(phi,sumy2=sumy2,apos=apos,a.phi=a.phi,l.phi=l.phi,shrinkage=shrinkage,u=u,umv=umv,momcoef=momcoef,g=g,priorModelBlock=priorModelBlock,blocksize=blocksize,logscale=TRUE) - maxphipost) }
        }
        myintegral <- integrate(f2int,phiseq[length(phiseq)],Inf)$value
    }
    phiseqpost <- phiseqpost / myintegral  # p(phi|y)
    logpy <- log(myintegral) + maxphipost  #log p(y)
    #Format
    phi <- data.frame(phi=phiseq,phipostprob=phiseqpost)
    phi <- phi[order(phi[,1],decreasing=TRUE),]
    #Remove unnecessary grid points
    dd <- abs(diff(phi[,2]))/max(phi[,2])
    ddsum <- 0; sel <- rep(TRUE,nrow(phi))
    for (i in 2:(nrow(phi)-1)) {
        ddsum <- ddsum + dd[i-1]
        if (ddsum < 0.0001) { sel[i] <- FALSE } else { ddsum <- 0 }
    }
    phi <- phi[sel,]
    #Exact posterior probabilities
    nvars <- models$nvars
    if (priorCoef@priorDistr=='zellner') {
        pp.upper <- priorModel(nvars) - 0.5*nvars*log(g+1) - logpy + lgamma(0.5*apos) - 0.5*apos * log((sumy2 - shrinkage*models$u.upper)/2)
        pp.upper <- exp(pp.upper)
        pp <- ifelse(is.na(models$u),NA,pp.upper)
    } else {
        modelidx <- sapply(strsplit(models$modelid,split=','), function(z) { ans= rep(FALSE,p); ans[as.numeric(z)] <- TRUE; return(ans) })
        pp <- double(nrow(models))
        w <- phi[,2]/sum(phi[,2])
        for (i in 1:length(pp)) {
            ppcond <- sapply(phi[,1],jointppMomKnownBlockDiag,sel=modelidx[,i],blocks=blocks,blocksize=blocksize,u=u,g=g,shrinkage=shrinkage,priorModelBlock=priorModelBlock,momcoef=momcoef,logscale=FALSE)
            pp[i] <- sum(ppcond * w)
        }
        pp.upper <- pp
    }
    models <- data.frame(models[,c('modelid','nvars')],pp=pp,pp.upper=pp.upper)
    ans <- list(models=models, phi=phi, logpy=logpy)
    #Coefficient estimates under each visited model
    modelidx <- sapply(strsplit(models$modelid,split=','), function(z) { ans= rep(FALSE,p); ans[as.numeric(z)] <- TRUE; return(ans) })
    pmcond <- matrix(0, nrow=nrow(models), ncol=p)
    for (i in 2:nrow(models)) {
        for (k in 1:length(umv)) {
            includeidx <- which(modelidx[blocks==k,i])
            if (length(includeidx)>0) {
                mincluded <- rep(0,blocksize[k])
                mincluded[includeidx] <- umv[[k]][[paste(includeidx,collapse=',')]]$m
                pmcond[i,blocks==k] <- mincluded
            }
        }
    }
    if (priorCoef@priorDistr=='zellner') pmcond <- shrinkage * pmcond
    ans$postmean.model <- data.frame(modelid=models$modelid,pmcond)
    #Marginal inclusion probabilities and BMA estimates
    if (bma) {
        margpp <- pm <- double(p)
        postphigrid <- phi[,2]/sum(phi[,2])
        if (priorCoef@priorDistr=='zellner') {
            for (k in 1:length(u)) {
                margppcond <- margppZellnerKnownBlockDiag(phi=phi[,1],u=u[[k]],umv=umv[[k]],g=g,shrinkage=shrinkage,priorModelBlock=priorModelBlock,logscale=FALSE)
                margpp[blocks==k] <- colSums(margppcond$margpp * postphigrid)
                pm[blocks==k] <- colSums(margppcond$coef * postphigrid)
            }
        } else {
            margpp <- pm <- double(p)
            for (k in 1:length(u)) {
                margppcond <- margppMOMKnownBlockDiag(phi=phi[,1],u=u[[k]],umv=umv[[k]],g=g,shrinkage=shrinkage,priorModelBlock=priorModelBlock,momcoef=momcoef[[k]],logscale=FALSE)
                margpp[blocks==k] <- colSums(margppcond$margpp * postphigrid)
                pm[blocks==k] <- colSums(margppcond$coef * postphigrid)
            }
        }
        ans$bma <- data.frame(margpp=margpp,coef=pm)
    }
    if (priorCoef@priorDistr=='pMOM') ans$momcoef <- momcoef
    return(ans)
}


blockuscores <- function(blocks, blocksize, y, Xty, XtX) {
    u <- umv <- ubest <- vector("list",length(blocksize))
    for (i in 1:length(u)) {
        Xtyblock <- Xty[blocks==i]
        modelsblock <- expand.grid(lapply(1:blocksize[i], function(z) c(FALSE,TRUE)))
        umv[[i]] <- apply(modelsblock[-1,,drop=FALSE], 1, function(sel) uscore(y,XtX[[i]][sel,sel],Xtyblock[sel]))
        ublock <- c(0,sapply(umv[[i]],'[[',1)) #null model has u-score=0
        nvarsblock <- rowSums(modelsblock)
        o <- order(nvarsblock,ublock,decreasing=c(TRUE,FALSE))
        modelid <- apply(modelsblock,1,function(z) paste(which(z),collapse=','))
        u[[i]] <- data.frame(modelid=modelid[o],u=ublock[o],nvars=nvarsblock[o],stringsAsFactors=FALSE)
        names(umv[[i]]) <- modelid[-1]
        ubest[[i]] <- do.call(rbind,by(u[[i]], INDICES=u[[i]][,'nvars'], FUN=function(z) z[1,]))[-1,] #best model of each size
    }
    return(list(u=u,umv=umv,ubest=ubest))
}


coolblock <- function(ubest, g, priorModelBlock, varidx, maxvars) {
    #Coolblock algorithm. Returns sequence of models with highest post prob under Zellner's prior conditional on any value for phi (residual variance)
    #Input
    # - ubest: list with as many elements as blocks. For each block it contains a data.frame with the best model (largest u-score) of each size
    # - g: Zellner prior dispersion parameter
    # - priorModelBlock: function taking a single integer argument k and returning the log-prior probability of including k variables within a block
    # - varidx: list with as many elements as blocks. For each block it indicates the indexes of the variables in that block
    # - maxvars: stop when >maxvars included into the model
    #Output: data.frame with info about best model of each size, containing the following columns
    # - nvars: number of variables
    # - u: u-score for best model of that size (may be missing for models not visited by coolblock)
    # - modelid: model identifier (comma-separated variable indexes for variables included in the model)
    # - phi: decreasing thresholds on residual variance. modelid[i] has highest posterior prob conditional on any residual variance <=phi[i] and >=phi[i+1]
    # - m: the posterior mode given phi[i] is obtained by setting active variables in block[i] to m[i]
    # - block: the posterior mode given phi[i] is obtained by setting active variables in block[i] to m[i]
    # - u.upper: upper bound for u (equals the exact u if this model size was visited by the algorithm)
    # - u.lower: lower bound for u (equals the exact u if this model size was visited by the algorithm)
    K <- length(ubest)
    shrinkage <- g/(1+g)
    log1g <- log(1+g)
    nvars <- sapply(ubest,nrow)
    #Store prior odds. The code takes advantage that for uniform/binom the odds don't depend on the block size
    #priorodds[[l]][j] has prior odds of l vars vs j vars
    priorodds <- lapply(1:max(nvars), function(l) { priorModelBlock(l,blocksize=max(nvars)) - sapply(0:(l-1), function(j) priorModelBlock(j,blocksize=max(nvars))) })
    #Step 1. Within-blocks optimal configuration
    #
    phiseq <- r <- m <- vector("list",K)
    for (k in 1:length(phiseq)) {
        #Obtain all pairwise r scores
        r[[k]] <- matrix(NA,nrow=nvars[k]+1,ncol=nvars[k]+1)
        rownames(r[[k]]) <- colnames(r[[k]]) <- 0:nvars[k]
        r[[k]][-1,1] <- r[[k]][1,-1] <- shrinkage * ubest[[k]][,'u'] / (ubest[[k]]$nvars * log1g - 2 * sapply(priorodds[ubest[[k]]$nvars],'[[',1))
        #r[[k]][-1,1] <- r[[k]][1,-1] <- shrinkage * ubest[[k]][,'u'] / (ubest[[k]]$nvars * log1g - 2 * priorodds[[nvars[k]]][ubest[[k]]$nvars])
        if (nvars[k]>=2) {
          for (l in 2:nvars[k]) {
              for (j in 1:(l-1)) {
                  r[[k]][l+1,j+1] <- r[[k]][j+1,l+1] <- shrinkage * (ubest[[k]][l,'u'] - ubest[[k]][j,'u']) / ((l-j) * log1g - 2 * priorodds[[l]][j+1])
              }
          }
        }
        phiseq[[k]] <- double(nvars[k])
        m[[k]] <- character(nvars[k])
        ll <- 1
        if (nvars[k]>=2) {
            for (l in 1:(nvars[k]-1)) {
                rmin <- min(r[[k]][l+1,1:l])
                rmax <- max(r[[k]][(l+2):(nvars[k]+1),l+1])
                if (rmax < rmin) {
                    phiseq[[k]][ll] <- rmin
                    m[[k]][ll] <- as.character(ubest[[k]][l,'modelid'])
                    ll <- ll+1
                }
            }
        }
        l <- nvars[k]  #treat last case separately (no larger models to compare with)
        phiseq[[k]][ll] <- min(r[[k]][l+1,1:l])
        m[[k]][ll] <- as.character(ubest[[k]][l,'modelid'])
        phiseq[[k]] <- phiseq[[k]][1:ll]
        m[[k]] <- m[[k]][1:ll]
    }
    #
    #Step 2. Combine blocks
    #
    phiseq <- data.frame(phi=unlist(phiseq),m=unlist(m),block=rep(1:length(phiseq),sapply(phiseq,length)),stringsAsFactors=FALSE)
    phiseq <- phiseq[order(phiseq[,1],decreasing=TRUE),]
    #
    #Step 3. Sequence of models
    #
    modelid <- character(nrow(phiseq))
    nvarsmodel <- integer(nrow(phiseq))
    useq <- double(nrow(phiseq))
    varidxWithinBlock <- strsplit(phiseq$m,split=',')
    curmodel <- curvars <- lapply(1:length(ubest), function(k) character(0))
    curu <- rep(0,length(ubest))
    i <- 1; stopvars <- FALSE
    while ((i <= nrow(phiseq) & (!stopvars))) {
        curblock <- phiseq$block[[i]]
        curmodel[[curblock]] <- phiseq$m[i]
        curvars[[curblock]] <- varidx[[curblock]][as.numeric(varidxWithinBlock[[i]])]
        nn <- unlist(curvars)
        modelid[i] <- paste(nn,collapse=',')
        nvarsmodel[i] <- length(nn)
        curu[curblock] <- ubest[[curblock]][length(varidxWithinBlock[[i]]),'u']
        useq[i] <- sum(curu)
        if (nvarsmodel[i]>maxvars) stopvars <- TRUE
        i <- i+1
    }
    if (!stopvars) {
        modelseq <- data.frame(nvars=nvarsmodel,u=useq,modelid=modelid,phiseq,stringsAsFactors=FALSE)
    } else {
        sel <- 1:(i-1)
        modelseq <- data.frame(nvars=nvarsmodel[sel],u=useq[sel],modelid=modelid[sel],phiseq[sel,],stringsAsFactors=FALSE)
    }
    rownames(modelseq) <- modelseq$nvars
    ans <- data.frame(nvars=1:max(modelseq$nvars),u=NA,modelid=NA,phi=NA,m=NA,block=NA)
    ans[modelseq$nvars,] <- modelseq
    #Best models of size 1 & 2 are always returned
    if (any(is.na(ans$u[1:2]))) onevar <- data.frame(block=1:length(ubest),do.call(rbind,lapply(ubest,function(z) z[1,])), stringAsFactors=FALSE)
    if (is.na(ans$u[1])) {
        sel <- which.max(onevar$u)
        ans$u[1] <- onevar$u[sel]
        ans$block[1] <- onevar$block[sel]
        ans$modelid[1] <- ans$m[1] <- varidx[[onevar$block[sel]]][as.integer(onevar$modelid[sel])]
    }
    if (is.na(ans$u[2])) {
        sel1 <- ((1:nrow(onevar))[order(onevar$u,decreasing=TRUE)])
        sel1 <- sel1[1:min(2,length(sel1))]
        u1 <- sum(onevar$u[sel1])
        if (any(nvars>1)) {
            twovar <- data.frame(block=1:length(ubest),do.call(rbind,lapply(ubest,function(z) z[2,])), stringAsFactors=FALSE)
            sel2 <- which.max(twovar$u)
            u2 <- twovar$u[sel2]
        }
        if (all(nvars==1) | u1>u2) {
            ans$u[2] <- u1
            ans$block[2] <- onevar$block[sel1][2]
            ans$modelid[2] <- paste(varidx[[onevar$block[sel1[1]]]][as.integer(onevar$modelid[sel1[1]])],varidx[[onevar$block[sel1[2]]]][as.integer(onevar$modelid[sel1[2]])],sep=',')
            ans$m[2] <- onevar$modelid[sel1[2]]
        } else {
            ans$u[2] <- u2
            ans$block[2] <- twovar$block[sel2]
            ans$modelid[2] <- paste(varidx[[twovar$block[sel2]]][as.integer(strsplit(twovar$modelid[sel2],split=',')[[1]])],collapse=',')
            ans$m[2] <- twovar$modelid[sel2]
        }
    }
    #
    #Step 4. Bound u-score for non-visited model sizes
    #
    mis <- which(is.na(ans$u[-nrow(ans)]))
    ans$u.upper <- ans$u.lower <- ans$u
    if (length(mis)>0) {
        for (i in 1:length(mis)) {
            l <- mis[i]+1
            while (is.na(ans$u[l]) & (l<=nrow(ans))) l <- l+1
            ans$u.upper[mis[i]] <- ans$u[l]
            l <- mis[i]-1
            while (is.na(ans$u[l]) & (l>=1)) l <- l-1
            ans$u.lower[mis[i]] <- ans$u[l]
        }
    }
    if (is.na(ans$u[nrow(ans)])) { ans$u.lower[nrow(ans)] <- ans$u[nrow(ans)-1] }
    return(ans)
}



## Auxiliary functions for block-diagonal design ##
###################################################

#Marginal inclusion probability & coefficient estimates for a block of variables variable conditional on a vector of phi values
margppZellnerKnownBlockDiag <- function(phi,u,umv,g,shrinkage,priorModelBlock,logscale=FALSE) {
    maxvars <- max(u$nvars)
    modelidx <- sapply(strsplit(u$modelid,split=','), function(z) { ans= rep(FALSE,maxvars); ans[as.numeric(z)] <- TRUE; return(ans) })
    priorpp <- priorModelBlock(0:maxvars,blocksize=maxvars)
    gpow <- (-(0:maxvars)/2) * log(g+1)
    margpp <- coefest <- matrix(NA,nrow=length(phi),ncol=maxvars)
    modelcoef <- shrinkage * sapply(u$modelid, function(z) { ans= rep(0,maxvars); ans[as.numeric(strsplit(z,split=',')[[1]])]= umv[[z]]$m; return(ans) })
    for (i in 1:nrow(margpp)) {
        h <- gpow[u$nvars+1] + .5*(shrinkage*u$u)/phi[i] + priorpp[u$nvars+1]
        hmax <- max(h)
        pp <- exp(h-hmax)/sum(exp(h-hmax))
        margpp[i,] <- as.vector(modelidx %*% pp)
        coefest[i,] <- as.vector(modelcoef %*% pp)
    }
    return(list(margpp=margpp,coef=coefest))
}


margppMOMKnownBlockDiag <- function(phi,u,umv,g,shrinkage,priorModelBlock,momcoef,logscale=FALSE) {
    #Note: posterior mean under MOM is approximated using MLE
    maxvars <- max(u$nvars)
    modelidx <- sapply(strsplit(u$modelid,split=','), function(z) { ans= rep(FALSE,maxvars); ans[as.numeric(z)] <- TRUE; return(ans) })
    priorpp <- priorModelBlock(0:maxvars,blocksize=maxvars)
    gpow <- (-(0:maxvars)/2) * log(g+1)
    margpp <- coefest <- matrix(NA,nrow=length(phi),ncol=maxvars)
    modelcoef <- sapply(u$modelid, function(z) { ans= rep(0,maxvars); ans[as.numeric(strsplit(z,split=',')[[1]])]= umv[[z]]$m; return(ans) })
    for (i in 1:length(phi)) {
        phipow <- 1/(phi[i]^(0:maxvars))
        logf <- log(sapply(momcoef, function(z) sum(z*phipow[1:length(z)])))
        logf <- c(logf[u$modelid[u$modelid!='']],0) #last model is null model
        h <- gpow[u$nvars+1] + .5*(shrinkage*u$u)/phi[i] + priorpp[u$nvars+1] + logf
        hmax <- max(h)
        pp <- exp(h-hmax)/sum(exp(h-hmax))
        margpp[i,] <- as.vector(modelidx %*% pp)
        coefest[i,] <- as.vector(modelcoef %*% pp)
    }
    return(list(margpp=margpp,coef=coefest))
}


#Joint model probability conditional on a single phi value
#Input
# - phi: single value for residual variance
# - sel: logical vector of length p indicating the active variables
# - block: vector of length p indicating the block that each variable belongs to
# - u: u-scores for all variables (i.e. u[sel] selects those for active variables)
# - g, shrinkage, rho, logscale: as in margppMomKnown
#Output: posterior probability of the model conditional on phi
jointppMomKnownBlockDiag <- function(phi,sel,blocks,blocksize,u,g,shrinkage=g/(1+g),priorModelBlock,momcoef,logscale=TRUE) {
    priorpp <- lapply(1:length(u), function(k) priorModelBlock(0:blocksize[k],blocksize=blocksize[k]))
    gpow <- (-(0:max(blocksize))/2) * log(g+1)
    phipow <- 1/(phi^(0:max(blocksize)))
    ans <- 0
    for (k in 1:length(u)) {
        nactive <- sum(sel[blocks==k])
        idx <- paste(which(sel[blocks==k]),collapse=',')
        logf <- log(sapply(momcoef[[k]], function(z) sum(z*phipow[1:length(z)])))
        logf <- c(logf[u[[k]]$modelid[u[[k]]$modelid!='']],0) #last model is null model
        h <- gpow[u[[k]]$nvars+1] + .5*(shrinkage*u[[k]]$u)/phi + priorpp[[k]][u[[k]]$nvars+1] + logf
        names(h) <- u[[k]]$modelid
        hmax <- max(h)
        ans <- ans + (ifelse(idx=="",h[length(h)],h[idx])-hmax) - log(sum(exp(h-hmax)))
        #exp(h[idx]) / sum(exp(h))
    }
    if (!logscale) ans <- exp(ans)
    return(ans)
}


#Evaluate quantity proportional to joint of (phi,y) on a grid
jointPhiyZellnerBlockDiag <- function(phiseq, sumy2, apos, a.phi, l.phi, shrinkage, u, g, priorModelBlock, blocksize, logscale=TRUE) {
    umax <- sapply(u, function(z) z$u[1])
    sumumax <- sum(umax)
    #Avoid overflow in cases where X'X not exactly block-diagonal
    if (sumumax > (sumy2-l.phi)) {
        umax <- umax/sumumax * (sumy2-l.phi)
        sumumax <- sum(umax)
    }
    ans <- -0.5*(sumy2-sumumax)/phiseq - (.5*apos+1)*log(phiseq)
    priorpp <- lapply(1:length(u), function(k) priorModelBlock(0:blocksize[k],blocksize=blocksize[k]))
    gpow <- (-(0:max(blocksize))/2) * log(g+1)
    for (i in 1:length(phiseq)) {
        hsum <- 0
        for (k in 1:length(u)) {
            hsum <- sum(exp(gpow[u[[k]]$nvars+1] + .5*(shrinkage*u[[k]]$u - umax[k])/phiseq[i] + priorpp[[k]][u[[k]]$nvars+1]))
            ans[i] <- ans[i] + sum(log(hsum))
        }
    }
    ans[is.infinite(ans)] <- -Inf  #avoid numerical overflow
    if (!logscale) ans <- exp(ans)
    return(ans)
}


jointPhiyMOMBlockDiag <- function(phiseq, sumy2, apos, a.phi, l.phi, shrinkage, u, umv, momcoef, g, priorModelBlock, blocksize, logscale=TRUE) {
    umax <- sapply(u, function(z) z$u[1])
    sumumax <- sum(umax)
    #Avoid overflow in cases where X'X not exactly block-diagonal
    if (sumumax > (sumy2-l.phi)) {
        umax <- umax/sumumax * (sumy2-l.phi)
        sumumax <- sum(umax)
    }
    ans <- -0.5*(sumy2-sumumax)/phiseq - (.5*apos+1)*log(phiseq)
    priorpp <- lapply(1:length(u), function(k) priorModelBlock(0:blocksize[k],blocksize=blocksize[k]))
    gpow <- (-(0:max(blocksize))/2) * log(g+1)
    for (i in 1:length(phiseq)) {
        phipow <- 1/(phiseq[i]^(0:max(blocksize)))
        for (k in 1:length(u)) {
            logf <- log(sapply(momcoef[[k]], function(z) sum(z*phipow[1:length(z)])))
            logf <- logf[u[[k]]$modelid]
            logf[length(logf)] <- 0 #last model is null model
            #logf <- sapply(umv[[k]][u[[k]]$modelid], function(z) log(eprod(shrinkage*z$m, z$V * (shrinkage*phiseq[i]), power=2)) - sum(log(diag(z$V)))) - u[[k]]$nvars * log(phiseq[i]*g) #same but slower
            hsum <- sum(exp(gpow[u[[k]]$nvars+1] + .5*(shrinkage*u[[k]]$u - umax[k])/phiseq[i] + priorpp[[k]][u[[k]]$nvars+1] + logf))
            ans[i] <- ans[i] + sum(log(hsum))
        }
    }
    ans[is.infinite(ans)] <- -Inf  #avoid numerical overflow
    if (!logscale) ans <- exp(ans)
    return(ans)
}


momPolyCoef <- function(z, shrinkage, g) {
    #Pre-compute polynomial coefficients for fast evaluation of pMOM marginal likelihood given phi
    # - z: z$m contains posterior mean, z$V the inverse of X'X, e.g. as stored in umv[[k]][[j]] and returned by uscore
    nvars <- nrow(z$m)
    ans <- double(nvars+1)
    v <- as.matrix(expand.grid(lapply(1:nvars, function(zz) 0:2)))
    h <- 1-v
    unitv <- (-2)^rowSums(v==1)
    R <- cov2cor(z$V)
    mstd <- shrinkage * z$m / sqrt(diag(z$V))
    #r=0
    ans[1]= sum(exp(sapply(1:nrow(h), function(i) { nvars*log((matrix(h[i,],nrow=1) %*% mstd)^2) }) - lfactorial(2*nvars) - nvars*log(g)) * unitv)
    if (nvars>1) {
        nvlog <- nvars*log(1+g)
        for (r in 1:(nvars-1)) {
            ss <- sapply(1:nrow(h), function(i) { r * (log(matrix(h[i,],nrow=1) %*% R %*% matrix(h[i,],ncol=1)) - log(2)) + (nvars-r)*log((matrix(h[i,],nrow=1) %*% mstd)^2) })
            ans[r+1] <- sum(exp(ss -lfactorial(r) - lfactorial(2*(nvars-r)) -nvlog + (nvars-r)*log(shrinkage)) * unitv)
        }
    }
    ans[nvars+1]= sum(exp(sapply(1:nrow(h), function(i) { nvars * (log(matrix(h[i,],nrow=1) %*% R %*% matrix(h[i,],ncol=1)) - log(2)) }) -lfactorial(r) - nvars*log(1+g)) * unitv)
    return(rev(ans))
}


enumblock <- function(l, blocksize, usedvars=0, curblocks=NULL, blockstart=1) {
    #Enumerate all models of given size l. Needless enumeration is avoided by using the within-block rankings defined by the u-statistics.
    if (blockstart==length(blocksize)) {
        ans <- c(curblocks,l)
    } else {
        minv <- max(c(0, l-sum(blocksize[(blockstart+1):length(blocksize)])))
        maxv <- min(c(blocksize[blockstart], l))
        ans <- vector("list", maxv-minv+1)
        for (i in minv:maxv) {
            ans[[i+1]] <- enumblock(l-i, blocksize=blocksize, usedvars=usedvars+i, curblocks=c(curblocks,i), blockstart=blockstart+1)
        }
        ans <- do.call(rbind,ans)
    }
    return(ans)
}


uscore <- function(y,XtX,Xty) {
    #Compute u-score, least squares estimate and its covariance for a given model
    V <- solve(XtX)
    m <- V %*% Xty
    u <- t(Xty) %*% m
    return(list(u=u,m=m,V=V))
}




## Auxiliary functions for orthogonal design ##
###############################################

#Marginal inclusion probability for a single variable conditional on a vector of phi values
# Input
# - phi: vector of phi values (residual variance)
# - u: u statistic for a single variable
# - g: prior dispersion parameter
# - shrinkage: g/(1+g)
# - rho: prior inclusion probability
# - logscale: set to TRUE to obtain log inclusion probability
# Output: vector of marginal inclusion probabilities conditional on the given phi's
margppZellnerKnownOrtho <- function(phi,u,g,shrinkage=g/(1+g),rho,logscale=TRUE) {
  ans <- -log(1 + sqrt(1+g) * exp(-.5 * shrinkage*u/phi) * (1-rho)/rho)
  if (!logscale) ans <- exp(ans)
  return(ans)
}

margppMomKnownOrtho <- function(phi,u,g,shrinkage=g/(1+g),rho,logscale=TRUE) {
  uu <- shrinkage*u/phi
  ans <- -log(1 + (1+g)^(1.5) * exp(-.5 * uu) * (1-rho)/(rho * (1+uu)))
  if (!logscale) ans <- exp(ans)
  return(ans)
}

#Joint model probability conditional on a single phi value
#Input
# - phi: single value for residual variance
# - sel: model indicator, i.e. vector indicating indexes of active variables under current model
# - u: u-scores for all variables (i.e. u[sel] selects those for active variables)
# - g, shrinkage, rho, logscale: as in margppMomKnown
#Output: posterior probability of the model conditional on phi
jointppZellnerKnownOrtho <- function(phi,sel,u,g,shrinkage=g/(1+g),rho,logscale=TRUE) {
    uu <- shrinkage*u/phi
    odds01 <- sqrt(1+g) * exp(-.5 * uu) * (1-rho)/rho
    if (length(sel)>0 & length(sel)<length(u)) {
        ans <- -sum(log(1+odds01[sel])) -sum(log(1+1/odds01[-sel]))
    } else if (length(sel)==0) {
        ans <- -sum(log(1+1/odds01))
    } else {
        ans <- -sum(log(1+odds01))
    }
    if (!logscale) ans <- exp(ans)
    return(ans)
}

#Joint model probability conditional on a single phi value
#Input
# - phi: single value for residual variance
# - sel: model indicator, i.e. vector indicating indexes of active variables under current model
# - u: u-scores for all variables (i.e. u[sel] selects those for active variables)
# - g, shrinkage, rho, logscale: as in margppMomKnown
#Output: posterior probability of the model conditional on phi
jointppMomKnownOrtho <- function(phi,sel,u,g,shrinkage=g/(1+g),rho,logscale=TRUE) {
    uu <- shrinkage*u/phi
    odds01 <- (1+g)^(1.5) * exp(-.5 * uu) * (1-rho)/(rho * (1+uu))
    if (length(sel)>0 & length(sel)<length(u)) {
        ans <- -sum(log(1+odds01[sel])) -sum(log(1+1/odds01[-sel]))
    } else if (length(sel)==0) {
        ans <- -sum(log(1+1/odds01))
    } else {
        ans <- -sum(log(1+odds01))
    }
    if (!logscale) ans <- exp(ans)
    return(ans)
}


#Evaluate quantity proportional to joint of (phi,y) on a grid
jointPhiyZellnerOrtho <- function(phiseq, sumy2, apos, shrinkage, u, g, rho, logscale=TRUE) {
    #Code version 1
    fseqnew <- do.call(cbind,lapply(phiseq, function(phi) exp(0.5 * shrinkage * u / phi)))
    ans <- -0.5*sumy2/phiseq - (.5*apos+1)*log(phiseq) + colSums(log(1 + rho*(fseqnew/sqrt(1+g) -1)))
    #Code version 2. Smarter algebra but often causes overflow
    #ans <- double(length(phiseq))
    #ct <- -0.5*sumy2/length(u)
    #for (i in 1:length(phiseq)) {
    #    fseqnew <- exp((ct + 0.5 * shrinkage * u) / phiseq[i])
    #    ans[i] <- -(.5*apos+1)*log(phiseq[i]) + sum(log((1-rho)*exp(ct/phiseq[i]) + rho*fseqnew/sqrt(1+g)))
    #}
    ans[is.infinite(ans)] <- -Inf  #avoid numerical overflow
    if (!logscale) ans <- exp(ans)
    return(ans)
}

jointPhiyMOMOrtho <- function(phiseq, sumy2, apos, shrinkage, u, g, rho, logscale=TRUE) {
    fseqnew <- do.call(cbind,lapply(phiseq, function(phi) exp(0.5 * shrinkage * u / phi + log(1 + shrinkage * u / phi) - log(1+g))))
    ans <- -0.5*sumy2/phiseq - (.5*apos+1)*log(phiseq) + colSums(log(1 + rho*(fseqnew/sqrt(1+g) -1)))
    ans[is.infinite(ans)] <- -Inf  #avoid numerical overflow
    if (!logscale) ans <- exp(ans)
    return(ans)
}


#Function int.simpson2 copied from R package fda.usc
# x: grid of x values
# y: f(x)
# equi: set to TRUE if grid has equally spaced points
# method: "TRAPZ" for trapezoidal rule, "CSR" for composite Simpson's rule, "ESR" for extended Simpson's rule
int.simpson2 <- function (x, y, equi=FALSE, method="CSR") {
    n = length(x)
    ny = length(y)
    if (n != ny) stop("Different length in the input data")
    if (n == 2 || ny == 2) method = "TRAPZ"
    out <- switch(method, TRAPZ = {
        idx = 2:length(x)
        value <- as.double((x[idx] - x[idx - 1]) %*% (y[idx] + y[idx - 1]))/2
    }, CSR = {
        if (!equi) {
            n = 2 * n - 1
            app = approx(x, y, n = n)
            x = app$x
            y = app$y
        }
        h = (max(x) - min(x))/(n - 1)
        value = (h/3) * (y[n] + y[1] + 2 * sum(y[2 * (1:((n-1)/2)) + 1]) + 4 * sum(y[2 * (1:((n - 1)/2))]))
    }, ESR = {
        if (!equi) {
            n = 2 * n - 1
            app = approx(x, y, n = n)
            x = app$x
            y = app$y
        }
        h = (max(x) - min(x))/(n - 1)
        if (n <= 4) stop("This method needs n>4")
        value = 17 * (y[1] + y[n]) + 59 * (y[2] + y[n - 1]) + 43 * (y[3] + y[n - 2]) + 49 * (y[4] + y[n - 3])
        value = value + 48 * sum(y[5:(n - 4)])
        value = (h/48) * value
    })
    return(out)
}

###
### msPriorSpec.R
###


##-----------------------------------------------------------------------------
## Invoked by validObject() method.
valid_msPriorSpec <- function(object) {

    msg <- NULL

    ## Ensure first two slots are scalar
    if (length(object@priorType) != 1) {
        msg <- c(msg,
                 sprintf("slot %s must be of length 1",
                         sQuote("priorType")))
        object@priorType <- object@priorType[1]         # process first only
    }
    if (length(object@priorDistr) != 1) {
        msg <- c(msg,
                 sprintf("slot %s must be of length 1",
                         sQuote("priorDistr")))
        object@priorDistr <- object@priorDistr[1]       # process first only
    }

    ## Ensure valid type
    valid_prior_types <- c("coefficients",
                           "modelIndicator",
                           "nuisancePars")
    found <- object@priorType %in% valid_prior_types
    if (!found) {
        msg <- c(msg,
                 sprintf("slot %s must be one of: %s",
                         sQuote("priorType"),
                         paste(dQuote(valid_prior_types), collapse=", ")))
    }

    ##-------------------------------------------------------------------------
    has_either_or <- function(this, that, among) {
        this_in <- this %in% among
        that_in <- that %in% among

        (all(this_in) & !any(that_in)) || (!any(this_in) & all(that_in))
    }


    ## Validate rest based on type
    switch(object@priorType,
           coefficients = {
               valid_coef_prior_distrs <- c(
                 "pMOM","groupMOM","piMOM","peMOM","zellner","groupzellner","normalid","bic","icarplus"
               )
               found <- object@priorDistr %in% valid_coef_prior_distrs
               if (!found) {
                   msg <- c(msg,
                            sprintf("slot %s must be one of: %s",
                                    sQuote("priorDistr"),
                                    paste(dQuote(valid_coef_prior_distrs),
                                          collapse=", ")))
               }
               if (object@priorDistr == "pMOM") {
                   reqd_pmom_prior_par <- "r"
                   found <- reqd_pmom_prior_par %in% names(object@priorPars)
                   if (!found) {
                       msg <- c(msg,
                                sprintf("slot %s must be vector with named element %s (when using %s distr)",
                                        sQuote("priorPars"),
                                        dQuote(reqd_pmom_prior_par),
                                        sQuote("pMOM")))
                   }
               }
               prPars_names <- names(object@priorPars)
               found <- (
                  (("a.tau" %in% prPars_names) & ("b.tau" %in% prPars_names)) ||
                  ("tau" %in% prPars_names) ||
                  ("taustd" %in% prPars_names)
               )
               if (!found) {
                   msg <- c(msg,
                            sprintf("slot %s must be vector with named element(s) tau, taustd or, a.tau and b.tau",
                                    sQuote("priorPars")))
               }
           },
           modelIndicator = {
               valid_ind_prior_distrs <- c("uniform",
                                           "binomial",
                                           "complexity")
               found <- object@priorDistr %in% valid_ind_prior_distrs
               if (!found) {
                   msg <- c(msg,
                            sprintf("slot %s must be one of: %s",
                                    sQuote("priorDistr"),
                                    paste(dQuote(valid_ind_prior_distrs), collapse=", ")))
               }
               if (object@priorDistr == "binomial") {
                   reqd_prior_pars <- c("p",
                                        "alpha.p",
                                        "beta.p")
                   found <- has_either_or(reqd_prior_pars[1],
                                          reqd_prior_pars[-1],
                                          names(object@priorPars))
                   if (!found) {
                       msg <- c(msg,
                                sprintf("slot %s must be vector with named element(s) %s, or %s (when using %s distr)",
                                        sQuote("priorPars"),
                                        dQuote(reqd_prior_pars[1]),
                                        paste(dQuote(reqd_prior_pars[-1]),
                                              collapse=" and "),
                                        sQuote("binomial")))
                   }
               } else if (object@priorDistr == "complexity") {
                   reqd_prior_pars <- "c"
                   found <- reqd_prior_pars %in% names(object@priorPars)
                   if (!found) {
                       msg <- c(msg,
                                sprintf("slot %s must be vector with names element %s",
                                        sQuote("priorPars"),
                                        dQuote(reqd_prior_pars[1])))
                   }
               }
           },
           nuisancePars = {
               valid_nuisance_prior_distrs <- c("gamma","invgamma")
               found <- object@priorDistr %in% valid_nuisance_prior_distrs
               if (!found) {
                 msg <- c(msg,
                          sprintf("slot %s must be %s",
                                  sQuote("priorDistr"),
                                  dQuote(valid_nuisance_prior_distrs)))
               }
               reqd_prior_pars <- c("alpha",
                                    "lambda")
               found <- reqd_prior_pars %in% names(object@priorPars)
               if (!all(found)) {
                   msg <- c(msg,
                            sprintf("slot %s must contain named elements %s",
                                    sQuote("priorPars"),
                                    paste(dQuote(reqd_prior_pars),
                                          collapse=", ")))
               }
           })

    if (is.null(msg)) {
        TRUE
    } else {
        msg
    }
}

setValidity("msPriorSpec", valid_msPriorSpec)


##-----------------------------------------------------------------------------
is.msPriorSpec <- function(x) {
    extends(class(x), "msPriorSpec")
}


##-----------------------------------------------------------------------------
## Generators for this object.
msPriorSpec <- function(priorType=c("coefficients",
                                    "modelIndicator",
                                    "nuisancePars"),
                        priorDistr,
                        priorPars) {
    ## Check arguments
    stopifnot(is.character(priorType)  && length(priorType) == 1)
    stopifnot(is.character(priorDistr) && length(priorDistr) == 1)
    stopifnot(is.vector(priorPars))

    ## Create new class
    new("msPriorSpec",
        priorType=match.arg(priorType),
        priorDistr=priorDistr,
        priorPars=priorPars)
}


bic <- function() {
    new("msPriorSpec", priorType="coefficients", priorDistr="bic", priorPars=c(tau=-1))
}

bicprior <- function() {
    bic()
}

aic <- function() {
    ic(2)
}

ic <- function(penalty) {
    new("msPriorSpec", priorType="coefficients", priorDistr="bic", priorPars=c(tau=penalty))
}


exponentialprior <- function(lambda=1) {
    gammaprior(alpha=1, lambda=lambda)
}

gammaprior <- function(alpha=.01, lambda=.01) {
    new("msPriorSpec", priorType='nuisancePars', priorDistr='gamma', priorPars=c(alpha=alpha,lambda=lambda))
}


igprior <- function(alpha=.01, lambda=.01) {
    new("msPriorSpec",priorType='nuisancePars',priorDistr='invgamma',priorPars=c(alpha=alpha,lambda=lambda))
}

momprior <- function(taustd=1, tau, tau.adj=10^6, r=1) {
  if (missing(tau)) {
    priorPars <- c(taustd=taustd, tau.adj=tau.adj,r=r)
  } else {
    priorPars <- c(tau=tau,tau.adj=tau.adj,r=r)
  }
  new("msPriorSpec", priorType="coefficients", priorDistr="pMOM", priorPars=priorPars)
}

icarplusprior <- function(a=0.5, taustd=1, tau.adj=10^6) {
    if ((a < 0) | (a>1)) stop("a must be in [0,1]")
    new("msPriorSpec", priorType="coefficients", priorDistr="icarplus", priorPars=c(a=a, taustd=taustd, tau.adj=tau.adj))
}


imomprior <- function(tau, tau.adj=10^6) {
    new("msPriorSpec", priorType="coefficients", priorDistr="piMOM", priorPars=c(tau=tau, tau.adj=tau.adj))
}

emomprior <- function(tau, tau.adj=10^6) {
    new("msPriorSpec", priorType="coefficients", priorDistr="peMOM", priorPars=c(tau=tau, tau.adj=tau.adj))
}

zellnerprior <- function(taustd=1, tau, tau.adj=10^6) {
  if (missing(tau)) {
    priorPars <- c(taustd=taustd, tau.adj=tau.adj)
  } else {
    priorPars <- c(tau=tau,tau.adj=tau.adj)
  }
  new("msPriorSpec", priorType="coefficients", priorDistr="zellner", priorPars=priorPars)
}

normalidprior <- function(taustd=1, tau, tau.adj=10^6) {
  if (missing(tau)) {
    priorPars <- c(taustd=taustd, tau.adj=tau.adj)
  } else {
    priorPars <- c(tau=tau,tau.adj=tau.adj)
  }
  new("msPriorSpec", priorType="coefficients", priorDistr="normalid", priorPars=priorPars)
}

groupmomprior <- function(taustd=1, tau, tau.adj=10^6) {
  if (missing(tau)) {
    priorPars <- c(taustd=taustd, tau.adj=tau.adj)
  } else {
    priorPars <- c(tau=tau,tau.adj=tau.adj)
  }
  new("msPriorSpec", priorType="coefficients", priorDistr="groupMOM", priorPars=priorPars)
}

groupimomprior <- function(tau, tau.adj=10^6) {
    new("msPriorSpec", priorType="coefficients", priorDistr="groupiMOM", priorPars=c(tau=tau, tau.adj=tau.adj))
}

groupemomprior <- function(tau, tau.adj=10^6) {
    new("msPriorSpec", priorType="coefficients", priorDistr="groupeMOM", priorPars=c(tau=tau, tau.adj=tau.adj))
}

groupzellnerprior <- function(taustd=1, tau, tau.adj=10^6) {
  if (missing(tau)) {
    priorPars <- c(taustd=taustd, tau.adj=tau.adj)
  } else {
    priorPars <- c(tau=tau,tau.adj=tau.adj)
  }
  new("msPriorSpec", priorType="coefficients", priorDistr="groupzellner", priorPars=priorPars)
}


modelunifprior <- function() {
    new("msPriorSpec",priorType='modelIndicator',priorDistr='uniform',priorPars=double(0))
}

modelbinomprior <- function(p=0.5) {
    new("msPriorSpec",priorType='modelIndicator',priorDistr='binomial',priorPars=list(p=p))
}

modelbbprior <- function(alpha.p=1, beta.p=1) {
    new("msPriorSpec",priorType='modelIndicator',priorDistr='binomial',priorPars=c(alpha.p=alpha.p,beta.p=beta.p))
}

modelcomplexprior <- function(c=1) {
    new("msPriorSpec",priorType='modelIndicator',priorDistr='complexity',priorPars=c(c=c))
}


igprior <- function(alpha=.01, lambda=.01) {
    new("msPriorSpec",priorType='nuisancePars',priorDistr='invgamma',priorPars=c(alpha=alpha,lambda=lambda))
}

context("Test modelSelection with Gibbs")
library("modelSelection")

source(test_path("data-for-tests.R"))
tolerance <- 5e-3

patrick::with_parameters_test_that(
  "modelSelection without groups works for", {
    pDelta <- modelbbprior(1,1)
    log <- capture.output(
      fit1 <- modelSelection(y=y3, x=X3, priorCoef=pCoef, priorModel=pDelta, enumerate=FALSE, family=family, priorSkew=pCoef),
      fit3 <- modelSelection(as.formula("y~X2+X3+X4"), data=data.frame(X3, y=y3), priorCoef=pCoef, priorModel=pDelta, enumerate=FALSE, family=family, priorSkew=pCoef)
    )
    pp1 <- postProb(fit1)
    pp3 <- postProb(fit3)
    expect_equal(as.character(pp1$modelid[1]), as.character(pp3$modelid[1]))
    expect_equal(as.numeric(pp1$pp[1]), as.numeric(pp3$pp[1]), tolerance=tolerance)
  },
  patrick::cases(
    mom_auto=list(family="auto", pCoef=momprior(tau=0.348)),
    mom_normal=list(family="normal", pCoef=momprior(tau=0.348)),
    mom_twopiecenormal=list(family="twopiecenormal", pCoef=momprior(tau=0.348)),
    mom_laplace=list(family="laplace", pCoef=momprior(tau=0.348)),
    mom_twopiecelaplace=list(family="twopiecelaplace", pCoef=momprior(tau=0.348)),
    imom_auto=list(family="auto", pCoef=imomprior(tau=0.348)),
    imom_normal=list(family="normal", pCoef=imomprior(tau=0.348)),
    imom_twopiecenormal=list(family="twopiecenormal", pCoef=imomprior(tau=0.348)),
    imom_laplace=list(family="laplace", pCoef=imomprior(tau=0.348)),
    imom_twopiecelaplace=list(family="twopiecelaplace", pCoef=imomprior(tau=0.348)),
    emom_auto=list(family="auto", pCoef=emomprior(tau=0.348)),
    emom_normal=list(family="normal", pCoef=emomprior(tau=0.348)),
    emom_twopiecenormal=list(family="twopiecenormal", pCoef=emomprior(tau=0.348)),
    emom_laplace=list(family="laplace", pCoef=emomprior(tau=0.348)),
    emom_twopiecelaplace=list(family="twopiecelaplace", pCoef=emomprior(tau=0.348))
  )
)

patrick::with_parameters_test_that(
  "modelSelection with groups (pCoef=pGroup) works for", {
    pDelta <- modelbbprior(1,1)
    groups <- c(1, 1, 2, 2, 3, 4, 4)
    log <- capture.output(
      fit <- modelSelection(
        y=y6, x=X6, priorCoef=pCoef, priorModel=pDelta, enumerate=FALSE,
        family=family, priorSkew=pCoef, priorGroup=pCoef, groups=groups
      )
    )
    pprobs <- postProb(fit)
    expect_equal(as.character(pprobs$modelid[1]), "3,4,6,7")
  },
  patrick::cases(
    mom_normal=list(family="normal", pCoef=momprior(tau=0.348)),
    mom_twopiecenormal=list(family="twopiecenormal", pCoef=momprior(tau=0.348)),
    mom_laplace=list(family="laplace", pCoef=momprior(tau=0.348)),
    mom_twopiecelaplace=list(family="twopiecelaplace", pCoef=momprior(tau=0.348)),
    imom_normal=list(family="normal", pCoef=imomprior(tau=0.348)),
    imom_twopiecenormal=list(family="twopiecenormal", pCoef=imomprior(tau=0.348)),
    imom_laplace=list(family="laplace", pCoef=imomprior(tau=0.348)),
    imom_twopiecelaplace=list(family="twopiecelaplace", pCoef=imomprior(tau=0.348)),
    emom_normal=list(family="normal", pCoef=emomprior(tau=0.348)),
#    emom_twopiecenormal=list(family="twopiecenormal", pCoef=emomprior(tau=0.348)),
    emom_laplace=list(family="laplace", pCoef=emomprior(tau=0.348)),
    emom_twopiecelaplace=list(family="twopiecelaplace", pCoef=emomprior(tau=0.348))
  )
)

patrick::with_parameters_test_that(
  "modelSelection with groups (pCoef!=pGroup), crossprodmat and covariancemat work for", {
    pDelta <- modelbbprior(1,1)
    groups <- c(1, 1, 2, 2, 3, 4, 4)
    log <- capture.output(
      fit <- modelSelection(
        y=y6, x=X6, priorCoef=pCoef, priorModel=pDelta, enumerate=FALSE, XtXprecomp=FALSE,
        family=family, priorSkew=pCoef, priorGroup=pGroup, groups=groups
      )
    )
    pprobs <- postProb(fit)
    expect_true("3,4,6,7" %in% pprobs$modelid[1:4])
  },
  patrick::cases(
    normid_gzell=list(family="normal", pCoef=normalidprior(tau=0.348), pGroup=groupzellnerprior(tau=0.4)),
    zell_gzell=list(family="normal", pCoef=zellnerprior(tau=0.348), pGroup=groupzellnerprior(tau=0.4))
  )
)


test_that(
  "modelSelection with smoothterms and no groups works", {
    pCoef=momprior(tau=0.348)
    pDelta <- modelbbprior(1,1)
    log <- capture.output(
      fit <- modelSelection(
        y6 ~ ., data=data.frame(y6, X6[,2:7]), priorCoef=pCoef, priorModel=pDelta, enumerate=FALSE,
        smoothterms= ~ X1 + X2 + X3 + X4 + X5 + X6, family="normal", priorSkew=pCoef, priorGroup=pCoef
        #y6~X6[,2:7], priorCoef=pCoef, priorModel=pDelta, enumerate=FALSE,
        #smoothterms= ~ X6[,2:7], family="normal", priorSkew=pCoef, priorGroup=pCoef
      )
    )
    pprobs <- postProb(fit)
    expect_equal(names(pprobs)[3], "pp")

    #expect_true(any(pprobs$modelid[1:5] == "3,4,6,7"))
  }
)

test_that(
  "modelSelection with includevars works", {
    log <- capture.output(
      fit <- modelSelection(
        y9, X9, enumerate=FALSE, includevars=c(rep(TRUE, 3), rep(FALSE, 7))
      )
    )
    models <- postProb(fit)$modelid
    expect_true(all(grepl("1,2,3", models)))
  }
)

context("Test modelSelection with enumerate=TRUE")
library("modelSelection")

source(test_path("data-for-tests.R"))
tolerance <- 1e-5

patrick::with_parameters_test_that(
  "modelSelection without groups works for", {
    pDelta <- modelbbprior(1,1)
    log <- capture.output(
      fit1 <- modelSelection(y=y3, x=X3, priorCoef=pCoef, priorModel=pDelta, enumerate=TRUE, family=family, priorSkew=pCoef),
      fit3 <- modelSelection(as.formula("y~X2+X3+X4"), data=data.frame(X3, y=y3), priorCoef=pCoef, priorModel=pDelta, enumerate=TRUE, family=family, priorSkew=pCoef)
    )
    pp1 <- postProb(fit1)
    pp3 <- postProb(fit3)
    expect_equal(pp1$modelid, pp3$modelid)
    expect_equal(pp1$pp, pp3$pp, tolerance=tolerance)
  },
  patrick::cases(
    mom_auto=list(family="auto", pCoef=momprior(tau=0.348)),
    mom_normal=list(family="normal", pCoef=momprior(tau=0.348)),
    mom_twopiecenormal=list(family="twopiecenormal", pCoef=momprior(tau=0.348)),
    mom_laplace=list(family="laplace", pCoef=momprior(tau=0.348)),
    mom_twopiecelaplace=list(family="twopiecelaplace", pCoef=momprior(tau=0.348)),
    imom_auto=list(family="auto", pCoef=imomprior(tau=0.348)),
    imom_normal=list(family="normal", pCoef=imomprior(tau=0.348)),
    imom_twopiecenormal=list(family="twopiecenormal", pCoef=imomprior(tau=0.348)),
    imom_laplace=list(family="laplace", pCoef=imomprior(tau=0.348)),
    imom_twopiecelaplace=list(family="twopiecelaplace", pCoef=imomprior(tau=0.348)),
    emom_auto=list(family="auto", pCoef=emomprior(tau=0.348)),
    emom_normal=list(family="normal", pCoef=emomprior(tau=0.348)),
    emom_twopiecenormal=list(family="twopiecenormal", pCoef=emomprior(tau=0.348)),
    emom_laplace=list(family="laplace", pCoef=emomprior(tau=0.348)),
    emom_twopiecelaplace=list(family="twopiecelaplace", pCoef=emomprior(tau=0.348))
  )
)

patrick::with_parameters_test_that(
  "modelSelection with groups works for", {
    pDelta <- modelbbprior(1,1)
    groups <- c(1, 1, 2, 2, 3, 4, 4)
    log <- capture.output(
      fit <- modelSelection(
        y=y6, x=X6, priorCoef=pCoef, priorModel=pDelta, enumerate=TRUE,
        family=family, priorSkew=pCoef, priorGroup=pCoef, groups=groups
      )
    )
    pprobs <- postProb(fit)
    expect_equal(length(pprobs$modelid), 16)
    expect_equal(as.character(pprobs$modelid[1]), "3,4,6,7")
  },
  patrick::cases(
    mom_normal=list(family="normal", pCoef=momprior(tau=0.348)),
    mom_twopiecenormal=list(family="twopiecenormal", pCoef=momprior(tau=0.348)),
    mom_laplace=list(family="laplace", pCoef=momprior(tau=0.348)),
    mom_twopiecelaplace=list(family="twopiecelaplace", pCoef=momprior(tau=0.348)),
    imom_normal=list(family="normal", pCoef=imomprior(tau=0.348)),
    imom_twopiecenormal=list(family="twopiecenormal", pCoef=imomprior(tau=0.348)),
    imom_laplace=list(family="laplace", pCoef=imomprior(tau=0.348)),
    imom_twopiecelaplace=list(family="twopiecelaplace", pCoef=imomprior(tau=0.348)),
    emom_normal=list(family="normal", pCoef=emomprior(tau=0.348)),
    emom_twopiecenormal=list(family="twopiecenormal", pCoef=emomprior(tau=0.348)),
    emom_laplace=list(family="laplace", pCoef=emomprior(tau=0.348)),
    emom_twopiecelaplace=list(family="twopiecelaplace", pCoef=emomprior(tau=0.348))
  )
)

patrick::with_parameters_test_that(
  "model space prior work in modelSelection:", {
    pCoef <- momprior(tau=0.348)
    log <- capture.output(
      fit <- modelSelection(y=y6, x=X6, priorCoef=pCoef, priorModel=pDelta, enumerate=TRUE, family="normal")
    )
    pprobs <- postProb(fit)
    expect_true(any(pprobs$modelid[1:5] == "3,4,6,7"))
  },
  .test_name=c("uniform", "binomial", "betabinomial", "complex"),
  pDelta=c(modelunifprior(), modelbinomprior(p=0.5), modelbbprior(alpha.p=1, beta.p=1), modelcomplexprior(c=1))
)

patrick::with_parameters_test_that(
  "asymetric binomial prior works in modelSelection (pDelta_pConstr):", {
    i0 <- integer(0)
    constraints <- list(i0, 1, i0, 3, i0, i0, 6)
    log <- capture.output(
      fit <- modelSelection(
        y=y6, x=X6, priorModel=pDelta, priorConstraints=pConstr,
        enumerate=TRUE, constraints=constraints
      )
    )
    log <- capture.output(
      fit_asym <- modelSelection(
        y=y6, x=X6, priorModel=modelbinomprior(p=c(0.7, 0.6, 0.2, 0.4)),
        priorConstraints=pConstr, enumerate=TRUE, constraints=constraints
      )
    )
    pprobs <- postProb(fit)
    pprobs_asym <- postProb(fit_asym)
    expect_true(pprobs[1, "modelid"] == "3,4,6,7")
    expect_true(pprobs_asym[1, "modelid"] == "3,4,6,7")
    expect_equal(pprobs[1, "pp"], .9957818, tolerance=tolerance)
    expect_true(abs(pprobs[1, "pp"] - pprobs_asym[1, "pp"]) > tolerance)
  },
  patrick::cases(
    vect_vect=list(pDelta=modelbinomprior(p=c(.5, .5, .5, .5)), pConstr=modelbinomprior(p=c(0.5, .5, .5))),
    vect_scalar=list(pDelta=modelbinomprior(p=c(.5, .5, .5, .5)), pConstr=modelbinomprior(p=0.5)),
    scalar_vect=list(pDelta=modelbinomprior(p=.5), pConstr=modelbinomprior(p=c(0.5, .5, .5))),
    scalar_scalar=list(pDelta=modelbinomprior(p=.5), pConstr=modelbinomprior(p=0.5))
  )
)

patrick::with_parameters_test_that(
  "modelSelection methods in normal family work:", {
    if (method == "Hybrid") {pCoef <- imomprior(tau=0.348)} else {pCoef <- momprior(tau=0.348)}
    pDelta <- modelbbprior(1,1)
    log <- capture.output(
      fit <- modelSelection(
        y=y6, x=X6, priorCoef=pCoef, priorModel=pDelta, enumerate=TRUE, family="normal",
        method=method, B=200, optimMethod=optimMethod, hess=hess
      )
    )
    pprobs <- postProb(fit)
    expect_true(any(pprobs$modelid[1:5] == "3,4,6,7"))
  },
  patrick::cases(
    auto=list(method="auto", optimMethod="CDA", hess="asymp"),
    laplace_cda=list(method="Laplace", optimMethod="CDA", hess="asymp"),
    laplace_cda_asympdiag=list(method="Laplace", optimMethod="CDA", hess="asympDiagAdj"),
    laplace_lma=list(method="Laplace", optimMethod="LMA", hess="asymp"),
    hybrid=list(method="Hybrid", optimMethod="CDA", hess="asymp"),
    mc=list(method="MC", optimMethod="CDA", hess="asymp"),
    bic=list(method="plugin", optimMethod="CDA", hess="asymp")
  )
)

patrick::with_parameters_test_that(
  "modelSelection methods in laplace family work:", {
    if (method == "Hybrid") {pCoef <- imomprior(tau=0.348)} else {pCoef <- momprior(tau=0.348)}
    pDelta <- modelbbprior(1,1)
    log <- capture.output(
      fit <- modelSelection(
        y=y6, x=X6, priorCoef=pCoef, priorModel=pDelta, enumerate=TRUE, family="laplace",
        method=method, B=200, optimMethod=optimMethod, hess=hess
      )
    )
    pprobs <- postProb(fit)
    expect_true(any(pprobs$modelid[1:5] == "3,4,6,7"))
  },
  patrick::cases(
    auto=list(method="auto", optimMethod="CDA", hess="asymp"),
    laplace_cda=list(method="Laplace", optimMethod="CDA", hess="asymp"),
    laplace_cda_asympdiag=list(method="Laplace", optimMethod="CDA", hess="asympDiagAdj"),
    laplace_lma=list(method="Laplace", optimMethod="LMA", hess="asymp"),
    hybrid=list(method="Hybrid", optimMethod="CDA", hess="asymp"),
    mc=list(method="MC", optimMethod="CDA", hess="asymp"),
    bic=list(method="plugin", optimMethod="CDA", hess="asymp")
  )
)

context("Test marginalLikelihood with groups")
library("modelSelection")

source(test_path("data-for-tests.R"))
tolerance <- 1e-6

patrick::with_parameters_test_that(
  "check_sel_groups works", {
    groups <- c(1,2,2,2,3,3,4,5)
    expect_error(check_sel_groups(sel, groups), error_msg)
  },
  patrick::cases(
    sel_outofbounds=list(sel=c(1,7,20), error_msg="sel larger than"),
    sel_incompatible_groups=list(sel=c(1,2,7), error_msg="incompatible with .+ groups"),
    sel_good_int=list(sel=c(2,3,4,7,8), error_msg=NA),
    sel_good_bool=list(sel=c(TRUE,FALSE,FALSE,FALSE,TRUE,TRUE,TRUE, FALSE), error_msg=NA)
  )
)

test_that(
  "marginalLikelihood ignores sel argument when its a formula", {
    pCoef <- momprior(tau=0.348)
    ans1 <- marginalLikelihood(seq(4), y3, X3, priorCoef=pCoef)
    formula <- y3 ~ .
    df <- data.frame(y3, X3[,2:4])
    expect_warning(ans2 <- marginalLikelihood(c(1,2), y=formula, data=df, priorCoef=pCoef), "ignoring sel")
    expect_equal(ans1, ans2)
    expect_warning(ans3 <- marginalLikelihood(y=formula, data=df, priorCoef=pCoef), NA)
    expect_equal(ans1, ans3)
  }
)

patrick::with_parameters_test_that(
  "marginalLikelihood with same kind of priorCoef and Group is correctly impemented for normal family:", {
    pVar <- igprior(alpha=0.01, lambda=0.01)
    ans_max <- marginalLikelihood(theta9_truth_idx, y9, X9, family="normal", priorCoef=pCoef, priorVar=pVar)
    ans_max_group <- marginalLikelihood(
      theta9_truth_idx, y9, X9, groups=groups9, family="normal",
      priorCoef=pCoef, priorGroup=pCoef, priorVar=pVar
    )

    ans_all <- marginalLikelihood(seq_along(theta9_truth), y9, X9, family="normal",priorCoef=pCoef, priorVar=pVar)
    ans_all_group <- marginalLikelihood(
      seq_along(theta9_truth), y9, X9, groups=groups9, family="normal",
      priorCoef=pCoef, priorGroup=pCoef, priorVar=pVar
    )
    expect_true(ans_max_group > ans_all_group)
    expect_equal(ans_max, ans_max_group)
    expect_equal(ans_all, ans_all_group)
  },
  .test_name=c("mom", "imom", "emom", "zellner", "normalid"),
  pCoef=c(momprior(tau=0.35), imomprior(tau=0.35), emomprior(tau=0.35), zellnerprior(tau=0.35), normalidprior(tau=0.35))
)

patrick::with_parameters_test_that(
  "marginalLikelihood with groups is correctly impemented for normal family:", {
    pVar <- igprior(alpha=0.01, lambda=0.01)
    ans_max <- marginalLikelihood(
      theta9_truth_idx, y9, X9, groups=groups9, family="normal",
      priorCoef=pCoef, priorGroup=pGroup, priorVar=pVar
    )

    ans_all <- marginalLikelihood(
      seq_along(theta9_truth), y9, X9, groups=groups9, family="normal",
      priorCoef=pCoef, priorGroup=pGroup, priorVar=pVar
    )
    expect_equal(ans_max, expected_max, tolerance=tolerance)
    expect_equal(ans_all, expected_all, tolerance=tolerance)
  },
  patrick::cases(
    normalid=list(pCoef=normalidprior(tau=0.3), pGroup=normalidprior(tau=0.4), expected_max=-231.111757, expected_all=-236.971966),
    normid_gzell=list(pCoef=normalidprior(tau=1), pGroup=groupzellnerprior(tau=0.4), expected_max=-316.658333, expected_all=-320.087106),
    zell_gzell=list(pCoef=zellnerprior(tau=0.3), pGroup=groupzellnerprior(tau=0.4), expected_max=-342.339794, expected_all=-342.19151),
    mom_gzell=list(pCoef=momprior(tau=0.3), pGroup=groupzellnerprior(tau=10), expected_max=-260.0427, expected_all=-269.2886), #from modelSelection
    #    mom_gzell=list(pCoef=momprior(tau=0.3), pGroup=groupzellnerprior(tau=10), expected_max=-262.450681, expected_all=-274.104395), #before mombf 4.0.0
    mom_gmom=list(pCoef=momprior(tau=0.3), pGroup=groupmomprior(tau=0.4), expected_max=-238.026357, expected_all=-252.308801)
  )
)

context("Test marginalLikelihood without groups")
library("modelSelection")

source(test_path("data-for-tests.R"))
tolerance <- 1e-5

patrick::with_parameters_test_that(
  "marginalLikelihood is correctly impemented for normal family and", {
    pVar <- igprior(alpha=0.01, lambda=0.01)
    ans_max <- marginalLikelihood(theta3_truth_idx, y3, X3, priorCoef=pCoef, priorVar=pVar)
    ans_all <- marginalLikelihood(
      seq_along(theta3_truth), y3, X3, priorCoef=pCoef, priorVar=pVar
    )
    expect_true(ans_max > ans_all)
    expect_equal(ans_max, expected_max, tolerance=tolerance)
    expect_equal(ans_all, expected_all, tolerance=tolerance)
  },
  patrick::cases(
    momprior=list(pCoef=momprior(tau=0.328, r=1), expected_max=-37.47804, expected_all=-41.73017),
    imomprior=list(pCoef=imomprior(tau=0.328), expected_max=-38.35765, expected_all=-43.9903),
    emomprior=list(pCoef=emomprior(tau=0.328), expected_max=-37.31882, expected_all=-42.475),
    zellnerprior=list(pCoef=zellnerprior(tau=0.328), expected_max=-39.81047, expected_all=-39.97039),
    normalidprior=list(pCoef=normalidprior(tau=0.328), expected_max=-38.35193, expected_all=-39.4416)
  )
)

patrick::with_parameters_test_that(
  "marginalLikelihood is correctly impemented for twopiecenormal family and", {
    pVar <- igprior(alpha=0.01, lambda=0.01)
    ans_max <- marginalLikelihood(
      theta3_truth_idx, y3, X3, family="twopiecenormal", priorCoef=pCoef,
      priorVar=pVar, priorSkew=pSkew
    )
    ans_all <- marginalLikelihood(
      seq_along(theta3_truth), y3, X3, family="twopiecenormal", priorCoef=pCoef,
      priorVar=pVar, priorSkew=pSkew
    )
    expect_true(ans_max > ans_all)
    expect_equal(ans_max, expected_max, tolerance=tolerance)
    expect_equal(ans_all, expected_all, tolerance=tolerance)
  },
  patrick::cases(
    momprior=list(pCoef=momprior(tau=0.328, r=1), pSkew=momprior(tau=0.3, r=1), expected_max=-38.64731, expected_all=-44.05285),
    imomprior=list(pCoef=imomprior(tau=0.328), pSkew=imomprior(tau=0.3), expected_max=-39.69978, expected_all=-46.27032),
    emomprior=list(pCoef=emomprior(tau=0.328), pSkew=emomprior(tau=0.3), expected_max=-38.33845, expected_all=-44.76512)
  )
)

patrick::with_parameters_test_that(
  "marginalLikelihood is correctly impemented for laplace family and",{
    pVar <- igprior(alpha=0.01, lambda=0.01)
    ans_max <- marginalLikelihood(
      theta3_truth_idx, y3, X3, family="laplace", priorCoef=pCoef, priorVar=pVar
    )
    ans_all <- marginalLikelihood(
      seq_along(theta3_truth), y3, X3, family="laplace", priorCoef=pCoef, priorVar=pVar
    )
    expect_true(ans_max > ans_all)
    expect_equal(ans_max, expected_max, tolerance=tolerance)
    expect_equal(ans_all, expected_all, tolerance=tolerance)
  },
  patrick::cases(
    momprior=list(pCoef=momprior(tau=0.328, r=1), expected_max=-38.25163, expected_all=-45.1899),
    imomprior=list(pCoef=imomprior(tau=0.328), expected_max=-39.13097, expected_all=-47.51524),
    emomprior=list(pCoef=emomprior(tau=0.328), expected_max=-37.92642, expected_all=-46.37662)
  )
)

patrick::with_parameters_test_that(
  "marginalLikelihood is correctly impemented for twopiecelaplace family and", {
    pVar <- igprior(alpha=0.01, lambda=0.01)
    ans_max <- marginalLikelihood(
      theta3_truth_idx, y3, X3, family="twopiecelaplace", priorCoef=pCoef,
      priorVar=pVar, priorSkew=pSkew
    )
    ans_all <- marginalLikelihood(
      seq_along(theta3_truth), y3, X3, family="twopiecelaplace", priorCoef=pCoef,
      priorVar=pVar, priorSkew=pSkew
    )
    expect_true(ans_max > ans_all)
    expect_equal(ans_max, expected_max, tolerance=tolerance)
    expect_equal(ans_all, expected_all, tolerance=tolerance)
  },
  patrick::cases(
    momprior=list(pCoef=momprior(tau=0.328, r=1), pSkew=momprior(tau=0.3, r=1), expected_max=-39.32618, expected_all=-45.58486),
    imomprior=list(pCoef=imomprior(tau=0.328), pSkew=imomprior(tau=0.3), expected_max=-40.31074, expected_all=-48.32312),
    emomprior=list(pCoef=emomprior(tau=0.328), pSkew=emomprior(tau=0.3), expected_max=-38.85072, expected_all=-47.06661)
  )
)

test_that(
  "logical or integer sel give equal results", {
    pVar <- igprior(alpha=0.01, lambda=0.01)
    pCoef=momprior(tau=0.328, r=1)
    ans_int <- marginalLikelihood(
      theta3_truth_idx, y3, X3, family="normal", priorCoef=pCoef, priorVar=pVar
    )
    ans_bool <- marginalLikelihood(
      theta3_truth_bool, y3, X3, family="normal", priorCoef=pCoef, priorVar=pVar
    )
    expect_equal(ans_int, ans_bool)
  }
)

#####################################################################
##
## ROUTINES RELATED TO GENERALIZED ADDITIVE MODELS
##
#####################################################################


bspline <- function(x, degree, knots) {
 # B-spline basis eval at vector of values x.
 # Normalized to sum to 1 at any x value.
 #
 # Input:
 #    x: vector of values at which to evaluate the B-spline basis
 #    degree: degree of the spline (0: piecewise constant, 1: linear etc.)
 #    knots: vector with positions of the knots
 #  Output: matrix[nx][nknots-degree-1] containing the B-spline basis
    ans= bsplineCI(as.double(x), as.integer(degree), as.double(knots))
    #ans= .Call("bsplineCI", as.double(x), as.integer(degree), as.double(knots))
    return(matrix(ans,nrow=length(x),byrow=TRUE))
}



tensorbspline <- function(x, degree, knots, maineffects) {
    #Create tensor product B-splines
    # - x: matrix with covariate values
    # - degree: degree of each marginal B-spline. If a single number is given, it is used for all columns of x
    # - knots: list of length equal to ncol(x) indicating the knots for each marginal B-spline. Alternatively, a single vector with the common knots to be used for all columns of x
    # - maineffects: if TRUE answer is returned in terms of intercept + main effects + tensor, where tensor is orthogonalized with respect to the main effects. If maineffects==FALSE no main effects are returned, only a tensor that already accounts for the main effects
    # Output: list with the following elements
    # - main: intercept and main effects given by the univariate splines associated to each column of x
    # - tensor: tensor product B-splines
    # Note: the element tensor already includes terms for the main effects. If you wish to include 
    if (missing(degree)) stop("degree must be specified")
    if (missing(knots)) stop("knots must be specified")
    if (is.vector(x)) x= matrix(x, ncol=1)
    p= ncol(x)
    if (length(degree)==1) degree= rep(degree,p)
    if (!is.list(knots) & !is.vector(knots)) stop("knots should either be a list or a vector")
    if (is.list(knots) & (length(knots) != ncol(x))) stop("If knots is a list, it should have length equal to ncol(x)")
    des= vector('list',p)
    for (j in 1:p) {
        if (is.list(knots)) {
            des[[j]]= bspline(x[,j], degree=degree[j], knots=knots[[j]])
        } else {
            des[[j]]= bspline(x[,j], degree=degree[j], knots=knots)
        }
        des[[j]]= des[[j]][,apply(des[[j]],2,'sd')>0]
        colnames(des[[j]])= paste('x',j,'.',1:ncol(des[[j]]),sep='')
    }
    f= paste(paste('des[[',1:p,']]',sep=''), collapse=':')
    f= as.formula(paste('~ -1 +',f))
    tensor= model.matrix(f)
    colnames(tensor)= gsub('des\\[\\[.\\]\\]','',colnames(tensor))
    if (!maineffects) {
        main= NULL
    } else {
        #remove redundant columns from main effects
        main= cbind(1,do.call(cbind, des))
        colnames(main)[1]= 'intercept'
        myqr= qr(main)
        main= main[,myqr$pivot[1:myqr$rank]]
        #remove redundant columns from tensor
        tensor= residuals(lm(tensor ~ main))
        myqr= qr(tensor)
        tensor= tensor[,myqr$pivot[1:myqr$rank],drop=FALSE]
    }
    ans= list(main=main, tensor=tensor)
    return(ans)
}



#Obtain ICAR penalty matrix D taking the difference between each element and its neighbours
#
# If z is p x 1, then D z is also p x 1 and its j^th entry is z[j] - mean(neighbours of j)
#
# INPUT
# - neighbours: list where entry j returns the indexes of the neighbours of j
# - scale: if TRUE, the entries in D are divided by a constant such that tr(t(D) %*% D)= p
#
# OUTPUT: matrix D of dimension length(neighbours) x length(neighbours)
#         If scale=FALSE, d[j,j]= 1 and d[j,i]= -1/N[j] if i is a neighbour of j, where N[j] is the number of neighbours of j
#         If scale= TRUE, D is multiplied by p / tr(t(D) %*% D)
icar_dmatrix= function(neighbours, scale=TRUE) {
    D= diag(length(neighbours))
    N= sapply(neighbours, length)
    for (j in 1:nrow(D)) {
        if (N[j] > 0) {
            D[j, neighbours[[j]]] = -1/N[j]
        }
    }
    if (scale) {
        D= D * sqrt(ncol(D) / sum(colSums(D^2)))
    }
    return(D)
}

##############################################################################################
##
## ROUTINES TO COMPUTE INTEGRATED LIKELIHOODS
##
##############################################################################################

marginalLikelihood <- function(
  sel, y, x, data, smoothterms, nknots=9, groups=1:ncol(x), family="normal",
  priorCoef, priorGroup, priorVar=igprior(alpha=0.01,lambda=0.01),
  priorSkew=momprior(tau=0.348), neighbours, phi, method='auto', adj.overdisp='intercept', hess='asymp',
  optimMethod, optim_maxit, initpar='none', B=10^5, logscale=TRUE, XtX, ytX
) {
  #Check input
  # format input data
  tmp <- formatInputdata(y=y,x=x,data=data,smoothterms=smoothterms,nknots=nknots,family=family)
  x <- tmp$x; y <- tmp$y; is_formula <- tmp$is_formula
  splineDegree <- tmp$splineDegree
  if (!is.null(tmp$groups)) groups <- tmp$groups
  hasgroups <- tmp$hasgroups
  if (!is.null(tmp$constraints)) constraints <- tmp$constraints
  outcometype <- tmp$outcometype; uncens <- tmp$uncens; ordery <- tmp$ordery
  typeofvar <- tmp$typeofvar
  p= ncol(x); n= length(y)
  if (missing(XtX)) { XtX <- t(x) %*% x } else { XtX <- as.matrix(XtX) }
  if (missing(ytX)) { ytX <- as.vector(matrix(y,nrow=1) %*% x) } else { ytX <- as.vector(ytX) }
  sumy2 <- as.double(sum(y^2)); sumy <- as.double(sum(y))
  colsumsx <- as.double(colSums(x))
  #
  familyint= formatFamily(family, issurvival= length(uncens)>0)$familyint
  if (familyint == 22) { sumlogyfact= as.double(sum(lgamma(y+1))) } else { sumlogyfact= as.double(0) } #Poisson regression
  # check prior and set defaults if necessary
  if (missing(priorCoef)) {
      defaultprior= defaultmom(outcometype=outcometype,family=family)
      priorCoef= defaultprior$priorCoef; priorVar= defaultprior$priorVar
  }
  if (missing(priorGroup)) { if (length(groups)==length(unique(groups))) { priorGroup= priorCoef } else { priorGroup= groupzellnerprior(tau=n) } }

  if (missing(phi)) { knownphi <- as.integer(0); phi <- double(0) } else { knownphi <- as.integer(1); phi <- as.double(phi) }

  if (!missing(neighbours)) { Dmat= icar_dmatrix(neighbours) } else { Dmat= diag(p) }

  # format arguments for .Call
  thinit= getthinit(y=y, x=x, family=family, initpar=initpar, enumerate=TRUE)
  usethinit= thinit$usethinit; thinit= thinit$thinit

  method <- formatmsMethod(method=method, usethinit=usethinit, optimMethod=optimMethod, optim_maxit=optim_maxit, priorCoef=priorCoef, priorGroup=priorGroup, knownphi=0, outcometype=outcometype, family=family, hasgroups=hasgroups, adj.overdisp=adj.overdisp, hess=hess)
  optimMethod <- method$optimMethod; optim_maxit <- method$optim_maxit; adj.overdisp <- method$adj.overdisp; hesstype <- method$hesstype; method <- method$method
  #hesstype <- as.integer(ifelse(hess=='asympDiagAdj',2,1)); optimMethod <- as.integer(ifelse(optimMethod=='CDA',2,1))
    
  B <- as.integer(B)
  tmp= codeGroupsAndConstraints(p=p,groups=groups)
  ngroups= tmp$ngroups; constraints= tmp$constraints; invconstraints= tmp$invconstraints; nvaringroup=tmp$nvaringroup; groups=tmp$groups
  tmp= formatmsPriorsMarg(priorCoef=priorCoef, priorGroup=priorGroup, priorVar=priorVar, priorSkew=priorSkew, n=n)
  r= tmp$r; prior= tmp$prior; priorgr= tmp$priorgr; tau=tmp$tau; taugroup=tmp$taugroup; alpha=tmp$alpha; lambda=tmp$lambda; taualpha=tmp$taualpha; fixatanhalpha=tmp$fixatanhalpha
  a= tmp$a

  if (!is_formula) {
    sel <- check_sel_groups(sel, groups)
    sel <- as.integer(sel-1); nsel <- as.integer(length(sel))
  } else {
    if (!missing(sel)) warning("y is of type formula: ignoring sel argument")
    sel <- as.integer(seq(p)-1)
    nsel <- length(sel)
  }

  ans <- marginalLikelihoodCI(knownphi, sel, nsel, familyint, prior, priorgr, n, p, y, uncens, sumy2, sumy, sumlogyfact, x, colsumsx, XtX, ytX, method, adj.overdisp, hesstype, optimMethod, optim_maxit, thinit, usethinit, B, alpha, lambda, tau, taugroup, taualpha, fixatanhalpha, r, a, groups, ngroups, nvaringroup, constraints, invconstraints, Dmat, logscale)
  return(ans)
}

check_sel_groups <- function(sel, groups) {
  p <- length(groups); seqp <- seq(p)
  if (is.logical(sel)) sel <- seqp[sel]
  if (any(sel > p)) stop("found index in sel larger than ncol(x). Please make sure all indexes refer to existing variables")
  invsel <- seqp[!(seqp %in% sel)]
  if (any(groups[sel] %in% groups[invsel])) stop("selected indexes incompatible with defined groups. Make sure each group is selected or discarded at once")
  return(sel)
}


##############################################################################################
## EXPECTATION OF PRODUCT OF SQUARES
##############################################################################################


eprod <- function(m, S, power=1, dof= -1) {
  #Mean of prod (x_i)^(2*power) when x_i ~ T_dof(mu,sigma). Set dof=-1 for N(mu,sigma). Written by John Cook
  ans <- eprod_I(as.double(m),as.double(S), as.integer(length(m)), as.integer(power), as.double(dof))
  ans
}

###
### ggm.R
###

### Methods for msfit_ggm objects

plot.msfit_ggm= function(x, y, ...) {
  postSample= x$postSample[,x$indexes[1,] != x$indexes[2,],drop=FALSE]
  margppcum= apply(postSample !=0, 2, cumsum) / seq_len(nrow(postSample))
  plot(margppcum[,1], type='l', ylim=c(0,1), xlab='Iteration', ylab='Marginal posterior inclusion probabilities')
  if (ncol(margppcum)>1) for (i in 2:min(5000,ncol(margppcum))) lines(margppcum[,i])
}


setMethod("show", signature(object='msfit_ggm'), function(object) {
  message('Gaussian graphical model (msfit_ggm object) with ',object$p,' variables\n')
  message("Use coef() to get BMA estimates, posterior intervals and posterior marginal prob of entries being non-zero\n")
  message("use postProb() for posterior model probabilities")
}
)

coef.msfit_ggm <- function(object,...) {
  m= as.vector(object$postmean) #use Rao-Blackwellized estimator
  #m= Matrix::colMeans(object$postSample) #use MCMC estimator
  ci= sparseMatrixStats::colQuantiles(object$postSample, prob=c(0.025,0.975))
  ans= cbind(t(object$indexes), m, ci, object$margpp) #use Rao-Blackwellized edge inclusion probabilities
  colnames(ans)[-1:-2]= c('estimate','2.5%','97.5%','margpp')
  return(ans)
}

icov <- function(fit, threshold) {
  if (!inherits(fit, 'msfit_ggm')) stop("Argument fit must be of class msfit_ggm")
  m= as.vector(fit$postmean) #use Rao-Blackwellized estimator
  #m= Matrix::colMeans(fit$postSample) #use MCMC estimator
  if (!missing(threshold)) {
      sel= (fit$margpp >= threshold) #use Rao-Blackwellized edge inclusion probabilities
      #sel= (Matrix::colMeans(fit$postSample != 0) >= threshold) #use MCMC edge inclusion probabilities
      ans= Matrix::sparseMatrix(i= fit$indexes[1,sel], j= fit$indexes[2,sel], x=m[sel], dims=c(fit$p,fit$p), symmetric=TRUE)
  } else {
      ans= matrix(nrow=fit$p,ncol=fit$p)
      ans[upper.tri(ans,diag=TRUE)]= m
      ans[lower.tri(ans)]= t(ans)[lower.tri(ans)]
  }
  return(ans)
}


proportion_visited_models= function(postSample) {
  modelpp <- apply(postSample != 0, 1, function(z) paste(which(z),collapse=','))
  modelpp <- table(modelpp)/length(modelpp)
  modelpp <- data.frame(modelid=names(modelpp), pp=as.numeric(modelpp))
  return(modelpp)
}

setMethod("postProb", signature(object='msfit_ggm'), function(object, nmax, method='norm') {
if (!is.null(object$models)) {
  ans= object$models
} else {
  param_ids= paste('(',object$indexes[1,], ',', object$indexes[2,], ')',sep='')
  npar= ncol(object$indexes)
  #Compute posterior probabilities
  modelpp = proportion_visited_models(object$postSample)
  #Compute model identifiers
  tmp= strsplit(modelpp$modelid, ',')
  modelid= t(sapply(tmp, function(z) { ans= rep(FALSE,npar); ans[as.integer(z)]= TRUE; return(ans) }))
  colnames(modelid)= param_ids
  modelid= Matrix::Matrix(modelid, sparse=TRUE)
  #Sort models in decreasing probability
  o= order(modelpp$pp,decreasing=TRUE)
  modelpp= modelpp[o,]
  modelid= modelid[o,]
  #Return nmax models    
  if (!missing(nmax)) {
      modelpp <- modelpp[1:nmax,]
      modelid = modelid[1:nmax,]
  }
  #Return output
  ans= list(modelid= modelid, pp=modelpp$pp)
}
return(ans)
}
)



### Model selection routines

modelSelectionGGM= function(y, priorCoef=normalidprior(tau=1), priorModel=modelbinomprior(1/ncol(y)), priorDiag=exponentialprior(lambda=1), center=TRUE, scale=TRUE, global_proposal= "none", prob_global=0.5, tempering=0.5, truncratio= 100, save_proposal=FALSE, niter=10^3, burnin= round(niter/10), updates_per_iter= ceiling(sqrt(ncol(y))), updates_per_column= 10, sampler='Gibbs', pbirth=0.75, pdeath=0.5*(1-pbirth), bounds_LIT, Omegaini='glasso-ebic', verbose=TRUE) {
  #Check input args
  if (!is.matrix(y)) y = as.matrix(y)
  p= ncol(y);
  if (p <=1) stop("y must have at least 2 columns")
  if (!is.numeric(y)) stop("y must be numeric")
  if (!(sampler %in% c('Gibbs','birthdeath','LIT'))) stop("sampler must be 'Gibbs', 'birthdeath', or 'LIT'")
  if (tempering < 0) stop("tempering cannot be negative")
  y = scale(y, center=center, scale=scale)
    
  #Format prior parameters
  prCoef= formatmsPriorsMarg(priorCoef=priorCoef, priorVar=priorDiag)
  prCoef= as.list(c(priorlabel=prCoef$priorCoef@priorDistr, prCoef[c('prior','tau','lambda')]))
  prModel= as.list(c(priorlabel=priorModel@priorDistr, priorPars= priorModel@priorPars))
  prModel[grep("priorPars", names(prModel))] <- as.double(prModel[grep("priorPars", names(prModel))])
    
  #Format posterior sampler parameters
  if (missing(bounds_LIT)) bounds_LIT= log(c("lbound_death"=1/p, "ubound_death"= 1, "lbound_birth"=1/p, "ubound_birth"=p))
  samplerPars= format_GGM_samplerPars(sampler, p=p, niter=niter, burnin=burnin, updates_per_iter=updates_per_iter, updates_per_column = updates_per_column, pbirth=pbirth, pdeath=pdeath, prob_global=prob_global, tempering=tempering, truncratio=truncratio, global_proposal=global_proposal, bounds_LIT=bounds_LIT, verbose=verbose)
    
  #Initial value for sampler
  if (verbose) message(" Obtaining initial parameter estimate...")
  Omegaini= initialEstimateGGM(y, Omegaini)
  if (verbose) message(" Done\n")
    
  #Call C++ function
  proposal= proposaldensity= NULL
  if (global_proposal == 'none') {
    ans= modelSelectionGGMC(y, prCoef, prModel, samplerPars, Omegaini)
    postSample= Matrix::t(ans$postSample)
    postmean = ans$postmean
    margpp= ans$margpp
    prop_accept= ans$prop_accept
  } else {
    ans= modelSelectionGGM_globalC(y, prCoef, prModel, samplerPars, Omegaini)
    postSample= Matrix::t(ans[[1]])
    postmean= ans[[2]]
    margpp= ans[[3]]
    prop_accept= ans[[4]]
    if (save_proposal) {
      proposal= lapply(ans[5:(5+p-1)], Matrix::t)
      for (i in seq_len(length(proposal))) proposal[[i]]@x = as.double(proposal[[i]]@x)
      proposaldensity= Matrix::t(ans[[length(ans)]])
    }
  }
    

  #Return output
  priors= list(priorCoef=priorCoef, priorModel=priorModel, priorDiag=priorDiag)

  A= diag(p)
  indexes= rbind(row(A)[upper.tri(row(A),diag=TRUE)], col(A)[upper.tri(row(A),diag=TRUE)])
  rownames(indexes)= c('row','column')

  ans= list(postSample=postSample, prop_accept=prop_accept, proposal=proposal, proposaldensity=proposaldensity, postmean=postmean, margpp=margpp, priors=priors, p=p, indexes=indexes, samplerPars=samplerPars, global_proposal=global_proposal)

  new("msfit_ggm",ans)
}



#Format posterior sampling parameters to pass onto C++
format_GGM_samplerPars= function(sampler, p, niter, burnin, updates_per_iter, updates_per_column, pbirth, pdeath, prob_global, tempering, truncratio, global_proposal, bounds_LIT, verbose) {
  if (missing(updates_per_column)) {
      updates_per_column= as.integer(min(p, max(log(p), 10)))
  } else {
      updates_per_column= as.integer(updates_per_column)
  }
  if (!(global_proposal %in% c('none','regression','in-sample'))) stop("global_proposal must be 'none', 'regression' or 'in-sample'")
  if ((updates_per_iter < 1) || (updates_per_iter > p)) stop("updates_per_iter must be between 1 and ncol(y)")
  if ((pbirth + pdeath) > 1) stop("pbirth + pdeath must be <=1")
  if (sampler == "LIT") {
    psum= (pbirth + pdeath)
    pbirth = pbirth / psum
    pdeath = pdeath / psum
  }

  if (!is.vector(bounds_LIT)) stop("bounds_LIT must be a vector")
  if (!all(c("lbound_death", "ubound_death", "lbound_birth", "ubound_birth") %in% names(bounds_LIT))) stop("bounds_LIT must be a named vector containing 'lbound_death', 'ubound_death', 'lbound_birth', 'ubound_birth'")
  
  samplerPars= list(sampler, as.integer(niter), as.integer(burnin), as.integer(updates_per_iter), as.integer(updates_per_column), as.double(pbirth), as.double(pdeath), as.double(prob_global), as.double(tempering), as.double(truncratio), global_proposal, as.double(bounds_LIT["lbound_death"]), as.double(bounds_LIT["ubound_death"]), as.double(bounds_LIT["lbound_birth"]), as.double(bounds_LIT["ubound_birth"]), as.integer(ifelse(verbose,1,0)))
  names(samplerPars)= c('sampler','niter','burnin','updates_per_iter','updates_per_column','pbirth','pdeath','prob_global','tempering','truncratio','global_proposal','lbound_death','ubound_death','lbound_birth','ubound_birth','verbose')
  return(samplerPars)
}

#Multivariate normal density given the precision matrix
dmvnorm_prec <- function(x, sigmainv, logdet.sigmainv, mu = rep(0, ncol(sigmainv)), log = FALSE) {
  if (missing(logdet.sigmainv)) logdet.sigmainv= determinant(sigmainv,logarithm=TRUE)$modulus
  d= mahalanobis(x, center=mu, cov=sigmainv, inverted=TRUE)
  ans= -0.5 * (d + ncol(sigmainv) * log(2*pi) - logdet.sigmainv)
  if(!log) ans= exp(ans)
  return(ans)
}

#Initial estimate of precision matrix
initialEstimateGGM= function(y, Omegaini) {
    
  if (is.character(Omegaini)) {
    ans= initGGM(y, Omegaini)
  } else {
    if (ncol(Omegaini) != nrow(Omegaini)) stop("Omegaini must be a square matrix")
    if (ncol(y) != ncol(Omegaini)) stop("ncol(Omegaini) must be equal to ncol(y)")
    if (inherits(Omegaini, "matrix")) {
    ans= Matrix::Matrix(Omegaini, sparse=TRUE)
  } else if (inherits(Omegaini, c("dgCMatrix", "ddiMatrix", "dsCMatrix"))) {
    ans= Omegaini
  } else stop("Invalid Omegaini. It must be of class matrix, dgCMatrix, dsCMatrix or ddiMatrix")
  }
  return(ans)
    
}



initGGM= function(y, Omegaini) {
  
  maxiter= ifelse (ncol(y) <=100, 5, 1)  #use 1 iteration for large p, to avoid excessive init time
  
  if (Omegaini=='null') {
    
    ans= Matrix::sparseMatrix(seq_len(ncol(y)), seq_len(ncol(y)), x=rep(1,ncol(y)), dims=c(ncol(y),ncol(y)))
    
  } else if (Omegaini %in% c('glasso-bic','glasso-ebic')) {
    if (Omegaini == 'glasso-bic') {
      method <- "BIC"
      #gamma = 0
    } else if (Omegaini == 'glasso-ebic') {
      #gamma = 0.5
      method <- "EBIC"
    }
    
    sfit= huge(y, method="glasso", scr=TRUE, verbose=FALSE, nlambda = 10)
    ebic= glasso_getBIC(y=y, sfit=sfit, method=method)
    
    #topbic= which.min(sfit2$ebic.score)
    topbic= which.min(ebic)
    found= (topbic != 1) & (topbic != 10)
    
    niter= 1
    while ((!found) && (niter<=maxiter)) {
      ## huge wants a decreasing sequence
      if((niter == 1) && (topbic == 1)){## No need for lambda bigger
        break
      }
      if (topbic==10) {
        rholist= seq(sfit$lambda[10], sfit$lambda[10]/10, length=10)
      } else {
        rholist= seq(10*sfit$lambda[1], sfit$lambda[1], length=10)
      }
      
      sfit= huge(y, lambda=rholist, method="glasso", scr=TRUE, verbose=FALSE)
      ebic= glasso_getBIC(y=y, sfit=sfit, method=method)
      
      topbic= which.min(ebic)
      found= (topbic != 1) & (topbic != 10)
      niter= niter+1
      
    }
    
    ans= Matrix::Matrix(sfit$icov[[topbic]], sparse=TRUE)
    
  } else {
    stop("Omegaini must be 'null', 'glasso-ebic' or 'glasso-bic'")
  }
  
  return(ans)
  
}


#Extraction of BIC/EBIC from object returned by huge
glasso_getBIC = function(y, sfit, method='EBIC') {
  n= nrow(y); d= ncol(y)
  if (method == 'BIC') {
    gamma = 0
  } else if (method == 'EBIC') {
    gamma = 0.5
  }
  #ans = -n * sfit$loglik + log(n) * sfit$df + 4 * gamma * log(d) * sfit$df
  #Manual implementation
  logl= npar= double(length(sfit$icov))
  for (i in seq_len(length(logl))) {
    logl[i]= sum(dmvnorm_prec(y, sigmainv=sfit$icov[[i]], log=TRUE))
    npar[i]= (sum(sfit$icov[[i]] != 0) - d)/2
  }
  if (method == 'BIC') {
    ans= -2*logl + npar * log(n)
  } else if (method == 'EBIC') {
    ans= -2*logl + npar * (log(n) + 4 * gamma * log(d))
  }
  return(ans)
}




#Model log-joint (log marginal likelihood + log model prior) for the non-zero entries in colsel of Omega
#This is an R version of the C++ function GGM_rowmarg, build for debugging purposes
GGM_rowmargR= function(y, colsel, Omega, model= (Omega[,colsel]!=0), priorCoef=normalidprior(tau=1), priorModel=modelbinomprior(1/ncol(y)), priorDiag=exponentialprior(lambda=1), center=TRUE, scale=TRUE) {
  #Format input
  y = scale(y, center=center, scale=scale)
  prCoef= formatmsPriorsMarg(priorCoef=priorCoef, priorVar=priorDiag)
  prCoef= as.list(c(priorlabel=prCoef$priorCoef@priorDistr, prCoef[c('prior','tau','lambda')]))
  prModel= as.list(c(priorlabel=priorModel@priorDistr, priorPars= priorModel@priorPars))
  lambda= prCoef[["lambda"]]
  tau= prCoef[["tau"]]
  if (sum(model)>1) {
    S= t(y) %*% y
    #Obtain Omegainv, select submatrix for model
    Omegainv= solve(Omega[-colsel,-colsel,drop=FALSE])
    Omegainv_model= Omegainv[model[-colsel], model[-colsel], drop=FALSE]
    #Obtain posterior mean m and covariance Uinv
    U= (S[colsel,colsel] + lambda) * Omegainv_model + diag(1/tau, nrow=nrow(Omegainv_model))
    Uinv= solve(U)
    idx= which(model)
    idx= idx[idx != colsel]
    s= S[idx, colsel, drop=FALSE]
    m= Uinv %*% s
    #Log marginal likelihood
    logmarg= 0.5 * t(m) %*% U %*% m - 0.5 * sum(model) * log(tau) - 0.5 * log(det(U))
  } else {
    logmarg= - 0.5 * sum(model) * log(tau)
    Omegainv= Omegainv_model= Uinv= m= NULL
  }
  #Log model prior
  p= 1/ncol(y); nsel= sum(model[-colsel])
  logprior= nsel * log(p) + (length(model) - 1 - nsel) * log(1-p)
  #Returns
  ans= list(logjoint=logmarg+logprior, Omegainv=Omegainv, Omegainv_model=Omegainv_model, m=m, Uinv=Uinv)
  return(ans)
}




## huge

huge <- function (x, lambda = NULL, nlambda = NULL, lambda.min.ratio = NULL, 
    method = "mb", scr = NULL, scr.num = NULL, cov.output = FALSE, 
    sym = "or", verbose = TRUE) {
    
    gcinfo(FALSE)
    est = list()
    est$method = method

    if (method == "glasso") {
        fit = huge.glasso(x, nlambda = nlambda, lambda.min.ratio = lambda.min.ratio, 
            lambda = lambda, scr = scr, cov.output = cov.output, 
            verbose = verbose)
        est$path = fit$path
        est$lambda = fit$lambda
        est$icov = fit$icov
        est$df = fit$df
        est$sparsity = fit$sparsity
        est$loglik = fit$loglik
        if (cov.output) 
            est$cov = fit$cov
        est$cov.input = fit$cov.input
        est$cov.output = fit$cov.output
        est$scr = fit$scr
        rm(fit)
        gc()
    } else {
        stop ("method not implemented")
    }

    est$data = x
    rm(x, scr, lambda, lambda.min.ratio, nlambda, cov.output, 
        verbose)
    gc()
    return(est)

}



#' The graphical lasso (glasso) using sparse matrix output 
#' 
#' @param x There are 2 options: (1) \code{x} is an \code{n} by \code{d} data matrix (2) a \code{d} by \code{d} sample covariance matrix. The program automatically identifies the input matrix by checking the symmetry. (\code{n} is the sample size and \code{d} is the dimension).
#' @param lambda A sequence of decreasing positive numbers to control the regularization when \code{method = "mb"}, \code{"glasso"} or \code{"tiger"}, or the thresholding in \code{method = "ct"}. Typical usage is to leave the input \code{lambda = NULL} and have the program compute its own \code{lambda} sequence based on \code{nlambda} and \code{lambda.min.ratio}. Users can also specify a sequence to override this. When \code{method = "mb"}, \code{"glasso"} or \code{"tiger"}, use with care - it is better to supply a decreasing sequence values than a single (small) value.
#' @param nlambda The number of regularization/thresholding parameters. The default value is \code{30} for \code{method = "ct"} and \code{10} for \code{method = "mb"}, \code{"glasso"} or \code{"tiger"}.
#' @param lambda.min.ratio If \code{method = "mb"}, \code{"glasso"} or \code{"tiger"}, it is the smallest value for \code{lambda}, as a fraction of the upperbound (\code{MAX}) of the regularization/thresholding parameter which makes all estimates equal to \code{0}. The program can automatically generate \code{lambda} as a sequence of length = \code{nlambda} starting from \code{MAX} to \code{lambda.min.ratio*MAX} in log scale. If \code{method = "ct"}, it is the largest sparsity level for estimated graphs. The program can automatically generate \code{lambda} as a sequence of length = \code{nlambda}, which makes the sparsity level of the graph path increases from \code{0} to \code{lambda.min.ratio} evenly.The default value is \code{0.1} when \code{method = "mb"}, \code{"glasso"} or \code{"tiger"}, and 0.05 \code{method = "ct"}.
#' @param scr If \code{scr = TRUE}, the lossy screening rule is applied to preselect the neighborhood before the graph estimation. The default value is  \code{FALSE}. NOT applicable when \code{method = "ct"}, {"mb"}, or {"tiger"}.
#' @param cov.output If \code{cov.output = TRUE}, the output will include a path of estimated covariance matrices. ONLY applicable when \code{method = "glasso"}. Since the estimated covariance matrices are generally not sparse, please use it with care, or it may take much memory under high-dimensional setting. The default value is \code{FALSE}.
#' @param verbose If \code{verbose = FALSE}, tracing information printing is disabled. The default value is \code{TRUE}.
huge.glasso = function(x, lambda = NULL, lambda.min.ratio = NULL, nlambda = NULL, scr = NULL, cov.output = FALSE, verbose = TRUE){

  gcinfo(FALSE)
  n = nrow(x)
  d = ncol(x)
  cov.input = isSymmetric(x)
  if(cov.input)
  {
    if(verbose) cat("The input is identified as the covariance matrix.\n")
    S = x
  }
  else
  {
    x = scale(x)
    S = cor(x)
  }
  rm(x)
  gc()
  if(is.null(scr)) scr = FALSE
  if(!is.null(lambda)) nlambda = length(lambda)
  if(is.null(lambda))
  {
    if(is.null(nlambda))
      nlambda = 10
    if(is.null(lambda.min.ratio))
      lambda.min.ratio = 0.1
    lambda.max = max(max(S-diag(d)),-min(S-diag(d)))
    lambda.min = lambda.min.ratio*lambda.max
    lambda = exp(seq(log(lambda.max), log(lambda.min), length = nlambda))
  }

  fit = .Call("_modelSelection_hugeglasso",S,lambda,scr,verbose,cov.output)

  fit$scr = scr
  fit$lambda = lambda
  fit$cov.input = cov.input
  fit$cov.output = cov.output

  rm(S)
  gc()
  if(verbose){
       cat("\nConducting the graphical lasso (glasso)....done.                                          \r")
       cat("\n")
      #flush.console()
  }
  return(fit)
}




## PRIOR ELICITATION ROUTINES


#\name{randompdmatrix_setpars}
#\alias{randompdmatrix_setpars}
#\title{
# 
#Set parameters such that symmetric random matrix 
#is positive-definite with high probability
# 
#}
#\description{
# 
#Obtain prior parameters s.t. separable symmetric random matrix Theta 
#is positive-definite with prob 0.95
# 
#Its off-diagonal entries are drawn as theta[i,j] = 0 with prob 1 - edge_prob 
#and theta[i,j] ~ N(0, sigma^2) otherwise, independently across i < j
# 
#Its diagonal entries theta[i,i] can either be fixed to diag_mean or randomly 
# 
# 
#}
#\usage{
# 
#randompdmatrix_setpars(p, edge_prob, diag_distrib = 'Exp', diag_mean, method='analytical', nsims=1000)
# 
#}
#\arguments{
# 
#\item{p}{Matrix dimension}
# 
#\item{edge_prob}{Probability that off-diagonal Theta[i,j] is non-zero}
# 
#\item{diag_distrib}{If "fixed" then Theta[i,i] = diag_mean. 
#If "Exp" then Theta[i,i] ~ Exp(rate= 1/diag_mean)}
# 
#\item{diag_mean}{Mean of the diagonal Theta[i,i]. By default it's set 
#to 1 for diag_distrib == "fixed", 
#and such that P(theta[i,i] >= 1) =0.95 for diag_distrib == "Exp"}
# 
#\item{method}{Set to "MC" to estimate the probability exactly,
#"analytical" for an asymptotic guess. See the details section.}
# 
#\item{nsims}{Number of simulations}
# 
#}
#\details{
# 
#The analytical estimate of the off-diagonal variance sigma 
#is obtained as follows. 
#If diag(Theta) is fixed, then sigma is set such that
# 
#thetamin > 2.01 sigma sqrt(edge_prob p)
# 
#where thetamin = min(diag(Theta)), since as p -> infty this guarantees
#that Theta is positive-definite with probability 1 (Carter & Rossell 2026)
# 
#If theta[i,i] ~ Exp(1/diag_mean[i]), then sigma is set such that
# 
#P(thetamin > 2.01 sigma sqrt(edge_prob p)) = 0.95, 
# 
#using that thetamin ~ Exp(rate= sum 1 / diag_mean).
#If the probability above goes to 1 as p -> infty, this guarantees
#that Theta is positive-definite with probability 1 (Carter & Rossell 2026)
# 
#The Monte Carlo estimate of sigma is based on an iterative logistic
#regression approach. Specifically, it simulates nsim matrices
#Theta for a range of sigma values around its analytical estimate,
#estimates P(Theta is pos-def) as a function of sigma via 
#logistic regression and updates sigma such that the predicted
#P(Theta is pos-def | sigma)= 0.95.
#The process is then repeated: given the updated sigma, 
#nsim new matrices are drawn around that sigma
#and a new logistic regression is fit to update sigma
# 
#If p > 200, Monte Carlo can take too long so instead a bias-corrected
#version of the analytical estimate is used
# 
#}
#\value{
# 
#A list with two entries
# 
#\item{diag_mean}{Equals the input parameter if it was non-missing. 
#Otherwise it's set to its default value, specified above}
# 
#\item{offdiag_variance}{sigma2, i.e. variance of non-zero 
#off-diagonal Theta[i,j] ~ N(0, edge_variance)}
# 
#}
#\references{
#Jack Carter, David Rossell. Positive-definiteness in separable priors: effects on prior interpretability and inference. arXiV 2026
#}
#\author{
#David Rossell
#}
#\seealso{
#\code{
#\link{modelSelectionGGM}} to perform model selection for graphical models
#}
#\examples{
# 
## Prior parameters based on asymptotics as p -> infty
#randompdmatrix_setpars(p=10, edge_prob=0.1, diag_distrib='Exp', method='analytical')
# 
## Prior parameters based on Monte Carlo for fixed p
##randompdmatrix_setpars(p=10, edge_prob=0.1, diag_distrib='Exp', method='MC')
# 
#}
#\keyword{ models }
#\keyword{ distribution }


randompdmatrix_setpars <- function(p, edge_prob, diag_distrib = 'Exp', diag_mean, method='analytical', nsims=1000) {
  # Check input parameters
  if (!method %in% c('analytical','MC')) stop("method must be 'analytical' or 'MC'")
  diag_mean <- checkpars_pdmatrix_prob(p=p, diag_distrib=diag_distrib, diag_mean=diag_mean, method=method)
  # Obtain initial guess based on analytical lower-bound
  if (diag_distrib == 'Exp') {
    fexp= function(v) return((log(0.95) - pexp(2.01 * sqrt(v * edge_prob * p), rate= sum(1 / diag_mean), lower.tail=FALSE, log.p=TRUE))^2)
    #fexp= function(v) return((0.95 - pexp(2.01 * sqrt(v * edge_prob * p), rate= sum(1 / diag_mean), lower.tail=FALSE))^2)
    offdiag_variance <- optim(par=1, fn=fexp, lower=0, upper=1, method="Brent")$par
  } else {
    offdiag_variance <- min(diag_mean)^2 / (2.01^2 * p * edge_prob)
  }
  # If method != 'analytical', estimate via Monte Carlo
  if (method != 'analytical') {
    if (p > 200) {
      offdiag_variance <- randompdmatrix_setpars_extrapolate(p=p, edge_prob=edge_prob, diag_distrib=diag_distrib, diag_mean=diag_mean, nsims=nsims)$offdiag_variance
    } else {
      logvseq <- seq(log(offdiag_variance/4), log(10000 * offdiag_variance), length=nsims)  #SD from 1/2 to 100 times the analytical value
      prob_pd <- priorprob_pdmatrix(p=p, edge_prob=edge_prob, offdiag_variance=exp(logvseq), diag_mean=diag_mean, diag_distrib=diag_distrib, nsims=1)
      b <- coef(glm(prob_pd ~ logvseq, family=binomial()))
      vlow  <- -(log(1 / 0.975 - 1) + b[1])/b[2]
      vup  <- -(log(1 / 0.925 - 1) + b[1])/b[2]
      #vguess0  <- -(log(1 / 0.95 - 1) + b[1])/b[2]
      logvseq <- seq(vlow, vup, length=nsims)  #SD within range of predicted prob of positive-def in [0.927, 0.975]
      prob_pd <- priorprob_pdmatrix(p=p, edge_prob=edge_prob, offdiag_variance=exp(logvseq), diag_mean=diag_mean, diag_distrib=diag_distrib, nsims=1)
      b <- coef(glm(prob_pd ~ logvseq, family=binomial()))
      offdiag_variance  <- as.double(exp(-(log(1 / 0.95 - 1) + b[1])/b[2]))
    }
  }
  ans <- list(diag_mean=diag_mean, offdiag_variance=offdiag_variance)
  return(ans)
}

# Same as randompdmatrix_setpars, using bias-corrected analytical guess of sigma to extrapolate to large p
randompdmatrix_setpars_extrapolate <- function(p, edge_prob, diag_distrib = 'Exp', diag_mean, nsims=1000) {
  diag_mean <- min(diag_mean)
  # Obtain analytical and Monte Carlo estimates of sigma2
  pseq = round(seq(10, 80, length=10))
  sigma2_an = sigma2_mc = double(length(pseq))
  for (i in 1:length(pseq)) {
    sigma2_an[i] <- randompdmatrix_setpars(p=pseq[i], edge_prob=edge_prob, diag_mean=diag_mean, diag_distrib=diag_distrib, method="analytical")$offdiag_variance
    sigma2_mc[i] <- randompdmatrix_setpars(p=pseq[i], edge_prob=edge_prob, diag_mean=diag_mean, diag_distrib=diag_distrib, method="MC", nsims=nsims)$offdiag_variance
  }
  # Estimate bias of the analytical estimate
  b <- coef(lm(log(sigma2_an) - log(sigma2_mc) ~ pseq))
  bias_pred <- as.double(b[1] + b[2] * p)
  # Obtain bias-corrected analytical estimate
  ans <- randompdmatrix_setpars(p=p, edge_prob=edge_prob, diag_mean=diag_mean, diag_distrib=diag_distrib, method="analytical")
  ans$offdiag_variance <- exp(log(ans$offdiag_variance) - bias_pred)
  return(ans)
}


# Check input parameters of randompdmatrix_setpars, and return default diag_mean when missing
checkpars_pdmatrix_prob <- function(p, diag_distrib, diag_mean, method) {
  # Check input parameters
  if (! diag_distrib %in% c('fixed','Exp')) stop("diag_distrib muse be 'fixed' or 'Exp'")
  if (missing(diag_mean)) {
    if (diag_distrib == 'fixed') {
      diag_mean <- rep(1, p) 
    } else {
      # Since theta_{ii} ~ Exp(1/diag_mean), set diag_mean such that P(theta_{ii} >= 1)= 0.99
      diag_mean <- rep(1/0.01, p) # for 0.99 tail prob
      #diag_mean <- rep(1/0.052, p) # for 0.95 tail prob
      # Since min_i theta_{ii} ~ Exp(p/diag_mean), setting diag_mean/p= 1/0.01 achieves P(min_i theta_{ii} >= 1) = 0.99
      # diag_mean <- rep(p/0.01, p)
    }
  } else {
    if (length(diag_mean) == 1) {
      diag_mean = rep(diag_mean, p)
    } else if (length(diag_mean) != p) {
      stop("diag_mean must have length equal 1 or p")
    }
  }
  return(diag_mean)
}


# Estimate the probability that a p x p separable symmetric random matrix Theta is positive-definite
# - p : matrix dimension
# - edge_prob: probability that off-diagonal Theta[i,j] is non-zero
# - offdiag_variance: variance of non-zero off-diagonal Theta[i,j] ~ N(0, edge_variance)
# - diag_mean: mean of the diagonal Theta[i,i]
# - diag_distrib: if "fixed" then Theta[i,i] = diag_mean. If 'Exp' then Theta[i,i] ~ Exp(rate= 1/diag_mean)
# - nsims: number of simulations
# Output: Monte Carlo estimate of the probability that Theta is positive-definite
priorprob_pdmatrix <- function(p, edge_prob, offdiag_variance, diag_mean, diag_distrib = 'Exp', nsims=5000) {
  diag_mean <- checkpars_pdmatrix_prob(p=p, diag_distrib=diag_distrib, diag_mean=diag_mean)
  ans <- double(length(offdiag_variance))
  for (j in 1:length(ans)) {
    # Format input parameters
    if (length(diag_mean) == 1) {
      diag_mean = rep(diag_mean, p)
    } else if (length(diag_mean) != p) {
      stop("diag_mean must have length equal 1 or p")
    }
    if (! diag_distrib %in% c('fixed','Exp')) stop("diag_distrib muse be 'fixed' or 'Exp'")
    # Simulate Theta
    n_offdiag <- p * (p-1) / 2
    Theta <- matrix(NA, nrow=p, ncol=p)
    if (diag_distrib == 'fixed') diag(Theta) = diag_mean
    pd <- logical(nsims)
    for (i in 1:nsims) {
      # Generate diagonal entries
      if (diag_distrib == 'Exp') {
        diag(Theta) = rexp(p, rate= 1/diag_mean)
      }
      # Generate off-diagonal
      nonzero <- (runif(n_offdiag) < edge_prob)
      Theta[upper.tri(Theta)][nonzero] <- rnorm(sum(nonzero), 0, sd=sqrt(offdiag_variance[j]))
      Theta[upper.tri(Theta)][!nonzero] <- 0
      Theta[lower.tri(Theta)] <- t(Theta)[lower.tri(Theta)]
      # Check positive-definiteness
      pd[i] <- (min(eigen(Theta)$values) > 0)
    }
    ans[j] <- mean(pd)
  }
  return(ans)
}

modelsearchBlockDiag <- function(y, x, priorCoef=momprior(tau=0.348), priorModel=modelbbprior(1,1), priorVar=igprior(0.01,0.01), blocksize=10, maxiter=10, maxvars=100, maxlogmargdrop=20, maxenum=10, verbose=TRUE) {
    n <- length(y); p <- ncol(x)
    #Check and format input parameters
    if (priorModel@priorDistr=='binomial' & ('p' %in% names(priorModel@priorPars))) {
        rho <- priorModel@priorPars[['p']]
        if (length(rho)>1) stop("modelsearchBlockDiag not implemented for vector p in modelbinomprior")
        priorModel <- function(nvar) nvar*log(rho) + (p-nvar)*log(1-rho)
        priorModelBlock <- function(nvar,blocksize) nvar*log(rho) + (blocksize-nvar)*log(1-rho)
    } else if (priorModel@priorDistr=='binomial' & !('p' %in% names(priorModel@priorPars))) {
        alpha=priorModel@priorPars['alpha.p']; beta=priorModel@priorPars['beta.p']
        priorModel <- function(nvar) lbeta(nvar + alpha, p - nvar + beta) - lbeta(alpha, beta)
        priorModelBlock <- function(nvar,blocksize) rep(-blocksize*log(2),length(nvar)) #set coolblock prior to unif. post prob based on correct priorModel
    } else if (priorModel@priorDistr=='uniform') {
        rho <- 0.5
        priorModel <- function(nvar) rep(-p*log(2),length(nvar))
        priorModelBlock <- function(nvar,blocksize) rep(-blocksize*log(2),length(nvar))
    } else { stop("Prior on model space not recognized. Use modelbbprior(), modelunifprior() or modelbinomprior()") }
    if (priorCoef@priorDistr == 'zellner') {
        g <- as.double(priorCoef@priorPars['tau'])
    } else if (priorCoef@priorDistr == 'pMOM') {
        tau <- as.double(priorCoef@priorPars['tau'])
        g <- n * tau
    } else {
        stop("priorCoef must be pMOM or Zellner. Use momprior() or zellnerprior()")
    }
    if (priorVar@priorDistr != 'invgamma') stop('priorVar must be an inverse gamma. Use igprior()')
    a.phi <- as.double(priorVar@priorPars['alpha'])
    l.phi <- as.double(priorVar@priorPars['lambda'])
    #Pre-compute useful quantities
    sumy2 <- l.phi + sum(y^2)
    apos <- a.phi+n
    shrinkage <- g/(1+g)
    Xty <- t(x) %*% matrix(y,ncol=1)
    #Block search forward
    bestlogpp <- bestmarg <- -Inf; iter <- 0; improved <- TRUE
    e <- y; selfixed <- integer(0)
    ans <- data.frame(nvars=integer(0), modelid=character(0), logpp=double(0), marglhood=double(0))
    while ((iter< maxiter) & improved) {
        improved <- FALSE
        #Forward pass
        ansfwd <- blocksearch(y=y,e=e,x=x,selfixed=selfixed,blocksize=blocksize,maxvars=maxvars,priorCoef=priorCoef,priorVar=priorVar,priorModel=priorModel,priorModelBlock=priorModelBlock,g=g,a.phi=a.phi,l.phi=l.phi,sumy2=sumy2,apos=apos,shrinkage=shrinkage,maxlogmargdrop=maxlogmargdrop)
        ansfwd <- ansfwd[!is.na(ansfwd$logpp),]
        selfwd.maxlogpp <- which.max(ansfwd$logpp)
        selfwd.bestmarg <- which.max(ansfwd$marglhood)
        if (ansfwd$logpp[selfwd.maxlogpp] > bestlogpp) { bestlogpp <- ansfwd$logpp[selfwd.maxlogpp]; sel.maxlogpp <- nrow(ans)+selfwd.maxlogpp; improved <- TRUE }
        if (ansfwd$marglhood[selfwd.bestmarg] > bestmarg) { bestmarg <- ansfwd$marglhood[selfwd.bestmarg]; sel.bestmarg <- nrow(ans)+selfwd.bestmarg }
        sel <- as.numeric(strsplit(ansfwd$modelid[max(selfwd.maxlogpp,selfwd.bestmarg)],split=',')[[1]])  #choose post mode or maximal integrated likelihood model, whichever is largest
        ans <- rbind(ans,ansfwd[!(ansfwd$modelid %in% ans$modelid),])
        #Backward pass
        if (length(sel)>1) {
            ansbackward <- blocksearch(y=y,x=x[,sel],blocksize=blocksize,maxvars=maxvars,priorCoef=priorCoef,priorVar=priorVar,priorModel=priorModel,priorModelBlock=priorModelBlock,g=g,a.phi=a.phi,l.phi=l.phi,sumy2=sumy2,apos=apos,shrinkage=shrinkage,maxlogmargdrop=maxlogmargdrop)
            ansbackward$modelid <- sapply(lapply(strsplit(ansbackward$modelid,','), function(z) sel[as.numeric(z)]), paste, collapse=',')
            ansbackward <- ansbackward[!(ansbackward$modelid %in% ans$modelid),]
            if (nrow(ansbackward)>0) {
                selback.maxlogpp <- which.max(ansbackward$logpp)
                selback.bestmarg <- which.max(ansbackward$marglhood)
                if (ansbackward$logpp[selback.maxlogpp] > bestlogpp) { bestlogpp <- ansbackward$logpp[selback.maxlogpp]; sel.maxlogpp <- nrow(ans)+selback.maxlogpp; improved <- TRUE }
                if (ansbackward$marglhood[selback.bestmarg] > bestmarg) { bestmarg <- ansbackward$marglhood[selback.bestmarg]; sel.bestmarg <- nrow(ans)+selback.bestmarg }
                ans <- rbind(ans,ansbackward)
            }
        }
        if (improved) {
            selfixed <- as.numeric(strsplit(ans[sel.bestmarg,'modelid'],split=',')[[1]])
            e <- y - predict(lm(y ~ x[,selfixed]))
        }
        if (verbose) {
            message("Iteration ",iter,"\n")
            message("  Posterior mode ",ans[sel.maxlogpp,'modelid'],ans[sel.maxlogpp,'logpp'],"\n")
            message("  Mode marg likelihood ",ans[sel.bestmarg,'modelid'],ans[sel.bestmarg,'marglhood'],"\n")
        }
        iter <- iter+1
    }
    #Exhaustive enumeration for submodels of highest marginal likelihood model (or highest post prob model, if the former has >maxenum variables)
    sel2enum <- which(max(ans$marglhood,na.rm=TRUE) - ans$marglhood <= maxlogmargdrop)
    sel2enum <- sel2enum[length(sel2enum)]
    if (ans$nvars[sel2enum] > maxenum) { sel2enum <- which(ans$nvars<= maxenum); sel2enum <- sel2enum[length(sel2enum)] }
    sel <- as.numeric(strsplit(ans$modelid[sel2enum],split=',')[[1]])
    sel <- sel[order(sel)]
    xsel <- x[,sel,drop=FALSE]
    if (is.null(colnames(xsel))) colnames(xsel) <- paste("x",sel,sep='')
    ms <- modelSelection(y=y,x=xsel,center=FALSE,scale=FALSE,enumerate=TRUE,maxvars=min(c(ncol(xsel),10)),priorCoef=priorCoef,priorModel=modelunifprior(),priorVar=priorVar,verbose=FALSE)
    nvarsenum <- sapply(strsplit(as.character(ms$models$modelid),split=','),length)
    modelid <- lapply(strsplit(as.character(ms$models$modelid),split=','), function(i) sel[as.numeric(i)])
    ct <- marginalLikelihood(sel=integer(0),family='normal',priorCoef=priorCoef,priorVar=priorVar,y=y,x=x,logscale=TRUE) - log(ms$models[ms$models$modelid=='','pp'])
    ansenum <- data.frame(nvars=sapply(modelid,length), modelid=sapply(modelid,paste,collapse=','), logpp=log(ms$models$pp) + priorModel(nvarsenum) + ct)
    ans <- unique(rbind(ans[,c('nvars','modelid','logpp')],ansenum[!(ansenum$modelid %in% ans$modelid),]))
    return(ans[order(ans$logpp,decreasing=TRUE),])
}


blocksearch <- function(y,e=y,x,selfixed=integer(0),blocksize,maxvars,priorCoef,priorVar,priorModel,priorModelBlock,g,a.phi,l.phi,sumy2,apos,shrinkage,maxlogmargdrop=20) {
    #Use spectral clustering followed by coolblock to define a sequence of models gamma of increasing size. Return actual p(y|gamma)p(gamma) for that sequence.
    notfixed <- setdiff(1:ncol(x), selfixed)
    nfixed <- length(selfixed)
    if (nfixed>0) {
        xn <- x[,notfixed,drop=FALSE]
    } else {
        xn <- x
    }
    #Cluster variables
    blocks <- spectralClus(xn, blocksize=blocksize)
    blocks <- as.integer(factor(blocks))
    nblocks <- max(blocks)
    blocksize <- table(blocks)
    nn <- 1:ncol(xn)
    varidx <- lapply(as.integer(names(blocksize)), function(k) nn[blocks==k])
    #Compute u-scores, least squares estimates and covariances within blocks
    Xty <- t(xn) %*% matrix(e,ncol=1)
    XtX <- lapply(1:nblocks,function(i) { sel <- which(blocks==i); t(xn[,sel]) %*% xn[,sel] })
    ubyblocks <- blockuscores(blocks, blocksize=blocksize, y=e, Xty=Xty, XtX=XtX)
    u <- ubyblocks$u; umv <- ubyblocks$umv; ubest <- ubyblocks$ubest
    #Enumerate models with coolblock and compute p(y,model)
    models <- coolblock(ubest,g=g,priorModelBlock=priorModelBlock,varidx=varidx,maxvars=maxvars)
    models <- models[!is.na(models$u),]
    modelid <- strsplit(models$modelid,split=',')
    models$modelid <- sapply(lapply(modelid,function(z) z[order(as.numeric(z))]), paste, collapse=',') #put variables ids in increasing order
    priorp <- priorModel(nfixed+models$nvars)
    marglhood <- pp <- rep(NA,nrow(models))
    i <- 1; bestpp <- -Inf; bestmarg <- -Inf; stopearly <- FALSE
    modelnames <- character(nrow(models))
    while ((i<=nrow(models)) & (!stopearly)) {
        sel <- c(selfixed,notfixed[as.numeric(modelid[[i]])])
        sel <- sel[order(sel)]
        modelnames[i] <- paste(sel,collapse=',')
        marglhood[i] <- marginalLikelihood(sel=sel,family='normal',priorCoef=priorCoef,priorVar=priorVar,y=y,x=x,logscale=TRUE)
        pp[i] <- priorp[i] + marglhood[i]
        if (pp[i] > bestpp) { bestpp <- pp[i] }
        if (marglhood[i] > bestmarg) { bestmarg <- marglhood[i] } else if (marglhood[i] + maxlogmargdrop < bestmarg) { stopearly <- TRUE }
        i <- i+1
    }
    ans <- data.frame(nvars=models[,'nvars'],modelid=modelnames,logpp=pp,marglhood=marglhood,stringsAsFactors=FALSE)
    return(ans)
}

initKmeans <- function(coord, nclus) {
    #Initialize clusters based on quantile grid on first component
    clini= as.numeric(cut(coord[,1], breaks=c(-Inf,quantile(coord[,1],probs=seq(1/nclus,1-1/nclus,length=nclus-1)),Inf)))
    centers= as.matrix(aggregate(coord,by=list(clini),FUN='mean')[,-1])
    centers= sapply(1:nclus, function(i) { sel= which(clini==i); coord[sel[which.min(colSums((t(coord[sel,])-centers[i,])^2))],] })
    if (ncol(coord)>1) centers= t(centers) else centers= matrix(centers,ncol=1)
    return(centers)
}

spectralClus <- function(x, blocksize=10, scale=FALSE, ndim= min(c(ncol(x)/blocksize,nrow(x)))) {
    #Spectral clustering of columns in x via k-means on largest eigenvalues of cor(x)^2. A deterministic initialization using quantiles in the first eigenvector is used. Clusters are iteratively subdivided until all clusters have size<=blocksize.
    # - x: data matrix whose columns we want to cluster
    # - blocksize: maximum cluster size
    # - scale: if TRUE
    # - ndim: number of eigenvectors to use in the cluster
    p <- ncol(x)
    if (p<blocksize) {
        cl <- rep(1,p)
    } else {
        xstd= scale(x,center=TRUE,scale=TRUE)
        S= cov(xstd)^2
        #if (!inverse) { S= cov(xstd)^2 } else { S= pseudoinverse(cov(xstd))^2 } #pseudoinv from package corpcor
        D= diag(p); diag(D)= 1/sqrt(rowSums(S))
        S= D %*% S %*% D
        eig= eigen(S)
        ndim <- min(c(ndim,nrow(x),p))
        if (!scale) {
            coord= t(t(eig$vectors[,1:ndim,drop=FALSE]) * sqrt(eig$values[1:ndim]))
        } else {
            coord= eig$vectors[,1:ndim,drop=FALSE]
        }
        #Initialize clusters based on quantile grid on first component
        centers= initKmeans(coord, nclus=max(round(p/blocksize),2))
        km= kmeans(coord, centers=centers)
        cl= km$cluster
        maxcl= max(cl)
        tab= table(cl)
        clussel= as.numeric(names(tab[tab>blocksize]))
        while (length(clussel)>0) {
            for (i in 1:length(clussel)) {
                sel= cl==clussel[i]
                centers= initKmeans(coord[sel,,drop=FALSE], nclus=2)
                cl[sel]= maxcl + kmeans(coord[sel,,drop=FALSE],centers=centers)$cluster
                tab[clussel[i]]= 0
                tab[maxcl+1:2]= table(cl[sel])
                names(tab)[maxcl+1:2]= as.character(maxcl+1:2)
                maxcl= maxcl+2
            }
            clussel= as.numeric(names(tab[tab>blocksize]))
        }
    }
    return(cl)
}

######################################################################################
## DIRICHLET
######################################################################################


#Density of a Dirichlet(q) ditribution evaluated at x
ddir= function (x, q, logscale=TRUE) {
    if (is.vector(x)) x= matrix(x,nrow=1)
    if (length(q)==1) q= rep(q,ncol(x))
    sel= rowSums((x<0) | (x>1))>0
    ans= double(nrow(x))
    ans[sel]= -Inf
    xnorm= t(x[!sel,,drop=FALSE])/rowSums(x[!sel,,drop=FALSE])
    ans[!sel]= colSums((q - 1) * log(xnorm)) + lgamma(sum(q)) - sum(lgamma(q))
    if (!logscale) ans= exp(ans)
    return(ans)
}


######################################################################################
## INVERSE GAMMA
######################################################################################

#Adapted from LaplacesDemon
dinvgamma= function (x, shape = 1, scale = 1, log = FALSE) {
    x <- as.vector(x)
    shape <- as.vector(shape)
    scale <- as.vector(scale)
    if (any(shape <= 0) | any(scale <= 0)) stop("The shape and scale parameters must be positive.")
    NN <- max(length(x), length(shape), length(scale))
    x <- rep(x, len = NN)
    shape <- rep(shape, len = NN)
    scale <- rep(scale, len = NN)
    alpha <- shape
    beta <- scale
    ans <- alpha * log(beta) - lgamma(alpha) - {alpha + 1} * log(x) - {beta/x}
    if (log == FALSE) ans <- exp(ans)
    return(ans)
}




######################################################################################
## INVERSE WISHART
######################################################################################

#log of the Multivariate gamma function
lmgamma= function(p,a) { 0.25*p*(p-1)*log(pi) + sum(lgamma(a + 0.5*(1-(1:p)))) }

diwish <- function(Sigma, nu, S, logscale=FALSE) {
    #Inverse Wishart density, adapted from LaplacesDemon package
    if (!is.matrix(Sigma)) Sigma <- matrix(Sigma)
    if (!is.matrix(S)) S <- matrix(S)
    if (!identical(dim(S), dim(Sigma))) stop("The dimensions of Sigma and S differ.")
    if (nu < nrow(S)) stop("The nu parameter is less than the dimension of S.")
    p <- nrow(Sigma)
    detSigma= as.numeric(determinant(Sigma,logarithm=TRUE)$modulus)
    detS= as.numeric(determinant(S,logarithm=TRUE)$modulus)
    ans <- -(nu * p/2) * log(2) - lmgamma(p,.5*nu) + (nu/2) * detS - ((nu + p + 1)/2) * detSigma - 0.5 * sum(diag((S %*% solve(Sigma))))
    if (!logscale) ans <- exp(ans)
    return(ans)
}


riwish <- function(nu, S, Sinv) {
    if (missing(Sinv)) Sinv <- solve(S)
    solve(rWishart(1, df=nu, Sigma=Sinv)[,,1])
}

##################################################################################
## Routines to simulate from the posterior distribution of parameters (BMA, and given one model)
## and to sample from non-local prior distributions
##################################################################################

setMethod("rnlp", signature(y='missing',x='missing',m='missing',V='missing',msfit='msfit',outcometype='missing',family='missing'), function(y, x, m, V, msfit, outcometype, family, priorCoef, priorGroup, priorVar, priorprec, niter=10^3, burnin=round(niter/10), thinning=1, pp='norm') {
  if ((msfit$family=='Continuous') & (msfit$family != 'normal')) stop("Posterior sampling only implemented for Normal residuals")
  y= msfit$ystd; x= msfit$xstd
  outcometype= msfit$outcometype; family= msfit$family
  isgroup= rep(FALSE, length(msfit$groups))
  isgroup[which(table(msfit$groups)>1)]= TRUE
  #Draw model
  pp <- postProb(msfit,method=pp)
  modelid <- strsplit(as.character(pp$modelid), split=',')
  ndraws <- as.numeric(rmultinom(1, size=niter, prob=pp$pp))
  sel <- ndraws>0; modelid <- modelid[sel]; ndraws <- ndraws[sel]
  priorCoef= msfit$priors$priorCoef
  priorGroup= msfit$priors$priorGroup
  priorVar= msfit$priors$priorVar
  priorprec= NULL
  #Draw coefficients
  idx <- c(0,cumsum(ndraws))
  if ((outcometype== 'Continuous') && (family== 'normal')) { ##Linear model
    ans <- matrix(0, nrow=niter, ncol=ncol(x)+1)
    for (i in 1:length(modelid)) {
      b <- min(50, ceiling((burnin/niter) * ndraws[i]))
      colsel <- as.numeric(modelid[[i]])
      if (priorCoef@priorDistr == "icarplus") priorprec = msfit$priorprec[colsel,colsel,drop=FALSE]
      ans[(idx[i]+1):idx[i+1],c(colsel,ncol(ans))] <- rnlp(y=y, x=x[,colsel,drop=FALSE], outcometype=outcometype, family=family, priorCoef=priorCoef, priorGroup=priorGroup, priorVar=priorVar, priorprec=priorprec, isgroup=isgroup[colsel], niter=b+idx[i+1]-idx[i], burnin=b)
    }
    if (is.null(colnames(x))) nn <- c(paste('beta',1:ncol(x),sep=''),'phi') else nn <- c(colnames(x),'phi')
    colnames(ans)= nn
  } else {                                                   ##GLM or Survival model
    ans <- matrix(0, nrow=niter, ncol=ncol(x))
    for (i in 1:length(modelid)) {
      b <- min(50, ceiling((burnin/niter) * ndraws[i]))
      colsel <- as.numeric(modelid[[i]])
      if (length(colsel)>0) {
        ans[(idx[i]+1):idx[i+1],colsel] <- rnlp(y=y, x=x[,colsel,drop=FALSE], outcometype=outcometype, family=family, priorCoef=priorCoef, priorGroup=priorGroup, priorVar=priorVar, isgroup=isgroup[colsel], niter=b+idx[i+1]-idx[i], burnin=b)
      }
    }
    if (is.null(colnames(x))) nn <- paste('beta',1:ncol(x),sep='') else nn <- colnames(x)
    colnames(ans)= nn
  }
  #Return posterior samples in non-standardized parameterization
  ans= unstdcoef(ans,p=ncol(x),msfit=msfit,coefnames=nn)
  return(ans)
}
)


#Return regression parameter estimates in the parameterization of non-standardized X's (i.e. where X does not have 0 mean, unit variance)
# Input
# - bstd: regression parameters, a matrix with columns corresponding to parameters and rows corresponding to different models / MCMC samples
# - p: columns 1:p in bstd contain regression coefficients for covariate effects, this is to consider that subsequent columns could contain nuisance parameters (phi, the residual variance in linear regression)
# - msfit: object returned by modelSelection.
# - coefnames: names that should be assigned to the output columns
#
# Output: matrix with same dimension as bstd, containing parameter estimates for non-standardized X's
unstdcoef <- function(bstd, p, msfit, coefnames) {
  my= msfit$stdconstants[1,'shift']; mx= msfit$stdconstants[-1,'shift']
  sy= msfit$stdconstants[1,'scale']; sx= msfit$stdconstants[-1,'scale']
  ct= (sx==0)
  b= bstd[,1:p,drop=FALSE]
  if (any(!ct)) b[,!ct]= t(t(b[,!ct])*sy/sx[!ct])  #re-scale regression coefficients
  if (any(ct)) {
      #adjust intercept, if already present
      if (any(!ct)) {
          b[,ct]= my + sy*b[,ct] - colSums(t(b[,!ct,drop=FALSE])*mx[!ct])
      } else {
          b[,ct]= my + sy*b[,ct]
      }
      bstd[,1:p]= b
  } else {
      #add intercept, if not already present
      intercept= my - colSums(t(b[,!ct,drop=FALSE])*mx[!ct]) 
      bstd= cbind(intercept,b,bstd[,-1:-p]); colnames(bstd)= c('intercept',coefnames)
  }
  if ('phi' %in% coefnames) bstd[,'phi']= sy^2*bstd[,'phi'] #re-scale residual variance
  return(bstd)
}



setMethod("rnlp", signature(y='ANY',x='matrix',m='missing',V='missing',msfit='msfit',outcometype='missing',family='missing'), function(y, x, m, V, msfit, outcometype, family, priorCoef, priorGroup, priorVar, priorprec, isgroup, niter=10^3, burnin=round(niter/10), thinning=1, pp='norm') {
    warning("If msfit is specified arguments y and x are ignored")
    rnlp(msfit=msfit, priorprec=priorprec, niter=10^3, burnin=round(niter/10), thinning=1, pp='norm')
}
)


setMethod("rnlp", signature(y='ANY',x='matrix',m='missing',V='missing',msfit='missing',outcometype='missing',family='missing'), function(y, x, m, V, msfit, outcometype, family, priorCoef, priorGroup, priorVar, priorprec, isgroup, niter=10^3, burnin=round(niter/10), thinning=1, pp='norm') {
    rnlp(y=y,x=x,family='Continuous',priorprec=priorprec)
}
)


setMethod("rnlp", signature(y='ANY',x='matrix',m='missing',V='missing',msfit='missing',outcometype='character',family='character'), function(y, x, m, V, msfit, outcometype, family, priorCoef, priorGroup, priorVar, priorprec, isgroup, niter=10^3, burnin=round(niter/10), thinning=1, pp='norm') {
  if ((outcometype== 'Continuous') && (family== 'normal')) {  ##Linear model
    ans <- rnlpLM(y=y,x=x,priorCoef=priorCoef,priorGroup=priorGroup,priorVar=priorVar,priorprec=priorprec,isgroup=isgroup,niter=niter,burnin=burnin,thinning=thinning)
  } else if (outcometype=='glm') { #GLM
    ans <- rnlpGLM(y=y,x=x,family=family,priorCoef=priorCoef,priorGroup=priorGroup,priorVar=priorVar,isgroup=isgroup,niter=niter,burnin=burnin,thinning=thinning)
  } else if ((outcometype=='Survival') && (family=='Cox')) {  #Cox model
    ans <- rnlpCox(y=y,x=x,priorCoef=priorCoef,priorGroup=priorGroup,isgroup=isgroup,niter=niter,burnin=burnin,thinning=thinning)
  } else {
    stop(paste("outcometype",outcometype,"and family=",family,"not implemented",sep=""))
  }
  return(ans)
}
)



rnlpLM <- function(y, x, priorCoef, priorGroup, priorVar, priorprec, isgroup, niter=10^3, burnin=round(niter/10), thinning=1) {
    if (missing(priorGroup)) priorGroup= priorCoef
    p <- ncol(x); n <- length(y)
    tau <- as.double(formatmsPriorsMarg(priorCoef=priorCoef, priorGroup=priorCoef, priorVar=priorVar, priorSkew=momprior(tau=0.348), n=n)$tau)
    if (nrow(x) != n) stop('Dimensions of y and x do not match')
    if (priorVar@priorDistr=='invgamma') {
        a_phi <- as.double(priorVar@priorPars['alpha'])
        b_phi <- as.double(priorVar@priorPars['lambda'])
    } else stop("Only invgamma prior for residual variance is currently implemented")
    if ((priorCoef@priorDistr %in% c('pMOM','peMOM','piMOM')) && (priorCoef@priorDistr == priorGroup@priorDistr)) {
      if (priorCoef@priorDistr=='pMOM') {
        prior <- as.integer(0); r <- as.integer(priorCoef@priorPars['r'])
      } else if (priorCoef@priorDistr=='piMOM') {
        prior <- as.integer(1); r <- as.integer(0)
      } else {
        prior <- as.integer(2); r <- as.integer(0)
      }
      if (p==0) {
        ans <- matrix(1/rgamma((niter-burnin)/thinning, .5*(a_phi+n), .5*(b_phi+sum(y^2))), ncol=1)
        colnames(ans) <- 'phi'
      } else {
        ans <- rnlpPostCI_lm(as.integer(niter),as.integer(burnin),as.integer(thinning),as.double(y),as.double(x),as.integer(p),as.integer(r),tau,a_phi,b_phi,prior)
        #ans <- .Call("rnlpPostCI_lm",as.integer(niter),as.integer(burnin),as.integer(thinning),as.double(y),as.double(x),as.integer(p),as.integer(r),tau,a_phi,b_phi,prior)
        ans <- matrix(ans,ncol=p+1)
        if (is.null(colnames(x))) colnames(ans) <- c(paste('beta',1:ncol(x),sep=''),'phi') else colnames(ans) <- c(colnames(x),'phi')
      }
    } else if (priorCoef@priorDistr %in% c('zellner','bic','normalid','icarplus')) {
      if (priorCoef@priorDistr == 'bic') tau= Inf
      if (p==0) {
        ans <- matrix(1/rgamma((niter-burnin)/thinning, .5*(a_phi+n), .5*(b_phi+sum(y^2))), ncol=1)
        colnames(ans) <- 'phi'
      } else {
        if (priorCoef@priorDistr == 'normalid') {
           S <- solve(t(x) %*% x + diag(p)/tau)
        } else if (priorCoef@priorDistr == 'icarplus') {
           S <- solve(t(x) %*% x + priorprec)
        } else {
           S <- solve((1+1/tau) * t(x) %*% x)
        }
        m <- as.vector(S %*% t(x) %*% matrix(y,ncol=1))
        ssr <- sum(y * (y - (x %*% m)))
        phi <- 1 / rgamma((niter - burnin)/thinning, 0.5*(n+a_phi), 0.5*(ssr + b_phi))
        beta <- matrix(rnorm(ncol(x)*(niter-burnin)/thinning),ncol=ncol(x)) %*% t(chol(S)) * phi
        beta <- t(t(beta)+m)
        ans <- cbind(beta, phi)
        if (is.null(colnames(x))) colnames(ans) <- c(paste('beta',1:ncol(x),sep=''),'phi') else colnames(ans) <- c(colnames(x),'phi')
      }
    } else {
        priorCoef <- zellnerprior(); priorGroup= groupzellnerprior()
        ans <- rnlpLM(y=y, x=x, priorCoef=priorCoef, priorGroup=priorGroup, priorVar=priorVar, isgroup=isgroup, niter=niter, burnin=burnin, thinning=thinning)
    }    
    return(ans)
}



setMethod("rnlp", signature(y='missing',x='missing',m='numeric',V='matrix',msfit='missing',outcometype='missing',family='missing'), function(y, x, m, V, msfit, outcometype, family, priorCoef, priorGroup, priorVar, priorprec=priorprec, isgroup, niter=10^3, burnin=round(niter/10), thinning=1, pp='norm') {
  if (missing(priorGroup)) priorGroup= priorCoef
  p <- ncol(V)
  tau <- as.double(priorCoef@priorPars['tau'])
  if ((priorCoef@priorDistr %in% c('pMOM','peMOM','piMOM')) && (priorCoef@priorDistr == priorGroup@priorDistr)) {
    if (priorCoef@priorDistr=='pMOM') {
      prior <- as.integer(0); r <- as.integer(priorCoef@priorPars['r'])
    } else if (priorCoef@priorDistr=='piMOM') {
      prior <- as.integer(1); r <- as.integer(0)
    } else if (priorCoef@priorDistr=='peMOM') {
      prior <- as.integer(2); r <- as.integer(0)
    } else stop("This kind of prior is not implemented")
    ans <- rnlpCI(as.integer(niter),as.integer(burnin),as.integer(thinning),as.double(m),as.double(V),as.integer(p),as.integer(r),tau,prior)
    ans <- matrix(ans,ncol=p)
  } else if (priorCoef@priorDistr %in% c('zellner','bic')) {
    ans <- t(m + t(chol(V)) %*% matrix(rnorm(p*(niter-burnin)/thinning),nrow=p))
  } else stop("This kind of prior is not implemented")
  if (is.null(colnames(ans))) {
      if (is.null(names(m))) colnames(ans) <- paste('beta',1:length(m),sep='') else colnames(ans) <- names(m)
  }
  return(ans)
}
)



glmfamilycode <- function(family) {
    #Convert string indicating GLM family to object of class family, as required by function glm
    if (length(grep("binomial",family))>0) {
        link= strsplit(family,split=" ")[[1]][2]
        if (is.na(link)) link= 'logit'
        ans= binomial(link=link)
    } else if (length(grep("gamma",family))>0) {
        link= strsplit(family,split=" ")[[1]][2]
        ans= Gamma(link=link)
        if (is.na(link)) link='inverse'
    } else if (length(grep("inverse.gaussian",family))>0) {
        link= strsplit(family,split=" ")[[1]][2]
        if (is.na(link)) link="1/mu^2"
        ans= inverse.gaussian(link=link)
    } else if (family=='normal') {
        ans= gaussian()
    } else if (length(grep("poisson",family))>0) {
        link= strsplit(family,split=" ")[[1]][2]
        if (is.na(link)) link="log"
        ans= poisson(link=link)
    } else { stop(paste("family",family,"not recognized",sep=" ")) }
    return(ans)
}



rnlpGLM <- function(y, x, family, priorCoef, priorGroup, priorVar, isgroup=rep(FALSE,ncol(x)), niter=10^3, burnin=round(niter/10), thinning=1) {
#Approximate posterior sampling for GLMs
    p <- ncol(x)
    if (p==0) {
      ans= matrix(double(0),nrow=(niter-burnin)/thinning,ncol=0)
    } else {
      fam= glmfamilycode(family)
      fit= glm(y ~ -1 + ., data=data.frame(x), family=fam)
      pm= postmomentsGLM(fit=fit, priorCoef=priorCoef, priorGroup=priorGroup, isgroup=isgroup)
      ans= rnlp(m=pm$m, V=pm$S, priorCoef=priorCoef, priorGroup=priorGroup, niter=niter, burnin=burnin, thinning=thinning)
    }
    return(ans)
}


postmomentsGLM <- function(fit, priorCoef, priorGroup, isgroup) {
# Extract approximate posterior mean and covariance from GLM fit
# Input
#  - fit: object returned by glm
#  - priorCoef: prior distribution on coefficients
#  - priorGroup: prior on coefficient groups
# Output: list with two elements, m containing the posterior mean and S the posterior covariance
  if (priorCoef@priorDistr != priorGroup@priorDistr) stop("priorGroup != priorCoef not currently implemented")
  tau <- as.double(priorCoef@priorPars['tau'])
  taugroup <- as.double(priorGroup@priorPars['tau'])
  thhat <- matrix(coef(fit),ncol=1)
  if (priorCoef@priorDistr == 'zellner') {
      Vinv <- (t(fit$R) %*% fit$R)
      S <- solve((t(fit$R) %*% fit$R)) / (1+1/tau)
      m <- S %*% Vinv %*% thhat
  } else if (priorCoef@priorDistr == 'bic') {
      S <- vcov(fit)
      m <-  thhat
  } else {
      #if (priorCoef@priorDistr %in% c('pMOM','peMOM','piMOM')) {
      Vinv <- (t(fit$R) %*% fit$R)
      d <- rep(1,nrow(thhat)); d[!isgroup] <- d[!isgroup]/tau; d[isgroup] <- d[isgroup] / taugroup
      D <- diag(nrow(thhat)); diag(D) <- d
      S <- solve(Vinv + D)
      #S <- solve(Vinv + diag(nrow(thhat))/tau)
      m <- S %*% Vinv %*% thhat
  }
  return(list(m=as.vector(m), S=S))
}

rnlpCox <- function(y, x, priorCoef, priorGroup, isgroup=isgroup, niter=10^3, burnin=round(niter/10), thinning=1) {
    tau <- as.double(priorCoef@priorPars['tau'])
    taugroup <- as.double(priorGroup@priorPars['tau'])
    p <- ncol(x)
    if (p==0) {
      ans <- matrix(double(0),nrow=(niter-burnin)/thinning,ncol=0)
    } else {
      fit <- survival::coxph(y ~ ., data=data.frame(x))
      thhat <- matrix(coef(fit),ncol=1); Vinv <- solve(fit$var)
      if (priorCoef@priorDistr == 'zellner') {
        Sinv <- Vinv * (1+1/tau)
      } else if (priorCoef@priorDistr == 'bic') {
        Sinv <- Vinv
      } else {
        #if (priorCoef@priorDistr %in% c('pMOM','peMOM','piMOM')) {
        d <- rep(1,p); d[!isgroup] <- d[!isgroup]/tau; d[isgroup] <- d[isgroup] / taugroup
        D <- diag(p); diag(D) <- d
        Sinv <- Vinv + D
        #Sinv <- Vinv + diag(p)/tau
      }
      S <- solve(Sinv)
      m <- S %*% Vinv %*% thhat
      ans <- rnlp(m=as.vector(m), V=S, priorCoef=priorCoef, priorGroup=priorGroup, niter=niter, burnin=burnin, thinning=thinning)
      if (is.null(colnames(x))) colnames(ans) <- paste('beta',1:ncol(x),sep='') else colnames(ans) <- colnames(x)
    }
    return(ans)
}



##################################################################################
## Routines to simulate from truncated Normal
##################################################################################

rtnorm <- function(n, m, sd, lower, upper) {
  if (length(lower) != length(upper)) stop('length of lower and upper must match')
  if (length(lower)>0) {
    lower[1] <- max(-1.0e10,lower[1])
    upper[length(upper)] <- min(1.0e10,upper[length(upper)])
    ans <- rnorm_truncMultCI(as.integer(n),as.double(lower),as.double(upper),as.double(m),as.double(sd))
    #ans <- .Call("rnorm_truncMultCI",as.integer(n),as.double(lower),as.double(upper),as.double(m),as.double(sd))
  } else {
    ans <- rnorm(n, m, sd)
  }
  return(ans)
}

rtmvnorm <- function(n, m, Sigma, SigmaInv, lower, upper, within, method='Gibbs', burnin=round(.1*n)) {
# Multivariate normal samples under rectangular constraint
# Input
# - n: number of draws
# - m: multivariate normal mean
# - Sigma: multivariate normal covariance
# - lower: vector with lower truncation points
# - upper: vector with upper truncation points
# - within: if TRUE, each variable is truncated to be >=lower and <= upper. If FALSE, it's truncated to be <lower or >upper
# - method: set method=='Gibbs' for Gibbs sampling, and method=='MH' for independent proposal MH
# - burnin: number of burn-in iterations
# Output: n draws obtained via Gibbs sampling after orthogonalization
  if (length(lower)==1) lower <- rep(1,length(m))
  if (length(upper)==1) upper <- rep(1,length(m))
  if (length(lower)!=length(m)) stop('Length of lower and m do not match')
  if (length(upper)!=length(m)) stop('Length of upper and m do not match')
  if (nrow(Sigma)!=length(m) | ncol(Sigma)!=length(m)) stop('Dimensions of m and Sigma do no match')
  if (!(method %in% c('Gibbs','MH'))) stop('Method should be Gibbs or MH')
  method <- as.integer(ifelse(method=='Gibbs',1,2))
  ans <- rtmvnormCI(as.integer(n), as.double(m), as.double(Sigma), as.double(lower), as.double(upper), as.integer(within), method)
  #ans <- .Call("rtmvnormCI",as.integer(n), as.double(m), as.double(Sigma), as.double(lower), as.double(upper), as.integer(within), method)
  matrix(ans,ncol=length(m))
}

rtmvnormProd <- function(n, m, Sigma, k=1, lower=0, upper=Inf, burnin=round(.1*n)) {
# Multivariate normal samples under product contraint  lower <= prod(x^k) <= upper
# Input
# - m: multivariate normal mean
# - Sigma: multivariate normal covariance
# - k: power of product
# - lower: lower truncation point
# - upper: upper truncation point
# - burnin: number of burn-in iterations
# Output: n draws obtained via Gibbs sampling
  lowtrunc <- ifelse(lower==0,as.integer(0),as.integer(1))
  uptrunc <- ifelse(upper==Inf,as.integer(0),as.integer(1))
  if (upper==Inf) { uptrunc <- as.integer(0); upper <- as.double(0) } else { uptrunc <- as.integer(1); upper <- as.double(upper) }
  ans= rtmvnormProdCI(as.integer(n), as.double(m), as.double(Sigma), as.integer(k), as.double(lower), upper, lowtrunc, uptrunc, as.integer(burnin));
  #ans= .Call("rtmvnormProdCI",as.integer(n), as.double(m), as.double(Sigma), as.integer(k), as.double(lower), upper, lowtrunc, uptrunc, as.integer(burnin));
  matrix(ans,nrow=n)
}

plot_taylor_approximation <- function(
    f,          # Original function (takes vector input)
    grad_f,     # Gradient function (returns vector)
    hessian_f,  # Hessian function (returns matrix)
    x0,         # Current point (vector)
    coord_idx,  # Index of coordinate to vary (e.g., 1 for x1)
    x_lim = c(x0[coord_idx] - 1, x0[coord_idx] + 1),  # Range for varying coordinate
    n_points = 100,  # Number of points for plotting
    ...         # Other arguments passed to f, grad_f and hessian_f
) {
  # Check inputs
  if (length(x0) < coord_idx)  stop("coord_idx exceeds the dimension of x0")
  
  # Generate sequence for varying coordinate
  x_var <- seq(x_lim[1], x_lim[2], length.out = n_points)
  
  # Evaluate original function along the varying coordinate
  f_values <- sapply(x_var, function(xi) {
    x <- x0
    x[coord_idx] <- xi
    f(x, ...)
  })
  
  # Compute Taylor approximations at x0
  grad <- grad_f(x0, ...)
  hess <- hessian_f(x0, ...)
  delta <- x_var - x0[coord_idx]
  
  # First-order Taylor approximation (linear)
  taylor1 <- f(x0, ...) + grad[coord_idx] * delta
  
  # Second-order Taylor approximation (quadratic)
  taylor2 <- f(x0, ...) + grad[coord_idx] * delta + 0.5 * hess[coord_idx, coord_idx] * delta^2
  
  # Plot
  plot(x_var, f_values, type = "l", lwd = 2, col = "black",
       xlab = paste("Coordinate", coord_idx, "(x", coord_idx, ")"),
       ylab = "Function value",
       main = paste("Function and Taylor Approximations at x[", coord_idx, "] =", x0[coord_idx]))
  abline(v = x0[coord_idx], lty = 2, col = "gray")  # Mark the current point
  lines(x_var, taylor1, col = "red", lwd = 2, lty = 2)    # 1st-order approx
  lines(x_var, taylor2, col = "blue", lwd = 2, lty = 3)   # 2nd-order approx
  legend("topright", legend = c("Original", "1st-order Taylor", "2nd-order Taylor"),
         col = c("black", "red", "blue"), lwd = 2, lty = c(1, 2, 3))
}



# - Z: In regression, p x q matrix containing the q meta-covariates for the p parameters of interest. It should not include an intercept, as it's already automatically added
#      In GGMs, a p x p matrix or list of p x p matrices
# - wini: Optional. Initial value for the q hyper-parameters
# - niter.mcmc= number of iterations in the final MCMC, run once after hyper-parameter estimates have been obtained
# - niter.mstep= number of MCMC iterations in each M-step required to update hyper-parameter estimates
# - niter.eBayes: max number of iterations in the empirical Bayes optimization algorithm. The algorithm also stop when the objective function didn't improve in >=2 iterations
# - priorvar.w: hyper-parameters w follow a prior w ~ N(0, w_priorvar I) where priorvar.w is the prior variance
# - verbose: if TRUE, progress in hyper-parameter estimation is printed
modelSelection_eBayes= function(Z, wini, niter.mcmc= 5000, niter.mstep= 1000, niter.eBayes= 20, priorvar.w, verbose=TRUE, ...) {
    # Add intercept and orthogonalize Z wrt intercept
    Z= model.matrix(~ ., data= as.data.frame(Z))
    zerosd= which(apply(Z, 2, 'sd')==0)
    if (length(zerosd) > 1) {
        Z= Z[, -zerosd[-1], drop=FALSE] #if there's >1 intercept, keep only the first
    }
    sel= (colnames(Z) != "(Intercept)")
    if (sum(sel) > 0) Z[,sel]= round(residuals(lm(Z[,sel] ~ 1)), 3)
    # Call generic eBayes function
    modelSelectionGeneric_eBayes(msfun="modelSelection", Z=Z, wini=wini, niter.mcmc=niter.mcmc, niter.mstep=niter.mstep, niter.eBayes=niter.eBayes, priorvar.w=priorvar.w, verbose=verbose, ...)
}

modelSelectionGGM_eBayes= function(Z, wini, niter.mcmc= 5000, niter.mstep= 1000, niter.eBayes= 20, priorvar.w, verbose=TRUE, ...) {
    # Check that Z has the right format
    if (missing(Z)) {
        params= eval(quote(list(...)))
        if ("y" %in% names(params)) p= ncol(params$y)
        Z = matrix(1, nrow= p*(p-1)/2, ncol=1)
    } else {
        if (is.matrix(Z)) {
          if (nrow(Z) != ncol(Z)) stop("Z must be a square matrix with the same dimensions as y")
          Z = cbind(1, matrix(Z[upper.tri(Z)], ncol=1))
        } else if (is.list(Z)) {
          if (!all(sapply(Z, is.matrix))) stop("Z is a list but some of its entries are not a matrix")
          Z = do.call(cbind, lapply(Z, function(A) A[upper.tri(A)]))
        } else {
          stop("If specified, Z must be a matrix or a list. If missing, it's taken to only contain the intercept")
        }
    }
    # Call generic eBayes function
    modelSelectionGeneric_eBayes(msfun="modelSelectionGGM", Z=Z, wini=wini, niter.mcmc=niter.mcmc, niter.mstep=niter.mstep, niter.eBayes=niter.eBayes, priorvar.w=priorvar.w, verbose=verbose, ...)
}

# - modselfun: function returning a list that has a component $margpp storing posterior marginal inclusion probabilities. 
#              Set "modelSelection" for linear regression, GLMs, GAMs, and "modelSelectionGGM" for Gaussian graphical models
#              The function must have input arguments priorModel, niter and verbose
modelSelectionGeneric_eBayes= function(msfun, Z, wini, niter.mcmc= 5000, niter.mstep= 1000, niter.eBayes= 20, priorvar.w, verbose=TRUE, ...) {
    
    if (msfun == "modelSelection") {
        modselfun <- modelSelection
        # Default hyper-parameters prior variance
        if (missing(priorvar.w)) priorvar.w= eBayes_priorelicit(Z=Z, targetprob=0.95, min_priorprob=0.001, prior="zellner")
    } else if (msfun == "modelSelectionGGM") {
        modselfun <- modelSelectionGGM
        # Default hyper-parameters prior variance
        if (missing(priorvar.w)) {
            if (ncol(Z)==1) {
                priorvar.w= eBayes_priorelicit(Z=Z, targetprob=0.95, min_priorprob=0.001, prior="zellner")
            } else {
                priorvar.w= eBayes_priorelicit(Z=Z, targetprob=0.95, min_priorprob=0.001, prior="zellner")
            }
        }
    } else stop("An invalid 'msfun' was provided")

    # Get initial parameter estimates
    if (verbose) message("Initializing hyper-parameter estimates... ")
    if (missing(wini)) {
        # Obtain posterior inclusion prob under Beta-Binomial(1,1) model prior
        ms= modselfun(priorModel=modelbbprior(), niter=niter.mstep, verbose=FALSE, ...)
        # Extract marginal posterior inclusion probabilities
        if (msfun == "modelSelectionGGM") {
            k = (1:ms$p)
            exclude_param <- k*(k+1)/2
            pip= ms$margpp[-exclude_param]  #diagonal entries are always non-zero (PIP=1)
        } else {
            pip= ms$margpp
        }
        # Set initial estimate using solution from full-rank/clustering case 
        wini= eBayes_mstep_init(Z=Z, postprob=pip, priorvar.w=priorvar.w)
        w= wini$w
    } else {
        w= wini
        wini= eBayes_mstep_init(Z=Z)
        if (msfun == "modelSelectionGGM") {
            p = (1 + sqrt(1 + 8 * nrow(Z)))/2  #figure out p from number of upper-triangular entries (rows in Z)
            k = (1:p)
            exclude_param <- k*(k+1)/2
        }
    }


    # Iterate
    i = 1
    found= FALSE
    Vinv= (t(Z) %*% Z) / ncol(Z)
    priorhess= - Vinv / priorvar.w
    fval= double(niter.eBayes + 1)
    fval[1]= fbest= wbest = NA
    noimprove= 0
    while (!found) {
        # Obtain current prior inclusion probabilities
        if ((msfun == "modelSelectionGGM") && (ncol(Z) == 1)) {
           priorprob= as.vector(1 / (1 + exp(- w)))
        } else {
           priorprob= 1 / (1 + exp(- Z %*% w))
        }
        # E-step: find posterior probabilities for current w
        priorModel= modelbinomprior(priorprob)
        ms= modselfun(priorModel=priorModel, niter=niter.mstep, verbose=FALSE, ...)
        # Extract marginal posterior inclusion probabilities
        if (msfun == "modelSelectionGGM") {
          pip <- ms$margpp[-exclude_param]  #diagonal entries are always non-zero (PIP=1)
        } else {
          pip <- ms$margpp
        }
        # M-step
        if (wini$fullrankcase) {
            wnew= eBayes_mstep_fullrank(Z=Z, Uinv=wini$Uinv, postprob=pip, wcur=w, priorvar.w=priorvar.w)
        } else {
            wnew= eBayes_mstep(wcurrent=w, Z=Z, postprob=pip, maxiter=10, priorvar.w=priorvar.w, Vinv=Vinv, priorhess=priorhess)$w
        }
        maxstep= max(abs(wnew - w))
        # Objective function
        if (msfun == "modelSelection") {
          fval[i+1]= eBayes_logit_objective(wnew, msfit=ms, Z=Z, priorvar.w=priorvar.w, Vinv=Vinv)
          if (is.na(fbest)) {
            fbest= fval[i]= eBayes_logit_objective(w, msfit=ms, Z=Z, priorvar.w=priorvar.w, Vinv=Vinv)
            wbest= w
            if (verbose) message(paste("Done\n\n","Iter", "Objective fun", "Hyper-parameter", "\n", i, fval[i], paste(w, collapse=" "), "\n"))
          }
          if (fval[i+1] >= fval[i]) { noimprove= 0 } else { noimprove= noimprove + 1 }
          if (fval[i+1] > fbest) { fbest= fval[i+1]; wbest= wnew }
          found = (i >= niter.eBayes) || (maxstep < 0.01) || (noimprove > 1)
        } else {
          if (is.na(wbest)) {
            wbest= w
            if (verbose) message(paste("Done\n\n","Iter", "Hyper-parameter", "\n", i, paste(w, collapse=" "), "\n"))
          }
          found = (i >= niter.eBayes) || (maxstep < 0.01)
        }
        w= wnew
        i= i + 1
        if (verbose) message(paste(" ", i, ifelse(msfun=="modelSelection", fval[i],""), paste(w, collapse=" "), "\n"))
    }
    # Run modelSelection with empirical Bayes prior probabilities
    if (verbose) message("Done. \n","Final MCMC with best hyper-parameter value... ")
    w= wbest
    # Obtain current prior inclusion probabilities
    if ((msfun == "modelSelectionGGM") & (ncol(Z) == 1)) {
      priorprob= as.vector(1 / (1 + exp(- w)))
    } else {
      priorprob= 1 / (1 + exp(- Z %*% w))
    }
    priorModel= modelbinomprior(priorprob)
    if (msfun == "modelSelection") {
      ms= modelSelection(priorModel=priorModel, niter=niter.mcmc, verbose=FALSE, ...)
    } else {
      ms= modelSelectionGGM(priorModel=priorModel, niter=niter.mcmc, verbose=FALSE, ...)
    }
    ms$eBayes_hyperpar= w
    ms$Z= Z
    if (verbose) message("Done\n")
    return(ms)
}


# Find prior variance of hyper-parameters such that all prior inclusion probabilities are in (min_priorprob, 1 - min_priorprob) with >= targetprob probability
# Hyper-parameters w follow a prior w ~ N(0, priorvar.w V), where V=I when prior == "normalshrinkage" and V= (Z^T Z / nrow(Z))^{-1} when prior == "zellner"
# The prior inclusion probability for a covariate that has meta-covariates z is m(w)= 1 / (1 + e^{-z^T w})
#
# Input
# - Z: p x q matrix containing the q meta-covariates for the p covariates. It should not include an intercept, as it's already automatically added
# - targetprob
# - min_priorprob
# Output: value of priorvar.w such that the prior prob that m(z) >= targetprob for all z values obtained by taking a row from Z
eBayes_priorelicit= function(Z, targetprob=0.95, min_priorprob=0.001, prior="zellner") {
    if (min_priorprob >= 0.5) stop("min_priorprob must be <= 0.5")
    #logit= function(x) log(x / (1-x))
    priorvar.w= (log(min_priorprob/(1-min_priorprob)) / qnorm((1-targetprob)/2))^2
    if (prior == "zellner") {
        V= solve((t(Z) %*% Z) / nrow(Z))
        v= mahalanobis(Z, center=rep(0,ncol(Z)), cov=V, inverted=TRUE)
        vmax= max(v)
    } else if (prior == "normalshrinkage") {
        vmax= max(rowSums(Z^2))
    }
    priorvar.w= priorvar.w / vmax
    #unique(apply(Z, 1, function(z) 1 - 2 * pnorm(logit(0.05) / sqrt(priorvar.w * mahalanobis(z, center=rep(0,ncol(Z)), cov=V, inverted=TRUE))))) #check: should all be >= targetprob
    return(priorvar.w)    
}



# Exact M-step solution when the number of unique rows in Z is ncol(Z), and U which is the ncol(Z) x ncol(Z) matrix containing these unique rows has full rank
# The M-step seeks w such that Z^T m(w) = Z^T postprob, where m(z) = 1 / (1 + exp{- Z w})
eBayes_mstep_fullrank= function(Z, Uinv, postprob, wcur, priorvar.w) {
    if (missing(Uinv)) {
        U= unique(Z)
        if (nrow(U) != ncol(U)) stop("The number of unique rows in Z is not ncol(Z)")
        Uinv= solve(U)
    } else {
        if (ncol(Uinv) != ncol(Z)) stop("Uinv must have the same number of columns as Z")
        if (nrow(Uinv) != ncol(Uinv)) stop("Uinv must be a square matrix")
    }
    Ztilde= Z %*% Uinv
    meanpp= (t(Ztilde) %*% postprob) / colSums(Ztilde)
    if (missing(wcur)) wcur= log(meanpp / (1-meanpp))
    wtilde= double(ncol(Z))
    for (j in 1:length(wtilde)) {
        wtilde[j]= eBayes_fullrank_solve_modifiedlogistic(a=meanpp[j], c= priorvar.w * nrow(Z), w_initial=wcur[j])
    }
    what= Uinv %*% wtilde
    #grad= eBayes_em_logit_grad(what, postprob=postprob, Z=Z)  #check: gradient should be 0
    return(what)
}


# Newton-Raphson solver for f(w) = a, where f(w) = 1 / (1 + exp(-w)) + w/c
eBayes_fullrank_solve_modifiedlogistic <- function(a, c, w_initial, tol = 1e-3, max_iter = 100) {
  if (missing(w_initial)) w_initial= log(a / (1-a))  #solution for the c=infinity case
  # Define f(w) and its derivative f'(w)
  f <- function(w) { 1 / (1 + exp(-w)) + w / c - a }
  f_prime <- function(w) { exp(-w) / (1 + exp(-w))^2 + 1 / c }
  # Initialize
  w <- w_initial
  iter <- 0
  converged <- FALSE
  # Newton-Raphson iterations
  while (iter < max_iter && !converged) {
    f_val <- f(w)
    f_deriv <- f_prime(w)
    if (abs(f_val) < tol) {
      converged <- TRUE
    } else {
      w <- w - f_val / f_deriv  # Newton update
      iter <- iter + 1
    }
  }
  return(w)
}



# Find an initial value for w solving the M-step equation Z^T m(w) = Z^T postprob, where m(z) = 1 / (1 + exp{- Z w})
#
# The exact solution is returned when the matrix U containing the unique rows in Z has ncol(Z) rows and is full-rank
# Otherwise, an approximated w is returned by clustering Z into ncol(Z) clusters and replacing Z by the cluster means (which, if they're full-rank, admit a closed-form solution
# Input
# - Z: p x q matrix recording q meta-covariates for p covariates
# - postprob: vector of length p with posterior probabilities for each covariate at current w
# - priorvar.w: the prior on w is N(0, priorvar.w * V) where V= (Z^T Z/nrow(Z))^{-1}
# Output: a list with the following components
# - w: value of w. Only returned if postprob is not missing
# - fullrankcase: TRUE if Z satisfies the full-rank case conditions
# - Uinv: inverse of U if fullrankcase==TRUE, and NULL otherwise
eBayes_mstep_init= function(Z, postprob, priorvar.w=priorvar.w) {
    if (!missing(postprob)) {
        if (is.vector(postprob)) postprob= matrix(postprob, ncol=1)
    }
    U= unique(Z)
    wini= NULL
    fullrankcase= FALSE
    if (nrow(U) == ncol(U)) {
        Uinv= try(solve(U), silent=TRUE)
        if (!inherits(Uinv, "try-error")) {
            if (!missing(postprob)) wini= eBayes_mstep_fullrank(Z, Uinv=Uinv, postprob=postprob, priorvar.w=priorvar.w)
            fullrankcase= TRUE
        }
    }
    #if (is.null(wini)) {
    #    clus= kmeans(Z, centers=ncol(Z))$cluster
    #    clusmeans= aggregate(Z, by = list(clus = clus), FUN = mean)
    #    Zc= merge(data.frame(rowid=1:nrow(Z), clus), clusmeans, by="clus")
    #    Zc= Zc[order(Zc$rowid,decreasing=FALSE),]
    #    Zc= as.matrix(Zc[,-1:-2]) #remove cluster and row id
    #    Uinv= try(solve(clusmeans[,-1]), silent=TRUE)
    #    if (!inherits(Uinv, "try-error")) wini= eBayes_mstep_fullrank(Zc, Uinv=Uinv, postprob=postprob, priorvar.w=priorvar.w)
    #}
    if (!missing(postprob)) {
        if (is.null(wini)) {
            logitpp= log((postprob + .001) / (1 - postprob + .001))
            wini= matrix(coef(lm(logitpp ~ -1 + Z)), ncol=1)
        }
    } else {
        wini= NULL
    }
    # Return output
    if (!fullrankcase) Uinv= NULL
    ans= list(w= wini, fullrankcase= fullrankcase, Uinv= Uinv)
    return(ans)
}


# Use Newton-Raphson to solve the M-step equation Z^T m(w) = Z^T postprob, where m(z) = 1 / (1 + exp{- Z w})
# Input
# - wcurrent: current value of w
# - Z: p x q matrix containing the q meta-covariates for the p covariates. It should not include an intercept, as it's already automatically added
# - postprob: vector of length p with posterior probabilities for each covariate at w= wcurrent
# - priorvar.w: prior on w is N(0, priorvar.w V) where V= (Z^T Z/nrow(Z))
# - maxiter: maximum number of iterations
# Output
# - w: solution of the M-step equation
# - grad: gradient of the objective function Z^T ( m(w) - postprob ) at w. Upon convergence, it should be zero
eBayes_mstep= function(wcurrent, Z, postprob, priorvar.w=priorvar.w, Vinv=Vinv, priorhess=priorhess, maxiter=10) {
    if (missing(priorhess)) priorhess= - Vinv / priorvar.w
    # Peform a first Newton-Raphson step
    w = wcurrent
    priorprob= 1 / (1 + exp(- Z %*% w))
    priorgrad= - (Vinv %*% w) / priorvar.w
    grad= priorgrad + eBayes_em_logit_grad(w, postprob=postprob, Z=Z, priorprob=priorprob)
    hess= priorhess + eBayes_em_logit_hess(w, postprob=postprob, Z=Z, priorprob=priorprob)
    delta= - base::solve(hess, grad)
    #Id= diag(ncol(Z))
    #delta= - base::solve(hess - Id, grad)
    w= w + delta
    maxstep= max(abs(delta))
    if (maxstep < 0.01) found= TRUE else found= FALSE
    iter= 0
    while (!found) {
        #eBayes_em_logit_objective(w, msfit=ms, Z=Z, priorvar.w=priorvar.w, Vinv=Vinv)
        priorprob= 1 / (1 + exp(- Z %*% w))
        priorgrad= - (Vinv %*% w) / priorvar.w
        grad= priorgrad + eBayes_em_logit_grad(w, postprob=postprob, Z=Z, priorprob=priorprob)                
        hess= priorhess + eBayes_em_logit_hess(w, postprob=postprob, Z=Z, priorprob=priorprob)
        #numDeriv::grad(eBayes_em_logit_objective, w, msfit=ms, Z=Z, priorvar.w=priorvar.w, Vinv=Vinv) #matches
        #numDeriv::hessian(eBayes_em_logit_objective, w, msfit=ms, Z=Z, priorvar.w=priorvar.w, Vinv=Vinv)  #matches
        #plot_taylor_approximation(f=eBayes_em_logit_objective, grad_f= eBayes_em_logit_grad, hessian_f= eBayes_em_logit_hess, x0= w, coord_idx=1, x_lim = c(-7,-4), n_points = 50, Z=Z, msfit=ms, postprob=pp0)
        #plot_taylor_approximation(f=eBayes_em_logit_objective, grad_f= eBayes_em_logit_grad, hessian_f= eBayes_em_logit_hess, x0= w, coord_idx=2, x_lim = c(2,5), n_points = 50, Z=Z, msfit=ms, postprob=pp0)
        #delta_damp= - base::solve(hess - Id, grad)
        delta= - base::solve(hess, grad)
        w= w + delta
        iter= iter + 1
        maxstep= max(abs(delta))
        if ((iter > maxiter) || maxstep < 0.001) found= TRUE
    }
    # Return output
    ans= list(w=w, grad=grad)
}


# Objective function in the empirical Bayes problem
# The objective function is sum_gamma pi(y | gamma) pi(gamma | w)= sum_gamma h(gamma) pi(gamma | y, w)
# where h(gamma)= pi(y | gamma) pi(gamma | w) / pi(gamma | y, w)
eBayes_logit_objective= function(w, msfit, Z, priorvar.w, Vinv) {
    ans= log(sum(exp(msfit$postProb)))
    ans= ans - (0.5 / priorvar.w) * (matrix(w,nrow=1) %*% Vinv %*% matrix(w,ncol=1))  #subtract log-prior
    return(ans)
}


# Objective function in the M-step under a logit link
# The objective function is f(w)= E[ log pi(gamma | w) | y, w=wcur ]
# - w: q-dimensional vector with hyper-parameter values
# - msfit: output of modelSelection when w=wcur
# - postprob: posterior inclusion probabilities pi(gamma_j = 1 | wcur), where wcur is the current hyper-parameter value
# - Z: p x q matrix containing the q meta-covariates for the p covariates. It should not include an intercept, as it's already automatically added
# - postprob, priorprob: ignored. Kept for compability with eBayes_em_logit_grad
# - priorvar.w, Vinv: prior on w is N(0, priorvar.w V), and Vinv= V^{-1}
# Output: f(w) at w
eBayes_em_logit_objective= function(w, msfit, Z, postprob, priorprob, priorvar.w, Vinv) {
    if (missing(priorprob)) priorprob= 1 / (1 + exp(- Z %*% w))
    ppmodel= postProb(msfit)
    logprior= double(nrow(ppmodel))
    for (i in 1:nrow(ppmodel)) {
        gamma= as.numeric(strsplit(ppmodel$modelid[i], split=",")[[1]])
        logprior[i]= sum(log(priorprob[gamma])) + sum(log((1-priorprob)[-gamma]))
    }
    ans= weighted.mean(logprior, w=ppmodel$pp)  #log marginal likelihood
    ans= ans - (0.5 / priorvar.w) * (matrix(w,nrow=1) %*% Vinv %*% matrix(w,ncol=1))  #subtract log-prior
    return(ans)
}


# Gradient of the objective function in the M-step under a logit link
# The objective function is f(w)= E[ log pi(gamma | w) | y, w=wcur ]
# - w: q-dimensional vector with hyper-parameter values
# - postprob: posterior inclusion probabilities pi(gamma_j = 1 | wcur), where wcur is the current hyper-parameter value
# - Z: p x q matrix containing the q meta-covariates for the p covariates. It should not include an intercept, as it's already automatically added
# - priorprob: prior inclusion probabilities 1/(1+exp(Z w)). If not specified, they're computed from (Z,w)
# - msfit: ignored (kept for compability with eBayes_em_logit_objective)
# Output: gradient of f(w) at w
eBayes_em_logit_grad= function(w, postprob, Z, priorprob, msfit) {
    if (missing(priorprob)) priorprob= 1 / (1 + exp(- Z %*% w))
    grad= t(Z) %*% (postprob - priorprob)
    return(grad)
}

# Same as eBayes_em_logit_grad, but returns the hessian of f(w)
eBayes_em_logit_hess= function(w, postprob, Z, priorprob, msfit) {
    if (missing(priorprob)) priorprob= 1 / (1 + exp(- Z %*% w))
    Zstd= sqrt(priorprob[,1]) * Z
    hess= - t(Zstd) %*% Zstd
    return(hess)
}

##############################################################################################
##
## DENSITY, QUANTILES, CDF AND PRIOR ELICITATION FOR NON-LOCAL PRIORS
##
##############################################################################################


###
### dmom.R
###

#Wrapper to call dpmom (product mom) and dqmom (quadratic mom)
dmom <- function(x, tau, a.tau, b.tau, phi=1, r=1, V1, baseDensity='normal', nu=3, logscale=FALSE, penalty='product') {
  if (penalty=='product') {
    ans <- dpmom(x, tau=tau, a.tau=a.tau, b.tau=b.tau, phi=phi, r=r, baseDensity=baseDensity, logscale=logscale)
  } else if (penalty=='quadratic') {
    if (r>1) stop("r>1 not implemented for penalty=='quadratic'. Try penalty=='product' instead")
    ans <- dqmom(x, V1=V1, g=tau, n=1, baseDensity=baseDensity, nu=nu)
  } else {
    stop("Only 'penalty==product' and 'penalty==quadratic' are implemented")
  }
  return(ans)
}

##Product MOM density

setMethod("dpmom", signature(x='vector'), function(x, tau, a.tau, b.tau, phi=1, r=1, baseDensity='normal', logscale=FALSE) {
    if (missing(tau) & (missing(a.tau) | missing(b.tau))) stop("Either tau or (a.tau,b.tau) must be specified")
    if (baseDensity=='normal') {
        if (!missing(tau)) {
            ans <- dnorm(x,0,sd=sqrt(tau*phi),log=TRUE) + r*log(x^2/(tau*phi)) - sum(log(seq(1,2*r-1,by=2)))
        } else {
            ct <- lgamma(r+.5*(a.tau+1)) + r*log(2) - lgamma(.5*a.tau) - .5*log(pi) - (r+.5)*log(b.tau) - .5*log(phi)
            ans <- ct + r*log(x^2/phi) - sum(log(seq(1,2*r-1,by=2))) - (r+.5*(a.tau+1))*log(1+x^2/(b.tau*phi))
        }
    } else if (baseDensity=='laplace') {
        if (r!=1) stop("Only r=1 is implemented for baseDensity='normal'")
        ans= log(x^2) + dalapl(x,0,scale=tau*phi,logscale=TRUE) - log(2*tau*phi)
    } else { stop("Only baseDensity=='normal' or 'laplace' is implemented for the product MOM") }
  if (!logscale) ans <- exp(ans)
  ans
}
)
setMethod("dpmom", signature(x='matrix'), function(x, tau, a.tau, b.tau, phi=1, r=1, baseDensity='normal', logscale=FALSE) {
  if (baseDensity!='normal') stop("Only baseDensity=='normal' is implemented for the product MOM")
  if (missing(tau) & (missing(a.tau) | missing(b.tau))) stop("Either tau or (a.tau,b.tau) must be specified")
  p <- ncol(x)
  normct <- p*sum(log(seq(1,2*r-1,by=2)))
  distval <- rowSums(x^2)
  if (!missing(tau)) {
    ans <- -(p * log(2 * pi) + p*(log(phi)+log(tau))  + distval/(phi*tau))/2 + r*rowSums(log(x^2/(tau*phi))) - normct
  } else {
    anew <- r*p + .5*p + .5*a.tau
    num <- lgamma(anew) - anew*log(1+rowSums(x^2)/(phi*b.tau))  + r*rowSums(log(2*x^2/(phi*b.tau))) - normct
    den <- .5*p*(log(pi)+log(phi)+log(b.tau)) + lgamma(.5*a.tau)
    ans <- num - den
  }
  if (!logscale) ans <- exp(ans)
  ans
}
)

##Quadratic MOM density
dqmom <- function(x,V1=1,g,n=1,theta0,baseDensity='normal',nu=3) {
if (missing(g)) stop("Prior dispersion must be specified for quadratic MOM")
if (!(baseDensity %in% c('normal','t'))) stop("The only implemented baseDensity values are 'normal' and 't'")
if (baseDensity=='t' & nu<3) stop('nu must be >=3, otherwise the prior is improper')
if (missing(V1)) {
  if (is.vector(x)) V1 <- 1 else V1 <- diag(ncol(x))
}
if (missing(theta0)) {
  if (is.vector(V1)) theta0 <- 0 else theta0 <- rep(0,ncol(V1))
}
    
if (is.vector(V1)) {
  qtheta <- (x-theta0)^2/(n*g*V1)
  if (baseDensity=='normal') {
      ans <- qtheta*dnorm(x,theta0,sd=sqrt(n*g*V1))
  } else if (baseDensity=='t') {
    normct <- exp(lgamma(.5*(nu+1))-lgamma(.5*nu)-.5*log(nu*pi*n*g*V1))
    ans <- qtheta*normct*(1+qtheta/nu)^(-.5*(nu+1))*(nu-2)/nu
  }
} else {
  #require(mvtnorm)
  qtheta <- mahalanobis(x,center=theta0,cov=n*g*V1)
  if (baseDensity=='normal') {
    ans <- qtheta * mvtnorm::dmvnorm(x,mean=theta0,sigma=n*g*V1)/ncol(V1)
  } else if (baseDensity=='t') {
    ans <- qtheta * mvtnorm::dmvt(x,delta=theta0,sigma=n*g*V1)*(nu-2)/(nu*length(theta0))
  }
}
return(ans)
}



###
### dimom.R
###

#Wrapper to call dpimom (product iMOM) and dqimom (quadratic imom)
dimom <- function(x, tau=1, phi=1, V1, logscale=FALSE, penalty='product') {
  if (penalty=='product') {
    ans <- dpimom(x, tau=tau, phi=phi, logscale=logscale)
  } else if (penalty=='quadratic') {
    ans <- dqimom(x, V1=V1, g=tau, n=1, nu=1, logscale=logscale)
  } else {
    stop("Only 'penalty==product' and 'penalty==quadratic' are implemented")
  }
  return(ans)
}

#Product iMOM

setMethod("dpimom", signature(x='vector'), function(x, tau=1, phi=1, logscale=FALSE) {
  ans <- .5*(log(tau)+log(phi)) - lgamma(.5) - log(x^2) - tau*phi/x^2
  ans[is.nan(ans)] <- -Inf
  if (!logscale) ans <- exp(ans)
  ans
}
)

setMethod("dpimom", signature(x='matrix'), function(x, tau=1, phi=1, logscale=FALSE) {
  x2 <- x^2
  ans <- ncol(x)*(.5*(log(tau)+log(phi)) - lgamma(.5)) - rowSums(log(x2)) - tau*phi*rowSums(1/x2)
  ans[is.nan(ans)] <- -Inf
  if (!logscale) ans <- exp(ans)
  ans
}
)

setMethod("dpimom", signature(x='data.frame'), function(x, tau=1, phi=1, logscale=FALSE) {
  dpimom(as.matrix(x),tau=tau,phi=phi,logscale=logscale)
}
)


#Quadratic iMOM
dqimom <- function(x,V1=1,g=1,n=1,nu=1,theta0,logscale=FALSE) {
if (is.vector(x)) {
  if (missing(theta0)) theta0 <- 0
  if (missing(V1)) V1 <- 1
  qtheta <- (x-theta0)^2/(n*g*V1)
  k <- -0.5*log(n*g*V1) - lgamma(nu/2)
  p1 <- 1
} else {
  if (missing(theta0)) theta0 <- rep(0,ncol(x))
  if (missing(V1)) V1 <- diag(ncol(x))
  qtheta <- t(matrix(x,nrow=nrow(x))) - theta0
  qtheta <- qtheta %*% solve(n*g*V1) %*% t(qtheta)
  k <- -0.5*log(n*g*det(V1)) - lgamma(nu/2) + lgamma(ncol(x)/2) - .5*ncol(x)*log(pi)
  p1 <- ncol(x)
}
ans <- (k - .5*(nu+p1)*log(qtheta) -1/qtheta)
if (!logscale) { ans <- exp(ans); ans[is.na(ans)] <- 0 }
return(ans)
}




###
### pmom.R
###

pmom <- function(q,V1=1,tau=1) {

  z <- .5-(pnorm(abs(q)/sqrt(V1*tau)) - abs(q)/sqrt(2*pi*V1*tau) * exp(-.5*q^2/(tau*V1)) - .5)
  return(z*(q<=0)+(1-z)*(q>0))
}


###
### qmom.R
###

qmom <- function(p, V1=1, tau=1) {

  e <- function(q) { return((pmom(q, V1, tau)-pneg)^2) }
  ans <- double(length(p))
  for (i in 1:length(p)) {
    pneg <- ifelse(p[i]<=.5, p[i], 1-p[i])
    ans[i] <- nlminb(start=-1, objective=e)$par
    if (p[i]>.5) {
      ans[i] <- -ans[i]
    }
  }
  ans
}


###
### pimom.R
###

pimom <- function(q,V1=1,tau=1,nu=1) {

  z <- (.5*pgamma(tau*V1/q^2,nu/2,1))
  return(z*(q<=0)+(1-z)*(q>0))
}


###
### qimom.R
###

qimom <- function(p, V1=1, tau=1, nu=1) {

  ans <- double(length(p))
  ans[p<=.5] <- -sqrt(tau*V1/qgamma(2*p[p<=.5], nu/2, 1))
  ans[p>.5] <- sqrt(tau*V1/qgamma(2*(1-p[p>.5]), nu/2, 1))
  ans
}



###
### emom.R
###


setMethod("demom",signature(x='vector'),function(x, tau, a.tau, b.tau, phi=1, logscale=FALSE) {
  V1 <- 1
  if (!missing(tau)) {
    pen <- -tau*phi/x^2
    normct <- sqrt(2)
    ans <- pen + dnorm(x,mean=0,sd=sqrt(tau*phi*V1),log=TRUE) + normct
  } else {
    p <- 1; x2phi <- x^2/phi
    anew <- .5*(a.tau+p); bnew <- .5*(b.tau+x2phi)
    num <- sqrt(2)*p + .5*a.tau*log(.5*b.tau)
    den <- lgamma(.5*a.tau) + .5*p*(log(2*pi)+log(phi)) + anew*log(bnew)
    bt <- bnew/x2phi
    ans <- num - den + log(2) + .5*anew*log(bt) + log(besselK(sqrt(4*bt),nu=anew,expon.scaled=TRUE)) - sqrt(4*bt)
  }
  if (!logscale) ans <- exp(ans)
  return(ans)
}
)

setMethod("demom",signature(x='data.frame'),function(x, tau, a.tau, b.tau, phi=1, logscale=FALSE) {
  demom(as.matrix(x),tau=tau,a.tau=a.tau,b.tau=b.tau,phi=phi,logscale=logscale)
}
)

setMethod("demom",signature(x='matrix'),function(x, tau, a.tau, b.tau, phi=1, logscale=FALSE) {
  p <- ncol(x)
  V1 <- diag(p)
  if (!missing(tau)) {
    pen <- -tau*phi*rowSums(1/x^2)
    normct <- p*sqrt(2)
    ans <- pen + mvtnorm::dmvnorm(x,mean=rep(0,p),sigma=tau*phi*V1,log=TRUE) + normct
  } else {
    anew <- .5*(a.tau+p); bnew <- .5*(b.tau+rowSums(x^2)/phi)
    num <- sqrt(2)*p + .5*a.tau*log(.5*b.tau)
    den <- lgamma(.5*a.tau) + .5*p*(log(2*pi)+log(phi)) + anew*log(bnew)
    bt <- bnew*phi*rowSums(1/x^2)
    ans <- num - den + log(2) + .5*anew*log(bt) + log(besselK(sqrt(4*bt),nu=anew,expon.scaled=TRUE)) - sqrt(4*bt)
  }
  if (!logscale) ans <- exp(ans)
  return(ans)
}
)


pemom <- function(q, tau, a.tau, b.tau) integrate(demom,-Inf,q,tau=tau,a.tau=a.tau,b.tau=b.tau)$value



##############################################################################################
## PRIOR ELICITATION
##############################################################################################


###
### g2mode.R
###

g2mode <- function(g,
                   prior=c('iMom', 'normalMom', 'tMom'),
                   nu=1,
                   dim=1) {
  prior <- match.arg(prior)
  switch(EXPR=prior,
         normalMom = {
           2*g
         },
         tMom = {
           g*2*nu/(nu-2+dim)
         },
         iMom = {
           2*g/(nu+dim)
         })
}


###
### mode2g.R
###

mode2g <- function(prior.mode,
                   prior=c('iMom', 'normalMom', 'tMom'),
                   nu=1,
                   dim=1) {
  prior <- match.arg(prior)
  switch(EXPR=prior,
         normalMom = {
           prior.mode / 2
         },
         iMom = {
           prior.mode * (nu+dim) / 2
         },
         tMom = {
           if (nu<3) {
             stop('tMom prior must have nu>2 degrees of freedom')
           }
           prior.mode * (nu-2+dim) / (2*nu)
         })
}


###
### priorp2g.R
###

priorp2g <- function(priorp,
                     q,
                     nu=1,
                     prior=c('iMom', 'normalMom','tMom')) {
  prior <- match.arg(prior)
  switch(EXPR=prior,
         normalMom = {
           e <- function(logg) {
             return((1-2*pmom(-abs(q), tau=exp(logg)) - priorp[i])^2)
           }

           ans <- double(length(priorp))
           for (i in 1:length(priorp)) {
             ans[i] <- exp(nlminb(start=0, objective=e)$par)
           }
           ans
         },
         tMom = {
           stop("prior=='tMom' is not currently implemented")
         },
         iMom = {
           p <- (1-priorp)/2
           qgamma(2*p,nu/2,1)*q^2
         })
}




##############################################################################################
## MARGINAL NLP * IG PRIORS
##############################################################################################


dmomigmarg= function(x,tau,a,b,logscale=FALSE) {
#Marginal MOM density p(x)= int MOM(x;0,tau*phi) IG(phi;a/2,b/2) dphi
    ans= log(2) + lgamma((a+3)/2) - 0.5*log(pi) - 1.5*log(b*tau) - lgamma(a/2) + log(x^2) - ((a+3)/2) * log(1 + x^2/(b*tau))
    if (!logscale) ans= exp(ans)
    return(ans)
}

#Marginal MOM cdf 
pmomigmarg= function(x,tau,a,b) { integrate(dmomigmarg,-Inf,x,tau=tau,a=a,b=b)$value }



#Marginal eMOM density p(x)= int eMOM(x;0,tau*phi) IG(phi;a/2,b/2) dphi
demomigmarg= function(x,tau,a,b,logscale=FALSE) {
    apos= (a+1)/2; bpos= (b+x^2/tau)/2
    ans= log(mgfIG(-tau/x^2,apos,bpos))
    ans= ans + sqrt(2) + lgamma((a+1)/2) - lgamma(a/2) - 0.5*log(pi*tau*b) - 0.5*(a+1) * log(1+x^2/(tau*b))
    if (!logscale) ans= exp(ans)
    return(ans)
}

#Marginal MOM cdf 
pemomigmarg= function(x,tau,a,b) { integrate(demomigmarg,-Inf,x,tau=tau,a=a,b=b)$value }


#Moment-generating function of IG(a,b) evaluated at t
mgfIG= function(t,a,b) {
    if (length(a)==1) a= rep(a,length(t))
    if (length(b)==1) b= rep(b,length(t))
    ans= double(length(t))
    for (i in 1:length(t)) {
        f= function(x) { exp(t[i] * x) * dinvgamma(x,a[i],b[i]) }
        ans[i]= integrate(f,0,Inf)$value
    }
    return(ans)
}



minbeta2tauMOMIGmarg= function(minbeta,a=3,b=3,targetprob=0.99) {
#Find tau such that P(|x| > minbeta)= targetprob, where p(x) is the MOM-IG marginal
    if (minbeta<0) stop("minbeta must be >0")
    f= function(z) abs(targetprob - 2*pmomigmarg(-minbeta,tau=z,a=a,b=b))
    optimize(f,interval=c(.01,5))$minimum
}


minbeta2taueMOMIGmarg= function(minbeta,a=3,b=3,targetprob=0.99) {
#Find tau such that P(|x| > minbeta)= targetprob, where p(x) is the MOM-IG marginal
    if (minbeta<0) stop("minbeta must be >0")
    f= function(z) abs(targetprob - 2*pemomigmarg(-minbeta,tau=z,a=a,b=b))
    tauseq= c(seq(.01,.3,length=10),seq(.4,2,by=.1))
    fseq= sapply(tauseq, f)
    o= order(fseq)
    optimize(f,interval=tauseq[o[1:2]])$minimum
}



##############################################################################################
## CONTINUOUS SPIKE & SLAB PRIORS
##############################################################################################


dmomss= function(x,tau0,tau1,pspike=0.5,baseDensity="normal",logscale=FALSE) {
#Spike & Slab MOM prior
# (1-pspike) * p(x; 0,tau0) + pspike * (x^2/c) * p(x; 0,tau1), where p(x;0,tauj) is Normal for baseDensity="normal" and Laplace for baseDensity="laplace" and c the MOM normalization constant
    if (baseDensity=="normal") {
        d0= dnorm(x,0,tau0,log=TRUE) + log(1-pspike)
        d1= dmom(x,tau=tau1,phi=1,baseDensity="normal",logscale=TRUE) + log(pspike)
    } else if (baseDensity=="laplace") {
        d0= dalapl(x,0,scale=tau0,logscale=TRUE) + log(1-pspike)
        d1= dmom(x,tau=tau1,baseDensity="laplace",logscale=TRUE) + log(pspike)
    }
    ans= exp(d0) + exp(d1)
    if (logscale) ans= log(ans)
    return(ans)       
}

dss= function(x,tau0,tau1,pspike=0.5,baseDensity="normal",logscale=FALSE) {
#Spike & Slab prior
# (1-pspike) * p(x; 0,tau0) + pspike * p(x; 0,tau1), where p(x;0,tauj) is Normal for baseDensity="normal" and Laplace for baseDensity="laplace"
    if (baseDensity=="normal") {
        d0= dnorm(x,0,tau0,log=TRUE) + log(1-pspike)
        d1= dnorm(x,0,tau1,log=TRUE) + log(pspike)
    } else if (baseDensity=="laplace") {
        d0= dalapl(x,0,scale=tau0,logscale=TRUE) + log(1-pspike)
        d1= dalapl(x,0,scale=tau1,logscale=TRUE) + log(pspike)
    }
    ans= exp(d0) + exp(d1)
    if (logscale) ans= log(ans)
    return(ans)       
}

###
### postMode.R
###

##-----------------------------------------------------------------------------
postMode <- function(y, x, priorCoef) {

tau <- as.double(priorCoef@priorPars['tau'])

if (priorCoef@priorDistr %in% c('pMOM','piMOM','peMOM')) {
  f2min <- function(th) {
    phi <- exp(th[length(th)])
    logl <- sum(dnorm(y,
                      x %*% matrix(th[-length(th)], ncol=1),
                      sd=sqrt(phi),
                      log=TRUE))
    logpr <- priorFun(th)
    return(-logl-logpr)
  }

  ## Set up priorFun
  priorFun <- switch(EXPR=priorCoef@priorDistr,
                     'pMOM' = {
                       r <- as.integer(priorCoef@priorPars['r'])
                       function(th) {
                         dmom(matrix(th[-length(th)], nrow=1),
                              tau=tau,
                              phi=exp(th[length(th)]),
                              r=r,
                              logscale=TRUE,
                              penalty="product")
                       }
                     },
                     'piMOM' = {
                       function(th) {
                         dimom(matrix(th[-length(th)], nrow=1),
                               tau=tau,
                               phi=exp(th[length(th)]),
                               logscale=TRUE,
                               penalty="product")
                       }
                     },
                     'peMOM' = {
                       function(th) {
                         demom(matrix(th[-length(th)], nrow=1),
                               tau=tau,
                               phi=exp(th[length(th)]),
                               logscale=TRUE)
                       }
                     },
                     stop('Prior specified in priorDistr not recognized'))

  ## Optimize
  n <- nrow(x)
  p <- ncol(x)
  lm1 <- lm(y~-1+x)
  thini <- c(coef(lm1), log(sum(residuals(lm1)^2/(n-p))))
  opt <- nlminb(thini, objective=f2min)$par
  ans <- list(coef=opt[-length(opt)],sigma2=exp(opt[length(opt)]))
} else if (priorCoef@priorDistr=='zellner') {
  lm1 <- lm(y~-1+x)
  thopt <- coef(lm1)*tau/(1+tau)
  ans <- list(coef=thopt, sigma2= sum((y - x %*% thopt)^2)/length(y))
} else {
  stop('Prior specified in priorDistr not recognized')
}
return(ans)
}

######################################################################################
## GENERAL FUNCTIONS FOR NORMAL-INVERSE WISHART
######################################################################################

dpostNIW <- function(mu,Sigma,x,g=1,mu0=rep(0,length(mu)),nu0=nrow(Sigma)+1,S0,logscale=FALSE) {
#Posterior Normal-IW density
#   x[i]       ~ N(mu,Sigma)
#   mu | Sigma ~ N(mu0, g Sigma)
#   Sigma ~ IW(nu0, S0)
# Input
# - x: n x p data matrix
# - g: prior dispersion parameter in the prior for mu
# - mu0: prior mean in the prior for mu
# - nu0: prior degrees of freedom for Sigma
# - S0: prior scale matrix for Sigma, by default set to I/nu0
# - logscale: set to TRUE to get the log-posterior density
# Output: Normal-IW posterior density evaluated at (mu,Sigma)
    if (!is.matrix(x)) stop("x must be a matrix")
    n= nrow(x); p= ncol(x)
    if (missing(S0)) { if (p==1) { S0= matrix(1/nu0) } else { S0= diag(p)/nu0 } }
    xbar= colMeans(x)
    nu1= nu0+n
    dm= matrix(xbar-mu0,ncol=1)
    S1= S0 + cov(x)*(n-1) + n/(1+n*g) * (dm %*% t(dm))
    w= n/(n+1/g)
    m1= w * xbar + (1-w) * mu0
    ans= mvtnorm::dmvnorm(mu,m1,Sigma/(n+1/g),log=TRUE) + diwish(Sigma,nu=nu1,S=S1,logscale=TRUE)
    if (!logscale) ans= exp(ans)
    return(ans)
}



rpostNIW <- function(n,x,g=1,mu0=0,nu0,S0,precision=FALSE) {
#Draws from posterior Normal-IW density
#   x[i]       ~ N(mu,Sigma)
#   mu | Sigma ~ N(mu0, g Sigma)
#   Sigma ~ IW(nu0, S0)
# Input
# - n: number of samples to be returned
# - x: n x p data matrix
# - g: prior dispersion parameter in the prior for mu
# - mu0: prior mean in the prior for mu
# - nu0: prior degrees of freedom for Sigma
# - S0: prior scale matrix for Sigma, by default set to I/nu0
# - precision: if set to TRUE, samples from the precision matrix Sigma^{-1} are returned instead
# Output: independent random draws from Normal-IW posterior
    if (!is.matrix(x)) stop("x must be a matrix")
    samplesize= nrow(x); p= ncol(x)
    if (missing(nu0)) { nu0= nrow(Sigma)+1 }
    if (missing(S0)) { if (p==1) { S0= matrix(1/nu0) } else { S0= diag(p)/nu0 } }
    xbar= colMeans(x)
    nu1= nu0+samplesize
    dm= matrix(xbar-mu0,ncol=1)
    S1= S0 + cov(x)*(samplesize-1) + samplesize/(1+samplesize*g) * (dm %*% t(dm))
    w= samplesize/(samplesize+1/g)
    m1= w * xbar + (1-w) * mu0
    ans= vector("list",2)
    if (!precision) { names(ans)= c("mu","Sigma") } else { names(ans)= c("mu","precision") }
    ans[[1]]= matrix(NA,nrow=samplesize,ncol=p)
    ans[[2]]= matrix(NA,nrow=samplesize,ncol=p*(p+1)/2)
    S1inv= solve(S1)
    for (i in 1:samplesize) {
        if (precision) {
            Sigmainv= rWishart(1, df=nu1, Sigma=S1inv)[,,1]
            Sigma= solve(Sigmainv)
            ans[[2]][i,]= Sigmainv[lower.tri(Sigmainv,diag=TRUE)]
        } else {
            Sigma= riwish(nu=nu1,Sinv=S1inv)
            ans[[2]][i,]= Sigma[lower.tri(Sigma,diag=TRUE)]
        }
        ans[[1]][i,]= mvtnorm::rmvnorm(1,m1,Sigma/(samplesize+1/g))
    }
    return(ans)
}




######################################################################################
## MARGINAL LIKELIHOOD CALCULATIONS FOR NORMAL-INVERSE WISHART
######################################################################################


setMethod(marginalNIW, signature("missing","ANY","matrix","numeric","missing"), function(x, xbar, samplecov, n, z, g,  mu0=rep(0,ncol(x)), nu0=ncol(x)+4, S0, logscale=TRUE) {
#Integrated likelihood for
#   x[i]       ~ N(mu,Sigma)
#   mu | Sigma ~ N(mu0, g Sigma)
#   Sigma      ~ IW(nu0, S0)
#
#
# Input
# - xbar: p-vector with sample mean
# - samplecov: sample covariance
# - n: sample size
# - g: prior dispersion parameter in the prior for mu
# - mu0: prior mean in the prior for mu
# - nu0: prior degrees of freedom for Sigma
# - S0: prior scale matrix for Sigma, by default set to I/nu0
# - logscale: set to TRUE to get the log-integrated likelihood
    if (n==0) {
        ans= 0
    } else {
        p= ncol(samplecov)
        if (missing(S0)) { if (p==1) { S0= matrix(1/nu0) } else { S0= diag(p)/nu0 } }
        nupost= nu0+n
        if (n>1) { S= samplecov * (n-1) } else { S= matrix(0,nrow=p,ncol=p) }
        m= matrix(xbar-mu0,ncol=1)
        Spost= S0 + S + n/(1+n*g) * (m %*% t(m))
        detS0= as.numeric(determinant(S0,logarithm=TRUE)$modulus)
        detSpost= as.numeric(determinant(Spost,logarithm=TRUE)$modulus)
        ans= -.5*n*p*log(pi) + lmgamma(p,.5*nupost) - lmgamma(p,.5*nu0) + .5*nu0*detS0 - .5*nupost*detSpost - 0.5*p*log(1+n*g)
    }
    if (!logscale) ans= exp(ans)
    return(ans)
}
)

setMethod("marginalNIW", signature("matrix","missing","missing","missing","missing"), function(x, xbar, samplecov, n, z, g,  mu0=rep(0,ncol(x)), nu0=ncol(x)+4, S0,logscale=TRUE) {
#Integrated likelihood for
#   x[i]       ~ N(mu,Sigma)
#   mu | Sigma ~ N(mu0, g Sigma)
#   Sigma      ~ IW(nu0, S0)
#
#
# Input
# - x: n x p data matrix
# - g: prior dispersion parameter in the prior for mu
# - mu0: prior mean in the prior for mu
# - nu0: prior degrees of freedom for Sigma^{-1}
# - S0: prior scale matrix for Sigma^{-1}, by default set to I/nu0
# - logscale: set to TRUE to get the log-integrated likelihood
    if (!is.matrix(x)) stop("x must be a matrix")
    n= nrow(x); p= ncol(x)
    if (n==0) {
        ans= 0
    } else {
        if (missing(S0)) { if (p==1) { S0= matrix(1/nu0) } else { S0= diag(p)/nu0 } }
        nupost= nu0+n
        txc= t(x)-colMeans(x); S= txc %*% t(txc)
        m= matrix(colMeans(x)-mu0,ncol=1)
        Spost= S0 + S + n/(1+n*g) * (m %*% t(m))
        detS0= as.numeric(determinant(S0,logarithm=TRUE)$modulus)
        detSpost= as.numeric(determinant(Spost,logarithm=TRUE)$modulus)
        ans= -.5*n*p*log(pi) + lmgamma(p,.5*nupost) - lmgamma(p,.5*nu0) + .5*nu0*detS0 - .5*nupost*detSpost - 0.5*p*log(1+n*g)
    }
    if (!logscale) ans= exp(ans)
    return(ans)
}
)



setMethod("marginalNIW", signature("matrix","missing","missing","missing","vector"), function(x, xbar, samplecov, n, z, g,  mu0=rep(0,ncol(x)), nu0=ncol(x)+4, S0,logscale=TRUE) {
#Integrated likelihood for Normal-IW given cluster allocations z
#   x[i] | z[i]=k      ~ N(mu[k],Sigma[k])
#   mu[k] | Sigma[k] ~ N(mu0, g Sigma[k])
#   Sigma[k]^{-1} ~ W(nu0, S0)
#
# Output: integrated likelihood conditional on z
    if (length(z) != nrow(x)) stop("length(z) must be equal to nrow(x)")
    cluslabels= unique(z)
    cluslabels= cluslabels[order(cluslabels)]
    xbar= samplecov= vector("list",length(cluslabels))
    n= integer(length(cluslabels))
    for (i in 1:length(cluslabels)) {
        sel= (z==cluslabels[i])
        xbar[[i]]= colMeans(x[sel,,drop=FALSE])
        n[i]=sum(sel)
        if (n[i]>1) { samplecov[[i]]=cov(x[sel,,drop=FALSE]) } else { samplecov[[i]]= matrix(0,nrow=ncol(x),ncol=ncol(x)) }
    }
    ans= marginalNIW(xbar=xbar,samplecov=samplecov,n=n,g=g,mu0=mu0,nu0=nu0,S0=S0,logscale=TRUE)
    if (!logscale) ans= exp(ans)
    return(ans)
}
)


setMethod("marginalNIW", signature("missing","list","list","numeric","missing"), function(x, xbar, samplecov, n, z, g,  mu0=rep(0,ncol(x)), nu0=ncol(x)+4, S0,logscale=TRUE) {
#Integrated likelihood for Normal-IW given cluster allocations z
#   x[i] | z[i]=k      ~ N(mu[k],Sigma[k])
#   mu[k] | Sigma[k] ~ N(mu0, g Sigma[k])
#   Sigma[k]^{-1} ~ W(nu0, S0)
#
# Output: integrated likelihood conditional on z
    k= length(xbar)
    if ((length(samplecov) != k) | (length(n) !=k)) stop("length of xbar, samplecov and n must match")
    p= ncol(samplecov[[1]])
    if (missing(S0)) { if (p==1) { S0= matrix(1/nu0) } else { S0= diag(p)/nu0 } }
    ans= sum(sapply(1:k, function(i) marginalNIW(xbar=xbar[[i]],samplecov=samplecov[[i]],n=n[i],g=g,mu0=mu0,nu0=nu0,S0=S0,logscale=TRUE)))
    if (!logscale) ans= exp(ans)
    return(ans)
}
)



######################################################################################
## NORMAL-INVERSE WISHART ACROSS MULTIPLE CLUSTERS WITH COMMON COVARIANCE
######################################################################################


dpostNIWCommon <- function(mu,Sigma,x,z,g=1,mu0=rep(0,length(mu)),nu0=nrow(Sigma)+1,S0,logscale=FALSE) {
#Posterior Normal-IW density
#   x[i] | z[i]=k      ~ N(mu[k],Sigma)
#   mu[k] | Sigma ~ N(mu0, g Sigma)
#   Sigma^{-1} ~ IW(nu0, S0)
# Input
# - mu: list with cluster means (length equal to number of distinct elements in z)
# - x: n x p data matrix
# - z: cluster allocations
# - g: prior dispersion parameter in the prior for mu
# - mu0: prior mean in the prior for mu
# - nu0: prior degrees of freedom for Sigma
# - S0: prior scale matrix for Sigma, by default set to I/nu0
# - logscale: set to TRUE to get the log-posterior density
# Output: Normal-IW posterior density evaluated at (mu,Sigma)
    if (!is.matrix(x)) stop("x must be a matrix")
    n= nrow(x); p= ncol(x)
    if (missing(S0)) { if (p==1) { S0= matrix(1/nu0) } else { S0= diag(p)/nu0 } }
    cluslabels= unique(z)
    if (length(cluslabels) != length(mu)) stop('mu must be a list with length equal to length(unique(z))')
    cluslabels= cluslabels[order(cluslabels)]
    if (!is.null(names(mu))) {
        if (any(cluslabels %in% names(mu))) stop("names(mu) must coindice with unique(z)")
        mu= mu[cluslabels]
    }
    xbar= vector("list",length(cluslabels))
    zcount= integer(length(cluslabels))
    S1= S0
    ans= 0
    for (i in 1:length(cluslabels)) {
        sel= (z==cluslabels[i])
        xbar[[i]]= colMeans(x[sel,,drop=FALSE])
        zcount[i]=sum(sel)
        dm= matrix(xbar[[i]]-mu0,ncol=1)
        txc= t(x[sel,,drop=FALSE])-xbar[[i]]
        S1= S1 + (txc %*% t(txc)) + zcount[i]/(1+zcount[i]*g) * (dm %*% t(dm))
        w= zcount[i]/(zcount[i]+1/g)
        m1= w * xbar[[i]] + (1-w) * mu0
        ans= ans + mvtnorm::dmvnorm(mu[[i]],m1,Sigma/(zcount[i]+1/g),log=TRUE)
    }
    nu1= nu0+n
    ans= ans + diwish(Sigma,nu=nu1,S=S1,logscale=TRUE)
    if (!logscale) ans= exp(ans)
    return(ans)
}


marginalNIWCommon <- function(x, z, g,  mu0=rep(0,ncol(x)), nu0=ncol(x)+4, S0,logscale=TRUE) {
#Integrated likelihood for common covariances model given cluster allocations z
#   x[i] | z[i]=k      ~ N(mu[k],Sigma)
#   mu[k] | Sigma ~ N(mu0, g Sigma)
#   Sigma ~ IW(nu0, S0)
#
# Input
# - x: n x p data matrix
# - g: prior dispersion parameter in the prior for mu
# - mu0: prior mean in the prior for mu
# - nu0: prior degrees of freedom for Sigma^{-1}
# - S0: prior scale matrix for Sigma^{-1}, by default set to I/nu0
# - logscale: set to TRUE to get the log-integrated likelihood
    if (!is.matrix(x)) stop("x must be a matrix")
    n= nrow(x); p= ncol(x)
    if (n==0) {
        ans= 0
    } else {
        if (missing(S0)) { if (p==1) { S0= matrix(1/nu0) } else { S0= diag(p)/nu0 } }
        nupost= nu0+n
        cluslabels= unique(z)
        cluslabels= cluslabels[order(cluslabels)]
        xbar= vector("list",length(cluslabels))
        zcount= integer(length(cluslabels))
        Spost= S0
        for (i in 1:length(cluslabels)) {
            sel= (z==cluslabels[i])
            xbar[[i]]= colMeans(x[sel,,drop=FALSE])
            zcount[i]=sum(sel)
            m= matrix(xbar[[i]]-mu0,ncol=1)
            txc= t(x[sel,,drop=FALSE])-xbar[[i]]
            Spost= Spost + (txc %*% t(txc)) + zcount[i]/(1+zcount[i]*g) * (m %*% t(m))
         }
         detS0= as.numeric(determinant(S0,logarithm=TRUE)$modulus)
         detSpost= as.numeric(determinant(Spost,logarithm=TRUE)$modulus)
         ans= -.5*n*p*log(pi) + lmgamma(p,.5*nupost) - lmgamma(p,.5*nu0) + .5*nu0*detS0 - .5*nupost*detSpost - 0.5*p*sum(log(1+zcount*g))
    }
    if (!logscale) ans= exp(ans)
    return(ans)
}

##############################################################################################
##
## PAIRWISE BAYES FACTORS BASED ON QUADRATIC PENALTIES
##
##############################################################################################


###
### mombf.R
###

mombf <- function(lm1,coef,g,prior.mode,baseDensity='normal',nu=3,theta0,logbf=FALSE,B=10^5) {
  if (!(baseDensity %in% c('normal','t'))) stop('baseDensity must either be normal or t')
  if (baseDensity=='t') {
    if (length(g)>1) stop("For baseDensity=='t' g is only allowed to have length 1")
    if (nu <= 2) stop('For t base density nu must be >=3, otherwise prior is improper')
    if (nu >= length(lm1$residuals)-length(coef)+2) warning(paste('Based on the amount of data available, nu should be <',length(lm1$residuals)-length(coef)+2,'in order to guarantee finite sample consistency'))
  }
  UseMethod("mombf")
}

###
### mombf.lm.R
###

mombf.lm <- function(lm1,coef,g,prior.mode,baseDensity='normal',nu=3,theta0,logbf=FALSE,B=10^5) {

if ((!missing(g)) & (!missing(prior.mode))) warning('Both g and prior.mode were specified. g will be ignored')
if ((missing(g)) & (missing(prior.mode))) stop('Either g or prior.mode must be specified')
if (missing(theta0)) theta0 <- rep(0,length(coef)) else if (length(theta0)!=length(coef)) stop('theta0 must have the same length as coef')

  thetahat <- coef(lm1)
  V <- summary(lm1)$cov.unscaled
  n <- length(lm1$residuals); p <- length(thetahat); p1 <- length(coef)
  if ((min(coef)<1) | (max(coef)>p)) stop('Non-valid value for coef. Use only values between 1 and the number of coefficients in lm1')
  ssr <- sum(residuals(lm1)^2); sr <- sqrt(ssr/(n-p))
  if (!missing(prior.mode)) g <- mode2g(prior.mode,prior=paste(baseDensity,'Mom',sep=''),nu=nu)
  if (baseDensity=='normal') {
    bf.mom <- momunknown(thetahat[coef],V[coef,coef],n=n,g=g,ssr=ssr,nuisance.theta=p-p1,theta0=theta0,logbf=logbf)
  } else if (baseDensity=='t') {
    randomg <- 1/rgamma(B,nu/2,nu/2)
    bf.mom <- momunknown(thetahat[coef],V[coef,coef],n=n,g=g*randomg,ssr=ssr,nuisance.theta=p-p1,theta0=theta0,logbf=logbf)
    bf.mom <- mean(bf.mom*randomg)*(nu-2)/nu
  }
  return(bf.mom)
}


###
### momknown.R
###

momknown <- function(theta1hat,V1,n,g=1,theta0,sigma,logbf=FALSE) {
if (missing(sigma)) stop('sigma must be specified')
if (missing(theta0)) theta0 <- rep(0,length(theta1hat))
p1 <- length(theta1hat)
l <- theta1hat-theta0; l <- matrix(l,nrow=1) %*% solve(V1) %*% matrix(l,ncol=1) * n*g/((1+n*g)*sigma^2) #noncentr param
muk <- p1+l
t1 <- matrix(theta1hat-theta0,nrow=1) %*% solve(V1) %*% matrix(theta1hat-theta0,ncol=1) * n*g/((1+n*g)*sigma^2)
bf <- .5*t1 + log(muk) - log(1+n*g) - (p1/2)*log(1+n*g) - log(p1)
if (!logbf) bf <- exp(bf)
return(bf)
}

###
### momunknown.R
###

momunknown <- function(theta1hat,V1,n,nuisance.theta,g=1,theta0,ssr,logbf=FALSE) {
if (missing(theta0)) theta0 <- rep(0,length(theta1hat))
p1 <- length(theta1hat); p <- p1 + nuisance.theta
l <- theta1hat-theta0; l <- matrix(l,nrow=1) %*% solve(V1) %*% matrix(l,ncol=1)
sigma2hat <- (ssr + l/(1+n*g))/(n-nuisance.theta)
muk <- p1+ l* n*g/((1+n*g)*sigma2hat)
bf <- (-(n-nuisance.theta)/2)*log(1+n*g*ssr/(ssr+l)) + log(muk) - log(1+n*g) + ((n-p)/2)*log(1+n*g) - log(p1)
if (!logbf) bf <- exp(bf)
return(bf)
}




###
### imombf.R
###

imombf <- function(lm1,
                   coef,
                   g,
                   prior.mode,
                   nu=1,
                   theta0,
                   method='adapt',
                   nquant=100,
                   B=10^5) {
    UseMethod("imombf")
}


###
### imombf.lm.R
###

imombf.lm <- function(lm1,coef,g,prior.mode,nu=1,theta0,method='adapt',nquant=100,B=10^5) {
if ((!missing(g)) & (!missing(prior.mode))) warning('Both g and prior.mode were specified. g will be ignored')
if ((missing(g)) & (missing(prior.mode))) stop('Either g or prior.mode must be specified')
if (missing(theta0)) theta0 <- rep(0,length(coef)) else if (length(theta0)!=length(coef)) stop('theta0 must have the same length as coef')

  thetahat <- coef(lm1)
  V <- summary(lm1)$cov.unscaled
  n <- length(lm1$residuals); p <- length(thetahat); p1 <- length(coef)
  if ((min(coef)<1) | (max(coef)>p)) stop('Non-valid value for coef. Use only values between 1 and the number of coefficients in lm1')
  ssr <- sum(residuals(lm1)^2); sr <- sqrt(ssr/(n-p))
  if (!missing(prior.mode)) g <- mode2g(prior.mode,prior='iMom')
  bf.imom <- imomunknown(thetahat[coef],V[coef,coef],n,nuisance.theta=p-p1,g=g,nu=nu,theta0=theta0,ssr=ssr,method=method,nquant=nquant,B=B)
  return(bf.imom)
}

###
### imomknown.R.R
###

imomknown <- function(theta1hat,V1,n,nuisance.theta,g=1,nu=1,theta0,sigma,method='adapt',B=10^5) {
if (missing(sigma)) stop('sigma must be specified')
if (missing(theta0)) theta0 <- rep(0,length(theta1hat))
f <- function(z) { ans <- (n*gi/z)^((nu+p1)/2) * exp(-n*gi/z); ans[z==0] <- 0; return(ans) }

p1 <- length(theta1hat)
l <- theta1hat-theta0; l <- matrix(l,nrow=1) %*% solve(V1) %*% matrix(l,ncol=1) / sigma^2 #noncentr param
m <- double(length(g))
if (method=='MC') {
  z <- rchisq(B,df=p1,ncp=l)
  for (i in 1:length(m)) { gi <- g[i]; m[i] <- mean(f(z)) }
} else if (method=='adapt') {
  f2 <- function(z) { return(f(z)*dchisq(z,df=p1,ncp=l)) }
  for (i in 1:length(m)) { gi <- g[i]; m[i] <- integrate(f2,0,Inf)$value }
} else {
  stop('method must be adapt or MC')
}
bf <- exp((p1/2)*log(2/(n*g)) + lgamma(p1/2)-lgamma(nu/2) + .5*l) * m
return(bf)
}


###
### imomunknown.R
###

imomunknown <- function(theta1hat,V1,n,nuisance.theta,g=1,nu=1,theta0,ssr,method='adapt',nquant=100,B=10^5) {

fncp <- function(sigma2) {
  l <- theta1hat-theta0; l <- as.vector(matrix(l,nrow=1) %*% solve(V1) %*% matrix(l,ncol=1)) / sigma2
  l[l==Inf] <- exp(80)
  return(l)
}
f <- function(z) { ans <- (n*gi/z)^((nu+p1)/2) * exp(-n*gi/z); ans[z==0] <- 0; return(ans) }

if (missing(theta0)) theta0 <- rep(0,length(theta1hat))
p1 <- length(theta1hat)
m <- double(length(g))
if (method=='MC') {
  sigma2 <- 1/rgamma(B,(n-nuisance.theta)/2,ssr/2)
  l <- fncp(sigma2)
  z <- rchisq(B,df=p1,ncp=l)
  for (i in 1:length(m)) { gi <- g[i]; m[i] <- mean(f(z)) }
} else if (method=='adapt') {
  f2 <- function(z,z2) { return(f(z)*dchisq(z,df=p1,ncp=fncp(z2))) }
  qseq <- 1/qgamma((2*(1:nquant)-1)/(2*nquant),(n-nuisance.theta)/2,ssr/2)
  for (i in 1:length(m)) {
    m[i] <- 0; for (j in 1:nquant) { gi <- g[i]; m[i] <- m[i]+integrate(f2,0,Inf,z2=qseq[j])$value }
    m[i] <- m[i]/nquant
  }
} else {
  stop('method must be adapt or MC')
}
t1 <- theta1hat-theta0; t1 <- matrix(t1,nrow=1) %*% solve(V1) %*% matrix(t1,ncol=1) / ssr #noncentr param
bf <- exp((p1/2)*log(2/(n*g)) + lgamma(p1/2) - lgamma(nu/2) + ((n-nuisance.theta)/2)*log(1+t1)) * m
return(bf)
}

###
### modelSelection.R
###

### Methods for msfit objects


plot.msfit= function(x, y, ...) {
  if (!x$enumerate) {
    margppcum= apply(x$postSample, 2, cumsum) / (1:nrow(x$postSample))
    plot(margppcum[,1], type='l', ylim=c(0,1), xlab='Iteration', ylab='Marginal posterior inclusion probabilities')
    if (ncol(margppcum)>1) for (i in 2:ncol(margppcum)) lines(margppcum[,i])
  } else {
    stop("plot.msfit produces an MCMC convergence diagnostic plot, but MCMC output is not available (you probably set enumerate=FALSE)")
  }
}


setMethod("show", signature(object='msfit'), function(object) {
  cat('msfit object with outcome of type ',object$outcometype,', ',object$p,' covariates and ',object$family,' error distribution\n')
  ifelse(any(object$postMode!=0), paste('  Posterior mode: covariate ',which(object$postMode==1)), '  Posterior mode: null model')
  cat("Use postProb() to get posterior model probabilities\n")
  cat("Use coef() or predict() to get BMA estimates and intervals for parameters or given covariate values\n")
  cat("Elements $margpp, $postMode, $postSample and $coef contain further information (see help('msfit') and help('modelSelection') for details)\n")
}
)


hasPostSampling <- function(object) {
#Sends an error message if posterior sampling is not implemented for the priors and outcome type of the msFit object
#
  #List combinations for which posterior sampling is implemented
  hassamples= data.frame(matrix(NA,nrow=10,ncol=4))
  names(hassamples)= c('outcometype','family','priorCoef','priorGroup')
  hassamples[1,] =    c('Continuous','normal',   'pMOM',   'pMOM')
  hassamples[2,] =    c('Continuous','normal',  'peMOM',  'peMOM')
  hassamples[3,] =    c('Continuous','normal',  'piMOM',  'piMOM')
  hassamples[4,] =    c('Continuous','normal','zellner','zellner')
  hassamples[5,] =    c('Continuous','normal','normalid','normalid')
  hassamples[6,] =    c('Continuous','normal','icarplus','icarplus')
  hassamples[7,] =    c('glm','binomial','bic','bic')
  hassamples[8,] =    c('glm','binomial logit','bic','bic')
  hassamples[9,] =    c('glm','binomial probit','bic','bic')
  hassamples[10,] =    c('glm','gamma inverse','bic','bic')
  hassamples[11,]=    c('glm','inverse.gaussian 1/mu^2','bic','bic')
  hassamples[12,]=    c('glm','poisson','bic','bic')
  hassamples[13,]=    c('glm','poisson log','bic','bic')
  hassamples[14,]=    c('Survival','Cox','bic','bic')
  #Check if there's variable groups
  hasgroups= (length(object$groups) > length(unique(object$groups)))
  outcometype= object$outcometype; family= object$family; priorCoef= object$priors$priorCoef@priorDistr; priorGroup= object$priors$priorGroup@priorDistr
  outcomefam= paste(outcometype,family,sep=',')
  if (hasgroups) {
      outcomefamprior= paste(outcomefam,priorCoef,priorGroup,sep=',')
      avail_outcomefamprior= apply(hassamples,1,paste,collapse=',')
  } else {
      outcomefamprior= paste(outcomefam,priorCoef,sep=',')
      avail_outcomefamprior= apply(hassamples[,1:3],1,paste,collapse=',')
  }
  found= outcomefam %in% apply(hassamples[,1:2],1,paste,collapse=',')
  exactsampling= outcomefamprior  %in% avail_outcomefamprior
  if (!found) {
    message("Inference on parameters currently only available for the following settings: \n\n")
    print(hassamples)
    message("\n")
    stop("Inference on parameters not implemented for outcometype= ",outcometype,", family=",family,", priorCoef=",priorCoef,", priorGroup=",priorGroup)
  } else {
    if (!exactsampling) warning("Exact posterior sampling not implemented, using Normal approx instead")
  }
}




coef.msfit <- function(object,...) {
    hasPostSampling(object)
    th= rnlp(msfit=object,niter=10^4)
    ct= (object$stdconstants[-1,'scale']==0)
    if (!is.null(names(object$margpp))) {
        nn= names(object$margpp)
    } else if (!is.null(colnames(th))) {
        nn= colnames(th)
    } else { nn= paste('beta',1:ncol(th))  }
    if (any(ct)) {
        if (ncol(th) > length(object$margpp)) {
            if (object$family %in% c('binomial','poisson')) {
              margpp= object$margpp
            } else {
              margpp= c(object$margpp,1)
              nn= c(nn,'phi')
            }
        } else { margpp= object$margpp }
    } else {
        if (ncol(th) > length(object$margpp)) {
            if (object$family %in% c('binomial','poisson')) {
              margpp= c(mean(th[,1]!=0),object$margpp)
              nn= c('intercept',nn)
            } else {
              margpp= c(mean(th[,1]!=0),object$margpp,1)
              nn= c('intercept',nn,'phi')
            }
        } else {
            margpp= c(mean(th[,1]!=0),object$margpp)
            nn= c('intercept',nn)
        }
    }
    ans= cbind(colMeans(th),t(apply(th,2,quantile,probs=c(.025,0.975))),margpp=margpp)
    colnames(ans)= c('estimate','2.5%','97.5%','margpp')
    rownames(ans)= nn
    return(ans)
}


#Return point estimate, 95% interval and posterior model prob for nmax top models
setMethod("coefByModel", signature(object='msfit'), function(object, maxmodels, alpha=0.05, niter=10^3, burnin=round(niter/10)) {

  if (!is.null(object$postmean) & (object$priors$priorCoef@priorDistr %in% c('bic','zellner'))) {

      postmean= object$postmean[1:min(maxmodels,nrow(object$postmean)),]
      postsd= sqrt(object$postvar[1:min(maxmodels,nrow(object$postvar)),])
      ans= list(postmean= postmean, ci.low= postmean + qnorm(alpha/2) * postsd, ci.up= postmean - qnorm(alpha/2) * postsd)
      nn= colnames(ans[[1]])
      
  } else {

    hasPostSampling(object)
    y= object$ystd; x= object$xstd
    outcometype= object$outcometype; family= object$family
    b= min(50, ceiling((burnin/niter) * niter))
    #List models for which estimates are to be obtained
    pp= postProb(object,method="norm")
    modelid= strsplit(as.character(pp$modelid), split=',')
    modelid= modelid[1:min(maxmodels,length(modelid))]
    priorCoef= object$priors$priorCoef
    priorGroup= object$priors$priorGroup
    priorVar= object$priors$priorVar
    #Obtain point estimates and posterior intervals
    ans= vector("list",3); names(ans)= c('postmean','ci.low','ci.up')
    if ((outcometype== 'Continuous') && (family== 'normal')) { ##Linear model
      ans[[1]]= ans[[2]]= ans[[3]]= matrix(0,nrow=length(modelid),ncol=ncol(x)+1)
      for (i in 1:length(modelid)) {  #for each model
        colsel= as.numeric(modelid[[i]])
        bm= coefOneModel(y=y, x=x[,colsel,drop=FALSE], outcometype=outcometype, family=family, priorCoef=priorCoef, priorGroup=priorGroup, priorVar=priorVar, alpha=alpha, niter=niter, burnin=b)
        colselphi= c(colsel,ncol(ans[[1]]))
        ans[[1]][i,colselphi]= bm[,1]
        ans[[2]][i,colselphi]= bm[,2]
        ans[[3]][i,colselphi]= bm[,3]
      }
      if (is.null(colnames(x))) nn= c(paste('beta',1:ncol(x),sep=''),'phi') else nn= c(colnames(x),'phi')
    } else {                                                   ##GLM or Survival model
      ans[[1]]= ans[[2]]= ans[[3]]= matrix(0, nrow=length(modelid), ncol=ncol(x))
      for (i in 1:length(modelid)) {   #for each model
        colsel= as.numeric(modelid[[i]])
        if (length(colsel)>0) {
          bm= coefOneModel(y=y, x=x[,colsel,drop=FALSE], outcometype=outcometype, family=family, priorCoef=priorCoef, priorGroup=priorGroup, priorVar=priorVar, alpha=alpha, niter=niter, burnin=b)
          ans[[1]][i,colsel]= bm[,1]
          ans[[2]][i,colsel]= bm[,2]
          ans[[3]][i,colsel]= bm[,3]
        }
      }
      if (is.null(colnames(x))) nn= paste('beta',1:ncol(x),sep='') else nn= colnames(x)
    }
    colnames(ans[[1]])= colnames(ans[[2]])= colnames(ans[[3]]) = nn
    rownames(ans[[1]])= rownames(ans[[2]])= rownames(ans[[3]]) = pp$modelid[1:nrow(ans[[1]])]

  }

  #Return parameter estimates in non-standardized parameterization
  ans[[1]]= unstdcoef(ans[[1]],p=ncol(ans[[1]]),msfit=object,coefnames=nn)
  ans[[2]]= unstdcoef(ans[[2]],p=ncol(ans[[2]]),msfit=object,coefnames=nn)
  ans[[3]]= unstdcoef(ans[[3]],p=ncol(ans[[3]]),msfit=object,coefnames=nn)
  return(ans)
}
)



setMethod("coefOneModel", signature(y='ANY',x='matrix',m='missing',V='missing',outcometype='character',family='character'), function(y, x, m, V, outcometype, family, priorCoef, priorGroup, priorVar, alpha=0.05, niter=10^3, burnin=round(niter/10)) {
  if ((outcometype== 'Continuous') && (family== 'normal')) {  ##Linear model
    b= rnlpLM(y=y, x=x, priorCoef=priorCoef, priorGroup=priorGroup, priorVar=priorVar, niter=niter, burnin=burnin)
  } else if (outcometype=='glm') { #GLM
    b= rnlpGLM(y=y, x=x, family=family, priorCoef=priorCoef, priorGroup=priorGroup, priorVar=priorVar, niter=niter, burnin=burnin)
  } else if ((outcometype=='Survival') && (family=='Cox')) {  #Cox model
    b= rnlpCox(y=y, x=x, priorCoef=priorCoef, priorGroup=priorGroup, niter=niter, burnin=burnin)
  } else {
    stop(paste("outcometype",outcometype,"and family=",family,"not implemented",sep=""))
  }
  ans= cbind(colMeans(b), t(apply(b,2,quantile,probs=c(alpha/2,1-alpha/2))))
  return(ans)
}
)



predict.msfit <- function(object, newdata, data, level=0.95, ...) {
    hasPostSampling(object)
    if (is.null(colnames(object$xstd))) colnames(object$xstd)= paste('x',1:ncol(object$xstd),sep='')
    th= rnlp(msfit=object,niter=10^4)
    mx= object$stdconstants[-1,'shift']; sx= object$stdconstants[-1,'scale']
    if (!missing(newdata)) {
        f= object$call$formula
        if ('formula' %in% class(f)) {
            if (missing(data)) stop("You must specify the argument 'data'")
            alldata= rbind(data,newdata)
            alldata[,as.character(f)[2]]= 0  #ensure there's no NAs in the response, so createDesign doesn't drop those rows from newdata
            nn= rownames(alldata)[(nrow(data)+1):(nrow(data)+nrow(newdata))]
            if (is.null(object$call$smoothterms)) {
                des= createDesign(f, data=alldata)
            } else {
                des= createDesign(f, data=alldata, smoothterms=object$call$smoothterms, splineDegree=object$call$splineDegree, nknots=object$call$nknots)
            }
            newdata= des$x[nn,,drop=FALSE]
        }
        #ct= (sx==0)
        #newdata[,!ct]= t((t(newdata[,!ct]) - mx[!ct])/sx[!ct])
        if (is.null(colnames(newdata))) colnames(newdata)= paste('x',1:ncol(newdata),sep='')
    } else {
        newdata= t(t(object$xstd) * sx + mx)
    }
    sel= colnames(th) %in% colnames(newdata)
    ypred= th[,sel] %*% t(newdata)
    if ("intercept" %in% colnames(th)[!sel]) ypred= ypred + th[,'intercept']
    ans= cbind(mean=colMeans(ypred), t(apply(ypred,2,quantile,probs=c((1-level)/2,1-(1-level)/2))))
    return(ans)
}


getmodelid= function(object) {
  if (!inherits(object, 'msfit')) stop("Function modelid requires an argument of type msfit")
  if (!is.null(object$models)) {
    ans= object$models[,c('modelid','family')]
  } else {
    modelid= apply(object$postSample==1, 1, function(z) paste(which(z),collapse=','))
    if (object$family=='auto') {
      twopiece <- laplace <- logical(length(modelid))
      twopiece[grep(as.character(object$p+1),modelid)] <- TRUE
      laplace[grep(as.character(object$p+2),modelid)] <- TRUE
      family <- character(length(modelid))
      family[(!twopiece) & (!laplace)] <- 'normal'
      family[twopiece & (!laplace)] <- 'twopiecenormal'
      family[(!twopiece) & laplace] <- 'laplace'
      family[twopiece & laplace] <- 'twopiecelaplace'
      modelid <- sub(paste(',',object$p+1,sep=''),'',modelid)
      modelid <- sub(as.character(object$p+1),'',modelid)  #for null model
      modelid <- sub(paste(',',object$p+2,sep=''),'',modelid)
      modelid <- sub(as.character(object$p+2),'',modelid)  #for null model
    } else {
      family= object$family
    }
    ans= data.frame(modelid=modelid, family=family)
  }
  return(ans)
}



setMethod("logjoint", signature(object='msfit'), function(object, return_models=TRUE, models_as_char=FALSE) {
  if (object$enumerate) {
      if (return_models) {
        ans= data.frame(object$models, object$postProb)
      } else {
        ans= object$postProb
      }
  } else {
      if (!models_as_char) {
          ans= unique(data.frame(object$postSample==1, logpp=object$postProb))
      } else {
          modelid= apply(object$postSample==1, 1, function(z) paste(which(z),collapse=','))
          ans= unique(data.frame(modelid, logpp= object$postProb))
      }
    if (!return_models) ans= ans[,ncol(ans)]
  }
return(ans)
}
)


setMethod("postProb", signature(object='msfit'), function(object, nmax, method='norm') {
if (!is.null(object$models)) {
    ans= object$models
} else {
  if (method=='norm') {
    modelpp <- unique(data.frame(object$postSample==1, logpp=object$postProb))
    modelpp <- data.frame(modelid= apply(modelpp[,1:(ncol(modelpp)-1)], 1, function(z) paste(which(z),collapse=',')), logpp=modelpp$logpp)
    modelpp$logpp <- modelpp$logpp - max(modelpp$logpp)
    modelpp$pp <- exp(modelpp$logpp)/sum(exp(modelpp$logpp))
  } else if (method=='exact') {
    modelpp <- apply(object$postSample==1, 1, function(z) paste(which(z),collapse=','))
    modelpp <- table(modelpp)/length(modelpp)
    modelpp <- data.frame(modelid=names(modelpp), pp=as.numeric(modelpp))
  } else {
    stop("Argument 'method' not recognized")
  }
  modelpp <- modelpp[order(modelpp$pp,decreasing=TRUE),]
  if (!missing(nmax)) modelpp <- modelpp[1:nmax,]
  if (object$family=='auto') {
    modelid <- as.character(modelpp[,'modelid'])
    twopiece <- laplace <- logical(nrow(modelpp))
    twopiece[grep(as.character(object$p+1),modelid)] <- TRUE
    laplace[grep(as.character(object$p+2),modelid)] <- TRUE
    family <- character(nrow(modelpp))
    family[(!twopiece) & (!laplace)] <- 'normal'
    family[twopiece & (!laplace)] <- 'twopiecenormal'
    family[(!twopiece) & laplace] <- 'laplace'
    family[twopiece & laplace] <- 'twopiecelaplace'
    modelid <- sub(paste(',',object$p+1,sep=''),'',modelid)
    modelid <- sub(as.character(object$p+1),'',modelid)  #for null model
    modelid <- sub(paste(',',object$p+2,sep=''),'',modelid)
    modelid <- sub(as.character(object$p+2),'',modelid)  #for null model
    modelpp <- data.frame(modelid=modelid,family=family,pp=modelpp[,'pp'])
  } else {
    modelpp <- data.frame(modelid=modelpp[,'modelid'],family=object$family,pp=modelpp[,'pp'])
  }
  ans= modelpp[,c('modelid','family','pp')]
}
return(ans)
}
)


#Convert text model identifier (e.g. "1,3,4") into logical identifier (e.g. c(TRUE,FALSE,TRUE,TRUE))
modelid2logical= function(modelid, nvars) {
  modelid= strsplit(modelid, split=',')
  ans= matrix(FALSE, nrow=length(modelid), ncol=nvars)
  for (i in 1:nrow(ans)) {
      sel= as.numeric(modelid[[i]])
      ans[i, sel[sel<=nvars]]= TRUE
  }
  return(ans)
}


#Obtain posterior probability for subsets of variables indicated in varsubset (the remaining parameters are passed on to postProb)
# varsubset can take one of the following 4 formats
# - A logical vector indicating which variables are in the subset and length equal to the number of variables, e.g. c(TRUE, FALSE, TRUE, TRUE)
# - A logical matrix where each row indicates a subset, as in the previous entry
# - A numeric vector indicating the indices of the variables in the subset, e.g. c(1,3,4)
# - A list where each entry is a numeric vector as in the previous entry
setMethod("postProbSubset", signature(object='msfit'), function(object, varsubset, nmax, method='norm') {
    if (!is.list(varsubset) & !is.matrix(varsubset)) {
      if (is.logical(varsubset)) varsubset= matrix(varsubset, nrow=1) else varsubset= list(varsubset)
    }
    if (is.matrix(varsubset)) nsubsets= nrow(varsubset) else if (is.list(varsubset)) nsubsets= length(varsubset) else stop("varsubset has the wrong format")
    modelpp= postProb(object, nmax=nmax, method=method)
    modelid= modelid2logical(modelpp$modelid, nvars= ncol(object$xstd))
    colnames(modelid)= colnames(object$xstd)
    ans= double(nsubsets)
    if (is.matrix(varsubset)) {
      for (i in 1:nsubsets) {
          nselvars= rowSums(modelid[,varsubset[i,],drop=FALSE]) #number of variables in the subset selected by each model
          selmodel= (nselvars== sum(varsubset[i,]))  #models selecting all variables in the subset
          ans[i]= sum(modelpp$pp[selmodel])
      }
    } else {
      for (i in 1:nsubsets) {
          nselvars= rowSums(modelid[,varsubset[[i]],drop=FALSE])    #number of variables in the subset selected by each model
          selmodel= (nselvars== length(varsubset[[i]]))  #models selecting all variables in the subset
          ans[i]= sum(modelpp$pp[selmodel])
      }
    }
    return(ans)
}
)


#Estimate marginal likelihood across all models, p(y)= sum_gamma p(y, gamma) where gamma denotes the considered models
#If all models can be enumerated, the exact result is obtained by summing across models
#If all models cannot be enumerated, the estimate is obtained via species re-sampling, as follows
# 1. Split the MCMC into two halves. Let S be the set of visited models in the 1st half, and m1= sum_{gamma in S} p(y,gamma)
# 2. The species re-sampling estimate is m1 * adj, where
#    adj= [1 + P(gamma not in S | y) / P(gamma in S | y)]
#    and P(gamma in S | y) is estimated from the 2nd half of the MCMC iterations
setMethod("marglhood_acrossmodels", signature(object='msfit'), function(object, logscale=TRUE) {
    m= max(object$postProb)
    ans= m + log(sum(exp(object$postProb - m)))
    if (!object$enumerate) {
        B= nrow(object$postSample)
        Bhalf= round(B/2)
        sampled_models1= object$postSample[1:Bhalf,]
        sampled_models2= object$postSample[-1:-Bhalf,]
        if (is.numeric(sampled_models1)) {
            modelids1= apply(sampled_models1, 1, function(z) paste(which(z==1),collapse=','))
            modelids2= apply(sampled_models2, 1, function(z) paste(which(z==1),collapse=','))
        } else if (is.logical(sampled_models1)) {
            modelids1= apply(unique(sampled_models1), 1, function(z) paste(which(z),collapse=','))
            modelids2= apply(sampled_models2, 1, function(z) paste(which(z),collapse=','))
        }
        prob.resampled1= mean(modelids2 %in% modelids1)
        prob.resampled2= mean(modelids1 %in% modelids2)
        adj1= (1 + (1-prob.resampled1) / prob.resampled1)
        adj2= (1 + (1-prob.resampled2) / prob.resampled2)
        ans= ans + mean(c(log(adj1),log(adj2)))
    }
    if (!logscale) ans= exp(ans)
    return(ans)
}
)


defaultmom= function(outcometype, family, verbose=TRUE) {
    if (outcometype=='Continuous') {
        if (verbose) message("Using default prior for continuous outcomes priorCoef=momprior(tau=0.348), priorVar=igprior(.01,.01)\n")
        priorCoef= momprior(tau=0.348)
        priorVar= igprior(alpha=.01,lambda=.01)
    } else if (outcometype=='Survival') {
        if (verbose) message("Using default prior for Normal AFT survival outcomes priorCoef=momprior(tau=0.192), priorVar=igprior(3,3)\n")
        priorCoef= momprior(tau=0.192)
        priorVar= igprior(alpha=3,lambda=3)
    } else if (outcometype=='glm') {
        if (verbose) message("Using default prior for GLMs priorCoef=momprior(tau=1/3), priorVar=igprior(.01,.01)\n")
        priorCoef= momprior(tau=1/3)
        priorVar= igprior(alpha=.01,lambda=.01)
    } else {
      stop("There is not default priorCoef for this outcome type")
    }
    ans= list(priorCoef=priorCoef, priorVar=priorVar)
    return(ans)
}



#### General model selection routines
modelSelection <- function(y, x, data, smoothterms, nknots=9, groups, constraints, center=TRUE, scale=TRUE, enumerate, includevars, models, maxvars, niter=5000, thinning=1, burnin=round(niter/10), family='normal', priorCoef, priorGroup, priorModel=modelbbprior(1,1), priorConstraints, priorVar=igprior(.01,.01), priorSkew=momprior(tau=0.348), neighbours, phi, deltaini, initSearch='CDA', method='auto', adj.overdisp='intercept', hess='asymp', optimMethod, optim_maxit, initpar='none', B=10^5, XtXprecomp, verbose=TRUE) {
# Input
# - y: either formula with the regression equation or vector with response variable. If a formula, arguments x, groups & constraints are ignored
# - x: design matrix with all potential predictors
# - data: data frame where the variables indicated in y (if it's a formula) and smoothterms can be found
# - smoothterms: formula indicating variables for which non-linear effects (splines) should be considered
# - nknots: number of knots
# - groups: vector indicating groups for columns in x (defaults to each variable in a separate group)
# - constraints: constraints on the model space. List with length equal to the number of groups; if group[[i]]=c(j,k) then group i can only be in the model if groups j and k are also in the model
# - center: if center==TRUE y and x are centered to have zero mean, therefore eliminating the need to include an intercept term in x.
# - scale: if scale==TRUE y and columns in x are scaled to have standard deviation 1
# - enumerate: if TRUE all models with up to maxvars are enumerated, else Gibbs sampling is used to explore the model space
# - includevars: set to TRUE for variables that you want to force into the model (for grouped variables, TRUE/FALSE is taken from 1st variable in each group)
# - maxvars: maximum number of variables in models to be enumerated (ignored if enumerate==FALSE)
# - niter: number of Gibbs sampling iterations
# - thinning: MCMC thinning factor, i.e. only one out of each thinning iterations are reported. Defaults to thinning=1, i.e. no thinning
# - burnin: number of burn-in MCMC iterations. Defaults to 10% of niter. Set to 0 for no burn-in.
# - family: assumed residual distribution ('normal','twopiecenormal','laplace','twopiecelaplace')
# - priorCoef: prior distribution for the coefficients. Must be object of class 'msPriorSpec' with slot priorType set to 'coefficients'. Possible values for slot priorDistr are 'pMOM', 'piMOM' and 'peMOM'.
# - priorGroup: prior on grouped coefficients, as indicated by groups
# - priorModel: prior on model indicator space. Must be object of class 'msPriorSpec' with slot priorType set to 'modelIndicator'. Possible values for slot priorDistr are 'uniform' and 'binomial'
# - priorVar: prior on residual variance. Must be object of class 'msPriorSpec' with slot priorType set to 'nuisancePars'. Slot priorDistr must be equal to 'invgamma'.
# - priorSkew: prior on residual skewness parameter. Ignored unless family=='twopiecenormal' or 'twopiecelaplace'
# - neighbours: Only used if priorCoef is an icarplus prior. neighbours is a list with the same length as the design matrix. Its entry j should be a vector indicating the neighbours of j, and have 0 length if j has no neighbours.
# - phi: residual variance. Typically this is unknown and therefore left missing. If specified argument priorVar is ignored.
# - deltaini: logical vector of length ncol(x) indicating which coefficients should be initialized to be non-zero. Defaults to all variables being excluded from the model
# - initSearch: algorithm to refine deltaini. initSearch=='CDA' uses a greedy coordinate descent algorithm. initSearch=='SCAD' sets deltaini to the non-zero elements in a SCAD fit with cross-validated regularization parameter. initSearch=='none' leaves deltaini unmodified.
# - method: method to compute marginal densities. method=='Laplace' for Laplace approx, method=='MC' for Importance Sampling, method=='Hybrid' for Hybrid Laplace-IS (the latter method is only used for piMOM prior with unknown residual variance phi), method='ALA' (former method=='plugin')
# - adj.overdisp: for method=='ALA' it indicates the over-dispersion adjustment to be made in models where the dispersion parameter is fixed, as in logistic and Poisson regression. adj.overdisp='none' for no adjustment (not recommended, particularly for Poisson models). adj.overdisp='intercept' to estimate over-dispersion from the intercept-only model. ad.overdisp='residuals' from the Pearson residuals of each model (slightly higher computational cost)
# - hess: only used for asymmetric Laplace errors. When hess=='asymp' the asymptotic hessian is used to compute the Laplace approximation to the marginal likelihood, when hess=='asympDiagAdj' a diagonal adjustment to the asymptotic Hessian is used
# - optimMethod: method to maximize objective function when method=='Laplace' or method=='MC'. Only used for family=='twopiecenormal'. optimMethod=='LMA' uses modified Newton-Raphson algorithm, 'CDA' coordinate descent algorithm
# - optim_maxit: maximum number of iterations in optimization method
# - initpar: initial value for regression parameters when finding the posterior mode to approximate the integrated likelihood. Either 'MLE', 'MLE-aisgd', 'L1', 'L2-aisgd', or a numeric vector specifying the initial values. If p<n/2 MLE is used, else L1 (regularization parameter set via BIC). If n>10,000 or p>200, then MLE-aisgd or L2-aisgd are used.
# - B: number of samples to use in Importance Sampling scheme. Ignored if method=='Laplace'.
# - verbose: set verbose==TRUE to print iteration progress
# Output: list
# - postSample: posterior samples
# - margpp: marginal posterior probability for inclusion of each covariate (approx by averaging marginal post prob for inclusion in each Gibbs iteration. This approx is more accurate than simply taking colMeans(postSample))
# - postMode: model with highest posterior probability amongst all those visited
# - postModeProb: unnormalized posterior prob of posterior mode (log scale)
# - postProb: unnormalized posterior prob of each visited model (log scale)

  #Check input
  tmp <- formatInputdata(y=y,x=x,data=data,smoothterms=smoothterms,nknots=nknots,family=family)
  x <- tmp$x; y <- tmp$y; formula <- tmp$formula;
  splineDegree <- tmp$splineDegree
  if (missing(groups)) groups <- 1:ncol(x)
  if (!is.null(tmp$groups)) groups <- tmp$groups
  if (length(groups) != ncol(x)) stop(paste("groups has the wrong length. It should have length",ncol(x)))
  hasgroups <- tmp$hasgroups; isgroups <- tmp$isgroups
  if (!is.null(tmp$constraints)) constraints <- tmp$constraints
  outcometype <- tmp$outcometype; uncens <- tmp$uncens; ordery <- tmp$ordery
  typeofvar <- tmp$typeofvar
  call <- list(formula=formula, smoothterms= NULL, splineDegree=splineDegree, nknots=nknots)
  if (!missing(smoothterms)) call$smoothterms <- smoothterms
  p= ncol(x); n= length(y)
  if (missing(includevars)) includevars= rep(FALSE,ncol(x))
  if (is.numeric(includevars)) {
      tmp= rep(FALSE,p)
      if (max(includevars) > p) stop(paste("includevars contains index ",max(includevars)," but the design matrix only has ",p," columns",sep=""))
      tmp[includevars]= TRUE
      includevars= tmp
  }
  if (length(includevars)!=ncol(x) | (!is.logical(includevars))) stop("includevars must be a logical vector of length ncol(x)")

  #Set default priors
  #If there are variable groups, count variables in each group, indicate 1st variable in each group, convert group and constraint labels to integers 0,1,...
  if (missing(priorCoef)) {
      defaultprior= defaultmom(outcometype=outcometype,family=family,verbose=verbose)
      priorCoef= defaultprior$priorCoef; priorVar= defaultprior$priorVar
  }
  if (missing(priorGroup)) { if (length(groups)==length(unique(groups))) { priorGroup= priorCoef } else { priorGroup= groupzellnerprior(tau=n) } }
  tmp= codeGroupsAndConstraints(p=p,groups=groups,constraints=constraints)
  ngroups= tmp$ngroups; constraints= tmp$constraints; invconstraints= tmp$invconstraints; nvaringroup=tmp$nvaringroup; groups=tmp$groups
  if (missing(models)) {
    if (missing(enumerate)) enumerate= ifelse(ngroups<15,TRUE,FALSE)
  } else { enumerate= TRUE }

  #Set maximum number of variables allowed in any model
  if (missing(maxvars)) maxvars= set_ms_maxvars(n=n, p=p, priorCoef=priorCoef, family=family, includevars=includevars)
  if (maxvars < sum(includevars)) stop("maxvars must be >= sum(includevars)")
  maxvars= as.integer(maxvars)

  #Standardize (y,x) to mean 0 and variance 1 (for continuous or log-survival time outcomes only)
  if (!is.vector(y)) { y <- as.double(as.vector(y)) } else { y <- as.double(y) }
  if (!is.matrix(x)) x <- as.matrix(x)
  mx= colMeans(x); sx= sqrt(colMeans(x^2) - mx^2) * sqrt(n/(n-1))
  ct= (sx==0)
  if (any(is.na(ct))) stop('x contains NAs, this is currently not supported, please remove the NAs')
  if (sum(ct)>1) stop('There are >1 constant columns in x (e.g. two intercepts)')
  if (!center) { my=0; mx= rep(0,p) } else { my= mean(y) }
  if (!scale) { sy=1; sx= rep(1,p) } else { sy= sd(y) }
  mx[typeofvar=='factor']=0; sx[typeofvar=='factor']= 1
  if (!(outcometype %in% c('Continuous','Survival'))) {
      my=0; sy= 1
  } else {
      if ((ncol(x)==1) & center) {
          if (ct) warning("You specified center=TRUE for an intercept-only model. This is rarely correct, to test whether the intercept is 0 you should not center the outcome. Consider setting center=FALSE.")
      }
  }
  ystd= (y-my)/sy; xstd= x; xstd[,!ct]= t((t(x[,!ct]) - mx[!ct])/sx[!ct])
  if (missing(phi)) { knownphi <- as.integer(0); phi <- double(0) } else { knownphi <- as.integer(1); phi <- as.double(phi) }
  stdconstants= rbind(c(my,sy),cbind(mx,sx)); colnames(stdconstants)= c('shift','scale')

  #Format arguments for .Call
  if (missing(deltaini)) {
    deltaini= as.integer(which(includevars)-1); ndeltaini= as.integer(length(deltaini))
  } else {
    if (length(deltaini)!=p) stop('deltaini must be of length ncol(x)')
    if (!is.logical(deltaini)) { stop('deltaini must be of type logical') } else { ndeltaini <- as.integer(sum(deltaini | includevars)); deltaini <- as.integer(which(deltaini | includevars)-1) }
  }

  thinit= getthinit(y=y, x=xstd, family=family, initpar=initpar, enumerate=enumerate)
  usethinit= thinit$usethinit; thinit= thinit$thinit

  method <-  formatmsMethod(method=method, usethinit=usethinit, optimMethod=optimMethod, optim_maxit=optim_maxit, priorCoef=priorCoef, priorGroup=priorGroup, knownphi=knownphi, outcometype=outcometype, family=family, hasgroups=hasgroups, adj.overdisp=adj.overdisp, hess=hess)
  optimMethod <- method$optimMethod; optim_maxit <- method$optim_maxit; adj.overdisp <- method$adj.overdisp; hesstype <- method$hesstype; method <- method$method

  niter <- as.integer(niter); burnin <- as.integer(burnin); thinning <- as.integer(thinning); B <- as.integer(B)
  sumy2 <- as.double(sum(ystd^2)); sumy <- as.double(sum(ystd)); ytX <- as.vector(matrix(ystd,nrow=1) %*% xstd); colsumsx <- as.double(colSums(xstd))
  if (missing(XtXprecomp)) XtXprecomp <- ifelse(ncol(xstd)<10^4,TRUE,FALSE)
  if (XtXprecomp) {
      XtX= t(xstd) %*% xstd
      hasXtX= as.logical(TRUE)
  } else {
      XtX= double(0)
      hasXtX= as.logical(FALSE)
  }

  ffamily= formatFamily(family, issurvival= length(uncens)>0)
  familyint= ffamily$familyint; familygreedy= ffamily$familygreedy
  if (familyint == 22) { sumlogyfact= as.double(sum(lgamma(ystd+1))) } else { sumlogyfact= as.double(0) } #Poisson regression
    
  if (!is.null(colnames(xstd))) { nn <- colnames(xstd) } else { nn <- paste('x',1:ncol(xstd),sep='') }

  tmp= formatmsPriorsMarg(priorCoef=priorCoef, priorGroup=priorGroup, priorVar=priorVar, priorSkew=priorSkew, n=n)
  r= tmp$r; prior= tmp$prior; priorgr= tmp$priorgr; tau=tmp$tau; taugroup=tmp$taugroup; a= tmp$a
  alpha=tmp$alpha; lambda=tmp$lambda; taualpha=tmp$taualpha; fixatanhalpha=tmp$fixatanhalpha
  priorCoef= tmp$priorCoef; priorGroup= tmp$priorGroup
    
  priorConstraints <- defaultpriorConstraints(priorModel, priorConstraints)
  tmp= formatmsPriorsModel(priorModel=priorModel, priorConstraints=priorConstraints, constraints=constraints)
  prDelta=tmp$prDelta; prDeltap=tmp$prDeltap; parprDeltap=tmp$parprDeltap
  prConstr=tmp$prConstr; prConstrp= tmp$prConstrp; parprConstrp= tmp$parprConstrp

  if (!missing(neighbours)) { Dmat= icar_dmatrix(neighbours) } else { Dmat= diag(p) }

  #Run model selection
  if (!enumerate) {
    #Initialize
    includevars <- as.integer(includevars)
    if (familyint==0) { postMode <- rep(as.integer(0),p+2) } else { postMode <- rep(as.integer(0),p) }
    postModeProb <- double(1)
    if (initSearch=='CDA') {
      niterGreed <- as.integer(100)
      ans= greedyVarSelCI(knownphi,familygreedy,prior,priorgr,niterGreed,ndeltaini,deltaini,includevars,n,p,ystd,uncens,sumy2,sumy,sumlogyfact,xstd,colsumsx,hasXtX,XtX,ytX,method,adj.overdisp,hesstype,optimMethod,optim_maxit,thinit,usethinit,B,alpha,lambda,phi,tau,taugroup,taualpha,fixatanhalpha,r,a,prDelta,prDeltap,parprDeltap,prConstr,prConstrp,parprConstrp,groups,ngroups,nvaringroup,constraints,invconstraints,maxvars,Dmat,as.integer(verbose))
      postMode <- ans[[1]]; postModeProb <- ans[[2]]
      if (familyint==0) { postMode <- as.integer(c(postMode,0,0)); postModeProb <- as.double(postModeProb - 2*log(2)) }
      postMode[includevars==1] <- TRUE
      ndeltaini <- as.integer(sum(postMode)); deltaini <- as.integer(which(as.logical(postMode))-1)
    } else if (initSearch=='SCAD') {
      if (verbose) message("Initializing via SCAD cross-validation...")
      deltaini <- rep(TRUE,ncol(xstd))
      cvscad <- ncvreg::cv.ncvreg(X=xstd[,!ct],y=ystd-mean(ystd),family="gaussian",penalty="SCAD",nfolds=10,dfmax=1000,max.iter=10^4)
      deltaini[!ct] <- ncvreg::ncvreg(X=xstd[,!ct],y=ystd-mean(ystd),penalty='SCAD',dfmax=1000,lambda=rep(cvscad$lambda[cvscad$cv],2))$beta[-1,1]!=0
      deltaini[includevars==1] <- TRUE
      ndeltaini <- as.integer(sum(deltaini)); deltaini <- as.integer(which(deltaini)-1)
      if (verbose) message(" Done\n")
    }

    #Run MCMC
    ans <- modelSelectionGibbsCI(postMode,postModeProb,knownphi,familyint,prior,priorgr,niter,thinning,burnin,ndeltaini,deltaini,includevars,n,p,ystd,uncens,sumy2,sumy,sumlogyfact,as.double(xstd),colsumsx,hasXtX,XtX,ytX,method,adj.overdisp,hesstype,optimMethod,optim_maxit,thinit,usethinit,B,alpha,lambda,phi,tau,taugroup,taualpha,fixatanhalpha,r,a,prDelta,prDeltap,parprDeltap,prConstr,prConstrp,parprConstrp,groups,ngroups,nvaringroup,constraints,invconstraints,maxvars,Dmat,as.integer(verbose))
    postSample <- matrix(ans[[1]],ncol=ifelse(familyint!=0,p,p+2))
    margpp <- ans[[2]]; postMode <- ans[[3]]; postModeProb <- ans[[4]]; postProb <- ans[[5]]
    margpp[includevars==1]= 1
    postmean= postvar= NULL
    modelid= apply(postSample[,1:ncol(xstd),drop=FALSE]==1, 1, function(z) paste(which(z),collapse=','))

  } else {

    #Model enumeration
    if (verbose) message("Enumerating models...\n")
    nincludevars= sum(includevars)
    nvars= ifelse(familyint==0,ncol(xstd)+2-nincludevars,ncol(xstd)-nincludevars)
    if (familyint==0) { includeenum= c(includevars[groups+1],FALSE,FALSE) } else { includeenum= includevars[groups+1] }
    if (missing(models)) {
      models= listmodels(vars2list=1:ngroups, includevars=includeenum, constraints=sapply(constraints,function(z) z+1), nvaringroup=nvaringroup, maxvars=maxvars) #listmodels expects group indexes 1,2,...
    } else {
      if (!is.logical(models)) stop("models must be a logical matrix")
      if (ncol(models) != ncol(xstd)) stop(paste("models has",ncol(models),"but x has",ncol(xstd),"columns"))
    }
    if (familyint==0) models= rbind(cbind(models,FALSE,FALSE),cbind(models,FALSE,TRUE),cbind(models,TRUE,FALSE),cbind(models,TRUE,TRUE))
    nmodels= as.integer(nrow(models))
    models= as.integer(models)
    includevars= as.integer(includevars)
    ans= modelSelectionEnumCI(nmodels,models,knownphi,familyint,prior,priorgr,n,p,ystd,uncens,sumy2,sumy,sumlogyfact,as.double(xstd),colsumsx,hasXtX,XtX,ytX,method,adj.overdisp,hesstype,optimMethod,optim_maxit,thinit,usethinit,B,alpha,lambda,phi,tau,taugroup,taualpha,fixatanhalpha,r,a,prDelta,prDeltap,parprDeltap,prConstr,prConstrp,parprConstrp,groups,ngroups,nvaringroup,constraints,invconstraints,Dmat,as.integer(verbose))
    postMode <- ans[[1]]; postModeProb <- ans[[2]]; postProb <- ans[[3]]
    postSample <- matrix(nrow=0,ncol=ifelse(familyint!=0,p,p+2))
    models <- matrix(models,nrow=nmodels)
    pp <- exp(postProb-postModeProb); pp <- matrix(pp/sum(pp),ncol=1)
    margpp <- as.vector(t(models) %*% pp)
    modelid= apply(models[,1:ncol(xstd),drop=FALSE]==1, 1, function(z) paste(which(z),collapse=','))
    if (familyint==0) {
        modelfam= models[,ncol(xstd)+1] + 2*models[,ncol(xstd)+2]
        margpp= c(margpp[1:ncol(xstd)],sum(pp[modelfam==0]),sum(pp[modelfam==1]),sum(pp[modelfam==2]),sum(pp[modelfam==3]))
        modeltxt= ifelse(modelfam==0,'normal',ifelse(modelfam==1,'twopiecenormal',ifelse(modelfam==2,'laplace','twopiecelaplace')))
        models= data.frame(modelid=modelid,family=modeltxt,pp=pp)
        postmean= postvar= NULL
    } else {
        models= data.frame(modelid=modelid,family=family,pp=pp)
        postmean= postvar= NULL
    }
    modelid= models$modelid
    models= models[order(models$pp,decreasing=TRUE),]
  }

  #Post-process output
  if (familyint!=0) {
    colnames(postSample) <- names(postMode) <- names(margpp) <- nn
  } else {
    colnames(postSample) <- names(postMode)<- c(nn,'asymmetry','laplace')
    names(margpp) <- c(nn,'family.normal','family.tpnormal','family.laplace','family.tplaplace')
  }

  priors= list(priorCoef=priorCoef, priorGroup=priorGroup, priorModel=priorModel, priorConstraints=priorConstraints, priorVar=priorVar, priorSkew=priorSkew)
  if (length(uncens)>0) { ystd[ordery]= ystd; uncens[ordery]= uncens; ystd= survival::Surv(time=ystd, event= uncens); xstd[ordery,]= xstd }
  names(constraints)= paste('group',0:(length(constraints)-1))

  if (priorCoef@priorDistr == 'icarplus') {
      priorprec= Matrix::Matrix(priorCoef@priorPars['a'] * Dmat + (1-priorCoef@priorPars['a']) * diag(ncol(xstd)), sparse=TRUE)
  } else {
      priorprec= NA
  }
    
  ans <- list(postSample=postSample,margpp=margpp,postMode=postMode,postModeProb=postModeProb,postProb=postProb,modelid=modelid,postmean=postmean,postvar=postvar,family=family,p=ncol(xstd),enumerate=enumerate,priors=priors,priorprec=priorprec,ystd=ystd,xstd=xstd,groups=groups,constraints=constraints,stdconstants=stdconstants,outcometype=outcometype,call=call)
  if (enumerate) { ans$models= models }
  new("msfit",ans)
}


# set default maximum number of variables
set_ms_maxvars = function(n, p, priorCoef, family, includevars) {
  if ((p <= n) && (priorCoef@priorDistr != 'bic')) {
      maxvars= p
  } else if ((p > n) && (priorCoef@priorDistr != 'bic')) {
      maxvars= n
  } else if ((p <= n) && (priorCoef@priorDistr == 'bic')) {  #for BIC, set smaller maxvars to avoid zero error variance
      maxvars= min(p, 0.5*n)
  } else if ((p > n) && (priorCoef@priorDistr == 'bic')) {   #for BIC, set smaller maxvars to avoid zero error variance
      maxvars= 0.5 * n
  }
  if (maxvars < sum(includevars)) maxvars= sum(includevars)
  if (family == 'auto') maxvars= maxvars + 2
  return(as.integer(maxvars))
}

# format input data from either formula (y), formula and data.frame (y,data) or matrix and vector (y, x)
# it accepts smoothterms, groups and survival data
formatInputdata <- function(y, x, data, smoothterms, nknots, family) {
  valid_families <- c('normal','twopiecenormal','laplace','twopiecelaplace','auto','binomial','binomial logit','poisson','poisson log')
  if (!(family %in% valid_families)) stop(paste("Invalid family. Valid values are", valid_families))
  call <- match.call()
  groups <- NULL; constraints <- NULL; ordery <- NULL
  if (inherits(y, "formula")) {
      if (missing(data)) stop("When y is a formula, you must supply the argument 'data'")
      formula= y; is_formula=TRUE; splineDegree= 3
      des= createDesign(y, data=data, smoothterms=smoothterms, splineDegree=splineDegree, nknots=nknots)
      x= des$x; groups= des$groups; constraints= des$constraints; typeofvar= des$typeofvar
      if ('Surv' %in% class(des$y)) {
          if (all(des$y[,1] >0)) {
              message("Response type is survival and all its values are >0. Remember that you should log-transform the response prior to running modelSelection\n")
          }
          outcometype= 'Survival'; uncens= as.integer(des$y[,2]); y= des$y[,1]
          ordery= c(which(uncens==1),which(uncens!=1)); y= y[ordery]; x= x[ordery,,drop=FALSE]; uncens= uncens[ordery]
          if (family !="normal") stop("For survival outcomes only family='normal' is currently implemented")
      } else {
          if (family %in% c('normal','twopiecenormal','laplace','twopiecelaplace','auto')) {
            outcometype= 'Continuous'
          } else {
            outcometype= 'glm'
          }
          y= des$y; uncens= integer(0)
      }
      nlevels <- apply(x,2,function(z) length(unique(z)))
      typeofvar[nlevels==2]= 'factor'
  } else {
      if ('Surv' %in% class(y)) {
          outcometype= 'Survival'; uncens= as.integer(y[,2]); y= y[,1]
          ordery= c(which(uncens==1),which(uncens!=1)); y= y[ordery]; x= x[ordery,,drop=FALSE]; uncens= uncens[ordery]
          if (family !="normal") stop("For survival outcomes only family='normal' is currently implemented")
      } else {
        if (family %in% c('normal','twopiecenormal','laplace','twopiecelaplace','auto')) {
          outcometype= 'Continuous'
        } else {
          outcometype= 'glm'
        }
        uncens= integer(0)
      }
      formula= splineDegree= NA; is_formula=FALSE; typeofvar= rep('numeric',ncol(x))
      if (!is.matrix(x)) x <- as.matrix(x)
  }
  if (nrow(x)!=length(y)) stop('nrow(x) must be equal to length(y)')
  if (any(is.na(y))) stop('y contains NAs, this is currently not supported, please remove the NAs')
  hasgroups <-  (length(groups) > length(unique(groups)))
  ningroup <- table(groups)
  isgroup <- (groups %in% as.numeric(names(ningroup)[ningroup>1]))
  y <- as.double(y)
  #Check that support of y is valid for the specified family
  if (family %in% c('binomial','binomial logit')) {
      if (any(!(y %in% c(0,1)))) stop("Invalid value for the response. For logistic regression it must be 0 or 1")
  } else if (family %in% c('poisson','poisson logit')) {
      if (any(y < 0) || any((y %% 1) != 0)) stop("Invalid value for the response. For Poisson regression it must be a natural number")
  }
  if (any(is.na(y)) | any(is.infinite(y))) stop("y contains NAs or infinite values")
  if (any(is.na(x)) | any(is.infinite(x))) stop("x contains NAs or infinite values")
  ans <- list(
    x=x, y=y, formula=formula, is_formula=is_formula, splineDegree=splineDegree,
    groups=groups, hasgroups=hasgroups, isgroup=isgroup, constraints=constraints, outcometype=outcometype, uncens=uncens,
    ordery=ordery, typeofvar=typeofvar
  )
  return(ans)
}



#Check that all variables appearing in myformula are in data
check_formula_variables <- function(myformula, data) {
  formula_vars <- all.vars(myformula)
  data_vars <- names(data)
  has_dot <- "." %in% formula_vars   # Check if formula contains a dot
  if (has_dot) {
    # Remove dot from variables to check
    formula_vars <- setdiff(formula_vars, ".")
    # For dot formulas, only check that explicitly mentioned variables exist
    missing_vars <- setdiff(formula_vars, data_vars)
    if (length(missing_vars) > 0) {
      stop("Some variable in the regression formula are not in the argument 'data'")
    }
    # Dot means "all other variables", so we're done
    return(TRUE)
  } else {
    # Regular formula - check all variables
    missing_vars <- setdiff(formula_vars, data_vars)
    if (length(missing_vars) > 0) {
      stop("Some variable in the regression formula are not in the argument 'data'")
    }
    return(TRUE)
  }
}



#Create a design matrix for the given formula. Return also covariate groups (e.g. from factors), covariate type (factor/numeric) and hierarchical constraints (e.g. from interaction terms), these are the parameters "groups" and "constraints" in modelSelection
createDesign <- function(formula, data, smoothterms, subset, na.action, splineDegree=3, nknots=14) {
    check_formula_variables(formula, data) #check that all variable names specified in formula are in data
    call <- match.call()
    if (missing(data)) data <- environment(formula)
    mf <- match.call(expand.dots = FALSE)
    if (!missing(subset)) {
        m <- match(c("formula", "data", "subset"), names(mf), 0L)
    } else {
        m <- match(c("formula", "data"), names(mf), 0L)
    }
    mf <- mf[c(1L, m)]
    mf$na.action = quote(na.pass)
    mf$drop.unused.levels <- TRUE
    mf[[1L]] <- quote(stats::model.frame)
    mt <- if (missing(data)) terms(formula) else terms(formula, data = data)
    mf$formula <- mt
    mf <- eval(mf, parent.frame())
    if (any(is.na(mf))) stop('Either the outcome or the covariates contain NAs. Please remove the NAs.')
    if (missing(na.action)) {
        naa = getOption("na.action", "na.fail")
        na.action = get(naa)
    }
    mf = na.action(mf)
    mt = attributes(mf)[["terms"]]  #mt is an object of class "terms" storing info about the model, see help(terms.object) for a description
    y <- model.response(mf, "any")
    x <- if (!is.empty.model(mt)) model.matrix(mt, mf) else matrix(, NROW(y), 0)
    groups <- attr(x,"assign") #group that each variable belongs to, e.g. for factors
    tab= table(groups);
    typeofvar= ifelse(groups %in% as.numeric(names(tab)[tab>1]),'factor','numeric')
    intercept= ifelse(min(groups)==0,1,0)
    if (length(attr(mt,"factors")) > 0) {
        groups2vars= attr(mt,"factors")[-1,] #for each variable group, hierarchical dependence on other groups
        nn= colnames(groups2vars)[!(colnames(groups2vars) %in% rownames(groups2vars))]
    } else {  #the model only contains an intercept
        nn= character(0)
    }
    if (length(nn)>0) { #there are interaction terms
        tmp= matrix(0,nrow=length(nn),ncol=ncol(groups2vars))
        rownames(tmp)= nn
        colnames(tmp)= colnames(groups2vars)
        for (i in 1:nrow(tmp)) {
            nni= paste(strsplit(nn[i],split=":")[[1]], collapse=":.*") #regular expression, e.g. if nn[i]="Xj:Xl" it checks for "Xj:.*Xl"
            tmp[i,grep(nni,colnames(tmp))]= 1
        }
        groups2vars= rbind(groups2vars,tmp)
        constraints= lapply(1:ncol(groups2vars), function(i) { ans= as.integer(which(groups2vars[,i]>0)); ans[ans!=i] + intercept })
        if (intercept==1) constraints= c(list(integer(0)),constraints)
    } else { #there are no interactions
        constraints= lapply(1:(max(groups)+intercept), function(i) integer(0))
    }
    groups= groups+intercept
    #Add spline terms
    if (!missing(smoothterms)) {
        if (!any(c('formula','matrix','data.frame') %in% class(smoothterms))) stop("smoothterms should be of class 'formula', 'matrix' or 'data.frame'")
        Snested= nestedSplines(x=x, groups=groups, smoothterms=smoothterms, data=data, subset=subset, na.action=na.action, splineDegree=splineDegree, nknots=nknots)
        x= cbind(x, Snested$L, Snested$W)
        groups= c(groups, Snested$groups, Snested$groupsW)
        typeofvar= c(typeofvar, Snested$typeofvar)
        constraints= c(constraints, Snested$constraintsL, Snested$constraintsW)
        #old code, before structuring nestedSplines as a separate function
        #x= cbind(x,L[,selL],W)
        #groups= c(groups, groupsL[selL], groupsW)
        #typeofvar= c(typeofvar, rep('numeric',sum(selL)+length(groupsW)))
        #constraints= c(constraints, constraintsL[selL], constraintsW)
    }
    return(list(y=y,x=x,groups=groups,constraints=constraints,typeofvar=typeofvar))
}


#Create design matrix for nested splines: linear within knots[1], knots[1] within knots[2], etc.
nestedSplines= function(x, groups, smoothterms, data, subset, na.action, splineDegree, nknots) {
    if (length(nknots)>1) nknots= nknots[order(nknots)]
    maxgroups= max(groups)
    if ('formula' %in% class(smoothterms)) {
        smoothterms= formula(paste("~ ",-1,"+",as.character(smoothterms)[2])) #remove intercept
        L= createDesign(smoothterms, data=data, subset=subset, na.action=na.action)$x
    } else {
        L= as.matrix(smoothterms)
        if (is.null(colnames(L))) colnames(L)= paste("L",1:ncol(L),sep="")
    }

    W= constraintsW= constraintsL= groupsW= groupsL= vector("list", length(nknots))
    for (kk in 1:length(nknots)) {
        W[[kk]] <- matrix(NA,nrow=nrow(L),ncol=(nknots[kk]-4)*ncol(L))
        namesW <- character(ncol(W[[kk]]))
        groupsW[[kk]] <- integer(ncol(W[[kk]])); constraintsW[[kk]]= vector("list",ncol(L))
        groupsL[[kk]] <- integer(ncol(L)); constraintsL[[kk]]= lapply(1:ncol(L), function(i) integer(0))
        selL= rep(FALSE,ncol(L)); nselL= 0
        for (j in 1:ncol(L)) {
            m= range(L[,j])
            tmp= bspline(L[,j], degree=splineDegree, knots=seq(m[1],m[2],length=nknots[kk])) #equally-spaced knots
            Lj= cbind(1,L[,j]); b= solve(t(Lj) %*% Lj) %*% (t(Lj) %*% tmp)
            idx= (1+(j-1)*(nknots[kk]-4)):(j*(nknots[kk]-4))
            W[[kk]][,idx]= tmp - Lj %*% b #project splines onto space orthogonal to linear term
            namesW[idx]= paste(colnames(L)[j],'.s',1:length(idx),sep='')
            repeated= which(colnames(x)==colnames(L)[j])
            if (length(repeated)==1) {  #linear term was already in x
                constraintsW[[kk]][[j]]= groups[repeated]
            } else {                    #linear term wasn't in x
                selL[j]= TRUE
                nselL= nselL+1
                groupsL[[kk]][j]= maxgroups + nselL
                constraintsW[[kk]][[j]]= groupsL[[kk]][j]
            }
            groupsW[[kk]][idx]= j
        }
        colnames(W[[kk]])= namesW
        groupsW[[kk]]= maxgroups + nselL + groupsW[[kk]]
        varW= (colMeans(W[[kk]]^2) - colMeans(W[[kk]])^2) * nrow(W[[kk]])/(nrow(W[[kk]])-1)
        if (any(varW<1.0e-4)) {
            W[[kk]]= W[[kk]][,varW>1.0e-4]; groupsW[[kk]]= groupsW[[kk]][varW>1.0e-4]; constraintsW[[kk]]= constraintsW[[kk]][unique(groupsW[[kk]]-min(groupsW[[kk]])+1)]
        }
    }

    #Combine design matrices hierarchically into single matrix with hierarchical constraints
    Wall= W[[1]]; groupsWall= groupsW[[1]]; constraintsWall= constraintsW[[1]]
    maxgroups= max(groupsWall)
    if (length(nknots)==1) {
        ans= list(L=L[,selL], W=Wall, groupsL=groupsL[selL], groupsW=groupsWall, typeofvar= rep('numeric',sum(selL)+length(groupsW)), constraintsL=constraintsL[selL], constraintsW=constraintsWall)
    } else {
        groupsWidx= unique(groupsW[[1]])
        for (kk in 2:length(nknots)) {
            keep= vector("list",length(groupsWidx))
            for (g in 1:length(keep)) {
                sel1= which(groupsW[[kk]] == groupsWidx[g])
                sel2= which(groupsWall == groupsWidx[g])
                r= cor(W[[kk]][,sel1], Wall[,sel2])  #correlation with basis with previous number of knots
                o= order(abs(apply(r, 1, max)))      #sort by max absolute correlation
                keep[[g]]= sel1[o[1:(length(sel1) - sum(groupsW[[kk-1]] == groupsWidx[g]))]]
                keep[[g]]= keep[[g]][order(keep[[g]])]
            }
            newgroups= groupsW[[kk]][unlist(keep)]; newgroups= newgroups + maxgroups - min(newgroups) + 1
            maxgroups= max(newgroups)
            newconstraintsW= as.list(unique(newgroups) - length(groupsWidx))
            Wall= cbind(Wall, W[[kk]][,unlist(keep)])
            groupsWall= c(groupsWall, newgroups)
            constraintsWall= c(constraintsWall, newconstraintsW)
        }
        for (j in 1:ncol(L)) {
            pattern= sub("\\[","\\\\[",colnames(L)[j]); pattern= sub("\\]","\\\\]",pattern)
            selj= grep(pattern, colnames(Wall))
            colnames(Wall)[selj]= paste(colnames(L)[j],'.s',1:length(selj), sep='')
        }
        ans= list(L=L[,selL], W=Wall, groupsL=groupsL[selL], groupsW=groupsWall, typeofvar= rep('numeric',sum(selL)+length(groupsWall)), constraintsL=constraintsL[selL], constraintsW=constraintsWall)
    }

    return(ans)
}





#Count variables in each group, indicate 1st variable in each group, convert group and constraint labels to integers 1,2,...
codeGroupsAndConstraints= function(p,groups,constraints) {
    groupsnum= as.numeric(factor(groups)); groupsnum= cumsum(c(TRUE,groupsnum[-1]!=groupsnum[-length(groupsnum)])) #re-code groups (in order of appearance)
    ngroups= max(groupsnum)
    if (ngroups>p) stop("There cannot be more groups than variables (columns in x)")
    if (missing(constraints)) {
        constraints= sapply(1:ngroups, function(i) integer(0))
    } else {
        if (length(constraints) != ngroups) stop("length(constraints) must be equal to number of variable groups")
        #Ensure that constraints match the group order in groupsnum
        g2code= sapply(names(constraints), function(nn) groupsnum[match(TRUE,groups==nn)])
        if (any(names(constraints) != as.character(g2code))) {
            constraints= constraints[g2code]
            names(constraints)= g2code
            for (i in 1:length(constraints)) { if (length(constraints[[i]]>0)) constraints[[i]]= as.integer(g2code[constraints[[i]]]) -as.integer(1) } #group codes start at 0
        } else {
            constraints= lapply(constraints,function(z) as.integer(z)-as.integer(1)) #group codes start at 0
        }
    }
    if (ngroups==p) {
        nvaringroup= as.integer(rep(1,p))
        groups= as.integer(0:(p-1))
    } else {
        nvaringroup= as.integer(table(groupsnum)) #number of variables in each group
        groups= as.integer(groupsnum-1) #group id that each variable belongs to
        #groups= as.integer(c(0,as.numeric(cumsum(nvaringroup[-length(nvaringroup)])))) #1st variable in each group (0-indexed)
    }
    #Determine inverse constraints
    invconstraints= vector("list",ngroups)
    tabconstr= cbind(group=rep(0:(ngroups-1), sapply(constraints,length)), requires= unlist(constraints))
    for (i in 1:ngroups) { invconstraints[[i]]= as.integer(tabconstr[tabconstr[,'requires']==(i-1), 'group']) }
    ans= list(ngroups=ngroups,constraints=constraints,invconstraints=invconstraints,nvaringroup=nvaringroup,groups=groups)
    return(ans)
}


#Routine to enumerate all models satisfying hierarchical constraints, e.g. x[i] can only be in model if x[j] and x[k] are in model
#
# - vars2list: vector indicating variable groups, e.g. 1:10 means there's 10 variable groups named 1-10
# - includevars: logical vector of length= length(vars2list). TRUE indicates that all models should include that variable group
# - constraints: list with length= length(vars2list). Each element indicates hierarchical restrictions. Restrictions must be given in order, i.e. a restriction on group i can only depend on groups < i
# - nvaringroup: number of variables in each group
# - fixedvars: for internal use only. When calling the function recursively, fixedvars are the variables that are currently included in the model
#
# OUTPUT: matrix with models in rows and variables in columns. If the (i,j) entry is TRUE, model i includes variable j
#
# EXAMPLE 1: list models under restriction that x3 included only when (x1,x2) also included
#
# listmodels(vars2list=1:3, constraints=list(integer(0),integer(0),c(1,2)))
#
# EXAMPLE 2: same but forcing inclusion of x2
#
# listmodels(vars2list=1:3, includevars= c(FALSE,TRUE,FALSE), constraints=list(integer(0),integer(0),c(1,2)))
#
# EXAMPLE 3: list models under restriction that group 3 included only when (group 1, group 2) also included
#
# listmodels(vars2list=1:3, constraints=list(integer(0),integer(0),c(1,2)), nvaringroup= c(1,3,2))
#
# EXAMPLE 4: list models under restrictions x4 requires (x1,x2), x5 requires (x1,x3), x6 requires (x2,x3), x7 requires (x1,...,x6)
#
# listmodels(vars2list=1:7, constraints=list(integer(0),integer(0),integer(0),c(1,2),c(1,3),c(2,3),1:6))

listmodels= function(vars2list, includevars=rep(FALSE,length(vars2list)), fixedvars=integer(0), constraints, nvaringroup=rep(1,length(vars2list)), maxvars) {
    var1= vars2list[1]
    if (includevars[var1]) { #forcing inclusion of this variable group
        if (length(vars2list)>1) {
            ans= listmodels(vars2list[-1], includevars=includevars, fixedvars=c(fixedvars,var1), constraints=constraints, nvaringroup=nvaringroup, maxvars=maxvars)
        } else {
            fixedLogical= rep(FALSE,length(constraints)-1)
            fixedLogical[fixedvars]= TRUE
            fixedLogical= rep(fixedLogical,nvaringroup[-length(nvaringroup)])
            ans= c(fixedLogical,rep(TRUE,nvaringroup[length(nvaringroup)]))
        }
    } else { #not forcing inclusion of this variable group
        nfixedvars= length(fixedvars)
        if (length(constraints[[var1]])>0) {
            var1constraint= all(constraints[[var1]] %in% fixedvars) #is constraint on var1 satisfied by fixedvars?
        } else { var1constraint= TRUE }
        if (length(vars2list)>1) { #if this is not the last variable, call function recursively
            ans1= listmodels(vars2list[-1], includevars=includevars, fixedvars=fixedvars, constraints=constraints, nvaringroup=nvaringroup, maxvars=maxvars)
            if (var1constraint & (nfixedvars<maxvars)) {
                ans2= listmodels(vars2list[-1], includevars=includevars, fixedvars=c(fixedvars,var1), constraints=constraints, nvaringroup=nvaringroup, maxvars=maxvars)
            } else { ans2= NULL }
            ans= rbind(ans1,ans2)
        } else {  #if this is the last variable, return entire variable inclusion vector
            fixedLogical= rep(FALSE,length(constraints)-1)
            fixedLogical[fixedvars]= TRUE
            fixedLogical= rep(fixedLogical,nvaringroup[-length(nvaringroup)])
            if (var1constraint & (nfixedvars<maxvars)) {
                ans= rbind(c(fixedLogical,rep(FALSE,nvaringroup[length(nvaringroup)])),c(fixedLogical,rep(TRUE,nvaringroup[length(nvaringroup)])))
            } else {
                ans= c(fixedLogical, rep(FALSE,nvaringroup[length(nvaringroup)]))
            }
        }
    }
    if (is.vector(ans)) ans= matrix(ans, nrow=1)
    return(ans)
}


#Routine to format method indicating how integrated likelihoods should be computed in modelSelection
#
#For any method other than 'auto', it sets a numeric code corresponding to that method to pass onto C.
#for method=='auto', the integration method is decided automatically according to the following rules
#
# 1. Continuous outcomes
# - if family normal, no groups: for pMOM exact / ALA; for other NLPs & LPs: Laplace
# - if family normal, groups:  for pMOMgMOM and pMOMgZell ALA; for other NLPs & LPs: Laplace
# - if family != normal: Laplace approximation (since ALA not currently implemented)
#
# 2. Survival outcomes. Use ALA when available, LA otherwise. Currently ALA available for pmom/groupMOM/groupzellner + pmom/groupMOM/groupzellner
#
# 3. GLMs. Laplace approximation
#
# Note: usethinit==3 indicates to use the initial parameter values in LA and ALA.
#
# OUTPUT
#
#   method
#   0 means Laplace approximation (note: for normal outcomes + normal priors this means the calculation is exact)
#   1 means Monte Carlo (Importance Sampling)
#   2 means ALA (approximate Laplace approximation)
#   -1 only available for pMOM, it means to use exact calculation for small models (<=3 parameters) and ALA for larger models
#  
#  optimMethod: 1 means Newton-Raphson type algorithm, 2 means Coordinate Descent Algorith, 0 means use the default. This argument is used in twopiecenormal models and GLMs, else it's ignored
#
#  optim_maxit: maximum number of iterations for optimization method. If not specified on input, a -1 is returned (so the C code uses a default maxit)
#
#  hesstype: what type of hessian to use in twopiecelaplace models, where the observed hessian is not defined
#
#  adj.overdisp
#  0: no over-dispersion adjustment
#  1: estimate over-dispersion from intercept-only model
#  2: estimate over-dispersion from Pearson residuals under each model
formatmsMethod= function(method, usethinit, initpar, optimMethod, optim_maxit=optim_maxit, priorCoef, priorGroup, knownphi, outcometype, family, hasgroups, adj.overdisp, hess) {
  hesstype <- as.integer(ifelse(hess=='asympDiagAdj',2,1))

  #Obtain code for the optimization method
  if (missing(optimMethod)) optimMethod <- 'auto'
  if (outcometype == 'glm') {
    if (optimMethod=='auto') {
        optimMethod <- as.integer(0)
    } else if (optimMethod %in% c('Newton','LMA')) {
        optimMethod <- as.integer(1)
    } else if (optimMethod == 'CDA') {
        optimMethod <- as.integer(2)
    } else { stop("Invalid optimMethod. For this family, only 'auto', 'Newton', 'LMA' or 'CDA' are implemented") }
  } else {
    if (optimMethod %in% c('Newton','LMA')) {
        optimMethod <- as.integer(1)
    } else {
        optimMethod <- as.integer(2)
    }
  }

  #Obtain code to be passed to C for the method to compute the integrated likelihood
  if ((method=='ALA') && (usethinit==3)) {
      method= 'Laplace'  #ALA init at theta != 0 is implemented in C as Laplace with limited optim_maxit
      if (missing(optim_maxit)) { optim_maxit= as.integer(0) } else { optim_maxit= as.integer(optim_maxit) } #maxit>0 tells C to use specified maxit
  } else {
      if (missing(optim_maxit)) { optim_maxit= as.integer(-1) } else { optim_maxit= as.integer(optim_maxit) } #maxit= -1 tells C to use its own defaults
  }
  
  if (method=='Laplace') {
    method <- as.integer(0)
  } else if (method=='MC') {
    method <- as.integer(1)
  } else if (method=='Hybrid') {
    if ((priorCoef@priorDistr!='piMOM') | (knownphi==1)) {
      warning("method=='Hybrid' is only available for 'piMOM' priors with unknown phi. Using method=='Laplace' instead")
      method <- as.integer(0)
    } else {
      method <- as.integer(2)
    }
  } else if (method=='ALA') {
      method <- as.integer(2)
  } else if (method=='auto') {
    if (outcometype=='Continuous') {
      if (family=='normal') {
          if (!hasgroups) {
            if (priorCoef@priorDistr=='pMOM') { method <- as.integer(-1) } else { method <- as.integer(0) }
          } else {
            if ((priorCoef@priorDistr=='pMOM') & (priorGroup@priorDistr %in% c('groupMOM','zellner','groupzellner'))) {
                method <- as.integer(2)
            } else { method <- as.integer(0) }
          }
      } else {
          method <- as.integer(0)
      }
    } else if (outcometype=='Survival') {
      if (family=='normal') {
        if ((priorCoef@priorDistr %in% c('pMOM','groupMOM','groupzellner')) & (priorGroup@priorDistr %in% c('pMOM','groupMOM','groupzellner'))) {
           method <- as.integer(2)
        } else {
           method <- as.integer(0)
        }
      } else { stop("For survival outcomes, only family=='normal' currently implemented") }
    } else if (outcometype=='glm') {
        method <- as.integer(0)
    } else { stop("outcometype must be 'Continuous', 'Survival' or 'glm'") }
  } else if ((method=='ALA') | (method=='plugin')) {
    method <- as.integer(2)
  } else {
    stop("Invalid 'method'")
  }

  adj.overdisp <- as.integer(ifelse(adj.overdisp=='none',0,ifelse(adj.overdisp=='intercept',1,2)))
  ans <- list(method=method, optimMethod=optimMethod, optim_maxit=optim_maxit, adj.overdisp=adj.overdisp, hesstype= hesstype)
  return(ans)
}

#Routine to format modelSelection prior distribution parameters for marginal likelihood
#Input: priorCoef, priorVar, priorGroup, priorSkew
#Output: parameters for prior on coefficients (r, prior, tau), prior on variance parameter (alpha, lambda), skewness parameter (taualpha, fixatanhalpha)
formatmsPriorsMarg <- function(priorCoef, priorGroup, priorVar, priorSkew, n) {
  r= as.integer(1)
  a= as.double(0.5)

  if (priorCoef@priorDistr=='bic') {
      
    prior <- priorgr <- as.integer(100)
    tau <- as.double(priorCoef@priorPars['tau'])
    taugroup <- alpha <- lambda <- taualpha <- fixatanhalpha <- as.double(-1)
      
  } else {
  
    if (missing(priorGroup)) priorGroup= priorCoef
    if (missing(priorSkew)) priorSkew= priorCoef
    has_taustd <- "taustd" %in% names(priorCoef@priorPars)
    has_taugroupstd <- "taustd" %in% names(priorGroup@priorPars)
    if (has_taustd) {
      taustd <- as.double(priorCoef@priorPars['taustd'])
    } else {
      tau <- as.double(priorCoef@priorPars['tau'])
    }
    if (has_taugroupstd) {
      taugroupstd <- as.double(priorGroup@priorPars['taustd'])
    } else {
      taugroup <- as.double(priorGroup@priorPars['tau'])
    }
    if (priorCoef@priorDistr=='pMOM') {
      r <- as.integer(priorCoef@priorPars['r'])
      prior <- as.integer(0)
      if (has_taustd) tau <- taustd * 1/3
    } else if (priorCoef@priorDistr=='piMOM') {
      prior <- as.integer(1)
    } else if (priorCoef@priorDistr=='peMOM') {
      prior <- as.integer(2)
    } else if (priorCoef@priorDistr=='zellner') {
      prior <- as.integer(3)
      if (has_taustd) tau <- taustd * n
    } else if (priorCoef@priorDistr=='normalid') {
      prior <- as.integer(4)
      if (has_taustd) tau <- taustd
    } else if (priorCoef@priorDistr=='icarplus') {
      prior <- as.integer(5)
      if (has_taustd) tau <- taustd
      a <- as.double(priorCoef@priorPars['a'])
    } else if (priorCoef@priorDistr=='groupMOM') {
      prior <- as.integer(10)
      if (has_taustd) tau <- taustd
    } else if (priorCoef@priorDistr=='groupzellner') {
      prior <- as.integer(13)
      if (has_taustd) tau <- taustd * n
    } else {
      stop('Prior specified in priorDistr not recognized')
    }
    if (priorGroup@priorDistr=='pMOM') {
      priorgr= as.integer(0)
      if (has_taugroupstd) taugroup <- taugroupstd * 1/3
    } else if (priorGroup@priorDistr=='piMOM') {
      priorgr= as.integer(1)
    } else if (priorGroup@priorDistr=='peMOM') {
      priorgr= as.integer(2)
    } else if (priorGroup@priorDistr=='zellner') {
      priorgr= as.integer(3)
      if (has_taugroupstd) taugroup <- taugroupstd * n
    } else if (priorGroup@priorDistr=='normalid') {
      priorgr= as.integer(4)
      if (has_taugroupstd) taugroup <- taugroupstd
    } else if (priorGroup@priorDistr=='icarplus') {
      priorgr <- as.integer(5)
      if (has_taustd) taugroup <- taugroupstd
    } else if (priorGroup@priorDistr=='groupMOM') {
      priorgr= as.integer(10)
      if (has_taugroupstd) taugroup <- taugroupstd
    } else if (priorGroup@priorDistr=='groupiMOM') {
      priorgr= as.integer(11)
    } else if (priorGroup@priorDistr=='groupeMOM') {
      priorgr= as.integer(12)
    } else if (priorGroup@priorDistr=='groupzellner') {
      priorgr= as.integer(13)
      if (has_taugroupstd) taugroup <- taugroupstd * n
    } else {
      stop('Prior in priorGroup not recognized')
    }
    alpha <- as.double(priorVar@priorPars['alpha']); lambda <- as.double(priorVar@priorPars['lambda'])
    #
    if ('msPriorSpec' %in% class(priorSkew)) {
        taualpha <- as.double(priorSkew@priorPars['tau'])
        fixatanhalpha <- as.double(-10000)
    } else {
        taualpha <- 0.358
        fixatanhalpha <- as.double(priorSkew)
    }
    if (has_taustd) priorCoef@priorPars['tau']= tau
    if (has_taugroupstd) priorGroup@priorPars['tau']= taugroup
  }
  
  ans= list(r=r,prior=prior,priorgr=priorgr,tau=tau,taugroup=taugroup,alpha=alpha,lambda=lambda,taualpha=taualpha,fixatanhalpha=fixatanhalpha,a=a,priorCoef=priorCoef,priorGroup=priorGroup)
  return(ans)
}

defaultpriorConstraints <- function(priorModel, priorConstraints) {
  if (missing(priorConstraints)) {
    if ((priorModel@priorDistr=='binomial') && ('p' %in% names(priorModel@priorPars)) && (length(priorModel@priorPars[['p']]) > 1)) {
      priorConstraints <- modelbinomprior(p=0.5)
    } else {
      priorConstraints <- priorModel
    }
  }
  return(priorConstraints)
}

#Routine to format modelSelection prior distribution parameters in model space
#Input: priorModel, priorConstraints, constraints
#Output: model space prior (prDelta, prDeltap, parprDeltap) and constraints (prConstr,prConstrp,parprConstrp)
formatmsPriorsModel <- function(priorModel, priorConstraints, constraints) {
  #Prior on model space (parameters not subject to hierarchical constraints)
  n_unconstrained <- sum(sapply(constraints, function(x) length(x) == 0))
  n_constrained <- length(constraints) - n_unconstrained
  if (priorModel@priorDistr=='uniform') {
    prDelta <- as.integer(0)
    prDeltap <- as.double(0)
    parprDeltap <- double(2)
  } else if (priorModel@priorDistr=='binomial') {
    if ('p' %in% names(priorModel@priorPars)) {
      prDelta <- as.integer(1)
      prDeltap <- as.double(priorModel@priorPars[['p']])
      if (any(prDeltap<=0) | any(prDeltap>1)) stop("For the binomial model prior the inclusion probabilities p must lie in (0,1]")
      if ((length(prDeltap) != 1) & (length(prDeltap) != n_unconstrained)) stop("p in priorModel must be a scalar or have length=number of unconstrained variables")
      parprDeltap <- as.double(length(prDeltap))
    } else {
      prDelta <- as.integer(2)
      prDeltap <- as.double(.5)
      parprDeltap <- as.double(priorModel@priorPars[c('alpha.p','beta.p')])
    }
  } else if (priorModel@priorDistr=='complexity') {
      prDelta <- as.integer(3)
      prDeltap <- as.double(priorModel@priorPars['c'])
      if (prDeltap<0) stop("c must be >0 for priorModel@priorDistr=='complexity'")
      parprDeltap <- double(2)
  } else {
    stop('Prior specified in priorModel not recognized')
  }
  #Prior on model space (parameters subject to hierarchical constraints)
  if (priorConstraints@priorDistr=='uniform') {
    prConstr <- as.integer(0)
    prConstrp <- as.double(0)
    parprConstrp <- double(2)
  } else if (priorConstraints@priorDistr=='binomial') {
    if ('p' %in% names(priorConstraints@priorPars)) {
      prConstr <- as.integer(1)
      prConstrp <- as.double(priorConstraints@priorPars[['p']])
      if (any(prConstrp<=0) | any(prConstrp>=1)) stop("p must be between 0 and 1 for priorConstraints@priorDistr=='binomial'")
      if ((length(prConstrp) != 1) & (length(prConstrp) != n_constrained)) stop("p in priorConstraints must be a scalar or have length=number of constrained variables")
      parprConstrp <- as.double(length(prConstrp))
    } else {
      prConstr <- as.integer(2)
      prConstrp <- as.double(.5)
      parprConstrp <- as.double(priorConstraints@priorPars[c('alpha.p','beta.p')])
    }
  } else if (priorConstraints@priorDistr=='complexity') {
      prConstr <- as.integer(3)
      prConstrp <- as.double(priorConstraints@priorPars['c'])
      if (prConstrp<0) stop("c must be >0 for priorConstraints@priorDistr=='complexity'")
      parprConstrp <- double(2)
  } else {
    stop('Prior specified in priorConstraints not recognized')
  }

  ans= list(prDelta=prDelta,prDeltap=prDeltap,parprDeltap=parprDeltap,prConstr=prConstr,prConstrp=prConstrp,parprConstrp=parprConstrp)
  return(ans)
}


#Assign a numerical code to the family of likelihoods, to pass on to C
formatFamily= function(family, issurvival) {
    
    if (family=='auto') {
        familyint= 0; familygreedy=1
    } else if (family=='normal') {
        familyint= familygreedy= ifelse(!issurvival,1,11)
    } else if (family=='twopiecenormal') {
        familyint= 2; familygreedy=1
    } else if (family=='laplace') {
        familyint= 3; familygreedy=1
    } else if (family=='twopiecelaplace') {
        familyint= 4; familygreedy=1
    } else if (family %in% c('binomial','binomial logit')) {
        familyint= familygreedy= 21
    } else if (family %in% c('poisson','poisson log')) {
        familyint= familygreedy= 22
    } else stop("family not available")

    ans= list(familyint= as.integer(familyint), familygreedy= as.integer(familygreedy))
    return(ans)
      
}



greedymodelSelectionR <- function(y, x, niter=100, marginalFunction, priorFunction, betaBinPrior, deltaini=rep(FALSE,ncol(x)), verbose=TRUE, ...) {
  #Greedy version of modelSelectionR where variables with prob>0.5 at current iteration are included deterministically (prob<.5 excluded)
  p <- ncol(x)
  if (length(deltaini)!=p) stop('deltaini must be of length ncol(x)')
  if (!missing(betaBinPrior)) {
    #Initialize probBin
    if ((betaBinPrior['alpha.p']>1) && (betaBinPrior['beta.p']>1)) {
      probBin <- (betaBinPrior['alpha.p']-1)/(betaBinPrior['alpha.p']+betaBinPrior['beta.p']-2)
    } else {
      probBin <- (betaBinPrior['alpha.p'])/(betaBinPrior['alpha.p']+betaBinPrior['beta.p'])
    }
    postOther <- matrix(NA,nrow=niter,ncol=1); colnames(postOther) <- c('probBin')
    priorFunction <- function(sel, logscale=TRUE) dbinom(x=sum(sel),size=length(sel),prob=probBin,log=logscale)
  } else {
    postOther <- matrix(NA,nrow=niter,ncol=0)
  }
  #Greedy iterations
  sel <- deltaini
  mcur <- marginalFunction(y=y,x=x[,sel,drop=FALSE],logscale=TRUE,...) + priorFunction(sel,logscale=TRUE)
  nchanges <- 1; itcur <- 1
  nn <- names(x)
  while (nchanges>0 & itcur<niter) {
    nchanges <- 0; itcur <- itcur+1
    for (i in 1:ncol(x)) {
      selnew <- sel; selnew[i] <- !selnew[i]
      mnew <- marginalFunction(y=y,x=x[,selnew,drop=FALSE],logscale=TRUE,...) + priorFunction(selnew,logscale=TRUE)
      if (mnew>mcur) { sel[i]=selnew[i]; mcur=mnew; nchanges=nchanges+1; if (verbose) message(paste(ifelse(sel[i],"Added","Dropped"),nn[i],"\n",collapse=" ")) }
    }
  }
  return(sel)
}

#Gibbs model selection using BIC to approximate marginal likelihood
# - y, x, xadj: response, covariates under selection and adjustment covariates
# - family: glm family, passed on to glm
# - niter: number of Gibbs iteration
# - burnin: burn-in iterations
# - modelPrior: function evaluating model log-prior probability. Takes a logical vector as input
# Returns:
# - postModel, postCoef1, postCoef2, margpp (analogous to pmomPM. postCoef1 & postCoef2 are MLEs under each visited model)
modelselBIC <- function(y, x, xadj, family, niter=1000, burnin= round(.1*niter), modelPrior, verbose=TRUE) {
  pluginJoint <- function(sel) {
    p <- sum(sel)
    ans <- vector("list",2); names(ans) <- c('marginal','coef')
    if (p>0 & p<=length(y)) {
      glm1 <- glm(y ~ x[,sel,drop=FALSE] + xadj -1, family=family)
      ans$marginal <- -.5*glm1$deviance - .5*log(length(y))*(glm1$df.null-glm1$df.residual) + modelPrior(sel)
      ans$coef1 <- coef(glm1)[1:p]; ans$coef2 <- coef(glm1)[-1:-p]
    } else if (p==0) {
      glm1 <- glm(y ~ xadj -1, family=family)
      ans$marginal <- -.5*glm1$deviance - .5*log(length(y))*(glm1$df.null-glm1$df.residual) + modelPrior(sel)
      ans$coef1 <- double(0); ans$coef2 <- coef(glm1)
    } else { ans$marginal <- -Inf; ans$coef1 <- ans$coef2 <- 0}
    return(ans)
  }
  #Greedy iterations
  if (verbose) message("Initializing...")
  sel <- rep(FALSE,ncol(x))
  mcur <- pluginJoint(sel)$marginal
  nchanges <- 1; it <- 1
  while (nchanges>0 & it<100) {
    nchanges <- 0; it <- it+1
    for (i in 1:ncol(x)) {
      selnew <- sel; selnew[i] <- !selnew[i]
      mnew <- pluginJoint(selnew)$marginal
      if (mnew>mcur) { sel[i] <- selnew[i]; mcur <- mnew; nchanges <- nchanges+1 }
    }
  }
  if (verbose) { message(" Done\nGibbs sampling") }
  #Gibbs iterations
  niter10 <- ceiling(niter/10)
  postModel <- matrix(NA,nrow=niter,ncol=ncol(x))
  postCoef1 <- matrix(0,nrow=niter,ncol=ncol(x))
  postCoef2 <- matrix(0,nrow=niter,ncol=ncol(xadj))
  curmod <- pluginJoint(sel)
  for (j in 1:niter) {
    for (i in 1:ncol(x)) {
      selnew <- sel; selnew[i] <- !selnew[i]
      newmod <- pluginJoint(selnew)
      mnew <- newmod$marginal
      if (runif(1) < 1/(1+exp(mcur-mnew))) { sel[i] <- selnew[i]; mcur <- mnew; curmod <- newmod }
    }
    postModel[j,] <- sel
    postCoef1[j,sel] <- curmod$coef1; postCoef2[j,] <- curmod$coef2
    if (verbose & ((j%%niter10)==0)) message(".")
  }
  if (verbose) message("Done\n")
  #Return output
  if (burnin>0) { postModel <- postModel[-1:-burnin,,drop=FALSE]; postCoef1 <- postCoef1[-1:-burnin,,drop=FALSE]; postCoef2 <- postCoef2[-1:-burnin,,drop=FALSE] }
  ans <- list(postModel=postModel, postCoef1=postCoef1, postCoef2=postCoef2, margpp=colMeans(postModel))
}


## Common prior distributions on model space

nselConstraints= function(sel, groups, constraints) {
    #Output
    # - ngroups0: number of unconstrained groups that are selected
    # - ngroups1: number of constrained groups that are selected
    # - ngroups: total number of groups (selected or unselected)
    # - ngroupsconstr: total number of groups that have a constraint
    # - violateConstraint: TRUE if sel violates a group constraint, FALSE otherwise
    ngroups= length(unique(groups))
    if (length(constraints) != ngroups) stop("length(constraints) must be equal to length(unique(groups))")
    if (!is.numeric(groups)) stop("Argument groups must be numeric")
    nconstraints= sapply(constraints,length)
    hasconstraint= (nconstraints>0)
    ngroupsconstr= sum(hasconstraint)
    tab= table(factor(sel,levels=c('FALSE','TRUE')),groups)
    selgroup= tab['TRUE',] >0
    violateConstraint= FALSE
    if (ngroupsconstr>0) { for (i in which(hasconstraint)) { if (selgroup[i] & any(!selgroup[constraints[[i]]])) violateConstraint= TRUE } }
    ngroups0= sum(selgroup[!hasconstraint]); ngroups1= sum(selgroup[hasconstraint])
    return(list(ngroups0=ngroups0, ngroups1=ngroups1, ngroups=ngroups, ngroupsconstr=ngroupsconstr, violateConstraint=violateConstraint, hasconstraint=hasconstraint))
}

#binomPrior <- function(sel, prob=.5, logscale=TRUE) {  dbinom(x=sum(sel),size=length(sel),prob=prob,log=logscale) }

binomPrior <- function(sel, prob=.5, logscale=TRUE, probconstr=prob, groups=1:length(sel), constraints=lapply(1:length(unique(groups)), function(z) integer(0))) {
    nsel= nselConstraints(sel=sel, groups=groups, constraints=constraints)
    if (!nsel$violateConstraint) {
      hasconstraint <- nsel$hasconstraint
      ans <- sum((log(prob) * sel)[!hasconstraint]) + sum((log(1-prob)*(1-sel))[!hasconstraint])
      if (nsel$ngroupsconstr>0) ans= ans+ sum((log(probconstr) * sel)[hasconstraint]) + sum((log(1-probconstr)*(1-sel))[hasconstraint])
    } else { ans= -Inf }
    if (!logscale) ans= exp(ans)
    return(ans)
}


#unifPrior <- function(sel, logscale=TRUE) { ifelse(logscale,-length(sel)*log(2),2^(-length(sel)))  }
unifPrior <- function(sel, logscale=TRUE, groups=1:length(sel), constraints=lapply(1:length(unique(groups)), function(z) integer(0))) {
    binomPrior(sel, prob=.5, logscale=logscale, probconstr=.5, groups=groups, constraints=constraints)
}


#bbPrior <- function(sel, alpha=1, beta=1, logscale=TRUE) {
#  ans <- lbeta(sum(sel) + alpha, sum(!sel) + beta) - lbeta(alpha,beta)
#  ifelse(logscale,ans,exp(ans))
#}

bbPrior <- function(sel, alpha=1, beta=1, logscale=TRUE, alphaconstr=alpha, betaconstr=beta, groups=1:length(sel), constraints=lapply(1:length(unique(groups)), function(z) integer(0))) {
  nsel= nselConstraints(sel=sel, groups=groups, constraints=constraints)
  if (!nsel$violateConstraint) {
      ans= lbeta(nsel$ngroups0 + alpha, nsel$ngroups-nsel$ngroupsconstr-nsel$ngroups0 + beta) - lbeta(alpha,beta)
      if (nsel$ngroupsconstr>0) ans= ans + lbeta(nsel$ngroups1 + alphaconstr, nsel$ngroupsconstr - nsel$ngroups1 + betaconstr) - lbeta(alphaconstr,betaconstr)
  } else { ans= -Inf }
  ifelse(logscale,ans,exp(ans))
}




bbPriorTrunc <- function (sel, logscale=TRUE, maxvars=10, ...) {
  #Same as bbPrior with prob=0 when variables > maxvars
  if (sum(sel)<=maxvars) { ans <- bbPrior(sel, logscale=logscale, ...) } else { ans <- ifelse(logscale, -Inf, 0) }
  return(ans)
}

unifPriorTrunc <- function (sel, logscale=TRUE, maxvars=10, ...) {
  #Same as unifPrior with prob=0 when variables > maxvars
  if (sum(sel)<=maxvars) { ans <- unifPrior(sel, logscale=logscale, ...) } else { ans <- ifelse(logscale, -Inf, 0) }
  return(ans)
}

context("Test msfit methods")
library("modelSelection")

source(test_path("data-for-tests.R"))
tolerance <- 1e-6


test_that(
  "msfit coef method works", {
    log <- capture.output(fit <- modelSelection(y=y3, x=X3))
    fit_coef <- coef(fit)
    expect_equal(names(fit_coef[1,])[4], "margpp")
    expect_true(all(fit_coef[,4]>=0))
    expect_true(all(fit_coef[,4]<=1))
  }
)

test_that(
  "msfit predict method works", {
    log <- capture.output(
      fit <- modelSelection(y3 ~ .,
      data = data.frame(y3, X3[,2:4])),
      ypred <- predict(fit)
    )
    expect_equal(names(ypred[1,])[1], "mean")
    expect_lt(mean((predict(fit)[,1]-y3)^2), 2)
  }
)

patrick::with_parameters_test_that(
  "msfit postProb method works", {
    log <- capture.output(fit <- modelSelection(y=y3, x=X3, enumerate=FALSE))
    pprobs <- postProb(fit, method=method)
    pprobs_cut <- postProb(fit, nmax=5, method=method)
    expect_equal(nrow(pprobs_cut), 5)
    expect_equal(pprobs[1:5,3], pprobs_cut[,3], tolerance=tolerance)
  },
  method=c("norm", "exact"),
  .test_name=method
)

## Routines for Cox proportional hazards model

pmomCoxMarginalR <- function(y, x, tau, r=1, method=ifelse(ncol(x)<=10,'Normal','plugin'), logscale=TRUE) {
  #Integrated Cox partial likelihood (Normal approximation) wrt product moment prior
  # - Partial Likelihood: f(y|th)= exp(-.5 (th-thhat)' solve(V) (th-thhat))
  # - Prior proportional to N(th; 0, tau*I) * prod(th^2/tau)^r
  # Input
  # - y: Surv object
  # - x: covariates
  # - tau: prior dispersion
  # - r: prior power parameter
  # - method: set to 'Normal' to compute E(prod(th^2)^i) exactly, to 'plugin' to use prod(E(th)^2). By default 'Normal' is used for up to 10 dimensions
  if (!("Surv" %in% class(y))) stop("y must be of class 'Surv'")
  if (r != 1) stop("Only r=1 currently implemented")
  if (missing(x)) {
    p <- 0
  } else {
    p <- ncol(x)
    if (!is.data.frame(x)) x <- data.frame(x)
  }
  if (p==0) {
    ans <- survival::coxph(y ~ 1)$loglik
  } else {
    fit <- survival::coxph(y ~ ., data=x)
    thhat <- matrix(coef(fit),ncol=1); Vinv <- solve(fit$var)
    Sinv <- Vinv + diag(p)/tau; S <- solve(Sinv)
    m <- S %*% Vinv %*% thhat
    ans <- as.numeric(fit$loglik[2] - (p+p/2)*log(tau) - .5*(t(thhat) %*% Vinv %*% thhat - t(m) %*% Sinv %*% m) + .5*as.numeric(determinant(S,logarithm=TRUE)$modulus))
    if (method=='Normal') {
      ans <- ans + log(eprod(m, S, power=2*r, dof=-1))
    } else {
      ans <- ans + prod(m^2+diag(S))
    }
  }
  if (!logscale) ans <- exp(ans)
  return(ans)
}




logPL <- function(beta, time, event, x, mc.cores=1) {
  #Cox log-partial likelihood for several values of beta
  # - beta: matrix with ncol(beta)==ncol(x) and rows containing the various beta values. When ncol(x)==1 beta can also be a vector
  # - time: survival times (censored or uncensored)
  # - event: event indicator
  # - x: covariate values
  # - mc.cores: number of cores, passed on to parallel::mclapply
  event <- as.numeric(event)
  if (!all(event %in% c(0,1))) stop("event must contain only 0's and 1's")
  if (!is.matrix(x) & !is.data.frame(x)) stop("x must be a matrix or data.frame")
  o <- order(time)
  time <- time[o]; event <- event[o]; x <- x[o,,drop=FALSE]

  if (is.vector(beta)) {
    if (ncol(x)!=1) stop('beta should be a matrix for ncol(x)>1')
    if ("parallel" %in% loadedNamespaces())  {
      ans <- parallel::mclapply(1:length(beta), function(i) { logPLSingle(length(time), event=event, x=x, beta[i]) }, mc.cores=mc.cores)
    } else {
      ans <- lapply(1:length(beta), function(i) { logPLSingle(length(time), event=event, x=x, beta[i]) })
    }
  } else {
    if ("parallel" %in% loadedNamespaces())  {
      ans <- parallel::mclapply(1:length(beta), function(i) { logPLSingle(length(time), event=event, x=x, beta[i,]) }, mc.cores=mc.cores)
    } else {
      ans <- lapply(1:length(beta), function(i) { logPLSingle(length(time), event=event, x=x, beta[i,]) })
    }
  }
  unlist(ans)
}

logPLSingle <- function(n, event, x, beta) {
  #Cox log-partial likelihood for a single value of beta
  #Note: event, x should be ordered according to the ranks of time
  #Downloaded from: http://www4.stat.ncsu.edu/~lu/research/example-pbc-cox.R
  L = 0.0
  for(i in 1:n){
     temp = 0.0
     for(j in i:n) temp = temp + exp(sum(beta*x[j,]))
     L = L - event[i]*(sum(beta*x[i,])-log(temp))
  }
  return(-L)
}

###
### greedyGLM.R
###

# Greedy algorithm to find approximate posterior mode
# Searches for model M that maximizes log-likelihood(theta_M) + log-prior(theta_M|M) + log-prior(M),
# where theta_M: MLE of theta under model M.
# The maximization starts at the null model and sequentially adds/drops any covariate increasing the target function.
# The algorigthm stops when no covariates are added/dropped after a full pass, i.e. a local mode has been found
#
# Note: the algorithm only considers models with at most length(y) variables.
#
# Input:
# - y, x, xadj: response, covariates and adjustment covariates
# - family: family argument passed on to glm
# - priorCoef: prior on regression coefficients
# - priorModel: prior on model space
# - maxit: maximum number of iterations
# Output: logical vector with length ncol(x) indicating the variables included in the model

greedyGLM <- function(y, x, xadj, family, priorCoef, priorModel, maxit=100) {
  pluginJoint <- function(sel) {
    p <- sum(sel)
    if (p>0 & p<=length(y)) {
      glm1 <- glm(y ~ x[,sel,drop=FALSE] + xadj -1, family=family)
      ans <- -.5*glm1$deviance + cfprior(matrix(coef(glm1)[1:p],nrow=1)) + modelprior(sel)
    } else if (p==0) {
      glm1 <- glm(y ~ xadj -1, family=family)
      ans <- -.5*glm1$deviance + modelprior(sel)
    } else { ans <- -Inf }
    return(ans)
  }
  #Set prior on model space
  if (priorModel@priorDistr=='uniform') {
    modelprior <- unifPrior
  } else if (priorModel@priorDistr=='binomial') {
    if (all(c('alpha.p','beta.p') %in% names(priorModel@priorPars))) {
      modelprior= function(sel) bbPrior(sel,alpha=priorModel@priorPars['alpha.p'],beta=priorModel@priorPars['beta.p'],logscale=TRUE)
    } else modelprior= function(sel) binomPrior(sel, prob=priorModel@priorPars[['p']], logscale=TRUE)
  } else {
    stop('Prior on model space prDelta not recognized')
  }
  #Set coefficients prior
  if (priorCoef@priorDistr=='pMOM') {
    if (all(c('a.tau','b.tau') %in% names(priorCoef@priorPars))) {
      cfprior <- function(th) dmom(th,a.tau=priorCoef@priorPars['a.tau'],b.tau=priorCoef@priorPars['b.tau'],r=priorCoef@priorPars['r'],logscale=TRUE)
    } else {
      cfprior <- function(th) dmom(th,tau=priorCoef@priorPars['tau'],r=priorCoef@priorPars['r'],logscale=TRUE)
    }
  } else if (priorCoef@priorDistr=='piMOM') {
    cfprior <- function(th) dimom(th,tau=priorCoef@priorPars['tau'],logscale=TRUE)
  } else if (priorCoef@priorDistr=='peMOM') {
    if (all(c('a.tau','b.tau') %in% names(priorCoef@priorPars))) {
      cfprior <- function(th) demom(th,a.tau=priorCoef@priorPars['a.tau'],b.tau=priorCoef@priorPars['b.tau'],logscale=TRUE)
    } else {
      cfprior <- function(th) demom(th,tau=priorCoef@priorPars['tau'],logscale=TRUE)
    }
  } else {
    stop('prior on coefficients not recognized')
  }
  #Greedy iterations
  sel <- rep(FALSE,ncol(x))
  mcur <- pluginJoint(sel)
  nchanges <- 1; niter <- 1
  while (nchanges>0 & niter<maxit) {
    nchanges <- 0; niter <- niter+1
    for (i in 1:ncol(x)) {
      selnew <- sel; selnew[i] <- !selnew[i]
      mnew <- pluginJoint(selnew)
      if (mnew>mcur) { sel[i] <- selnew[i]; mcur <- mnew; nchanges <- nchanges+1 }
    }
  }
  return(sel)
}

######################################################################################
## FUNCTIONS TO PERFORM MODEL SELECTION ON NORMAL MIXTURES
######################################################################################


setMethod("show", signature(object='mixturebf'), function(object) {
  message('mixturebf object with ',object@p,' variables\n\n')
  message("Use draw() to obtain posterior samples. postProb() returns posterior probabilities as given below\n\n")
  print(object@postprob)
}
)


setMethod("postProb", signature(object='mixturebf'), function(object, nmax, method='norm') {
  object@postprob
}
)

setMethod("postSamples", signature(object='mixturebf'), function(object) {
    if (!is.null(object@mcmc)) { ans= object@mcmc } else { ans= NULL }
    return(ans)
}
)

#setMethod("coef", signature(object='mixturebf'), function(object, ...) {})
coef.mixturebf <- function(object,...) {
    if (length(object@mcmc)==0) stop('No MCMC samples are available, posterior means cannot be computed')
    ans= vector("list",length(object@mcmc))
    names(ans)= names(object@mcmc)
    for (i in 1:length(ans)) {
        k= as.integer(sub('k=','',names(object@mcmc)[i]))
        if ((k>1) & (!is.null(object@mcmc[[i]]))) {
            sel= which(names(object@mcmc[[i]])=='cholSigmainv')
            sel2= which(names(object@mcmc[[i]])=='momiw.weight')
            ans[[i]]= lapply(object@mcmc[[i]][-c(sel,sel2)],colMeans)
            cholSigmainv= object@mcmc[[i]][[sel]]
            ans[[i]]$Sigmainv= vector("list",k)
            for (j in 1:k) { ans[[i]]$Sigmainv[[j]]= Reduce("+",lapply(1:nrow(cholSigmainv),function(z) getSigmainv(j,p=object@p,cholSigmainv[z,])))/nrow(cholSigmainv) }
        }
        if ((k==1) & (length(grep('Normal',object@postprob[i,'model']))==1)) {
            ans[[i]]= list(eta=1, mu= object@postpars$mu1, Sigmainv=solve(object@postpars$S1)*object@postpars$nu1)
        }
    }
    return(ans)
}


getSigmainv= function(j,p,cholSigmainv) {
    ans= matrix(0,nrow=p,ncol=p)
    ans[lower.tri(ans,diag=TRUE)]= cholSigmainv[(1+(j-1)*p*(p+1)/2):(j*p*(p+1)/2)]
    return(ans %*% t(ans))
}


######################################################################################
## BAYES FACTORS
######################################################################################

bfnormmix <- function(x, k=1:2, mu0=rep(0,ncol(x)), g, nu0, S0, q=3, q.niw=1, B=10^4, burnin= round(B/10), logscale=TRUE, returndraws=TRUE, verbose=TRUE) {
    #Bayes factor comparing a Normal mixture with k-1 versus k components (different component-specific covariances)
    #
    #
    # Likelihood p(x[i,] | mu,Sigma,eta)= sum_j eta_j N(x[i,]; mu_j,Sigma_j)
    # Prior: p(mu_j, Sigma_j)= N(mu_j; mu0, g Sigma) IW(Sigma_j; nu0, S0) indep j=1,...,k
    #        p(eta)= Dir(eta; q)
    #
    # Input arguments
    # - x: n x p input data matrix
    # - k: number of components
    # - mu0, g, S0, nu0, q: prior parameters
    # - B: number of MCMC iterations
    # - burnin: number of burn-in iterations
    # - logscale: if set to TRUE the log-BF is returned
    # Output
    # - k: number of components
    # - pp.momiw: posterior prob of k components under a MOM-IW-Dir(q) prior
    # - pp.niw: posterior prob of k components under a Normal-IW-Dir(q.niw) prior
    # - probempty: probability that any one cluster is empty under a Normal-IW-Dir(q.niw) prior
    # - bf.momiw: Bayes factor comparing 1 vs k components under a MOM-IW-Dir(q) prior
    # - logpen: log of the posterior mean of the MOM-IW-Dir(q) penalty term
    # - logbf.niw: Bayes factor comparing 1 vs k components under a Normal-IW-Dir(q.niw) prior
    if (length(k)<2) stop('k must have length >=2 specifying the number of components to be compared')
    if (any(k)<1) stop('All values of k must be >=1')
    if (!is.matrix(x)) x= as.matrix(x)
    n= nrow(x); p= ncol(x)
    if (missing(g)) {
        f= function(g) (pgamma(4,p/2+1,1/(4*g)) - 0.05)^2
        g= optimize(f,c(0,5.7))$minimum
    }
    if (missing(nu0)) nu0= p+4
    if (missing(S0)) { if (p==1) { S0= matrix(1/nu0) } else { S0= diag(p)/nu0 } }
    logprobempty= logpen= double(length(k))
    logbf.niw= double(length(k)-1)
    mcmcout= vector("list",length(k)); names(mcmcout)= paste('k=',k,sep='')
    for (i in length(k):2) {
        ak= lgamma(k[i]*q.niw) - lgamma(k[i]*q.niw-q.niw) + lgamma(n+k[i]*q.niw-q.niw) - lgamma(n+k[i]*q.niw)
        #Initialize clusters via MLE
        em= mclust::Mclust(data=x, G=k[i], modelNames=ifelse(p==1,'V','VVV'),verbose=0)
        #if MLE failed, set prior to prevent 0 variances
        if ("NULL" %in% class(em)) em= try(mclust::Mclust(data=x,G=k[i],modelNames=ifelse(p==1,'V','VVV'),prior=mclust::priorControl(functionName="defaultPrior"),verbose=0))
        if ('try-error' %in% class(em)) { z= as.integer(kmeans(x, centers=k[i])$cluster) } else { z= as.integer(em$classification) }
        #Gibbs sampling (version using mombf)
        mcmcfit= "normalmixGibbsCI"(as.double(x),as.integer(n),as.integer(p),as.integer(k[i]),z,as.double(mu0),as.double(g),as.integer(nu0),as.double(S0),as.double(q.niw),as.integer(B),as.integer(burnin),as.integer(verbose))
        #mcmcfit= .Call("normalmixGibbsCI",as.double(x),as.integer(n),as.integer(p),as.integer(k[i]),z,as.double(mu0),as.double(g),as.integer(nu0),as.double(S0),as.double(q.niw),as.integer(B),as.integer(burnin),as.integer(verbose))
        names(mcmcfit)= c('logprobempty','logpen','eta','mu','cholSigmainv')
        logprobempty[i]= mcmcfit$logprobempty
        eta= t(matrix(mcmcfit$eta,ncol=(B-burnin)))
        mu= t(matrix(mcmcfit$mu,ncol=(B-burnin)))
        cholSigmainv= t(matrix(mcmcfit$cholSigmainv,ncol=(B-burnin))) #Cholesky decomp. Sigma^{-1}= cholSigmainv %*% t(cholSigmainv)
        mcmcout[[i]]= list(eta=eta,mu=mu,cholSigmainv=cholSigmainv)
        #probone= ppOneEmpty(x=x,g=g,q=q,q.niw=q.niw,mcmcfit=mcmcout[[i]],logscale=TRUE,verbose=TRUE) #to debug only, can be ignored
        qdif= q-q.niw; constddir= lgamma(k[i]*q) - lgamma(k[i]*q.niw) + k[i]*(lgamma(q.niw)-lgamma(q))
        logpen.mcmc= mcmcfit$logpen - normctNMix(k=k[i],p=p) + constddir + qdif * rowSums(log(eta))
        mcmcout[[i]]$momiw.weight= exp(logpen.mcmc-max(logpen.mcmc))
        logpen[i]= max(logpen.mcmc) + log(mean(mcmcout[[i]]$momiw.weight)) #log MOM-IW penalty
        mcmcout[[i]]$momiw.weight= mcmcout[[i]]$momiw.weight / sum(mcmcout[[i]]$momiw.weight)
        #Bayes factors
        logbf.niw[i-1]= mcmcfit$logprobempty - ak
    }
    logbf.niw= -c(0,cumsum(logbf.niw))  #BF of all models vs k=1 under Normal-IW-Dir(q.niw) prior
    logbf.momiw= logbf.niw + logpen    #BF of all models vs k=1 under MOM-IW-Dir(q) prior
    logprobempty[1]= -Inf
    pp.momiw= exp(logbf.momiw - max(logbf.momiw)); pp.momiw= pp.momiw/sum(pp.momiw)
    pp.niw= exp(logbf.niw - max(logbf.niw)); pp.niw= pp.niw/sum(pp.niw)
    ans= cbind(k,pp.momiw,pp.niw,logprobempty,logbf.momiw,logpen,logbf.niw)
    if (!logscale) { ans[,-1:-3]= exp(ans[,-1:-3]); names(ans)= sub('log','',names(ans)) }
    ans= data.frame(ans, model='Normal, VVV')
    #For 1 component, (mu,Sigma)|x ~ N(mu; mu1, Sigma/prec) IWishart(Sigma; nu=nu1, S=S1)
    xbar = colMeans(x)
    nu1 = nu0 + n
    S1 = S0 + cov(x)*(n-1) + n/(1+n*g) * (matrix(xbar-mu0,ncol=1) %*% matrix(xbar-mu0,nrow=1))
    w = n/(n + 1/g)
    m1 = w * xbar + (1 - w) * mu0
    postpars= list(mu1=m1, prec= n+1/g, nu1=nu1, S1=S1)
    #Return answer
    ans= new("mixturebf", postprob=ans, p=p, n=n, priorpars=list(mu0=mu0, g=g, nu0=nu0, S0=S0, q=q, q.niw=q.niw), postpars=postpars)
    if (returndraws) ans@mcmc= mcmcout
    return(ans)
}








######################################################################################
## POSTERIOR PROBABILITY OF EMPTY CLUSTERS
######################################################################################

ppOneEmpty <- function(x,g,q,q.niw,mcmcfit,logscale=TRUE,verbose=TRUE) {
    # Posterior probability that one cluster is empty and posterior mean of MOM-IW penalty
    # Input arguments
    # - x: n * p data matrix
    # - mcmcfit: MCMC output fitting a Normal mixture. A list with elements list(eta=eta,mu=mu,cholSigmainv=cholSigmainv), where Sigma^{-1}= cholSigmainv %*% t(cholSigmainv) is the precision matrix
    # - logscale: if TRUE log-prob is returned
    # - verbose: set to TRUE to print iteration progress
    # Output
    # - logprobempty: mean P(n_j=0|y,k) under a Dir(q.niw) prior, where n_j is the number of individuals in cluster j
    # - logpen: posterior mean of the MOM penalty d(theta) under a product Normal-IW-Dirichlet prior, that is E(d(theta) | y,k)
    #   where d(theta)= (1/C_k) [ Dir(eta; q) / Dir(eta; q.niw)] [ prod_{i<j} (mu_i-mu_j)' A^{-1} (mu_i-mu_j)/g ] [ prod_j N(mu_j; 0, g A) IW(Sigma_j; nu0, S0) ]
    #         C_k: MOM-IW prior norm constant
    #         g: prior dispersion parameter
    #         A^{-1}= sum_j Sigma_j^{-1} / k
    niter= nrow(mcmcfit$eta) #number of MCMC draws
    p= ncol(x); k= ncol(mcmcfit$eta)
    probOneEmpty= matrix(NA,nrow=niter,ncol=k)
    logpen= double(niter)
    if (verbose) message("Post-processing MCMC output")
    qdif= q-q.niw; constddir= lgamma(k*q) - lgamma(k*q.niw) + k*(lgamma(q.niw)-lgamma(q))
    for (i in 1:niter) {
        #Extract (eta,mu,Sigma)
        eta= mcmcfit$eta[i,]
        mu= lapply(1:k, function(j) mcmcfit$mu[i,(1+(j-1)*p):(j*p)])
        Sigmainv= lapply(1:k, function(j) { getSigmainv(j,p,mcmcfit[[3]][i,]) })
        Sigma= lapply(Sigmainv, solve)
        #Cluster allocation probabilities (n x k matrix)
        pp= sapply(1:k, function(j) mvtnorm::dmvnorm(x,mu[[j]],sigma=Sigma[[j]],log=TRUE) + log(eta[j]))
        pp= exp(pp-apply(pp,1,'max')); pp= pp/rowSums(pp)
        pp[pp> 1-1e-13]= 1-1e-13
        #log-probability of each cluster being empty
        probOneEmpty[i,]= colSums(log(1-pp))
        #MOM-IW penalty
        Ainv= Reduce("+",Sigmainv) / k
        mumat= do.call(rbind,mu)
        logprior= sum(mvtnorm::dmvnorm(mumat,sigma=g*solve(Ainv),log=TRUE)) + constddir + qdif * sum(log(eta))
        #same as logprior= sum(mvtnorm::dmvnorm(mumat,sigma=g*solve(Ainv),log=TRUE)) + ddir(eta,q=q,logscale=TRUE) - ddir(eta,q=q.niw,logscale=TRUE)
        for (jj in 1:k) logprior= logprior - mvtnorm::dmvnorm(mu[[jj]],sigma=g*Sigma[[jj]],log=TRUE)
        d= as.vector(dist(mumat %*% t(chol(Ainv))))^2 #Pairwise Mahalanobis distances between mu's
        logpen[i]= logprior + sum(log(d)) - k*(k-1)/2*log(g) - normctNMix(k=k,p=p)
        if (verbose & ((i %% (niter/10))==0)) message(".")
    }
    if (verbose) message("\n")
    #Compute mean log-prob, in a way that helps prevent numerical overflow
    logprob= double(k)
    for (i in 1:k) {
        m= max(probOneEmpty[,i])
        logprob[i]= m + log(mean(exp(probOneEmpty[,i] - m)))
    }
    logprob= max(logprob) + log(mean(exp(logprob - max(logprob))))
    logpen= max(logpen) + log(mean(exp(logpen-max(logpen))))
    ans= c(logprobempty=logprob, logpen=logpen)
    if (!logscale) ans= exp(ans)
    return(ans)
}





######################################################################################
## PRIOR NORMALIZATION CONSTANT
######################################################################################

normctNMix= function(k,p) {
#Prior normalization constant for MOM-IW prior
    if (p==1) {
        ans= sum(lgamma(2:(k+1)))
    } else if (k==2) {
        ans= log(2*p)
    } else {
        if ((p<=ncol(Cktab)) & (k<=nrow(Cktab)+1)) { ans= Cktab[k-1,p] } else { ans= normctNMixEstimate(p=p,k=k,B=20000) }
    }
    return(ans)
}

#Pre-tabulated log norm constant for MOM-IW
Cktab= matrix(NA,nrow=19,ncol=20)
rownames(Cktab)= paste('k=',2:(nrow(Cktab)+1),sep=''); colnames(Cktab)= paste('p=',1:ncol(Cktab),sep='')
Cktab[1,]= c(0.693,1.386,1.792,2.079,2.303,2.485,2.639,2.773,2.890,2.996,3.091,3.178,3.258,3.332,3.401,3.466,3.526,3.584,3.638,3.689)
Cktab[2,]= c(2.485,4.561,5.709,6.507,7.133,7.655,8.095,8.481,8.817,9.118,9.386,9.647,9.878,10.092,10.295,10.484,10.663,10.827,10.984,11.140)
Cktab[3,]= c(5.663,9.810,11.976,13.504,14.713,15.658,16.526,17.253,17.900,18.478,19.024,19.498,19.962,20.374,20.759,21.132,21.479,21.812,22.111,22.412)
Cktab[4,]= c(10.450,17.050,20.791,23.242,25.145,26.677,28.074,29.204,30.284,31.207,32.042,32.816,33.576,34.252,34.892,35.487,36.039,36.586,37.097,37.565)
Cktab[5,]= c(17.030,27.187,32.119,35.893,38.505,40.914,42.929,44.680,46.042,47.294,48.619,49.744,50.860,51.782,52.796,53.619,54.470,55.258,55.968,56.681)
Cktab[6,]= c(25.555,38.463,45.795,51.076,54.848,58.037,60.621,62.954,65.149,67.026,68.901,70.553,71.715,73.189,74.438,75.502,76.727,77.797,78.748,79.829)
Cktab[7,]= c(36.159,52.974,61.186,69.241,73.942,79.432,81.386,85.021,87.891,90.849,92.470,94.466,98.722,98.475,100.097,101.348,103.371,104.420,105.544,107.230)
Cktab[8,]= c(48.961,66.164,80.042,88.514,96.708,101.179,109.560,110.135,113.193,117.850,119.457,122.496,124.638,127.080,128.651,131.149,132.972,134.709,136.385,137.994)
Cktab[9,]= c(64.066,82.847,102.055,113.733,120.787,126.917,135.411,138.371,143.527,146.264,151.659,152.687,156.542,159.355,161.416,164.206,166.930,168.785,171.930,173.261)
Cktab[10,]= c(81.568,97.246,124.129,136.741,144.772,157.996,161.171,169.065,174.822,179.265,183.386,187.168,191.950,195.910,199.041,203.199,203.853,207.579,211.241,212.259)
Cktab[11,]= c(101.555,122.362,146.287,162.528,180.388,187.270,192.937,203.938,212.889,217.734,221.541,228.209,231.441,236.452,238.341,243.116,246.672,249.123,252.716,254.760)
Cktab[12,]= c(124.107,141.233,168.917,196.092,214.977,221.311,234.298,240.367,248.477,260.534,260.992,265.749,272.312,278.072,283.667,287.527,291.691,296.176,301.515,301.772)
Cktab[13,]= c(149.299,156.957,206.438,225.427,243.774,259.805,267.883,285.804,291.234,299.880,309.264,311.079,316.570,323.971,331.077,334.229,342.464,344.108,348.812,354.336)
Cktab[14,]= c(177.198,179.557,237.431,253.155,274.733,298.454,315.369,323.422,336.471,341.744,353.027,359.312,367.176,377.951,378.114,386.572,393.042,400.376,404.191,406.108)
Cktab[15,]= c(207.870,202.500,250.975,288.691,316.324,340.346,351.251,370.607,382.489,395.351,402.748,411.758,422.648,431.317,438.742,441.203,447.649,451.989,461.505,467.732)
Cktab[16,]= c(241.375,229.919,285.081,338.441,357.618,381.691,401.358,415.652,436.766,442.482,465.513,470.246,477.489,480.464,493.833,501.363,506.500,513.625,519.481,527.751)
Cktab[17,]= c(277.770,254.477,321.324,370.048,395.518,425.777,449.232,463.807,483.476,503.058,510.875,527.850,533.347,547.950,555.441,564.276,575.912,578.512,585.708,593.824)
Cktab[18,]= c(317.110,274.729,354.350,418.900,445.156,478.831,500.711,524.482,547.148,557.461,578.541,584.673,594.984,611.518,618.742,628.901,638.195,648.375,656.415,666.447)
Cktab[19,]= c(359.446,317.930,397.421,452.398,504.381,536.868,556.158,578.169,600.491,627.437,633.916,657.388,661.210,671.309,683.771,700.666,706.412,725.335,732.537,739.385)


normctNMixEstimate= function(k,p,B=10^4) {
#Monte Carlo estimate of MOM-IW normalization constant (B is the number of MC samples)
    d= double(B)
    for (i in 1:B) {
        mu= matrix(rnorm(p*k),ncol=p)
        d[i]= sum(log(as.vector(dist(mu))^2)) #Pairwise Mahalanobis distances between mu's
    }
    max(d) + log(mean(exp(d-max(d))))
}

################################################################################
# List of functions
################################################################################
# * inside
# * lasso.bic
# * binary.id
# * compact.id
# * pinc.mt
# * grid.opt.mt
# * Of.EB.cil
# * Gf.EB.cil
# * Of.EP.cil
# * Gf.EP.cil
# * check.parameter.format
# * exposureCoef
# * model.pprobs.cil
# * bma.cil.teff
# * check.input.format
# * cil.teff
################################################################################
# List of methods
################################################################################
# * postProb
# * coef
################################################################################

# ##############################################################################
# # Dependencies
# library(modelSelection)
# library(pracma)
# library(glmnet)
# library(hdm)
# ##############################################################################

################################################################################
# Test if value "x" is inside interval ab = (a, b)
inside <- function(x, ab, include.ab = TRUE) {
################################################################################
  ifelse(include.ab == TRUE,
    (x >= ab[1] && x <= ab[2]), (x > ab[1] && x < ab[2]))
}



biclm <- function(y, pred, x, beta) {
  if (missing(pred)) pred <- x %*% beta
  n <- ifelse(is.vector(y), length(y), nrow(y))
  npar <- sum(beta != 0)
  ans <- n * log(colSums((y - pred)^2) / length(y)) + n * (log(2 * pi) + 1) + log(n) * npar
  return(ans)
}


biclogreg <- function(y, pred, x, beta) {
  if (missing(pred)) pred <- x %*% beta
  n <- ifelse(is.vector(y), length(y), nrow(y))
  npar <- sum(beta != 0)
  ans <- - 2 * sum(dbinom(y, size=1, prob=expit(-pred), log=TRUE)) + n * log(npar)
  return(ans)
}

################################################################################
# LASSO estimation with BIC
lasso.bic <- function(y, x, intercept = TRUE, standardize = TRUE, family = 'gaussian') {
################################################################################
  if (!is.matrix(x)) x <- as.matrix(x)
  fit <- glmnet::glmnet(x = x, y = y, family = family, alpha = 1, intercept = intercept, standardize = standardize)
  if (intercept == TRUE) {
    ct <-  which(apply(x, 2, 'sd') == 0)
    if (length(ct)==0) x <- cbind(1,x)
    beta <- as.matrix(rbind(fit[['a0']], fit[['beta']]))
  } else {
    beta <- as.matrix(fit[['beta']])
  }
  
  n <- length(y)
  p <- colSums(beta != 0)

  if (family == 'gaussian') {
    bic <-  sapply(1:ncol(beta), function(i) biclm(y, x=x, beta=beta[,i,drop=FALSE]))
  } else if (family == 'binomial') {
    bic <- sapply(1:ncol(beta), function(i) biclogreg(y=y, x=x, beta=beta[,i,drop=FALSE]))
  } else {
     stop(paste("family=",family,"not implemented"))
  }
  sel <- which.min(bic)
  ypred <- x %*% beta[, sel, drop=FALSE]
  if (intercept) {
    beta <- beta[-1,sel] #if intercept was added by glmnet, exclude it from the estimated coef
  } else {
    beta <- beta[,sel]
  }
  ans <- list(coef = beta, ypred = ypred, lambda.opt = fit[['lambda']][sel], lambda = data.frame(lambda = fit[['lambda']], bic = bic, nvars = p))
  return(ans)
}

################################################################################
# Converting model indexes to binary strings
binary.id <- function(x, p, nt = 1, conc = FALSE) {
################################################################################
  out <- rep(0, p + nt)
  out[as.numeric(unlist(strsplit(x, ',')))] <- 1
  if (conc == TRUE) { out <- paste(out, collapse = '') }
  return(out)
}

################################################################################
# Converting binary strings to model indexes
compact.id <- function(x) {
################################################################################
  paste(which(unlist(strsplit(as.character(x), '')) == 1), collapse = ',')
}

################################################################################
# Prior inclusion probability function (conditional on coefficient size)
pinc.mt <- function(x, th, rho.min = 0, rho.max = 1, squared = FALSE, includeX) {
################################################################################
  #  If there are interactions (CAREFUL: main effects + ORDERED interactions)
  th <- as.numeric(th)
  if (ncol(x) / (length(th) - 1) > 1) {
    th <- c(th, rep(th[-1], each = ncol(x) / (length(th) - 1) - 1))
  }

  # If using abs(beta) or beta^2
  if (squared == TRUE) {
    probs <- (1 + exp(-apply(x, 1, function(z) {
        sum(c(1, z^2) * th)
      })))^(-1)
  } else {
    probs <- (1 + exp(-apply(x, 1, function(z) {
        sum(c(1, abs(z)) * th)
      })))^(-1)
  }

  # Limit upper and lower bounds
  probs <- rho.min + (rho.max - rho.min) * probs

  # Force inclusion of variables indicated by includeX
  probs[includeX] <- 1
    
  # End
  return(probs)
}

################################################################################
grid.opt.mt <- function(G0, betad, pj1, method = 'EB', ws = NA, th.grid,
  th.prior, rho.min = 0, rho.max = 1, ret.plotly = FALSE, includeX, Dvars) {
################################################################################
  opt.th <- th.grid
  if (th.prior == 'tunif') {
    opt.th <- opt.th[which(opt.th[, 1] >= -log(length(betad))), ]
    opt.th <- opt.th[which(opt.th[, 2] >= -log(length(betad))), ]
    opt.th <- opt.th[which(opt.th[, 1] <= log(length(betad))), ]
    opt.th <- opt.th[which(opt.th[, 2] <= log(length(betad))), ]
  }

  #opt.th <- expand.grid(th0s, th1s)
  if (method == 'EB') {
    # Evaluate -log of objective function
    fval <- exp(-apply(opt.th, 1, Of.EB.cil, betad = betad, G0 = G0, ws = ws, th.prior = th.prior, rho.min = rho.min, rho.max = rho.max, includeX = includeX, Dvars = Dvars))
  } else if (method == 'EP') {
    fval <- exp(-apply(opt.th, 1, Of.EP.cil, betad = betad, pj1 = pj1, th.prior = th.prior, rho.min = rho.min, rho.max = rho.max, includeX = includeX))
  } else {
    stop('supported theta search methods are "EB" and "EP".')
  }

  # Optimal values. If there are multiple modes, choose the one with smallest L2 norm
  th.opt <- opt.th[fval == max(fval),,drop=FALSE]
  th.opt <- as.numeric(th.opt[which.min(th.opt[,1]^2 + th.opt[,2]^2),])
  #th.opt <- as.numeric(opt.th[which.max(fval),])
  names(th.opt) <- paste('th', 1:length(th.opt) - 1, sep = '')

  # Output
  return(list(th.opt = th.opt))
}#; grid.opt.mt <- compiler::cmpfun(grid.opt.mt)

################################################################################
# OBJECTIVE FUNCTIONS FOR EB METHOD
# - x: theta hyper-parameter value
# - betad: measures of association between treatment d and controls
# - G0: model identifiers. Each element is a character string where 1's indicate inclusion and 0's exclusion
# - ws: marginal likelihood for each model in G0
# - th.prior: prior on theta, currently not used
# - rho.min, rho.max: the prior inclusion prob given by any theta is restricted to [rho.min, rho.max], to avoid prior inclusion prob=0 or 1
# - includeX: variables whose inclusion is forced into the model
# - Dvars: index of variables corresponding to treatment. These have prior inclusion prob = 1/2, regardless of theta
Of.EB.cil <- function(x, betad, G0, ws, th.prior, rho.min = 0, rho.max = 1, includeX, Dvars) {
################################################################################
  # log p(y | theta)
  p1 <- rep(NA, nchar(G0)[1])
  p1[-Dvars] <- pinc.mt(betad, th = x, rho.min = rho.min, rho.max = rho.max, includeX = includeX)
  p1[Dvars] <- 1/2
  foo <- function(m) {
    gm <- as.numeric(unlist(strsplit(m, '')))
    pg <- prod((p1 ** gm) * (1 - p1) ** (1 - gm))  #prior model probability
    wg <- ws[which(G0 == m)] #marginal likelihood of the model
    return(wg * pg)
  }
  sc <- sum(sapply(G0, foo))
  pt1 <- log(sum(sc))  # log p(y | theta)

  # log p(theta)
  pt2 <- 0

  return(-(pt1 + pt2))
}

################################################################################
Gf.EB.cil <- function(x, betad, G0, ws, th.prior, rho.min = 0, rho.max = 1, includeX, Dvars) {
################################################################################
  # Compute (unnormalised) model inclusion probabilities
  p1 <- rep(NA, nchar(G0)[1])
  p1[-Dvars] <- pinc.mt(betad, th = x, rho.min = rho.min, rho.max = rho.max, includeX = includeX)
  p1[Dvars] <- 1/2
  #sD <- nchar(G0[1]) - length(p1)
  #p1 <- c(rep(1/2, sD), p1)
  sc <- sapply(G0, function(m) {
    gm <- as.numeric(unlist(strsplit(m, '')))
    wg <- ws[which(G0 == m)]
    pg <- (p1 ** gm) * (1 - p1) ** (1 - gm)
    return(wg * prod(pg))
  })

  # Marginal inclusion probabilities
  pprobs <- data.frame('model_id' = G0, 'pprob' = sc / sum(sc))
  pprobs[, 1] <- sapply(pprobs[, 1], compact.id)
  mpprobs <- apply(t(sapply(pprobs[, 1], binary.id,
    p = nchar(G0[1]) - ncol(betad), nt = ncol(betad))), 2,
    function(x) { sum(x * pprobs[, ncol(pprobs)]) })

  # grad log p(y | theta)
  gs <- rep(NA, length(x))
  gs[1] <- sum((mpprobs - p1)[-Dvars])
  #gs[1] <- sum((mpprobs - p1)[-(1:sD)])
  for (k in 2:length(gs)) {
    aux0 <- (ncol(betad) / (length(x) - 1) - 1)
    idxs <- c(k - 1)
    if (aux0 > 0) { idxs <- c(idxs, (length(x) - 1) + aux0 * (k - 2) + 1:aux0) }
    aux1 <- mpprobs - p1
    aux2 <- rowSums(abs(as.matrix(betad[, idxs])))
    gs[k] <- sum(aux1[-Dvars] * aux2)
    #gs[k] <- sum(aux1[-(1:sD)] * aux2)
  }

  # grad log p(theta)
  t0 <- t1 <- 0

  # - grad log p(theta | y) = -(grad log p(y | theta) + grad log p(theta))
  return(-(gs + t0))
}

################################################################################
# OBJECTIVE FUNCTION FOR EP METHOD
# - x: theta hyper-parameter value
# - betad: measures of association between treatment d and controls
# - pj1: marginal posterior inclusion probabilities in outcome model under uniform model prior
# - th.prior: prior on theta, currently not used
# - rho.min, rho.max: the prior inclusion prob given by any theta is restricted to [rho.min, rho.max], to avoid prior inclusion prob=0 or 1
# - includeX: variables whose inclusion is forced into the model
Of.EP.cil <- function(x, betad, pj1, th.prior, rho.min = 0, rho.max = 1, includeX) {
################################################################################
  # log p(y | theta)
  p1 <- pinc.mt(betad, th = x, rho.min = rho.min, rho.max = rho.max, includeX = includeX)
  #David: the line below seems wrong: the EP objective fun is a sum involving control prob in p1, excluding the treatment
  #p1 <- c(rep(1/2, length(pj1) - length(p1)), p1)
  #David: the corrected line is below (make pj1 shorter, rather than make p1 longer)
  sD <- length(pj1) - length(p1); pj1 <- pj1[-(1:sD)]
  fj <- pj1 * p1 + (1 - pj1) * (1 - p1)
  pt1 <- sum(log(fj))

    # log p(theta)
  pt2 <- 0

  # - log p(theta | y) \propto -(log p(y | theta) + log p(theta))
  return(-(pt1 + pt2))
}

################################################################################
Gf.EP.cil <- function(x, betad, nt, pj1, th.prior, rho.min = 0, rho.max = 1, includeX) {
################################################################################
  # grad log p(y | theta)
  p1 <- pinc.mt(betad, th = x, rho.min = rho.min, rho.max = rho.max, includeX= includeX)
  sD <- length(pj1) - length(p1)
  p1 <- c(rep(1/2, sD), p1)
  fj <- pj1 * p1 + (1 - pj1) * (1 - p1)

  gs <- rep(NA, length(x))
  gs[1] <- sum(((((1 - p1) * p1) / fj) * (2 * pj1 - 1))[-(1:sD)])
  for (k in 2:length(gs)) {
    aux0 <- (ncol(betad) / (length(x) - 1) - 1)
    idxs <- c(k - 1)
    if (aux0 > 0) { idxs <- c(idxs, (length(x) - 1) + aux0 * (k - 2) + 1:aux0) }
    aux1 <- (((1 - p1) * p1) / fj) * (2 * pj1 - 1)
    aux2 <- rowSums(abs(as.matrix(betad[, idxs])))
    gs[k] <- sum(aux1[-(1:sD)] * aux2)
  }

  # grad log p(theta)
  t0 <- 0

  # - grad log p(theta | y) = -(grad log p(y | theta) + grad log p(theta))
  return(-(gs + t0))
}

################################################################################
check.parameter.format <- function(rho.min, rho.max, th.range, max.mod, lpen, eps, bvs.fit0, th.EP, D) {
################################################################################
  # Error messages accounted for
  err1 <- 'if informed, argument "rho.min" must be a scalar in the interval (0, 1/2).'
  err2 <- 'if informed, argument "rho.max" must be in the interval (rho.min, 1).'
  err3 <- 'if informed, argument "th.range" must be numeric and have at least two distinct values.'
  #err4 <- 'if informed, argument "tau" must be a positive scalar.'
  err5 <- 'parameter "max.mod" must be a positive integer scalar.'
  err6 <- 'parameter "lpen" must be set to "lambda.min" or "lambda.1se" (see documentation for "glmnet").'
  err7 <- 'argument "eps" must be a scalar in the interval (0, 1/2).'
  err8 <- 'if informed, parameter "bvs.fit0" must be of class "msfit".'
  err9 <- 'if informed, parameter "th.EP" must be a numeric vector of length equal to the number of columns in "D" plus 1.'

  # Parameter: "rho.min"
  if (! is.null(rho.min) | length(rho.min) > 1) {
    if (! is.numeric(rho.min) | length(rho.min) > 1) {
      stop(err1)
    } else if (! inside(rho.min, c(0, 1/2), include.ab = FALSE)) {
      stop(err1)
    }
  }

  if (rho.min >= rho.max) stop(err2)

  # Parameter: "th.range"
  if (! is.null(th.range)) {
    if (! is.numeric(th.range)) {
      stop(err3)
    } else if (! length(unique(th.range)) == length(th.range)) {
      stop(err3)
    }
  }

  # Parameter: "tau"
  #if (! is.null(tau) | length(tau) > 1) {
  #  if (! is.numeric(tau) | length(tau) > 1) {
  #    stop(err4)
  #  } else if (sign(tau) != 1) {
  #    stop(err4)
  #  }
  #}

  # Parameter: "max.mod"
  if (! is.null(max.mod) | length(max.mod) > 1) {
    if (! is.numeric(max.mod) | length(max.mod) > 1) {
      stop(err5)
    } else if (sign(max.mod) != 1) {
      stop(err5)
    }
  }

  # Parameter: "lpen"
  if (length(lpen) != 1) {
    stop(err6)
  } else if (! lpen %in% c('lambda.min', 'lambda.1se')) {
    stop(err6)
  }

  # Parameter: "eps"
  if (! is.null(eps) | length(eps) > 1) {
    if (! is.numeric(eps) | length(eps) > 1) {
      stop(err7)
    } else if (! inside(eps, c(0, 1/2), include.ab = FALSE)) {
      stop(err7)
    }
  }

  # Object: "bvs.fit0"
  if (! is.null(bvs.fit0)) {
    if (!inherits(bvs.fit0, 'msfit')) {
      stop(err8)
    }
  }

  # Object: "th.EP"
  if (! is.null(th.EP)) {
    if (! is.numeric(th.EP) | length(th.EP) != ncol(D) + 1) {
      stop(err9)
    }
  }
}



################################################################################
# Estimate coefficients regressing exposure (or treatments) on controls
# - Di: exposure (or treatment) variables
# - A: controls
# - familyD: type of outcome for each column in Di, e.g. c('normal','binomial')
# - typeofvar: does each column in A correspond to a 'numeric' or 'factor'. The latter are not standardized to zero mean, unit variance
# - addintcpt: should an intercept be added to A? (TRUE/FALSE)
# - mod1: method to estimate the regression model
# - priorCoef, Rinit: only used if mod1=='bvs', 'bma' or 'bms'. Then it's the prior on the coefficients and number of MCMC iterations used by modelSelection to estimate the coefficients
# - lpen: only used if mod1=='lasso'. It's an option when choosing lambda in LASSO with cross-validation, 'lambda.min' or 'lambda.1se'
exposureCoef <- function(Di, A, familyD, typeofvar = rep('numeric',ncol(A)), addintcpt, mod1, priorCoef, Rinit, lpen) {
  n <- nrow(A); p <- ncol(A)
  if (length(familyD)==1) familyD <- rep(familyD, ncol(Di))
  familyD.glmnet <- familyD
  familyD.glmnet[familyD.glmnet == 'normal'] <- 'gaussian'

  #Scale design matrix
  mx <- colMeans(A); sx <- sqrt(colMeans(A^2) - mx^2) * sqrt(n/(n-1))
  isct <- (sx==0)
  mx[typeofvar=='factor'] <- 0; sx[typeofvar=='factor'] <- 1
  A[,!isct]= t((t(A[,!isct]) - mx[!isct])/sx[!isct])

  #Obtain coefficients for each exposure (treatment) variable
  betad <- matrix(NA, nrow = ncol(A), ncol = ncol(Di))
  for (i in 1:ncol(Di)) {
    # Define matrices
    b <- Di[, i]
    exc <- 2 * p  # phantom column: not exclude anything
    if (length(unique(Di[, i])) <= 3 &  # This controls sum-to-zero constraint
        all(unique(Di[, i]) %in% c(-1, 0, 1))) {
      b[which(b == -1)] <- 0
    }

    # Exposure family
    ntreatvals <- length(unique(b))
    if (ntreatvals > 1) {
    #if (ntreatvals > 1 & min(table(b)) > 1) {  #David: this line seemed incorrect, for continuous b we had min(table(d))==1 and hence betad wasn't initialized
      if (mod1 == 'ginv') {
        if (familyD[i] != 'normal') stop("The generalized inverse method cannot be applied when familyD != 'normal'")
        betad[, i] <- as.matrix(pracma::pinv(t(A) %*% A) %*% t(A) %*% b)[-exc] 
      } else if (mod1 %in% c('bvs', 'bma', 'bms')) {
        m1.fit <- modelSelection(b, A, family=familyD[i], priorCoef = priorCoef, priorVar = igprior(0.01,0.01), priorModel = modelbbprior(1,1),
          niter = Rinit, verbose = FALSE)
        betad[,i] <- coef(m1.fit)[1:nrow(betad),1]
        #post.th <- colMeans(rnlp(msfit = m1.fit, priorCoef = priorCoef, niter = 1e4))
        #betad[, i] <- unname(post.th[2:(length(post.th) - 1)])[-exc]
      } else if (mod1 == 'lasso') {
        m1.fit <- glmnet::cv.glmnet(x=A, y=b, intercept = addintcpt, family = familyD.glmnet[i])
        m1.coef <- as.numeric(coef(m1.fit, s = m1.fit[[lpen]])[-exc])
        if (nrow(betad) == length(m1.coef)) betad[,i] <- m1.coef else betad[,i] <- m1.coef[-1]
        #betad[, i] <- unname(coef(m1.fit, s = m1.fit[[lpen]])[-1, 1])[-exc]
      } else if (mod1 == 'lasso_bic') {
        m1.fit <- lasso.bic(y=b, x=A, intercept = addintcpt, family = familyD.glmnet[i])
        betad[, i] <- unname(m1.fit[['coef']][-exc])
        #if (addintcpt == FALSE) { m1.fit[['coef']][2] <- m1.fit[['coef']][1] }
        #betad[, i] <- unname(m1.fit[['coef']][-1][-exc])
      } else if (mod1 == 'ridge') {
        m1.fit <- glmnet::cv.glmnet(A, b, alpha = 0, intercept = addintcpt, family = familyD.glmnet[i])
        betad[, i] <- unname(coef(m1.fit, s = m1.fit[['lambda.min']]))[-exc]
        #betad[, i] <- unname(coef(m1.fit, s = m1.fit[['lambda.min']])[-1, ])[-exc]
      } else {
        stop('currently unsupported "mod1" method.')
      }
    } else {
      betad[, i] <- rep(0, nrow(betad))  # If treatment has no variability
    }
  }
  colnames(betad) <- colnames(Di)
  rownames(betad) <- colnames(A)
  return(betad)
}
################################################################################


################################################################################
#Set range of values to consider for hyper-parameter theta in the EB/EP optimization
setThetaRange <- function(ncolD) {
  if (ncolD == 1) {
    th.range <- seq(-40, 40, 1)
  } else if (ncolD == 2) {
    th.range <- seq(-40, 40, 2)
  } else if (ncolD <= 4) {
    th.range <- seq(-10, 10, 2)
  } else if (ncolD <= 7) {
    th.range <- seq(-7.5, 7.5, 2.5)
  } else if (ncolD <= 14) {
    th.range <- seq(-2, 2, 2)
    #th.range <- seq(-5, 5, 2.5)
  } else {
    th.range <- c(-2, 2)
  }
  return(th.range)
}
################################################################################


################################################################################
model.pprobs.cil <- function(y, D, X, I = NULL, family = 'normal', familyD = 'normal',
  mod1, th.search = 'EB', th.prior = 'unif', priorCoef, rho.min = NULL, rho.max,
  th.range = NULL, max.mod = 2^20, lpen = 'lambda.1se', eps = 1e-10,
  R, Rinit= 500, bvs.fit0 = NULL, th.EP = NULL, center = center, scale = scale,
  includevars = includevars, verbose = TRUE) {
################################################################################

  # Renaming (DEPENDENCIES: modelSelection, pracma, glmnet, hdm)
  ms <- modelSelection       #modelSelection::modelSelection
  igp <- igprior(0.01, 0.01) #modelSelection::igprior(0.01, 0.01)
  bbp <- modelbbprior(1, 1)  #modelSelection::modelbbprior(1, 1)
  ufp <- modelunifprior()    #modelSelection::modelunifprior()
  mlik <- marginalLikelihood #modelSelection::marginalLikelihood

  # These parameters must be integers 
  max.mod <- round(max.mod, 0)
  R <- round(R, 0)

  # Create design matrices
  if (!is.null(I)) {
    Z <- cbind(D, I, X)
    Di <- cbind(D, I)
    ncolI <- ncol(I)
  } else {
    Z <- cbind(D, X)
    Di <- D
    ncolI <- 0
  }

  if (is.data.frame(X)) {
    f <- as.formula(paste('~', paste(names(X), collapse=' + ')))
    A <- createDesign(formula=f, data=X)
    typeofvar <- A$typeofvar
    A <- A$x
  } else if (is.matrix(X)) {
      A <- X
      typeofvar <- rep('numeric',ncol(A))
  } else {
    stop("X must be a matrix or a data.frame")
  }

  # Default lower bound on prior inclusion probability
  if (is.null(rho.min)) rho.min <- 1 / (ncol(Z) + 1)

  # Make sure parameter inputs are in the correct format
  check.parameter.format(rho.min = rho.min, rho.max = rho.max, th.range = th.range, max.mod = max.mod, lpen = lpen, eps = eps, bvs.fit0 = bvs.fit0, th.EP = th.EP, D = D)

  # Set covariates that must be included in the model
  if (missing(includevars)) includevars <- rep(FALSE, ncol(D)+ncolI+ncol(A))
  includeX <-  includevars[(ncol(D)+ncolI+1):length(includevars)]
  if (length(includevars) != ncol(D) + ncolI + ncol(A)) stop(paste("includevars has length",length(includevars),"but there are",ncol(D)+ncolI+ncol(A),"columns in (D,I,X). Note: if X is a data.frame, an intercept was automatically added"))

  isct <- (apply(A, 2, 'sd') == 0)
  if (sum(isct) > 1) stop("There are >1 constant columns (e.g. intercepts) in X. Try removing the intercept")
  addintcpt <- sum(isct)==0
  includeX[isct] <- TRUE #always add the intercept (so it doesn't affect EB/EP estimates)
   
  # Estimate coefficients on exposure model
  if (verbose) message("ESTIMATING ASSOCIATION BETWEEN TREATMENTS AND CONTROLS\n\n")
  betad <- exposureCoef(Di=Di, A=A, familyD=familyD, typeofvar=typeofvar, addintcpt=addintcpt, mod1=mod1, priorCoef=priorCoef, Rinit=Rinit, lpen=lpen)
    
  # THETA SEARCH ###############################################################
  # Initial fit: th0 = 0; th1 = 0 (UNIFORM MODEL PRIOR)
  if (is.null(bvs.fit0)) {
    if (verbose) message("INITIAL FIT UNDER UNIFORM MODEL PRIOR \n\n")
    if (is.data.frame(Z)) {
      f <- as.formula(paste('y ~', paste(names(Z), collapse=' + ')))
      data <- cbind(y, Z)
      Zdes <- formatInputdata(f, data=data, family=family)$x
      p <- ncol(Zdes) - ncol(Di)
      isct <- apply(Zdes, 2, var) == 0
      includevars[isct] <- TRUE #always include the intercept (so it doesn't affect EB/EP estimates)
      Dvars <- 2:(ncol(D)+1) #columns in Z corresponding to D (1st column is the intercept)
      bvs.fit0 <- ms(f, data=data, priorCoef = priorCoef, priorVar = igp, priorModel = ufp, niter = Rinit, verbose = verbose, center = center, scale = scale, includevars = includevars)
    } else {
        p <- ncol(Z) - ncol(Di)
        isct <- apply(Z, 2, var) == 0
        includevars[isct] <- TRUE #always include the intercept (so it doesn't affect EB/EP estimates)
        Dvars <- 1:ncol(D)  #columns in Z corresponding to D
        bvs.fit0 <- ms(y, Z, priorCoef = priorCoef, priorVar = igp, priorModel = ufp, niter = Rinit, verbose = verbose, center = center, scale = scale, includevars = includevars)
    }
  }
  # Posterior model probabilities
  pprobs <- postProb(bvs.fit0)[, c(1, 3)]

  # Marginal inclusion probabilities
  pj1 <- bvs.fit0[['margpp']]
  pj1[which(is.nan(pj1))] <- 0  # Potential problems in modelSelection with undefined
  pj1[which(pj1 == 1 & !includevars)] <- 1 - eps  # In case there are extreme values
  pj1[which(pj1 == 0)] <- eps

  # Set of visited models
  max.mod <- min(max.mod, nrow(pprobs))
  G0 <- unname(sapply(as.character(pprobs[1:max.mod, 1]), binary.id, p = p, nt = ncol(Di), conc = TRUE)) # Prevent a dictionary too big

  if (verbose) message("\n ESTIMATING HYPER-PARAMETER VALUES \n\n")
  # Determine the values of hat{theta} #########################################
  # Set limits for optimisation (in principle: unbounded)
  lws <- rep(-Inf, ncol(D) + 1)
  ups <- rep(+Inf, ncol(D) + 1)

  # Range for grid search
  if (is.null(th.range)) th.range <- setThetaRange(ncol(D))

  if (is.null(th.EP)) {
    # EP approximation (to set starting point)
    ths <- vector(ncol(D) + 1, mode = 'list')
    for (i in 1:length(ths)) { ths[[i]] <- th.range }
    EP.is <- grid.opt.mt(G0, betad, pj1, th.grid = expand.grid(ths), method = 'EP', th.prior = th.prior, rho.min = rho.min, rho.max = rho.max, includeX = includeX, Dvars = Dvars)

    # Load gradient functions and optimise
    st <- unlist(EP.is[[1]])
    opt.EP <- nlminb(st, objective = Of.EP.cil, gradient = Gf.EP.cil, 
      lower = lws, upper = ups, pj1 = pj1, betad = betad, 
      th.prior = th.prior, rho.min = rho.min, rho.max = rho.max, includeX = includeX)

    # Set values if there is convergence
    if (opt.EP[['convergence']] %in% 0:1) {
      th.EP <- opt.EP[['par']]
    } else {
      stop('theta search under "EP" did not converge.')
    }
  }

  # Empirical Bayes search (starting at EP solution)
  if (th.search == 'EB') {
    # Model search at EP optimum
    auxprpr <- pinc.mt(betad, th = th.EP, rho.min = rho.min, rho.max = rho.max, includeX = includeX)
    auxprpr <- c(rep(1/2, ncol(betad)), auxprpr)
    auxprpr[which(auxprpr == 1)] <- 1 - eps
    auxprpr[which(auxprpr == 0)] <- eps
    auxmp <- modelbinomprior(p = auxprpr)#modelSelection::modelbinomprior(p = auxprpr)

    if (is.data.frame(Z)) {
      update.fit <- ms(f, data=data, priorCoef=priorCoef, priorVar=igp, priorModel=auxmp, niter=round(R/2), verbose=verbose, center=center, scale=scale, includevars=includevars)
    } else {
      update.fit <- ms(y, Z, priorCoef = priorCoef, priorVar = igp, priorModel = auxmp, niter = round(R/2), verbose = verbose, center = center, scale = scale, includevars = includevars)
    }

    # Append model set
    upprobs <- postProb(update.fit)[, c(1, 3)]
    G1 <- unname(sapply(
      as.character(upprobs[, 1]), binary.id, p = p, nt = ncol(D), conc = TRUE))
    G0 <- unique(c(G0, G1[1:min(max.mod, length(G1))]))

    # Pre-compute marginal likelihoods
    ws <- unname(unlist(sapply(G0, function(m) {
      js <- which(unlist(strsplit(m, '')) == 1)
      return(mlik(js, y, update.fit$xstd, priorCoef = priorCoef, logscale = TRUE)) #David: line below seemed wrong, ws was exponentiated later
      #return(mlik(js, y, Z, priorCoef = priorCoef, logscale = FALSE))
    })))

    # Avoid numerical problems with MLs that are too small
    ws <- exp(ws - max(ws)) # Multiply by constant exp(-max(ws)) (NO EFFECT ON ARG MAX)

    # Optimise starting at the EP optimum
    opt.EB <- try(nlminb(th.EP, objective = Of.EB.cil, gradient = Gf.EB.cil,
      lower = lws, upper = ups, betad = betad, G0 = G0,
      ws = ws, th.prior = th.prior, rho.min = rho.min, rho.max = rho.max, includeX = includeX, Dvars = Dvars), silent = TRUE)
    if (inherits(opt.EB, 'try-error')) {
      opt.EB <- try(nlminb(th.EP, objective = Of.EB.cil,
        gradient = Gf.EB.cil, lower = lws, upper = ups, betad = betad,
        G0 = G0, ws = ws, th.prior = th.prior, rho.min = rho.min,
        rho.max = rho.max, includeX = includeX, Dvars = Dvars))
    }

    # Optimal values
    if (opt.EB[['convergence']] %in% 0:1) {
      th.hat <- opt.EB[['par']]
    } else {
      stop('theta search under th.search=="EB" did not converge.')
    }
  } else if (th.search == 'EP') {
    # Optimal values
    th.hat <- th.EP
  } else {
    stop('th.search method unsupported -- choose amongst "EB" and "EP".')
  }

  # MODEL SEARCH (under fixed theta hat) #######################################
  # Prior probabilities under theta
  pg1cth <- rep(NA, nchar(G0)[1])
  pg1cth[-Dvars] <- pinc.mt(betad, th = th.hat, rho.min = rho.min, rho.max = rho.max, includeX = includeX)
  pg1cth[Dvars] <- 1/2
  #pg1cth[which(pg1cth == 1 & !includevars)] <- 1 - eps

  # Bernoulli model prior with unequal probabilities
  nbp <- modelbinomprior(p = pg1cth)#modelSelection::modelbinomprior(p = pg1cth)
 
  # New model search
  #new.fit <- ms(y, Z, priorCoef = priorCoef, priorVar = igp, priorModel = nbp, niter = R, verbose = verbose, center = center, scale = scale, includevars = includevars)
  if (verbose) message("\n FINAL FIT UNDER ESTIMATED ASYMMETRIC BERNOULLI PRIOR \n\n")
  if (is.data.frame(Z)) {
    new.fit <- ms(f, data=data, priorCoef = priorCoef, priorVar = igp, priorModel = nbp, niter = R, verbose = verbose, center = center, scale = scale, includevars = includevars)
    treatidx <- Dvars
  } else {
    new.fit <- ms(y, Z, priorCoef = priorCoef, priorVar = igp, priorModel = nbp, niter = R, verbose = verbose, center = center, scale = scale, includevars = includevars)
    treatidx <- Dvars + ifelse(all(apply(Z,2,'sd') > 0), 1, 0) #if Z has no intercept, coef(new.fit) will add one automatically
  }

  # BMA inference and Posterior model probabilities
  beta <- coef(new.fit)
  teff <- beta[treatidx,]
  pprobs <- postProb(new.fit)#modelSelection::postProb(new.fit)[, c(1, 3)]
  mpprobs <- new.fit[['margpp']]

  # Finish
  return(list(teff = teff, beta = beta, pprob.y = pprobs, mpprob.y = mpprobs, th.hat = th.hat, priorprobs = pg1cth, 
              msfit = new.fit, init.msfit = bvs.fit0, th.EP = th.EP, mod1.coef = betad, Dvars=Dvars, includeX=includeX, rho.min=rho.min, rho.max=rho.max))
}#; model.pprobs.cil <- compiler::cmpfun(model.pprobs.cil)


################################################################################
check.input.format <- function(y, D, X, I, R) {
################################################################################
  # Format errors that can be encountered
  err1 <- 'argument "y" must be of class "matrix" and contain one column.'
  err2 <- 'argument "D" must have at least one column and the same number of rows as "y".'
  #err2 <- 'argument "D" must be of class "matrix" with at least one column and the same number of rows as "y".'
  err3 <- 'argument "X" must have at least one column and the same number of rows as "y".'
  #err3 <- 'argument "X" must be of class "matrix" with at least one column and the same number of rows as "y".'
  err4 <- 'argument "I" must have at least one column and the same number of rows as "y".'
  #err4 <- 'if argument "I" is informed, it must be of class "matrix" with at least one column and the same number of rows as "y".'
  err5 <- 'parameter "R" must be a single positive integer.'

  # Object "y"
  if (! 'matrix' %in% class(y)) {
    stop(err1)
  } else if (ncol(y) != 1) {
    stop(err1)
  }

  # Object "D"
  if (nrow(D) != nrow(y) | ncol(D) == 0) stop(err2)

  # Object "X"
  if (nrow(X) != nrow(y) | ncol(X) == 0) stop(err3)

  # Object "I"
  if (! is.null(I)) {
    if (nrow(I) != nrow(y) | ncol(I) == 0) stop(err4)
    ncolI= ncol(I)
  } else {
      ncolI= 0
  }
  
  # Parameter "R"
  if (! is.numeric(R) | length(R) != 1) {
    stop(err5)
  } else if (R < 1) {
    stop(err5)
  }

  # NA warnings
  if (any(is.na(y))) { stop('"y" cannot contain NAs.') }
  if (any(is.na(D))) { stop('"D" cannot contain NAs.') }
  if (any(is.na(X))) { stop('"X" cannot contain NAs.') }
  if (! is.null(I) & any(is.na(I))) {
    stop('if informed, "I" cannot contain NAs.')
  }

  # End
  return(format.is.ok = TRUE)
}

################################################################################
cil <- function(y, D, X, I = NULL, family = 'normal', familyD = 'normal', R = 1e4, Rinit = 500, th.search = 'EB',
  mod1 = 'lasso_bic', th.prior = 'unif', priorCoef = momprior(taustd=1), rho.min = NULL, rho.max = 0.95, th.range = NULL, max.mod = 2^20,
  lpen = 'lambda.1se', eps = 1e-10, bvs.fit0 = NULL, th.EP = NULL, center = TRUE, scale = TRUE, includevars, verbose=TRUE) {
################################################################################
  # Assert inputs are in the correct format
  check.input.format(y, D, X, I, R)

  # Posterior model probabilities
  pprobs <- model.pprobs.cil(y, D, X, I, family = family, familyD = familyD, R = R, Rinit = Rinit, th.search = th.search,
    mod1 = mod1, th.prior = th.prior, priorCoef = priorCoef,
    rho.min = rho.min, rho.max = rho.max, th.range = th.range,
    max.mod = max.mod, lpen = lpen, eps = eps, bvs.fit0 = bvs.fit0,
    th.EP = th.EP, center = center, scale = scale, includevars = includevars, verbose = verbose)

  # BMA computation
  #teff <- bma.cil(pprobs[['msfit']], nt = ncol(cbind(D, I)))

  # Output object
  out <- list(cil.teff = pprobs[['teff']],  # BMA estimates, 95\% CIs and marginal posterior prob for treatment variables
              coef = pprobs[['beta']], # BMA estimates, 95\% CI's and marginal posterior probs for all variables
#cil.teff = teff[['bma.e.cil']],  # BMA Point estimates
#              cil.teff.postdist = teff[['bma.d.cil']],  # BMA Distribution
#              cil.bma.mcmc = teff[['mcmc.cil']],  # BMA MCMC runs
              model.postprobs = pprobs[['pprob.y']],  # Model post. probs.
              margpp = pprobs[['mpprob.y']],  # Marginal post. probs under CIL prior
              margprior = pprobs[['priorprobs']], # CIL-estimated marginal prior inclusion probabilities
              margprior.limits = c(pprobs[['rho.min']], pprobs[['rho.max']]), # (lower, upper) limits on margprior
              theta.hat = pprobs[['th.hat']],  # Estimated theta hat
              treat.coefs = pprobs[['mod1.coef']],  # Coefs. treatment models
              treat.varindex = pprobs[['Dvars']], # Indexes of columns in full design matrix corresponding to treatments
              msfit = pprobs[['msfit']],  # Model selection fit of the model
              theta.EP = pprobs[['th.EP']],  # Estimated theta for EP method
              includeX = pprobs[['includeX']], # Covariates forced to be included in the outcome model
              init.msfit = pprobs[['init.msfit']])  # Initial model sel. fit

  # Exit
  return(invisible(new('cilfit', out)))
  #return(invisible(out))
}



################################################################################
# DEPRECATED FUNCTION
#bma.cil <- function(msfit, nt = 1, ret.mcmc = TRUE) {
#################################################################################
#  # Posterior draws
#  draws <- rnlp(msfit = msfit, niter = 1e4)
# 
#  # BMA Treatment Effect Estimates
#  idxs <- 1 + 1:nt
#  if (nt == 1) {
#    teff.est <- mean(draws[, idxs])
#  } else {
#    teff.est <- colMeans(draws[, idxs])
#  }
# 
#  # BMA distribution
#  if (nt == 1) {
#    bma.distr <- quantile(draws[, idxs], seq(0, 1, 0.005))
#  } else {
#    bma.distr <- apply(draws[, idxs], 2, quantile, seq(0, 1, 0.005))
#  }
# 
#  # Output
#  out <- list(bma.e.cil = teff.est, bma.d.cil = bma.distr, mcmc.cil = NA)
#  if (ret.mcmc == TRUE) { out[['mcmc.cil']] <- draws[, 1 + 1:nt] }
# 
#  # End
#  return(invisible(out))
#}



################################################################################
# Adapt relevant methods for objects of class "cilfit"
################################################################################

# plot()
setMethod("plotprior", signature(object='cilfit'), function(object, xlab, ylab, ylim=c(0,1), ...) {
  if (missing(xlab)) xlab <- 'Treatment regression coefficients'
  if (missing(ylab)) ylab <- 'Posterior inclusion probability under uniform model prior'
  ntreat <- ncol(object$treat.coefs)
  treat.varindex <- object$treat.varindex
  bmean <- colMeans(object$treat.coefs)
  b <- t(matrix(bmean, nrow=ntreat, ncol=200))
  for (i in 1:ntreat) {
    if (ntreat > 1) devAskNewPage(ask = TRUE)
    pp.unifprior <- object$init.msfit$margpp[-treat.varindex]
    pp.unifprior <- pp.unifprior[!object$includeX]
    treat.coef <- object$treat.coef[!object$includeX,i]
    isct <- (apply(object$init.msfit$xstd[,,drop=FALSE], 2, 'sd') == 0)[-treat.varindex]
    isct <- isct[!object$includeX]
    pp.unifprior <- pp.unifprior[!isct]
    treat.coef <- treat.coef[!isct]
    plot(treat.coef, pp.unifprior, xlab=xlab, ylab=ylab, ylim=ylim, ...)
    b[,i] <- matrix(seq(min(treat.coef), max(treat.coef), length=200), ncol=1)
    priorprobfit <- pinc.mt(b, th=object$theta.hat, rho.min=object$margprior.limits[1], rho.max=object$margprior.limits[2], includeX=rep(FALSE,length(b)))
    lines(b, priorprobfit)
    b[,i] <- bmean[i]
  }
  devAskNewPage(ask = FALSE)
}
)


# show()
setMethod("show", signature(object='cilfit'), function(object) {
  message('cilfit object with outcome of type',object$msfit$outcometype,',',object$msfit$p,'covariates and',object$msfit$family,'error distribution\n')
  message(" - Use coef() to obtain BMA estimates for treatment variables\n")
  message(" - Use plotprior() to plot the prior inclusion probabilities\n")
  message(" Methods available for 'msfit' objects can be applied to element 'msfit', e.g. coef(object$msfit) returns BMA estimates for all variables\n")
  message(" Elements $margpp and $margprior contain marginal variable posterior and prior inclusion probabilities\n")
}
)


# postProb()
setMethod('postProb', signature(object = 'cilfit'),
  function(object, nmax, method = 'norm') {
    ans <- object[['model.postprobs']]
    if (missing(nmax)) {
      nmax <- nrow(ans)
    }
    return(ans[1:nmax, ])
  }
)

# coef()
# Returns BMA inference for treatment variables. For inference on all parameters use coef(object$msfit)
coef.cilfit <- function(object, ...) {
  return(object$cil.teff)
  #qls <- c(0.025, 0.975)
  #pes <- as.matrix(object[['cil.teff']], ncol = 1)
  #if (length(object[['cil.teff']]) == 1) {
  #  cis <- quantile(object[['cil.teff.postdist']], probs = qls)
  #} else {
  #  cis <- apply(object[['cil.teff.postdist']], 2, quantile, probs = qls)
  #}
  #mpp <- as.matrix(object[['marg.postprobs']][1:length(pes)], ncol = 1)
  #out <- cbind(pes, t(as.matrix(cis, nrow = 1, ncol = 2)), mpp)
  #colnames(out) <- c('estimate', '2.5%', '97.5%', 'margpp')
  #rownames(out) <- paste('treat', 1:nrow(out), sep = '')
  #return(out)
}

# # coef()
# setMethod('coef', signature(object = 'cilfit'), function(object) {
#   qls <- c(0.025, 0.975)
#   pes <- as.matrix(object[['cil.teff']], ncol = 1)
#   if (length(object[['cil.teff']]) == 1) {
#     cis <- quantile(object[['cil.teff.postdist']], probs = qls)
#   } else {
#     cis <- apply(object[['cil.teff.postdist']], 2, quantile, probs = qls)
#   }
#   mpp <- as.matrix(object[['marg.postprobs']][1:length(pes)], ncol = 1)
#   out <- cbind(pes, t(as.matrix(cis, nrow = 1, ncol = 2)), mpp)
#   colnames(out) <- c('estimate', '2.5%', '97.5%', 'margpp')
#   rownames(out) <- paste('treat', 1:nrow(out), sep = '')
#   return(out)
# })
# END OF SCRIPT

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

rnorm_truncMultCI <- function(n, ltrunc, rtrunc, m, s) {
    .Call(`_modelSelection_rnorm_truncMultCI`, n, ltrunc, rtrunc, m, s)
}

rtmvnormCI <- function(n, mu, Sigma, lower, upper, within, method) {
    .Call(`_modelSelection_rtmvnormCI`, n, mu, Sigma, lower, upper, within, method)
}

rtmvnormProdCI <- function(n, mu, Sigma, k, lower, upper, is_low_trunc, is_up_trunc, burnin) {
    .Call(`_modelSelection_rtmvnormProdCI`, n, mu, Sigma, k, lower, upper, is_low_trunc, is_up_trunc, burnin)
}

rnlpPostCI_lm <- function(niter, burnin, thinning, y, x, p, r, tau, a_phi, b_phi, prior) {
    .Call(`_modelSelection_rnlpPostCI_lm`, niter, burnin, thinning, y, x, p, r, tau, a_phi, b_phi, prior)
}

rnlpCI <- function(niter, burnin, thinning, m, V, p, r, tau, prior) {
    .Call(`_modelSelection_rnlpCI`, niter, burnin, thinning, m, V, p, r, tau, prior)
}

modelSelectionGGMC <- function(y, prCoef, prModel, samplerPars, Omegaini) {
    .Call(`_modelSelection_modelSelectionGGMC`, y, prCoef, prModel, samplerPars, Omegaini)
}

modelSelectionGGM_globalC <- function(y, prCoef, prModel, samplerPars, Omegaini) {
    .Call(`_modelSelection_modelSelectionGGM_globalC`, y, prCoef, prModel, samplerPars, Omegaini)
}

hugeglasso <- function(S, lambda, scr, verbose, cov_output) {
    .Call(`_modelSelection_hugeglasso`, S, lambda, scr, verbose, cov_output)
}

marginalLikelihoodCI <- function(Sknownphi, Ssel, Snsel, Sfamily, SpriorCoef, SpriorGroup, Sn, Sp, Sy, Suncens, Ssumy2, Ssumy, Ssumlogyfact, Sx, Scolsumsx, SXtX, SytX, Smethod, Sadjoverdisp, Shesstype, SoptimMethod, Soptim_maxit, Sthinit, Susethinit, SB, Salpha, Slambda, Stau, Staugroup, Staualpha, Sfixatanhalpha, Sr, Sa, Sgroups, Sngroups, Snvaringroup, Sconstraints, Sinvconstraints, SDmat, Slogscale) {
    .Call(`_modelSelection_marginalLikelihoodCI`, Sknownphi, Ssel, Snsel, Sfamily, SpriorCoef, SpriorGroup, Sn, Sp, Sy, Suncens, Ssumy2, Ssumy, Ssumlogyfact, Sx, Scolsumsx, SXtX, SytX, Smethod, Sadjoverdisp, Shesstype, SoptimMethod, Soptim_maxit, Sthinit, Susethinit, SB, Salpha, Slambda, Stau, Staugroup, Staualpha, Sfixatanhalpha, Sr, Sa, Sgroups, Sngroups, Snvaringroup, Sconstraints, Sinvconstraints, SDmat, Slogscale)
}

normalmixGibbsCI <- function(Sx, Sn, Sp, Sncomp, Sz, Smu0, Sg, Snu0, SS0, Sq, SB, Sburnin, Sverbose) {
    .Call(`_modelSelection_normalmixGibbsCI`, Sx, Sn, Sp, Sncomp, Sz, Smu0, Sg, Snu0, SS0, Sq, SB, Sburnin, Sverbose)
}

testfunctionCI <- function(A, oldcol, newcol) {
    .Call(`_modelSelection_testfunctionCI`, A, oldcol, newcol)
}

eprod_I <- function(m, S, n, power, dof) {
    .Call(`_modelSelection_eprod_I`, m, S, n, power, dof)
}

modelSelectionEnumCI <- function(Snmodels, Smodels, Sknownphi, Sfamily, SpriorCoef, SpriorGroup, Sn, Sp, Sy, Suncens, Ssumy2, Ssumy, Ssumlogyfact, Sx, Scolsumsx, ShasXtX, SXtX, SytX, Smethod, Sadjoverdisp, Shesstype, SoptimMethod, Soptim_maxit, Sthinit, Susethinit, SB, Salpha, Slambda, Sphi, Stau, Staugroup, Staualpha, Sfixatanhalpha, Sr, Sa, SpriorModel, SprDeltap, SparprDeltap, SpriorConstr, SprConstrp, SparprConstrp, Sgroups, Sngroups, Snvaringroup, Sconstraints, Sinvconstraints, SDmat, Sverbose) {
    .Call(`_modelSelection_modelSelectionEnumCI`, Snmodels, Smodels, Sknownphi, Sfamily, SpriorCoef, SpriorGroup, Sn, Sp, Sy, Suncens, Ssumy2, Ssumy, Ssumlogyfact, Sx, Scolsumsx, ShasXtX, SXtX, SytX, Smethod, Sadjoverdisp, Shesstype, SoptimMethod, Soptim_maxit, Sthinit, Susethinit, SB, Salpha, Slambda, Sphi, Stau, Staugroup, Staualpha, Sfixatanhalpha, Sr, Sa, SpriorModel, SprDeltap, SparprDeltap, SpriorConstr, SprConstrp, SparprConstrp, Sgroups, Sngroups, Snvaringroup, Sconstraints, Sinvconstraints, SDmat, Sverbose)
}

modelSelectionGibbsCI <- function(SpostModeini, SpostModeiniProb, Sknownphi, Sfamily, SpriorCoef, SpriorGroup, Sniter, Sthinning, Sburnin, Sndeltaini, Sdeltaini, Sincludevars, Sn, Sp, Sy, Suncens, Ssumy2, Ssumy, Ssumlogyfact, Sx, Scolsumsx, ShasXtX, SXtX, SytX, Smethod, Sadjoverdisp, Shesstype, SoptimMethod, Soptim_maxit, Sthinit, Susethinit, SB, Salpha, Slambda, Sphi, Stau, Staugroup, Staualpha, Sfixatanhalpha, Sr, Sa, SpriorModel, SprDeltap, SparprDeltap, SpriorConstr, SprConstrp, SparprConstrp, Sgroups, Sngroups, Snvaringroup, Sconstraints, Sinvconstraints, Smaxvars, SDmat, Sverbose) {
    .Call(`_modelSelection_modelSelectionGibbsCI`, SpostModeini, SpostModeiniProb, Sknownphi, Sfamily, SpriorCoef, SpriorGroup, Sniter, Sthinning, Sburnin, Sndeltaini, Sdeltaini, Sincludevars, Sn, Sp, Sy, Suncens, Ssumy2, Ssumy, Ssumlogyfact, Sx, Scolsumsx, ShasXtX, SXtX, SytX, Smethod, Sadjoverdisp, Shesstype, SoptimMethod, Soptim_maxit, Sthinit, Susethinit, SB, Salpha, Slambda, Sphi, Stau, Staugroup, Staualpha, Sfixatanhalpha, Sr, Sa, SpriorModel, SprDeltap, SparprDeltap, SpriorConstr, SprConstrp, SparprConstrp, Sgroups, Sngroups, Snvaringroup, Sconstraints, Sinvconstraints, Smaxvars, SDmat, Sverbose)
}

greedyVarSelCI <- function(Sknownphi, Sfamily, SpriorCoef, SpriorGroup, Sniter, Sndeltaini, Sdeltaini, Sincludevars, Sn, Sp, Sy, Suncens, Ssumy2, Ssumy, Ssumlogyfact, Sx, Scolsumsx, ShasXtX, SXtX, SytX, Smethod, Sadjoverdisp, Shesstype, SoptimMethod, Soptim_maxit, Sthinit, Susethinit, SB, Salpha, Slambda, Sphi, Stau, Staugroup, Staualpha, Sfixatanhalpha, Sr, Sa, SpriorModel, SprDeltap, SparprDeltap, SpriorConstr, SprConstrp, SparprConstrp, Sgroups, Sngroups, Snvaringroup, Sconstraints, Sinvconstraints, Smaxvars, SDmat, Sverbose) {
    .Call(`_modelSelection_greedyVarSelCI`, Sknownphi, Sfamily, SpriorCoef, SpriorGroup, Sniter, Sndeltaini, Sdeltaini, Sincludevars, Sn, Sp, Sy, Suncens, Ssumy2, Ssumy, Ssumlogyfact, Sx, Scolsumsx, ShasXtX, SXtX, SytX, Smethod, Sadjoverdisp, Shesstype, SoptimMethod, Soptim_maxit, Sthinit, Susethinit, SB, Salpha, Slambda, Sphi, Stau, Staugroup, Staualpha, Sfixatanhalpha, Sr, Sa, SpriorModel, SprDeltap, SparprDeltap, SpriorConstr, SprConstrp, SparprConstrp, Sgroups, Sngroups, Snvaringroup, Sconstraints, Sinvconstraints, Smaxvars, SDmat, Sverbose)
}

pmomMarginalKI <- function(Ssel, Snsel, Sn, Sp, Sy, Ssumy2, SXtX, SytX, Sphi, Stau, Sr, Smethod, SB, Slogscale, Sngroups, Snvaringroup) {
    .Call(`_modelSelection_pmomMarginalKI`, Ssel, Snsel, Sn, Sp, Sy, Ssumy2, SXtX, SytX, Sphi, Stau, Sr, Smethod, SB, Slogscale, Sngroups, Snvaringroup)
}

pmomMarginalUI <- function(Ssel, Snsel, Sn, Sp, Sy, Ssumy2, Sx, SXtX, SytX, Stau, Sr, Smethod, SB, Slogscale, Salpha, Slambda, Sngroups, Snvaringroup) {
    .Call(`_modelSelection_pmomMarginalUI`, Ssel, Snsel, Sn, Sp, Sy, Ssumy2, Sx, SXtX, SytX, Stau, Sr, Smethod, SB, Slogscale, Salpha, Slambda, Sngroups, Snvaringroup)
}

pimomMarginalKI <- function(Ssel, Snsel, Sn, Sp, Sy, Ssumy2, SXtX, SytX, Sphi, Stau, Smethod, SB, Slogscale, Sngroups, Snvaringroup) {
    .Call(`_modelSelection_pimomMarginalKI`, Ssel, Snsel, Sn, Sp, Sy, Ssumy2, SXtX, SytX, Sphi, Stau, Smethod, SB, Slogscale, Sngroups, Snvaringroup)
}

pimomMarginalUI <- function(Ssel, Snsel, Sn, Sp, Sy, Ssumy2, Sx, SXtX, SytX, Stau, Smethod, SB, Slogscale, Salpha, Slambda, Sngroups, Snvaringroup) {
    .Call(`_modelSelection_pimomMarginalUI`, Ssel, Snsel, Sn, Sp, Sy, Ssumy2, Sx, SXtX, SytX, Stau, Smethod, SB, Slogscale, Salpha, Slambda, Sngroups, Snvaringroup)
}

bsplineCI <- function(x, degree, knots) {
    .Call(`_modelSelection_bsplineCI`, x, degree, knots)
}

# Estimate GLM parameters under the full model
# - y: response variable
# - x: design matrix
# - family: 'normal', 'binomial', 'poisson'. For families 'laplace', 'twopiecenormal', 'twopiecelaplace', initialization is based on the normal model
# - initpar: method to initialize the parameters. It can be 'MLE', 'L1', 'MLE-aisgd', 'L2-aisgd'. If missing, it uses MLE when p <= n/2 and L1 when p > n/2. If p>200 or n>10,000 then ai-sgd is used (averaged intrinsic stochatic gradient descent)
initParameters <- function(y, x, family, initpar) {
    n= length(y); p= ncol(x)
    if (!(initpar %in% c('none','auto','MLE','L1'))) stop("initpar must be 'none', 'auto', 'MLE'")
    #Disabled sgd since sgd package was discontinued
    #if (!(initpar %in% c('none','auto','MLE','MLE-aisgd','L1','L2-aisgd'))) stop("initpar must be 'none', 'auto', 'MLE', 'MLE-aisgd', 'L1' or 'L2-aisgd'")
    if (initpar =='auto') {
        initpar= ifelse(p <= n/2, 'MLE', 'L1')
        #large= (n > 10000) || (p > 200)
        #if ((p<=n/2) && (!large)) {
        #    initpar= 'MLE'
        #} else if ((p<=n/2) && large) {
        #    initpar= 'MLE-aisgd'
        #} else if ((p>n/2) && (!large)) {
        #    initpar= 'L1'
        #} else {
        #    initpar= 'L2-aisgd'
        #}
    }
    if (!(family %in% c('binomial','poisson'))) family= 'gaussian'
    if (initpar == 'none') {
        ans= rep(0,p)
    } else if (initpar == 'MLE') {
        fit= glm(y ~ x -1, family=family)
        ans= coef(fit)
    #} else if (initpar == 'MLE-aisgd') {
    #    fit= sgd(x=x, y=y, model="glm", model.control=list(family=family), sgd.control=list(method="ai-sgd"))
    #    ans= coef(fit)
    }  else if (initpar == 'L1') {
        int= (colSums(x==1) == nrow(x))
        fit= glmnet::glmnet(x=x[,!int], y=y, intercept=TRUE, family=family, nlambda=50, alpha=1)
        b= matrix(NA, nrow=ncol(x), ncol=length(fit$lambda))
        b[!int,]= as.matrix(coef(fit)[-1,])
        b[int,]= coef(fit)[1,]
        bicvals= glmBIC(b, y=y, x=x, family=family)
        ans= coef(fit)[,which.min(bicvals)]
    #} else if (initpar == 'L1-aisgd') {
        #Function sgd does not appear to return correct results for aisgd with L1
      #} else if (initpar == 'L2-aisgd') {
      #  #Ridge penalty
      #  fit= sgd(x=x, y=y, model="glm", model.control=list(family=family, lambda2=0.001), sgd.control=list(method="ai-sgd"))
      #  ans= coef(fit)
    #} else stop("Invalid 'initpar'. Allowed values are 'MLE', 'L1', 'MLE-aisgd', 'L2-aisgd'")
    } else stop("Invalid 'initpar'. Allowed values are 'MLE', 'L1'")
    return(ans)    
}


#Wrapper to initParameters, returning 0 when initpar=='none'
getthinit = function(y, x, family, initpar, enumerate) {
  if ((!missing(initpar)) && is.numeric(initpar)) {
      thinit= as.double(initpar)
      if (length(thinit) != ncol(x)) stop(paste("x has",ncol(x),"columns but initpar has",length(thinit),"elements"))
  } else {
      if ((!missing(initpar)) && (initpar=='none')) {
          usethinit= as.integer(ifelse(enumerate, 0, 1)) #tell C code to not init or do its own init
      } else {
          usethinit= as.integer(3) #tell C code to use thinit
      }
      thinit= as.double(initParameters(y=y, x=x, family=family, initpar=initpar))
  }
  ans= list(thinit=thinit, usethinit=usethinit)
  return(ans)
}


#Obtain BIC for GLMs
# - beta: regression coefficients. Either a vector with length equal to ncol(x), or a matrix with nrow equal to ncol(x) and different estimates in the columns of beta
# - y: response variable
# - x: design matrix
# - family: 'binomial', 'poisson' or otherwise the normal model is used
glmBIC = function(beta, y, x, family) {
    n= length(y); p= ncol(x)
    if (is.vector(beta)) {
        beta= matrix(beta,ncol=1)
        npars= sum(beta!=0)
    } else {
        npars= colSums(beta!=0)
    }
    lp= x %*% beta
    if (family=='binomial') {
        logl= double(ncol(lp))
        for (i in 1:ncol(lp)) logl[i]= logL_logreg(beta[,i], ytX= t(y) %*% x, Xbeta=lp[,i], n=n, X=x, logscale=TRUE)
        ans = -2 * logl + log(n)*p
    } else if (family=='poisson') {
        logl= double(ncol(lp))
        for (i in 1:ncol(lp)) logl[i]= logL_poisreg(beta[,i], ytX= t(y) %*% x, Xbeta=lp[,i], n=n, X=x, sumlfactorialy=sum(lfactorial(y)), logscale=TRUE)
        ans = -2 * logl + log(n)*p
    } else {
        ans= n * log(colSums((y-lp)^2)/n) + n*(log(2*pi)+1) + log(n)*p
    }
    return(ans)
}


###########################################################################################
# AUXILIARY FUNCTIONS FOR LOGISTIC REGRESSION
###########################################################################################

logit= function(z) log(z/(1-z))  #logit function
expit= function(z) 1/(1+exp(-z)) #inverse of logit function

#Negative logistic regression log-likelihood
logL_logreg= function(beta, ytX, Xbeta, n, X, logscale=TRUE) {
    if (any(beta != 0)) {
        if (missing(Xbeta)) Xbeta= as.vector(X %*% matrix(beta,ncol=1))
        ans= sum(ytX * beta) - sum(log(1+exp(Xbeta)))
    } else {
        if (missing(n)) stop("If beta==0, n must be specified")
        ans= -n * log(2)
    }
    if (!logscale) ans= exp(ans)
    return(ans)
}



###########################################################################################
# AUXILIARY FUNCTIONS FOR POISSON REGRESSION
###########################################################################################

#Negative Poisson regression log-likelihood
logL_poisreg= function(beta, ytX, Xbeta, n, X, sumlfactorialy, logscale=TRUE) {
    #if (missing(sumlfactorialy)) sumlfactorialy= sum(lfactorial(y))
    if (any(beta != 0)) {
        if (missing(Xbeta)) Xbeta= as.vector(X %*% matrix(beta,ncol=1))
        ans= sum(ytX * beta) + sum(exp(Xbeta)) - sumlfactorialy
    } else {
        if (missing(n)) stop("If beta==0, n must be specified")
        ans= - n - sumlfactorialy
    }
    if (!logscale) ans= exp(ans)
    return(ans)
}

###
### AllGenerics.R
###

if (!isGeneric("coefByModel")) {
  setGeneric("coefByModel", function(object, maxmodels, alpha=0.05, niter=10^3, burnin=round(niter/10)) standardGeneric("coefByModel"))
}

if (!isGeneric("coefOneModel")) {
  setGeneric("coefOneModel", function(y, x, m, V, outcometype, family, priorCoef, priorGroup, priorVar, alpha=0.05, niter=10^3, burnin=round(niter/10)) standardGeneric("coefOneModel"))
}


if (!isGeneric("dpimom")) {
    setGeneric("dpimom",
               function(x,
                        tau=1,
                        phi=1,
                        logscale=FALSE) standardGeneric("dpimom"))

}

if (!isGeneric("dpmom")) {
    setGeneric("dpmom",
               function(x,
                        tau,
                        a.tau,
                        b.tau,
                        phi=1,
                        r=1,
                        baseDensity='normal',
                        logscale=FALSE) standardGeneric("dpmom"))
}

if (!isGeneric("demom")) {
    setGeneric("demom",
               function(x,
                        tau,
                        a.tau,
                        b.tau,
                        phi=1,
                        logscale=FALSE) standardGeneric("demom"))
}


if (!isGeneric("getAIC")) {
  setGeneric("getAIC", function(object) standardGeneric("getAIC"))
}

if (!isGeneric("getBIC")) {
  setGeneric("getBIC", function(object) standardGeneric("getBIC"))
}

if (!isGeneric("getEBIC")) {
  setGeneric("getEBIC", function(object) standardGeneric("getEBIC"))
}

if (!isGeneric("getIC")) {
  setGeneric("getIC", function(object) standardGeneric("getIC"))
}


if (!isGeneric("marginalIW")) {
  setGeneric("marginalNIW", function(x, xbar, samplecov, n, z, g,  mu0=rep(0,ncol(x)), nu0=ncol(x)+4, S0,logscale=TRUE) standardGeneric("marginalNIW"))
}

if (!isGeneric("plotprior")) {
  setGeneric("plotprior", function(object, xlab, ylab, ylim=c(0,1), ...) standardGeneric("plotprior"))
}

if (!isGeneric("logjoint")) {
  setGeneric("logjoint", function(object, return_models=TRUE, models_as_char=FALSE) standardGeneric("logjoint"))
}

if (!isGeneric("postProb")) {
  setGeneric("postProb", function(object, nmax, method='norm') standardGeneric("postProb"))
}

if (!isGeneric("postProbSubset")) {
  setGeneric("postProbSubset", function(object, varsubset, nmax, method='norm') standardGeneric("postProbSubset"))
}

if (!isGeneric("marglhood_acrossmodels")) {
  setGeneric("marglhood_acrossmodels", function(object, logscale=TRUE) standardGeneric("marglhood_acrossmodels"))
}

if (!isGeneric("postSamples")) {
    setGeneric("postSamples", function(object) standardGeneric("postSamples"))
}



if (!isGeneric("rnlp")) {
  setGeneric("rnlp", function(y, x, m, V, msfit, outcometype, family, priorCoef, priorGroup, priorVar, priorprec, isgroup, niter=10^3, burnin=round(niter/10), thinning=1, pp='norm') standardGeneric("rnlp"))
}

modelSelectionGLM= function(y, x, data, smoothterms, nknots=9, groups=1:ncol(x), constraints, center=TRUE, scale=TRUE, includevars=rep(FALSE,ncol(x)), maxvars, familyglm= gaussian(), priorCoef, priorGroup, priorVar, priorModel= modelbbprior(1,1), models, verbose= TRUE) {
    #Enumerate all or specified models and return posterior prob as approximated by BIC. Enumerated models satisfy group and hierarchical restrictions from factors and interaction terms
    #Input
    # - data: data.frame with a column named 'y' containing the response
    # - family: type of GLM, passed onto function 'glm'
    # - maxvars: enumeration is limited to models with <=maxvars
    # - verbose: set to TRUE to display progress information
    # - models: logical matrix with ncol(x) columns selecting variables for each included model
    # Output: object of class msfit, as returned by modelSelection in mombf

    if (missing(priorCoef)) priorCoef= bic()
    if (missing(priorGroup)) priorGroup= bic()
    if (missing(priorVar)) priorVar= igprior(0,0)
    if ((priorCoef@priorDistr != 'bic') | (priorGroup@priorDistr != 'bic')) stop("Only priorCoef=bic() and priorGroup=bic() currently implemented")

    enumerate= TRUE #currently only full enumeration is supported
    family= paste(familyglm$family,familyglm$link)
    familyint= -1 #argument currently ignored, set to value != 0 to prevent inference on the family

    ################# Start of code copied from modelSelection #################

    #Check input
    tmp <- formatInputdata(y=y,x=x,data=data,smoothterms=smoothterms,nknots=nknots,family=family)
    x <- tmp$x; y <- tmp$y; formula <- tmp$formula;
    splineDegree <- tmp$splineDegree
    if (!is.null(tmp$groups)) groups <- tmp$groups
    isgroup <- tmp$isgroup
    if (!is.null(tmp$constraints)) constraints <- tmp$constraints
    outcometype <- tmp$outcometype; uncens <- tmp$uncens; ordery <- tmp$ordery
    typeofvar <- tmp$typeofvar
    call <- list(formula=formula, smoothterms= NULL, splineDegree=splineDegree, nknots=nknots)
    if (!missing(smoothterms)) call$smoothterms <- smoothterms
    p= ncol(x); n= length(y)
    if (is.numeric(includevars)) {
        tmp= rep(FALSE,p)
        if (max(includevars) > p) stop(paste("includevars contains index ",max(includevars)," but the design matrix only has ",p," columns",sep=""))
        tmp[includevars]= TRUE
        includevars= tmp
    }
    if (length(includevars)!=ncol(x) | (!is.logical(includevars))) stop("includevars must be a logical vector of length ncol(x)")
    if (missing(maxvars)) maxvars= ifelse(family=='auto', p+2, p)
    if (maxvars <= sum(includevars)) stop("maxvars must be >= sum(includevars)")

    #If there are variable groups, count variables in each group, indicate 1st variable in each group, convert group and constraint labels to integers 0,1,...
    tmp= codeGroupsAndConstraints(p=p,groups=groups,constraints=constraints)
    ngroups= tmp$ngroups; constraints= tmp$constraints; invconstraints= tmp$invconstraints; nvaringroup=tmp$nvaringroup; groups=tmp$groups

    #Standardize (y,x) to mean 0 and variance 1
    if (!is.vector(y)) { y <- as.double(as.vector(y)) } else { y <- as.double(y) }
    if (!is.matrix(x)) x <- as.matrix(x)
    mx= colMeans(x); sx= sqrt(colMeans(x^2) - mx^2) * sqrt(n/(n-1))
    ct= (sx==0)
    if (any(is.na(ct))) stop('x contains NAs, this is currently not supported, please remove the NAs')
    if (sum(ct)>1) stop('There are >1 constant columns in x (e.g. two intercepts)')
    if (!center) { my=0; mx= rep(0,p) } else { my= mean(y) }
    if (!scale) { sy=1; sx= rep(1,p) } else { sy= sd(y) }
    mx[typeofvar=='factor']=0; sx[typeofvar=='factor']= 1
    if (!(outcometype %in% c('Continuous','Survival'))) { my=0; sy= 1 }
    ystd= (y-my)/sy; xstd= x; xstd[,!ct]= t((t(x[,!ct]) - mx[!ct])/sx[!ct])
    #################    End of code copied from modelSelection #################

    #if (missing(phi)) { knownphi <- as.integer(0); phi <- double(0) } else { knownphi <- as.integer(1); phi <- as.double(phi) }
    stdconstants= rbind(c(my,sy),cbind(mx,sx)); colnames(stdconstants)= c('shift','scale')

    #Enumerate models
    if (missing(models)) {
      if (familyint==0) { includeenum= c(includevars[groups+1],FALSE,FALSE) } else { includeenum= includevars[groups+1] }
      models= listmodels(vars2list=1:ngroups, includevars=includeenum, constraints=sapply(constraints,function(z) z+1), nvaringroup=nvaringroup, maxvars=maxvars) #listmodels expects group indexes 1,2,...
      if (familyint==0) models= rbind(cbind(models,FALSE,FALSE),cbind(models,FALSE,TRUE),cbind(models,TRUE,FALSE),cbind(models,TRUE,TRUE))
    } else {
      if (!is.matrix(models) | !is.logical(models)) stop("models must be a matrix of logicals")
      if (dim(models)[2] != ncol(x)) stop("models must have ncol(x) columns")
    }
    nmodels= nrow(models)
    postmean= postvar= matrix(0, nrow=nmodels, ncol=ncol(xstd))
    bicmodel= double(nmodels)
    if (verbose) message("Enumerating ",nmodels," models")
    iterprogress= round(nmodels/10)
    for (i in 1:nmodels) {
      sel= models[i,]
      if (any(sel)>0) {
        fit1= glm(y ~ -1 + xstd[,sel], data=data, family=familyglm)
      } else { fit1= glm(y ~ -1, data=data, family=familyglm) }
      pm= postmomentsGLM(fit1, priorCoef=priorCoef, priorGroup=priorGroup, isgroup=isgroup)
      postmean[i,sel]= pm$m
      postvar[i,sel]= diag(pm$S)
      bicmodel[i]= BIC(fit1)
      if (verbose && ((i %% iterprogress)==0)) message('.')
    }
    postProb= -bicmodel/2

    #Add model prior probabilities
    if (priorModel@priorDistr == "uniform") {
        priorp= apply(models, 1, function(z) { unifPrior(sel=z, logscale=TRUE, groups=groups, constraints=constraints) })
    } else if (priorModel@priorDistr == "binomial") {
        if ('p' %in% names(priorModel@priorPars)) {
            priorp= apply(models, 1, function(z) { binomPrior(sel=z, prob=priorModel@priorPars[['p']], logscale=TRUE, groups=groups, constraints=constraints) })
        } else if (all(c('alpha.p','beta.p') %in% names(priorModel@priorPars))) {
            priorp= apply(models, 1, function(z) { bbPrior(sel=z, alpha=priorModel@priorPars['alpha.p'], beta=priorModel@priorPars['beta.p'], logscale=TRUE, groups=groups, constraints=constraints) })
        } else { stop("priorModel not recognized") }
    }
    postProb= postProb + priorp
    postMode= models[which.max(postProb),]
    postModeProb= max(postProb)

    #Format output
    postSample= matrix(nrow=0,ncol=p)
    pp= exp(postProb-postModeProb); pp= matrix(pp/sum(pp),ncol=1)
    margpp= as.vector(t(models) %*% pp)
    modelid= apply(models[,1:ncol(xstd),drop=FALSE]==1, 1, function(z) paste(which(z),collapse=','))
    models= data.frame(modelid=modelid,family=family,pp=pp)
    rownames(postmean)= rownames(postvar)= modelid
    colnames(postmean)= colnames(postvar)= colnames(xstd)
    o= order(models$pp,decreasing=TRUE)
    models= models[o,]; postmean= postmean[o,]; postvar= postvar[o,]

    priors= list(priorCoef=priorCoef, priorGroup=priorGroup, priorModel=priorModel, priorConstraints=priorModel, priorVar=priorVar, priorSkew=NULL)
    names(constraints)= paste('group',0:(length(constraints)-1))
    ans= list(postSample=postSample,margpp=margpp,postMode=postMode,postModeProb=postModeProb,postProb=postProb,postmean=postmean,postvar=postvar,family=family,p=ncol(xstd),enumerate=enumerate,priors=priors,ystd=ystd,xstd=xstd,groups=groups,constraints=constraints,stdconstants=stdconstants,outcometype=outcometype,call=call)
    ans$models= models
    new("msfit",ans)
}

#######################################################################
##
## ASYMMETRIC LAPLACE ROUTINES
##
#######################################################################

dalapl <- function(x, th=0, scale=1, alpha=0, logscale=FALSE) {
  ans <- -abs(th-x)/(sqrt(scale)*ifelse(x<th,1+alpha,1-alpha)) - 0.5*log(scale) - log(2)
  if (!logscale) ans <- exp(ans)
  return(ans)
}

palapl <- function(x, th=0, scale=1, alpha=0) {
    ans <- ifelse(x<=th, exp(-abs(x-th)/(sqrt(scale)*(1+alpha))) * (1+alpha)/2, (1+alpha)/2 + (1 - exp(-abs(x-th)/(sqrt(scale)*(1-alpha)))) * (1-alpha)/2)
    return(ans)
}

ralapl <- function(n, th=0, scale=1, alpha=0) {
    s <- runif(n) < palapl(0,scale=scale,alpha=alpha)
    ans <- double(n)
    ans[s] <- th - rexp(sum(s)) * sqrt(scale)*(1+alpha)
    ans[!s] <- th + rexp(sum(!s)) * sqrt(scale)*(1-alpha)
    return(ans)
}

### Methods for icfit objects

setMethod("show", signature(object='icfit'), function(object) {
  cat("icfit object\n\n")
  cat("Model with best ",object$criterion,": ",paste(object$topmodel, collapse=' '),"\n\n")
  cat("Use summary(), coef() and predict() to get inference for the top model\n")
  cat("Use coef(object$msfit) and predict(object$msfit) to get BMA estimates and predictions\n")
}
)

confint.icfit <- function(object, ...) {
    confint(object$topmodel.fit)
}

#Return non-zero coefficient estimates
coef.icfit <- function(object,...) {
    coef(object$topmodel.fit)
}

#Return coefficient estimates for all covariates, including the non-zeroes
coefall <- function(object) {  UseMethod("coefall") }

coefall.icfit <- function(object) {
  b <- rep(0, length(object$varnames))
  names(b) <- object$varnames
  b[object$topmodel] <- coef(object$topmodel.fit)    
  return(b)
}


predict.icfit <- function(object, ...) {
    predict(object$topmodel.fit, ...)
}

summary.icfit <- function(object, ...) {
    summary(object$topmodel.fit, ...)
}




## FIND THE MODEL ATTAINING THE BEST VALUE OF AN INFORMATION CRITERION

checkargs_IC <- function(...) {
    params= eval(quote(list(...)))
    forbiddenpars= c('priorCoef','priorModel','center','scale')
    if (any(forbiddenpars %in% names(params))) stop(paste("Arguments",paste(forbiddenpars,collapse=", "),"have set values so they cannot be passed on to modelSelection"))
}


topmodelnames <- function(ans) {
    topvarids= as.numeric(strsplit(ans$models$modelid[1], ',')[[1]])
    ans= list(topvarids= topvarids, varnames= ans$varnames[topvarids])
    return(ans)
}

family2glm <- function(family) {
    if (family=="normal") {
        ans= gaussian()
    } else if (family=="binomial") {
        ans= binomial()
    } else if (family=="poisson") {
        ans= poisson()
    } else stop("Only the Normal, Binomial and Poisson families are currently implemented")
    return(ans)
}


#Extract info criteria from an msfit and return icfit object
extractmsIC <- function(ms, getICfun) {
    ans= vector("list",5)
    names(ans)= c('topmodel','topmodel.fit','models','varnames','msfit')
    ans$models= getICfun(ms)
    if (!is.null(colnames(ms$xstd))) {
        ans$varnames= colnames(ms$xstd)
    } else {
        ans$varnames= paste('x[,',1:ncol(ms$xstd),']',sep='')
    }
    tm= topmodelnames(ans)
    ans$topmodel= tm$varnames
    ans$msfit= ms
    data= data.frame(y=ms$ystd, ms$xstd[,tm$topvarids])
    names(data)[-1]= ans$varnames[tm$topvarids]
    f= formula(y ~ -1 + .)
    ans$topmodel.fit= glm(f , data=data, family=family2glm(ms$family))
    new("icfit", ans)
}

#Combine two icfit objects into a single one
combineICfit <- function(fit1, fit2) {
    if (any(fit1$varnames != fit2$varnames)) stop("Cannot combine two icfit objects with different varnames")
    ans= vector("list",5)
    names(ans)= c('topmodel','topmodel.fit','models','varnames','msfit')
    ans$varnames = fit1$varnames
    ans$models <- rbind(fit1$models, fit2$models)
    ans$models <- dplyr::arrange(ans$models, ans$models[[2]])  #sort increasingly by value of the information criterion
    if (fit1$models[1,2] < fit2$models[1,2]) {
        ans$topmodel = fit1$topmodel
        ans$topmodel.fit = fit1$topmodel.fit
    } else {
        ans$topmodel = fit2$topmodel
        ans$topmodel.fit = fit2$topmodel.fit
    }
    new("icfit", ans)
}



bestBIC <- function(...) {
    checkargs_IC(...)
    ms= modelSelection(..., priorCoef=bic(), priorModel=modelunifprior(), center=FALSE, scale=FALSE)
    ans= extractmsIC(ms, getBIC)
    ans$criterion= 'BIC'
    return(ans)
}

bestAIC <- function(...) {
    checkargs_IC(...)
    ms= modelSelection(..., priorCoef=aic(), priorModel=modelunifprior(), center=FALSE, scale=FALSE)
    ans= extractmsIC(ms, getAIC)
    ans$criterion= 'AIC'
    return(ans)
}

bestEBIC <- function(...) {
    checkargs_IC(...)
    ms= modelSelection(..., priorCoef=bic(), priorModel=modelbbprior(), center=FALSE, scale=FALSE)
    ans= extractmsIC(ms, getEBIC)
    ans$criterion= 'EBIC'
    return(ans)
}

bestIC <- function(..., penalty) {
    if (missing(penalty)) stop("penalty must be specified. Alternatively consider using bestBIC(), bestEBIC() or bestAIC()")
    checkargs_IC(...)
    ms= modelSelection(..., priorCoef=ic(penalty), priorModel=modelunifprior(), center=FALSE, scale=FALSE)
    ans= extractmsIC(ms, getIC)
    ans$criterion= paste('GIC (penalty=',penalty,')',collapse='')
    return(ans)
}

bestBIC_fast <- function(..., fastmethod='all') {
  args <- list(...)
  args$models <- findmodels_fast(..., priorCoef=bic(), priorModel=modelunifprior(), fastmethod=fastmethod)
  ans <- do.call(bestBIC, args)
  if (fastmethod %in% c('adaptiveL1','all')) {
    newmodels <- findmodels_alasso(ans, ...)
    if (!is.null(newmodels)) {
      args$models <- newmodels
      ansnew <- do.call(bestBIC, args)
      ans <- combineICfit(ans, ansnew)
    }
  } 
  return(ans)  
}

bestAIC_fast <- function(..., fastmethod='all') {
  args <- list(...)
  args$models <- findmodels_fast(..., priorCoef=aic(), priorModel=modelunifprior(), fastmethod=fastmethod)
  ans <- do.call(bestAIC, args)
  if (fastmethod %in% c('adaptiveL1','all')) {
    newmodels <- findmodels_alasso(ans, ...)
    if (!is.null(newmodels)) {
      args$models <- newmodels
      ansnew <- do.call(bestAIC, args)
      ans <- combineICfit(ans, ansnew)
    }
  } 
  return(ans)  
}

bestEBIC_fast <- function(..., fastmethod='all') {
  args <- list(...)
  args$models <- findmodels_fast(..., priorCoef=bic(), priorModel=modelbbprior(), fastmethod=fastmethod)
  ans <- do.call(bestEBIC, args)
  if (fastmethod %in% c('adaptiveL1','all')) {
    newmodels <- findmodels_alasso(ans, ...)
    if (!is.null(newmodels)) {
      args$models <- newmodels
      ansnew <- do.call(bestEBIC, args)
      ans <- combineICfit(ans, ansnew)
    }
  } 
  return(ans)  
}

bestIC_fast <- function(..., penalty, fastmethod='all') {
  args <- list(...)
  args$models <- findmodels_fast(..., priorCoef=ic(penalty), priorModel=modelunifprior(), fastmethod=fastmethod)
  ans <- do.call(bestIC, args, penalty)
  if (fastmethod %in% c('adaptiveL1','all')) {
    newmodels <- findmodels_alasso(ans, ...)
    if (!is.null(newmodels)) {
      args$models <- newmodels
      ansnew <- do.call(bestIC, args, penalty)
      ans <- combineICfit(ans, ansnew)
    }
  } 
  return(ans)  
}


findmodels_fast <- function(y, x, data, smoothterms, nknots=9, groups, constraints, enumerate, includevars, maxvars, niter=5000, family='normal', priorCoef, priorGroup, priorModel=modelbbprior(1,1), priorConstraints, priorVar=igprior(.01,.01), priorSkew=momprior(tau=0.348), neighbours, phi, deltaini, adj.overdisp='intercept', hess='asymp', optimMethod, optim_maxit, initpar='none', B=10^5, XtXprecomp, verbose=TRUE, fastmethod) {
  if (!fastmethod %in% c("L0Learn","L1","adaptiveL1","CDA","all")) stop("fastmethod must be 'L0Learn', 'L1', 'adaptiveL1', 'CDA' or 'all'")
  #Determine what model search methods to use
  L0Learn_available <- ifelse(family %in% c("normal","binomial"), TRUE, FALSE)
  L1_available <- ifelse(family %in% c("normal","twopiecenormal","laplace","twopiecelaplace","auto","binomial","binomial logit","poisson","poisson log","negbinomial","multinomial","cox","mnormal"), TRUE, FALSE)
  CDA_available <- ifelse(family %in% c("normal","twopiecenormal","laplace","twopiecelaplace","auto","binomial","binomial logit","poisson","poisson log"), TRUE, FALSE)
  use_L0Learn <- ifelse((fastmethod %in% c("L0Learn","all")) & L0Learn_available, TRUE, FALSE)
  use_L1 <- ifelse((fastmethod %in% c("L1","adaptiveL1","all")) & L1_available, TRUE, FALSE)
  use_CDA <- ifelse((fastmethod %in% c("CDA","all")) & CDA_available, TRUE, FALSE)
  if (!use_L0Learn & !use_L1 & !use_CDA) stop("The specified fastmethod is unavailable for the specified family")
  #Run L0Learn
  if (use_L0Learn) {
    loss <- ifelse(family == "normal", "SquaredError", "Logistic")
    # Build design matrix
    tmp <- formatInputdata(y=y,x=x,data=data,smoothterms=smoothterms,nknots=nknots,family=family)
    x <- tmp$x; y <- tmp$y
    # Call L0Learn
    if (missing(includevars)) includevars <- rep(FALSE, ncol(x))
    if (missing(maxvars)) maxvars <- set_ms_maxvars(n=nrow(x), p=ncol(x), priorCoef=bicprior(), family=family, includevars=includevars)
    fit <- L0Learn.fit(x=x, y=y, penalty="L0", maxSuppSize=maxvars, loss=loss, algorithm="CDPSI")
    models <- unique(t(as.matrix(coef(fit) != 0)))
    if (ncol(models) > ncol(x)) models <- models[,-1] #remove intercept
    models_L0Learn <- models
  } else {
    models_L0Learn <- NULL
  }
  #Run L1
  if (use_L1) {
    models_L1 <- findmodels_lasso(y=y, x=x, data=data, smoothterms, nknots=nknots, groups=groups, constraints=constraints, enumerate=enumerate, includevars=includevars, maxvars=maxvars, niter=niter, family=family, priorCoef=priorCoef, priorGroup=priorGroup, priorModel=priorModel, priorConstraints=priorConstraints, priorVar=priorVar, priorSkew=priorSkew, neighbours=neighbours, phi=phi, deltaini=deltaini, adj.overdisp=adj.overdisp, hess=hess, optimMethod=optimMethod, optim_maxit=optim_maxit, initpar=initpar, B=B, XtXprecomp=XtXprecomp, verbose=verbose)
  } else {
    models_L1 <- NULL
  }
  #Run CDA
  if (use_CDA) {
    fit <- modelSelection(y=y, x=x, data=data, initSearch='CDA', burnin=0, niter=1, center=FALSE, scale=FALSE, smoothterms=smoothterms, nknots=nknots, groups=groups, constraints=constraints, enumerate=enumerate, includevars=includevars, maxvars=maxvars, priorCoef=priorCoef, priorGroup=priorGroup, priorModel=priorModel, priorConstraints=priorConstraints, priorVar=priorVar, priorSkew=priorSkew, neighbours=neighbours, phi=phi, deltaini=deltaini, adj.overdisp=adj.overdisp, hess=hess, optimMethod=optimMethod, optim_maxit=optim_maxit, initpar=initpar, XtXprecomp=XtXprecomp, verbose=verbose)
    models_CDA <- matrix(fit$postMode == 1, nrow=1)
  } else {
    models_CDA <- NULL
  }
  models <- unique(rbind(models_L0Learn, models_L1, models_CDA))
  return(models)
}


# Map GLM family from the format required by modelSelection to that used by glmnet
# - Families twopiecenormal, laplace and twopiecelaplace are mapped to normal, since these are unavailable in glmnet
# - Family negbinomial is mapped to Poisson, since the negative binomial is unavailable in glmnet
family2glmnet <- function(family) {
    if (family %in% c("normal","twopiecenormal","laplace","twopiecelaplace","auto")) {
        family_glmnet = "gaussian"
    } else if (family %in% c("binomial","binomial logit")) {
        family_glmnet = "binomial"
    } else if (family %in% c("poisson", "poisson log", "negbinomial")) {
        family_glmnet = "poisson"
    } else if (family == "multinomial") {
        family_glmnet = "multinomial"
    } else if (family == "cox") {
        family_glmnet = "cox"
    } else if (family == "mgaussian") {
        family_glmnet = "mgaussian"
    } else {
        stop("This family is unavailable in glmnet")
    }
    return(family_glmnet)
}


# Return all models visited by the LASSO regularization path. If bini is not missing, adaptive LASSO is used (only on variables such that bini != 0)
findmodels_lasso <- function(bini, y, x, data, smoothterms, nknots=9, groups, constraints, enumerate, includevars, maxvars, niter=5000, family='normal', priorCoef, priorGroup, priorModel=modelbbprior(1,1), priorConstraints, priorVar=igprior(.01,.01), priorSkew=momprior(tau=0.348), neighbours, phi, deltaini, adj.overdisp='intercept', hess='asymp', optimMethod, optim_maxit, initpar='none', B=10^5, XtXprecomp, verbose=TRUE) {
  # Build design matrix
  tmp <- formatInputdata(y=y,x=x,data=data,smoothterms=smoothterms,nknots=nknots,family=family)
  x <- tmp$x; y <- tmp$y
  family_glmnet <- family2glmnet(family)
  if (missing(bini)) { # LASSO
    fit <- glmnet::glmnet(x=x, y=y, family=family_glmnet, alpha=1, intercept=FALSE, standardize=TRUE)
    models <- unique(t(as.matrix(coef(fit)[-1,,drop=FALSE] != 0)))
  } else {             # Adaptive LASSO based on initial parameter estimate bini
    sel <- (bini != 0)
    xstd <- scale(x[,sel,drop=FALSE])
    xstd <- t(t(xstd) * abs(bini[sel]))  
    fit <- glmnet::glmnet(x=xstd, y=y, family=family_glmnet, alpha=1, intercept=FALSE, standardize=FALSE)
    msel <- unique(t(as.matrix(coef(fit)[-1,,drop=FALSE] != 0)))
    models <- matrix(FALSE, nrow=nrow(msel), ncol=ncol(x))
    models[,sel] <- msel
  }
  return(models)
}


# Given icfit object, search for new models using adaptive LASSO and add them to the icfit object
findmodels_alasso <- function(ans, ...) {
  newmodels <- NULL
  if (length(coef(ans)) > 1) {
    bini <- coefall(ans)
    newmodels <- findmodels_lasso(bini, ...)
    newmodelid <- apply(newmodels, 1, function(z) paste(which(z), collapse = ","))
    # Keep only new models that aren't already in args$models
    isnew <- !(newmodelid %in% ans$models$modelid)
    if (any(isnew)) {
      newmodels <- newmodels[isnew,,drop=FALSE]
    }
  }
  return(newmodels)
}


## EXTRACT INFORMATION CRITERIA FROM AN msfit object

setMethod("getAIC", signature(object='msfit'), function(object) {
    ans= getBIC(object)
    names(ans)[names(ans)=='bic']= 'aic'
    return(ans)
}
)


setMethod("getBIC", signature(object='msfit'), function(object) {
    pc= object$priors$priorCoef
    pm= object$priors$priorModel
    if ((pc@priorDistr != 'bic') || (pm@priorDistr != 'uniform')) stop("To obtain BIC you should set priorCoef=bic() and priorModel=modelunifprior() when calling modelSelection")
    ans= getIC(object)
    names(ans)[names(ans)=='ic']= 'bic'
    return(ans)
}
)

setMethod("getIC", signature(object='msfit'), function(object) {
    ic = -2 * ( object$postProb + object$p * log(2) )
    ans= dplyr::tibble(modelid=object$modelid, ic=ic)
    ans= ans[order(ans$ic),]
    return(ans)
}
)


setMethod("getEBIC", signature(object='msfit'), function(object) {
    pc= object$priors$priorCoef
    pm= object$priors$priorModel
    isbbprior= (pm@priorDistr == 'binomial') && all(pm@priorPars == c(1,1))
    if ((pc@priorDistr != 'bic') || !isbbprior) stop("To obtain BIC you should set priorCoef=bic() and priorModel=modelbbprior() when calling modelSelection")
    ebic = -2  * ( object$postProb + log(object$p+1) )
    ans= dplyr::tibble(modelid=object$modelid, ebic=ebic)
    ans= ans[order(ans$ebic),]
    return(ans)
}
)

### Methods for localtest objects

setMethod("show", signature(object='localtest'), function(object) {
  message("Estimated local covariate effects\n\n")
  print(object$covareffects[1:15,,drop=FALSE])
  if (nrow(object$covareffects)>15) message("...")
  message("\n\n")
  message("Use coef() to extract point estimates, intervals and posterior probabilities for local effects \n")
}
)

coef.localtest <- function(object,...) {
    object$covareffects
}

predict.localtest <- function(object, newdata, data, level=0.95, ...) {
    nres= length(object$ms) #number of resolution levels
    if (!missing(newdata)) {
        if (!all(c("x","z") %in% names(newdata))) stop("newdata must be a list with two entries named 'x' and 'z'")
        if (!is.matrix(newdata$x)) stop("newdata$x must be a matrix")
        if (!is.matrix(newdata$z)) newdata$z= as.matrix(newdata$z)
        w= predictDesign(object=object, newdata=newdata)
        ypred= matrix(NA, nrow=nres, ncol=nrow(newdata$x))
        for (i in 1:nres) ypred[i,]= predict(object$ms[[i]], newdata=w[[i]])[,1]
     } else {
        ypred= do.call(rbind,lapply(object$ms, function(z) predict(z)[,1]))
    }
    ypred= colSums(ypred * object$pp_localknots)
    return(ypred)
}


#Create design matrix for newdata given fitted object of type localtest
predictDesign <- function(object, newdata) {
    if (object$Sigma != 'identity') stop("predict method is currently only implemented for independent errors")
    nres= length(object$ms) #number of resolution levels
    ans= vector("list",nres)
    for (j in 1:nres) {
        zdiscrete= zdiscretef= matrix(NA,nrow=nrow(newdata$z),ncol=ncol(newdata$z))
        for (i in 1:ncol(newdata$z)) {
            zdiscretef= cut(newdata$z[,i], breaks=object$regionbounds[[j]][[i]])
            zdiscrete[,i]= as.numeric(zdiscretef)
        }
        region= apply(zdiscrete,1,paste,collapse='.')       
        ans[[j]]= createDesignLocaltest(x=newdata$x, z=newdata$z, region=region, regionbounds=object$regionbounds[[j]], basedegree=object$basedegree, cutdegree=object$cutdegree, knots=object$knots, usecutbasis=object$usecutbasis)$w
    }
    return(ans)
}


setMethod("postProb", signature(object='localtest'), function(object, nmax, method='norm') {
    object$pplocalgrid
}
)


# Local null test for the effect of x on y, at each various regions given by the coordinates in z, using cut spline basis
# Input
# - y: outcome
# - x: matrix with covariate values with nrow(x)==length(y)
# - z: matrix with coordinates with nrow(z)==length(y)
# - x.adjust: optionally, further adjustment covariates to be included in the model with no testing being performed
# - function_id: function identifier. It is assumed that one observes multiple functions over z, this is the identifier of each individual function
# - Sigma: error covariance. By default 'identity', other options are 'MA', 'AR' or 'AR/MA' (meaning that BIC is used to choose between MA and AR). Alternatively the user can supply a function such that Sigma(z[i,],z[j,]) returns the within-function cov(y[i,], y[j,])
# - localgridsize: local test probabilities will be returned for a grid of z values of size localgridsize for each dimension
# - localgrid: regions at which tests will be performed. Defaults to dividing each [min(z[,i]),  max(z[,i])] into 10 equal intervals. If provided, localgrid must be a list with one entry for each z[,i], containing a vector with the desired grid for that z[,i]
# - nbaseknots: number of knots for the spline approximation to the baseline effect of x on y
# - nlocalknots: number of knots for the basis capturing the local effects
# - basedegree: degree of the spline approximation to the baseline
# - cutdegree: degree of the cut spline basis used for testing
# - usecutbasis: if FALSE, then the basis is not cut and a standard spline basis is returned
# - verbose: if set to TRUE some progress information is printed
# - ...: other arguments to be passed on to modelSelection, e.g. family='binomial' for logistic regression
# Output
# - covareffects: estimated local covariate effects at different z's, along with 0.95 posterior intervals and posterior probability for the existence of an effect
# - pplocalgrid: posterior probabilities for the existence of an effect for regions of z values
# - covareffects.mcmc: MCMC output used to build covareffects. Only returned if return.mcmc=TRUE
# - ms: objects of class 'msfit' returned by modelSelection
# - pp_localknots: posterior probability for each resolution level (value of nlocalknots)
# - nlocalknots, basedegree, cutdegree, knots: input parameters
# - regionbounds: list with region bounds defined by the local testing knots at each resolution level
# - Sigma: input parameter

localnulltest= function(y, x, z, x.adjust, localgridsize, localgrid, nbaseknots=20, nlocalknots=c(5,10,15), localknots, basedegree=3, cutdegree=0, usecutbasis=TRUE, priorCoef=normalidprior(), priorGroup=priorCoef, priorModel=modelbbprior(), mc.cores=min(4,length(nlocalknots)), return.mcmc=FALSE, verbose=FALSE, ...) {
    #Check & format input arguments
    check= checkargs_localnulltest(y=y,x=x,x.adjust=x.adjust,z=z,localknots=localknots)
    y= check$y; x= check$x; z= check$z; x.adjust= check$x.adjust
    #Define local grid
    if (missing(localgridsize)) { if (ncol(z)==1) localgridsize= 100 else localgridsize= 10 }
    if (missing(localgrid)) localgrid= define_localgrid(localgrid=localgrid, localgridsize=localgridsize, z=z)
    if (missing(localknots)) {
        localknots= NULL
    } else {
        if (!is.list(localknots)) localknots= list(localknots)
    }
    #Perform local null tests
    localnulltest_core(y=y, x=x, z=z, x.adjust=x.adjust, localgridsize=localgridsize, localgrid=localgrid, nbaseknots=nbaseknots, nlocalknots=nlocalknots, localknots=localknots, basedegree=basedegree, cutdegree=cutdegree, usecutbasis=usecutbasis, priorCoef=priorCoef, priorGroup=priorGroup, priorModel=priorModel, mc.cores=mc.cores, return.mcmc=return.mcmc, verbose=verbose, ...)
}


#Same as localnulltest, but for data observed on a regular grid (e.g. time series, functional data analysis) where errors may be correlated
localnulltest_fda= function(y, x, z, x.adjust, function_id, Sigma='AR/MA', localgridsize, localgrid, nbaseknots=20, nlocalknots=c(5,10,15), localknots, basedegree=3, cutdegree=0, usecutbasis=TRUE, priorCoef=momprior(), priorGroup=groupmomprior(), priorModel=modelbbprior(), mc.cores=min(4,length(nlocalknots)), return.mcmc=FALSE, verbose=FALSE, ...) {
    #Check & format input requirements
    check= checkargs_localnulltest(y=y,x=x,z=z,x.adjust=x.adjust,function_id=function_id,localknots=localknots)
    y= check$y; x= check$x; z= check$z; x.adjust= check$x.adjust
    #Define local grid
    if (missing(localgridsize)) { if (ncol(z)==1) localgridsize= 100 else localgridsize= 10 }
    if (missing(localgrid)) localgrid= define_localgrid(localgrid=localgrid, localgridsize=localgridsize, z=z) #define localgrid
    if (missing(localknots)) {
        localknots= NULL
    } else {
        if (!is.list(localknots)) localknots= list(localknots)
    }
    #Perform local null tests
    localnulltest_core(y=y, x=x, z=z, x.adjust=x.adjust, function_id=function_id, Sigma=Sigma, localgridsize=localgridsize, localgrid=localgrid, nbaseknots=nbaseknots, nlocalknots=nlocalknots, localknots=localknots, basedegree=basedegree, cutdegree=cutdegree, usecutbasis=usecutbasis, priorCoef=priorCoef, priorGroup=priorGroup, priorModel=priorModel, mc.cores=mc.cores, return.mcmc=return.mcmc, verbose=verbose, ...)
}

#Check input arguments to localnulltest
checkargs_localnulltest= function(y, x, z, x.adjust, function_id, localknots) {
    if (is.matrix(y)) y= as.vector(y)
    if (!is.matrix(x)) x= as.matrix(x)
    if (!is.matrix(z)) z= as.matrix(z)
    n= length(y)
    if (nrow(x) != n) stop("nrow(x) must be equal to length(y)")
    if (nrow(z) != n) stop("nrow(z) must be equal to length(y)")
    if (!missing(function_id)) if (length(function_id) != n) stop("length(function_id) must be equal to length(y)")
    if (!is.numeric(x)) stop("x must be numeric")
    if (!is.numeric(z)) stop("z must be numeric")
    if (missing(x.adjust)) {
        x.adjust= NULL
    } else {
        if (!is.null(x.adjust)) {
            x.adjust= as.matrix(x.adjust)
            x.adjust= x.adjust[, apply(x.adjust, 2, sd) > 0, drop=FALSE]  #drop the intercept (and constant columns)
            if (!is.numeric(x.adjust)) stop("x.adjust must be numeric")
        }
    }
    if (!missing(localknots)) {
        if (!is.list(localknots)) {
            b= range(localknots)
            for (i in 1:ncol(z)) if ((min(z) < b[1]) || (max(z) > b[2])) stop(paste("Specified localknots do not span the range of z[,",i,"]",sep=""))
        } else {
            for (j in 1:length(localknots)) {
                b= range(localknots[[j]])
                for (i in 1:ncol(z)) if ((min(z) < b[1]) || (max(z) > b[2])) stop(paste("Specified localknots do not span the range of z[,",i,"]",sep=""))
            }
        }
    }
    return(list(y=y, x=x, z=z, x.adjust=x.adjust))
}

#Core function to run local null tests at multiple resolutions.
# It calls localnulltest_givenknots (for iid errors) or localnulltest_fda_givenknots (for dependent errors observed on regular grids)
localnulltest_core= function(y, x, z, x.adjust, function_id, Sigma, localgridsize, localgrid, nbaseknots=20, nlocalknots=c(5,10,15), localknots, basedegree=3, cutdegree=0, usecutbasis=TRUE, priorCoef=normalidprior(), priorGroup=priorCoef, priorModel=modelbbprior(), mc.cores=min(4,length(nlocalknots)), return.mcmc=FALSE, verbose=FALSE, ...) {
    #Define function to be run at each resolution level
    if (is.null(localknots)) {
        nresolutions= length(nlocalknots)
        if (missing(Sigma)) {
            foo= function(i) { localnulltest_givenknots(y=y, x=x, z=z, x.adjust=x.adjust, localgrid=localgrid, nbaseknots=nbaseknots, nlocalknots=nlocalknots[i], basedegree=basedegree, cutdegree=cutdegree, usecutbasis=usecutbasis, priorCoef=priorCoef, priorGroup=priorGroup, priorModel=priorModel, verbose=verbose, ...) }
            Sigma= "identity"
        } else {
            foo= function(i) { localnulltest_fda_givenknots(y=y, x=x, z=z, x.adjust=x.adjust, function_id=function_id, Sigma=Sigma, localgrid=localgrid, nbaseknots=nbaseknots, nlocalknots=nlocalknots[i], basedegree=basedegree, cutdegree=cutdegree, usecutbasis=usecutbasis, priorCoef=priorCoef, priorGroup=priorGroup, priorModel=priorModel, verbose=verbose, ...) }
        }
    } else {
        nresolutions= length(localknots)
        if (missing(Sigma)) {
            foo= function(i) { localnulltest_givenknots(y=y, x=x, z=z, x.adjust=x.adjust, localgrid=localgrid, nbaseknots=nbaseknots, localknots=localknots[[i]], basedegree=basedegree, cutdegree=cutdegree, usecutbasis=usecutbasis, priorCoef=priorCoef, priorGroup=priorGroup, priorModel=priorModel, verbose=verbose, ...) }
            Sigma= "identity"
        } else {
            foo= function(i) { localnulltest_fda_givenknots(y=y, x=x, z=z, x.adjust=x.adjust, function_id=function_id, Sigma=Sigma, localgrid=localgrid, nbaseknots=nbaseknots, localknots=localknots[[i]], basedegree=basedegree, cutdegree=cutdegree, usecutbasis=usecutbasis, priorCoef=priorCoef, priorGroup=priorGroup, priorModel=priorModel, verbose=verbose, ...) }
        }
    }
    #Run analysis for each resolution level
    if (("parallel" %in% loadedNamespaces()) & mc.cores>1)  {
      alltests= parallel::mclapply(1:nresolutions, foo, mc.cores=mc.cores)
    } else {
      alltests= lapply(1:nresolutions, foo)
    }
    #Posterior probability of each resolution level
    logpp= double(length(alltests))
    for (i in 1:length(logpp)) {
        logpp[[i]]= marglhood_acrossmodels(alltests[[i]]$ms[[1]], logscale=TRUE) - 0.5 * alltests[[i]]$logdetSinv
    }
    pp_localknots= exp(logpp - max(logpp))
    pp_localknots= pp_localknots / sum(pp_localknots)
    #Average across resolution levels       
    if(nresolutions==1) {
        ans= alltests[[1]]
    } else {
        #Posterior probabilities for the local tests
        pplocalgrid= alltests[[1]]$pplocalgrid
        pplocalgrid[,-1]= pplocalgrid[,-1] * pp_localknots[1]
        for (i in 2:nresolutions) pplocalgrid[,-1]= pplocalgrid[,-1] + alltests[[i]]$pplocalgrid[,-1] * pp_localknots[i]
        #BMA estimate and 95% intervals
        covareffects= alltests[[1]]$covareffects
        p= ncol(alltests[[1]]$covareffects.mcmc)
        B= sapply(alltests, function(zz) nrow(zz$covareffects.mcmc))
        w= rep(pp_localknots, B)
        for (i in 1:p) {
            samples= sapply(alltests, function(zz) zz$covareffects.mcmc[,i])
            covareffects[i,'estimate']= weighted.mean(samples, w)
            covareffects[i,4:5]= quantile_weighted(samples, weights=w, probs = c(0.025,0.975))
        }
        if (return.mcmc) {
            covareffects.mcmc= cbind(do.call(rbind, lapply(alltests, "[[", "covareffects.mcmc")), weight=w/sum(w))
        } else {
            covareffects.mcmc= NULL
        }
        ms= lapply(alltests, function(zz) zz[["ms"]][[1]])
        regionbounds= lapply(alltests, function(zz) zz[["regionbounds"]][[1]])
        knots= lapply(alltests, function(zz) zz[["knots"]][[1]])
        ans= list(pplocalgrid=pplocalgrid, covareffects=covareffects, covareffects.mcmc=covareffects.mcmc, ms=ms, pp_localknots=pp_localknots, nlocalknots=nlocalknots, regionbounds=regionbounds, basedegree=basedegree, cutdegree=cutdegree, usecutbasis=usecutbasis, knots=knots, Sigma=Sigma)
    }
    new("localtest",ans)
}




localnulltest_givenknots= function(y, x, z, x.adjust, localgridsize, localgrid, nbaseknots=20, nlocalknots=10, localknots, basedegree=3, cutdegree=0, usecutbasis=TRUE, priorCoef=normalidprior(), priorGroup=priorCoef, priorModel=modelbbprior(), verbose=FALSE, ...) {
    #Check & format input requirements
    if (missing(localknots)) {
        if (length(nlocalknots)>1) stop("nlocalknots must have length 1. Consider using localnulltest")
        localknots= NULL
    }
    check= checkargs_localnulltest(y=y, x=x, z=z, x.adjust=x.adjust)
    y= check$y; x= check$x; z= check$z; x.adjust= check$x.adjust
    # Define local tests
    if (missing(localgridsize)) { if (ncol(z)==1) localgridsize= 100 else localgridsize= 10 }
    if (missing(localgrid)) localgrid= define_localgrid(localgrid=localgrid, localgridsize=localgridsize, z=z)
    # Define knots and corresponding knot-based testing regions
    kk= define_knots_localnulltest(z=z, localgrid=localgrid, nbaseknots=nbaseknots, basedegree=basedegree, nlocalknots=nlocalknots, localknots=localknots)
    knots= kk$knots; regionbounds= kk$regionbounds; region=kk$region; regioncoord= kk$regioncoord; testov= kk$testov; testxregion= kk$testxregion; testIntervals= kk$testIntervals
    #Create design matrix & run Bayesian model selection
    desnew= estimationPoints(x=x, regioncoord=regioncoord, regionbounds=regionbounds, testov=testov) #Points at which local effects will be estimated
    if (any(is.na(region)) | any(is.na(desnew$region))) warning("Some testing regions had too few observations. Consider using less local knots\n")
    des= createDesignLocaltest(x=rbind(x,desnew$x), z=rbind(z,desnew$z), y=y, region=c(region,desnew$region), regionbounds=regionbounds, basedegree=basedegree, cutdegree=cutdegree, knots=knots, usecutbasis=usecutbasis, useSigma=FALSE, rowids.fullrank=1:nrow(x))
    w= des$w[1:nrow(x),]; wnew= des$w[-1:-nrow(x),]; neighbours= des$neighbours
    if (!is.null(x.adjust)) {
        w= cbind(w, x.adjust)
        groups= c(des$vargroupsn, rep(max(des$vargroupsn)+1, ncol(x.adjust)))
    } else {
        groups= des$vargroupsn
    }
    ms= modelSelection(y, x=w, groups=groups, priorCoef=priorCoef, priorGroup=priorGroup, priorModel=priorModel, neighbours=neighbours, verbose=verbose, ...)        
    pp= postProb(ms)
    #Obtain posterior probability for each region defined by the knots. Store in regionid, ppregion
    modelid= modelid2logical(pp$modelid, nvars=ncol(ms$xstd))
    regionnames= unique(des$vargroups[-1:-des$ncolw0])
    firstvaringroup= match(regionnames, des$vargroups)
    regionid= modelid[,firstvaringroup,drop=FALSE]
    colnames(regionid)= des$vargroups[firstvaringroup]
    regionidtext= apply(regionid, 1, function(z) paste(which(z), collapse=','))
    ppregion= aggregate(pp$pp ~ regionidtext, FUN=sum)
    names(ppregion)= c('regionid','pp')
    regionid= modelid2logical(ppregion$regionid, nvars=ncol(regionid))
    colnames(regionid)= des$vargroups[firstvaringroup]
    #Find posterior probability for each local test
    varids= unique(des$w1varname)
    pplocaltest= matrix(NA, nrow=length(testxregion), ncol=ncol(x))
    colnames(pplocaltest)= varids
    for (i in 1:length(varids)) {
        sel= grep(paste("^",varids[i],sep=""), colnames(regionid)) #inclusion indicators for variable varids[i]
        nn= colnames(regionid)[sel]
        nncut= sub("\\.","", sub(varids[i],'',nn))
        colnames(regionid)[sel]= nncut
        pplocaltest[,i]= postProblocaltest(regionid[,sel,drop=FALSE], ppregion$pp, testxregion)
        colnames(regionid)[sel]= nn
    }
    #Estimated effect at the center of each local test region
    if (verbose) message("Estimating local effects... ")
    if (!is.null(x.adjust)) { m.adjust= colMeans(x.adjust) } else { m.adjust= NULL }
    est= estimateLocaleffect(ms, wnew, desnew, x.adjust=m.adjust)
    if (verbose) message("Done\n")
    #Return output
    pplocalgrid= data.frame(localtest=1:nrow(testIntervals), testIntervals, pplocaltest)
    ans= list(pplocalgrid=pplocalgrid, covareffects=est$covareffects, covareffects.mcmc=est$covareffects.mcmc, ms=list(ms), pp_localknots=1, logdetSinv=0, nlocalknots=nlocalknots, regionbounds=list(regionbounds), basedegree=basedegree, cutdegree=cutdegree, usecutbasis=usecutbasis, knots=knots, Sigma='identity')
    new("localtest",ans)
}



localnulltest_fda_givenknots= function(y, x, z, x.adjust, function_id, Sigma='AR/MA', localgridsize, localgrid, nbaseknots=20, nlocalknots=10, localknots, basedegree=3, cutdegree=0, usecutbasis=TRUE, priorCoef=momprior(), priorGroup=groupmomprior(), priorModel=modelbbprior(), verbose=FALSE, ...) {
    #Check & format input requirements
    if (missing(localknots)) {
        if (length(nlocalknots)>1) stop("nlocalknots must have length 1. Consider using localnulltest_fda")
        localknots= NULL
    }
    check= checkargs_localnulltest(y=y, x=x, z=z, x.adjust=x.adjust, function_id=function_id)
    y= check$y; x= check$x; z= check$z; x.adjust= check$x.adjust
    if (inherits(Sigma, "character")) {
        if (!(Sigma %in% c('MA','AR','AR/MA'))) stop("This Sigma is not implemented. Use 'MA', 'AR', 'AR/MA', or a user-supplied function")
    } else {
        if (!inherits(Sigma, "function")) stop("Sigma must either be character or a function such that Sigma(i,j)=cov(epsilon_i,epsilon_j)")
    }
    # Sort data according to function_id and z
    o= do.call(order, data.frame(function_id, z))
    y= y[o]; x= x[o,,drop=FALSE]; z= z[o,,drop=FALSE]; function_id= function_id[o]
    if (!is.null(x.adjust)) x.adjust= x.adjust[o,,drop=FALSE]
    # Define local tests
    if (missing(localgridsize)) { if (ncol(z)==1) localgridsize= 100 else localgridsize= 10 }
    if (missing(localgrid)) localgrid= define_localgrid(localgrid=localgrid, localgridsize=localgridsize, z=z)
    # Define knots and corresponding knot-based testing regions
    kk= define_knots_localnulltest(z=z, localgrid=localgrid, nbaseknots=nbaseknots, basedegree=basedegree, nlocalknots=nlocalknots, localknots=localknots)
    knots= kk$knots; regionbounds= kk$regionbounds; region= kk$region; regioncoord= kk$regioncoord; testov= kk$testov; testxregion= kk$testxregion; testIntervals= kk$testIntervals
    #Create design matrix & run Bayesian model selection
    xc= scale(x, center=TRUE, scale=FALSE)
    desnew= estimationPoints(x=xc, regioncoord=regioncoord, regionbounds=regionbounds, testov=testov) #Points at which local effects will be estimated
    if (any(is.na(region)) | any(is.na(desnew$region))) warning("Some testing regions had too few observations. Consider using less knots\n")
    des= createDesignLocaltest(x=xc, z=z, y=y, x.adjust=x.adjust, xnew=desnew$x, znew=desnew$z, function_id=function_id, region=region, regionnew=desnew$region, regionbounds=regionbounds, basedegree=basedegree, cutdegree=cutdegree, knots=knots, usecutbasis=usecutbasis, useSigma=TRUE, Sigma=Sigma)
    ytilde= des$ytilde; wtilde= des$wtilde; logdetSinv= des$logdetSinv; neighbours= des$neighbours
    wnew= des$wnew
    if (!is.null(x.adjust)) {
        wtilde= cbind(wtilde, des$wtilde.adjust)
        groups= c(des$vargroupsn, rep(max(des$vargroupsn)+1, ncol(x.adjust)))
    } else {
        groups= des$vargroupsn
    }
    ms= modelSelection(y=ytilde, x=wtilde, groups=groups, priorCoef=priorCoef, priorGroup=priorGroup, priorModel=priorModel, neighbours=neighbours, verbose=verbose, ...)
    pp= postProb(ms)
    #Obtain posterior probability for each region defined by the knots. Store in regionid, ppregion
    modelid= modelid2logical(pp$modelid, nvars=ncol(ms$xstd))
    regionnames= unique(des$vargroups[-1:-des$ncolw0])
    firstvaringroup= match(regionnames, des$vargroups)
    regionid= modelid[,firstvaringroup,drop=FALSE]
    colnames(regionid)= des$vargroups[firstvaringroup]
    regionidtext= apply(regionid, 1, function(z) paste(which(z), collapse=','))
    ppregion= aggregate(pp$pp ~ regionidtext, FUN=sum)
    names(ppregion)= c('regionid','pp')
    regionid= modelid2logical(ppregion$regionid, nvars=ncol(regionid))
    colnames(regionid)= des$vargroups[firstvaringroup]
    #Find posterior probability for each local test
    varids= unique(des$w1varname)
    pplocaltest= matrix(NA, nrow=length(testxregion), ncol=ncol(x))
    colnames(pplocaltest)= varids
    for (i in 1:length(varids)) {
        sel= grep(paste("^",varids[i],sep=""), colnames(regionid)) #inclusion indicators for variable varids[i]
        nn= colnames(regionid)[sel]
        nncut= sub("\\.","", sub(varids[i],'',nn))
        colnames(regionid)[sel]= nncut
        pplocaltest[,i]= postProblocaltest(regionid[,sel,drop=FALSE], ppregion$pp, testxregion)
        colnames(regionid)[sel]= nn
    }
    #Estimated effect at the center of each local test region
    if (verbose) message("Estimating local effects... ")
    if (!is.null(x.adjust)) { m.adjust= colMeans(x.adjust) } else { m.adjust= NULL }
    est= estimateLocaleffect(ms, wnew, desnew, x.adjust=m.adjust)
    if (verbose) message("Done\n")
    #Return output
    pplocalgrid= data.frame(localtest=1:nrow(testIntervals), testIntervals, pplocaltest)
    ans= list(pplocalgrid=pplocalgrid, covareffects=est$covareffects, covareffects.mcmc=est$covareffects.mcmc, ms=list(ms), pp_localknots=1, logdetSinv=logdetSinv, nlocalknots=nlocalknots, regionbounds=list(regionbounds), basedegree=basedegree, cutdegree=cutdegree, usecutbasis=usecutbasis, knots=knots)
    new("localtest",ans)
}


# Define knots and corresponding knot-based testing regions
# Input
# - z: matrix with n rows and d colums indicating the d-dimensional coordinates for each observation
# - localgrid: regions at which tests will be performed. Defaults to dividing each [min(z[,i]),  max(z[,i])] into 10 equal intervals. If provided, localgrid must be a list with one entry for each z[,i], containing a vector with the desired grid for that z[,i]
# - nbaseknots: number of knots for the baseline basis
# - basedegree: degree of the baseline basis
# - nlocalknots: number of knots for the local testing basis. Ignored when localknots are specified
# - localknots: knots for the local testing basis. A list with ncol(z) entries giving the knots for each dimension
# Output
# - knots: a list with d elements containing the knots for each dimension
# - regionbounds: if localknots is specified, regionbounds is equal to localknots after adding regions at the extremes when needed. 
#                 If missing at input, a list with d elemants containing the local testing region bounds based on equi-spaced nlocalknots knots
# - region: character vector indicating the region of each observation (row in z)
# - regioncoord: region coordinates
# - testov: overlaps between localgrid and regioncoord
# - testxregion: list with an entry for each test, indicating the regions overlapping that test
# - testIntervals: local test grid, formatted as intervals
define_knots_localnulltest= function(z, localgrid, nbaseknots, basedegree, nlocalknots, localknots) {
    knots= vector("list", ncol(z))
    regionbounds= vector("list",ncol(z))
    for (i in 1:ncol(z)) {
        #Knots for baseline
        zlim= range(z[,i])
        baseknotwidth= (zlim[2] - zlim[1]) / nbaseknots
        knots[[i]]= seq(zlim[1] - baseknotwidth*(0.5+basedegree), zlim[2] + baseknotwidth*(0.5+basedegree), by=baseknotwidth)
        if (zlim[2] - zlim[1] > 0.01) knots[[i]]= round(knots[[i]], 3)
        #Knots for local tests
        if (is.null(localknots)) {
            zseq= seq(zlim[1], zlim[2], length=nlocalknots)
            regionbounds[[i]]= c(zseq[1] - (zseq[2]-zseq[1]), zseq, zseq[length(zseq)] + (zseq[2]-zseq[1]))
            if (zlim[2] - zlim[1] > 0.01) regionbounds[[i]]= round(regionbounds[[i]], 3)
        } else {
            regionbounds[[i]]= localknots
        }
    }
    zdiscrete= zdiscretef= matrix(NA,nrow=nrow(z),ncol=ncol(z))
    regionlabels= vector("list", length(ncol(z)))
    for (i in 1:ncol(z)) {
        zdiscretef= cut(z[,i], breaks=regionbounds[[i]])
        regionlabels[[i]]= data.frame(regionid=1:length(levels(zdiscretef)), start=regionbounds[[i]][-length(regionbounds[[i]])], end=regionbounds[[i]][-1])
        zdiscrete[,i]= as.numeric(zdiscretef)
    }
    region= apply(zdiscrete,1,paste,collapse='.')
    #Store coordinates for each region
    regioncoord= regionCoordinates(unique(zdiscrete), regionlabels)
    #Find regions overlapping each local test
    testov= testOverlaps(localgrid, regioncoord)
    testxregion= testov$testxregion                           #regions overlapping each local test
    testIntervals= do.call(data.frame, testov$testIntervals)  #local tests formatted as intervals
    label= rep(paste('z',1:ncol(z),sep=''), each=2)
    label= paste(label, rep(c('.low','.high'),ncol(z)), sep='')
    names(testIntervals)= label
    #Return output
    ans= list(knots=knots, regionbounds=regionbounds, region=region, regioncoord=regioncoord, testov=testov, testxregion=testxregion, testIntervals=testIntervals)
    return(ans)
}


#Estimate error correlation matrix
# Input
# - y: outcome
# - w: basis to be fed into modelSelection
# - z: matrix with coordinates with nrow(z)==length(y)
# - function_id: function identifier. Independence is assumed across functions
# - region: factor indicating what region (based on a partition of z) each observation belongs to
# - method: method to estimate the covariance. 'MA', 'AR', or 'AR/MA' (meaning trying both AR and MA and choose the one with best BIC)
# Output: a function Sigma such that Sigma(z[i,], z[j,])= cov(z[i,], z[j,])
#   For method=='MA', based on estimating the MA1 parameter y_t= theta * epsilon_{t-1} + epsilon_t
#   For method=='AR', based on estimating the AR1 parameter y_t= theta * y_{t-1} + epsilon_t

estimateSigma_localtest= function(y, w, z, function_id, region, method='AR/MA') {
    e= residuals(lm(y ~ w))
    if (ncol(z)>1) stop("If z is univariate, Sigma must be provided")
    if (!(method %in% c('MA','AR','AR/MA'))) stop("Method to estimate Sigma must be 'MA', 'AR', or 'AR/MA'")
    #
    ids= unique(function_id)
    theta= bics= matrix(NA, nrow=length(ids), ncol=2)
    colnames(theta)= colnames(bics)= c('MA','AR')
    for (i in 1:length(ids)) {
        sel= (function_id == ids[i])
        if (method %in% c('MA','AR/MA')) {
            fit.ma= try(arima(e[sel], order=c(0,0,1), method='ML'), silent=TRUE)
            if (inherits(fit.ma, "try-error")) fit.ma= try(arima(e[sel], order=c(0,0,1)), silent=TRUE)
            if (!inherits(fit.ma, "try-error")) {
                theta[i,'MA']= coef(fit.ma)['ma1']
                bics[i,'MA']= BIC(fit.ma)
            }
        }
        if (method %in% c('AR','AR/MA')) {
            fit.ar= try(arima(e[sel], order=c(1,0,0), method='ML'), silent=TRUE)
            if (inherits(fit.ar, "try-error")) fit.ar= try(arima(e[sel], order=c(1,0,0)), silent=TRUE)
            if (!inherits(fit.ar, "try-error")) {
                theta[i,'AR']= coef(fit.ar)['ar1']
                bics[i,'AR']= BIC(fit.ar)
            }
        }
    }
    if (method=='AR/MA') method= ifelse(which.min(colMeans(bics, na.rm=TRUE)) == 1, 'MA', 'AR')
    #Store the correlation matrix
    zregion= unique(cbind(as.numeric(region), z))
    if (method=='MA') {
        theta= mean(theta[,'MA'], na.rm=TRUE)
        ans= evalSigma_MA1_localtest(theta=theta, region=zregion[,1])   
        #fun= function(z1, z2) { if (z1==z2) return(1) else if (abs(z1-z2)==zdif) return(theta/(1+theta^2)) else return(0) }
    } else {
        theta= mean(theta[,'AR'], na.rm=TRUE)
        ans= evalSigma_AR1_localtest(theta=theta, region=zregion[,1])   
    }
    return(ans)
}



#Compute correlation matrix for MA1 model. The covariance is cov(i,i)= 1+theta^2, cov(i,i+1)= theta, cov(i,j)=0 otherwise
# Input
# - theta: MA1 parameter, i.e. y[t]= epsilon[t-1] * theta + epsilon[t]
# - region: factor indicating what region each observation belongs to
# - function_id: function identifier. Independence is assumed across functions
# Output: a list with one element per region, containing Sigma= corr(e) for that region. The list names match the unique values in region
evalSigma_MA1_localtest= function(theta, region) {
    regionids= unique(region)
    ans= vector("list", length(regionids))
    names(ans)= regionids
    for (i in 1:length(regionids)) {
        sel= which(region == regionids[i])
        ans[[i]]= diag(length(sel))
        ans[[i]][abs(col(ans[[i]]) - row(ans[[i]])) == 1]= theta / (1+theta^2)
        rownames(ans[[i]])= colnames(ans[[i]])= sel #set row/column names, for later use
    }
    return(ans)
}


#Compute correlation matrix for AR1 model, i.e. is cor(i,j)= theta^{|i-j|}
# Input
# - theta: AR1 parameter, i.e. y[t]= epsilon[t-1] * theta + epsilon[t]
# - region: factor indicating what region each observation belongs to
# - function_id: function identifier. Independence is assumed across functions
# Output: a list with one element per region, containing Sigma= corr(e) for that region. The list names match the unique values in region
evalSigma_AR1_localtest= function(theta, region) {
    regionids= unique(region)
    ans= vector("list", length(regionids))
    names(ans)= regionids
    for (i in 1:length(regionids)) {
        sel= which(region == regionids[i])
        ans[[i]]= matrix(NA, nrow=length(sel), ncol=length(sel))
        ans[[i]]= theta^abs(col(ans[[i]]) - row(ans[[i]]))
        rownames(ans[[i]])= colnames(ans[[i]])= sel #set row/column names, for later use
    }
    return(ans)
}


#Compute covariance matrix from a user-given function
# Input
# - Sigma: function such that Sigma(i,j)= cov(y[i],y[j]), which is assumed to only depend on z[i,] and z[j,]
# - region: factor indicating what region (based on a partition of z) each observation belongs to
# - z: coordinates
# Output: a list with one element per region, containing Sigma= cov(e) for that region. The list names match the unique values in region
evalSigma_localtest= function(Sigma, region, z) {
    regionids= unique(region)
    ans= vector("list", length(regionids))
    names(ans)= regionids
    for (i in 1:length(regionids)) {
        sel= which(region == regionids[i])
        ans[[i]]= matrix(NA, nrow=length(sel), ncol=length(sel))
        ans[[i]][1,1]= Sigma(z1=z[sel[1],], z2=z[sel[1],])
        if (length(sel)>1) {
            for (j in 2:length(sel)) {
                for (k in 1:j) {
                    ans[[i]][j,k]= ans[[i]][k,j]= Sigma(z1=z[sel[j],], z2=z[sel[k],])
                }
            }
        }
        rownames(ans[[i]])= colnames(ans[[i]])= sel #set row/column names, for later use
    }
    return(ans)
}



#Compute uncorrelated outcome and design matrix given inverse of error covariance, considering only within-region terms
# Input
# - y: outcome
# - w: design matrix
# - x.adjust: optional argument. Adjustment covariates (no testing performed on these)
# - region: factor indicating the regions to which each observation in y belongs to
# - S: error covariance. A list with an entry for each region with Sigma for that region
# - function_id: function identifier. Independence across functions is assumed
# Output: a list with elements
# - ytilde: block-diag(S)^{-1/2} y, where the blocks are defined by region
# - wtilde: block-diag(S)^{-1/2} w, where the blocks are defined by region
# - wtilde.adjust: block-diag(S}^{-1/2} x.adjust
# - logdetSinv: log-determinant of Sinv
uncorrelate_outcome= function(y, w, x.adjust, region, S, function_id) {
    n= length(y)
    ytilde= double(n)
    wtilde= matrix(NA, nrow=nrow(w), ncol=ncol(w))
    if (!is.null(x.adjust)) {
        wtilde.adjust= matrix(NA, nrow=nrow(x.adjust), ncol=ncol(x.adjust))
    } else {
        wtilde.adjust= NULL
    }
    logdetSinv= 0
    regionids= unique(region)
    functionids= unique(function_id)
    nfunctions= length(functionids)
    for (i in 1:length(regionids)) {
        e= eigen(S[[regionids[i]]])
        sqSinv= e$vectors %*% diag(1/sqrt(e$values), nrow=nrow(S[[regionids[i]]])) %*% t(e$vectors)
        logdetSinv= logdetSinv - nfunctions * sum(log(e$values))
        seli= (region == regionids[i])
        for (j in functionids) {
            sel= seli & (function_id == j)
            ytilde[sel]= sqSinv %*% matrix(y[sel],ncol=1)
            wtilde[sel,]= sqSinv %*% w[sel,,drop=FALSE]
            if (!is.null(x.adjust)) wtilde.adjust[sel,]= sqSinv %*% x.adjust[sel,,drop=FALSE]
        }
    }
    ans= list(ytilde=ytilde, wtilde=wtilde, wtilde.adjust=wtilde.adjust, logdetSinv=logdetSinv)
    return(ans)
}



#Compute quantiles from weighted sample. Adapted from function Quantile in package DescTools
quantile_weighted= function (x, weights = NULL, probs = c(0.025,0.975), na.rm = FALSE, names = TRUE, type = 7, digits = 7) {
    format_perc <- function(x, digits = max(2L, getOption("digits")), 
        probability = TRUE, use.fC = length(x) < 100, ...) {
        if (length(x)) {
            if (probability) x <- 100 * x
            ans <- paste0(if (use.fC) 
                formatC(x, format = "fg", width = 1, digits = digits)
            else format(x, trim = TRUE, digits = digits, ...), 
                "%")
            ans[is.na(x)] <- ""
            ans
        }
        else character(0)
    }
    if (!is.numeric(x)) stop("'x' must be a numeric vector")
    n <- length(x)
    if (n == 0 || (!isTRUE(na.rm) && any(is.na(x)))) {
        return(rep.int(NA, length(probs)))
    }
    if (!is.null(weights)) {
        if (!is.numeric(weights)) stop("'weights' must be a numeric vector")
        else if (length(weights) != n) {
            stop("'weights' must have the same length as 'x'")
        }
        else if (!all(is.finite(weights))) 
            stop("missing or infinite weights")
        if (any(weights < 0)) stop("negative weights")
        if (!is.numeric(probs) || all(is.na(probs)) || isTRUE(any(probs < 0 | probs > 1))) {
            stop("'probs' must be a numeric vector with values in [0,1]")
        }
        if (all(weights == 0)) {
            warning("all weights equal to zero")
            return(rep.int(0, length(probs)))
        }
    }
    if (isTRUE(na.rm)) {
        indices <- !is.na(x)
        x <- x[indices]
        n <- length(x)
        if (!is.null(weights)) weights <- weights[indices]
    }
    order <- order(x)
    x <- x[order]
    weights <- weights[order]
    if (is.null(weights)) rw <- (1:n)/n
    else rw <- cumsum(weights)/sum(weights)
    if (type == 5) {
        qs <- sapply(probs, function(p) {
            if (p == 0) 
                return(x[1])
            else if (p == 1) 
                return(x[n])
            select <- min(which(rw >= p))
            if (rw[select] == p) 
                mean(x[select:(select + 1)])
            else x[select]
        })
    }
    else if (type == 7) {
        if (is.null(weights)) {
            index <- 1 + max(n - 1, 0) * probs
            lo <- pmax(floor(index), 1)
            hi <- ceiling(index)
            x <- sort(x, partial = if (n == 0) numeric() else unique(c(lo, hi)))
            qs <- x[lo]
            i <- which((index > lo & x[hi] != qs))
            h <- (index - lo)[i]
            qs[i] <- (1 - h) * qs[i] + h * x[hi[i]]
        }
        else {
            n <- sum(weights)
            ord <- 1 + (n - 1) * probs
            low <- pmax(floor(ord), 1)
            high <- pmin(low + 1, n)
            ord <- ord%%1
            allq <- approx(cumsum(weights), x, xout = c(low, high), method = "constant", f = 1, rule = 2, ties="mean")$y
            k <- length(probs)
            qs <- (1 - ord) * allq[1:k] + ord * allq[-(1:k)]
        }
    }
    else {
        qs <- NA
        warning(gettextf("type %s is not implemented", type))
    }
    if (names && length(probs) > 0L) {
        stopifnot(is.numeric(digits), digits >= 1)
        names(qs) <- format_perc(probs, digits = digits)
    }
    return(qs)
}


#Define the local testing regions. In ncol(z)>=2 dimensions these are rectangular-shaped
# Input
# - localgrid: if not missing, this is returned as the function's output after checking it has the right format
# - localgridsize: number of local tests that one wishes to perform. The number of grid points in each dimension is localgridsize^(1/dim)
# - z: coordinates
# Output: list of length ndim defining a grid for each coordinate
define_localgrid= function(localgrid, localgridsize, z) {
    ndim= ncol(z)
    if (!missing(localgrid)) {
        if (!list(localgrid)) stop(paste("localgrid should be a list of length",ndim,"defining a grid for each coordinate in z (e.g. just 1 if z is univariate"))
    } else {
        if (localgridsize <= 1) stop("You defined a too small localgridsize")
        splitrange= function(zz) {
            zlim= range(zz)
            ans= seq(zlim[1], zlim[2], length=localgridsize)
            if (zlim[2] - zlim[1] > 0.01) ans= round(ans,3)
            return(ans)
        }
        localgrid= apply(z, 2, splitrange, simplify=FALSE)
        #localgrid= apply(z, 2, function(zz) seq(min(zz), max(zz), length=localgridsize), simplify=FALSE)
    }
    names(localgrid)= paste('dim', 1:length(localgrid), sep='')
    return(localgrid)
}


#Estimate via MCMC the local effects at wnew
# Input
# - ms: output from modelSelection
# - wnew: design points at which to estimate the local effects
# - desnew: output of estimationPoints containing further info about wnew
# - x.adjust: optional argument. The mean value of adjustment covariates included in ms, for which no testing is performed
# Output
# - covareffects: data.frame indicating the effect of each covariate at each region (posterior mean and 95% intervals)
# - covareffects.mcmc: mcmc draws for the effects summarized in covareffects
estimateLocaleffect= function(ms, wnew, desnew, x.adjust) {
    th= rnlp(msfit=ms, niter=max(nrow(ms$postSample), 1000))
    newdata= wnew[desnew$covareffect$value==1,] - wnew[desnew$covareffect$value==0]
    if (!is.null(x.adjust)) {
        newdata= cbind(newdata, matrix(0, nrow=nrow(newdata), ncol=length(x.adjust)))
        #newdata= cbind(newdata, matrix(rep(x.adjust, nrow(newdata)), nrow=nrow(newdata), byrow=TRUE))
    }
    sel= (desnew$covareffect$value==1)
    covareffects= data.frame(covariate=desnew$covareffect$covariate[sel], desnew$z[sel,,drop=FALSE])
    thsel= !(colnames(th) %in% c('intercept','phi'))
    covareffects.mcmc= th[,thsel] %*% t(newdata)
    covareffects$estimate= colMeans(covareffects.mcmc)
    qq= t(apply(covareffects.mcmc, 2, quantile, probs=c(.025,.975)))
    colnames(qq)= c("2.5%","97.5%")
    margpp= colMeans(covareffects.mcmc != 0)
    covareffects= cbind(covareffects, qq, margpp=margpp)
    ans= list(covareffects= covareffects, covareffects.mcmc=covareffects.mcmc)
    return(ans)
}


#Obtain points at which local effects will be estimated
# - If z is not missing, local effects are estimated at these z coordinates
# - If z is missing, these are the center of each local test's region with each x set to 0 or 1
# Input
# - x: x values
# - z: z values at which to estimate the local effects
# - regioncoord: region coordinates
# - regionbounds: boundaries of each region
# - testov: overlap between local tests and regions, as returned by testOverlaps
# Output
# - x: x values at which to estimate the effect. These will be (0,1) for each test region
# - z: z values at which to estimate the effect. If z is missing these are the local test region centers, else the input z's
# - region: region corresponding to each row in z
# - covareffect: data.frame indicating the covariate corresponding to each row in x, and its value (0 or 1)
estimationPoints= function(x, z, regioncoord, regionbounds, testov) {
    if (missing(z)) {
        #Obtain center point of each test interval
        z= lapply(testov$testIntervals, rowMeans)
        z= as.matrix(expand.grid(z))
        z= rbind(z, z) #duplicate matrix for x at mean(x) and at mean(x)+1
    }
    colnames(z)= paste('z',1:ncol(z),sep='')
    #Figure out region for each row in z
    if (ncol(z)==1) {
        regionid= intervals::interval_overlap(z[,1], intervals::Intervals(regioncoord[[1]],closed=c(TRUE,FALSE)))
        regionid= sapply(regionid, '[[', 1) #if there are multiple hits, then return the 1st one
    } else {
        zoverlap= vector("list", ncol(z))
        for (i in 1:ncol(z)) zoverlap[[i]]= intervals::interval_overlap(z[,i], intervals::Intervals(regioncoord[[i]]))
        regionid= integer(nrow(z))
        for (j in 1:nrow(z)) {
            zoverlapj= lapply(zoverlap, '[[', j)
            tmp= do.call(intersect, zoverlapj) #same as intersect(zoverlapj[[1]], ..., zoverlapj[[ncol(z)]])
            regionid[j]= tmp[1] #if there are multiple hits, then return the 1st one
        }
    }
    zregion= rownames(regioncoord[[1]])[regionid]
    zregion= sub("R","",zregion)
    #Build design matrix
    m= colMeans(x)
    xmid= matrix(rep(m,each=nrow(z)), nrow=nrow(z))
    xdes= zdes= vector("list",ncol(x))
    for (j in 1:ncol(x)) {
        xmid[,j]= rep(0:1, each=nrow(xmid)/2)
        xdes[[j]]= xmid
        zdes[[j]]= z
        xmid[,j]= m[j]
    }
    covariate= rep(1:ncol(x), each=nrow(z))
    value= rep(rep(0:1,each=nrow(z)/2), ncol(x))
    ans= list(x=do.call(rbind,xdes), z=do.call(rbind,zdes), region=rep(zregion,ncol(x)), covareffect=data.frame(covariate,value))
    return(ans)
}


#Return coordinates of each region
# Input:
# - zdiscrete: discretized z coordinates, with each dimension in a separate column
# - regionlabels: a list with an entry for each dimension, indicating the start and end of the interval for each discretized z coordinate
# Output: a list with an entry per dimension, indicating the coordinates of each region in that dimension
regionCoordinates= function(zdiscrete, regionlabels) {
    #zdiscrete= unique(zdiscrete)
    nn= paste('R', apply(zdiscrete,1,paste,collapse='.'), sep='')
    regioncoord= lapply(1:ncol(zdiscrete), function(i) { ans= matrix(NA, nrow=nrow(zdiscrete), ncol=2); rownames(ans)= nn; return(ans) })
    names(regioncoord)= paste('dim',1:length(regioncoord),sep='')
    for (j in 1:ncol(zdiscrete)) {
        for (i in 1:nrow(regioncoord[[j]])) {
            sel= zdiscrete[i,j]
            regioncoord[[j]][i,]= as.double(regionlabels[[j]][sel,c('start','end')])
        }
        regioncoord[[j]]= regioncoord[[j]][order(nn),]
    }
    return(regioncoord)
}


#Determine neighbouring regions (needed for P-spline prior)

neighbours_1d= function(ids) {
  sapply(ids, function(i) { if (i==1) { ans= i+1 } else if (i==length(ids)) { ans= i-1 } else { ans= c(i-1,i+1) }; return(ans) })
}

#Return neighbours of main effects in multi-dimensional case (ncol(z) > 1)
neighbours_multid_main= function(w) {
   w0names= strsplit(colnames(w), split="\\.")
   w0names.vname= sapply(w0names, '[[', 1)
   w0names.ids= as.numeric(sapply(w0names, function(zz) if (length(zz)>=2) { return(zz[2]) } else { return(NA) }))
   nn= unique(w0names.vname)
   neighbours0= vector("list", length(nn))
   for (i in 1:length(nn)) {
       sel= (w0names.vname == nn[i]) #select variable
       selwhich= which(sel)
       neighbours0[[i]]= vector("list",sum(sel))
       if (!any(is.na(w0names.ids[sel]))) { #if main effect has some neighbours
           for (j in 1:length(selwhich)) {
               if (j==1) { ans= selwhich[j]+1 } else if (j==length(selwhich)) { ans= selwhich[j]-1 } else { ans= selwhich[c(j-1,j+1)] }
               neighbours0[[i]][[j]]= ans
           }
       } else {
           for (j in 1:length(selwhich)) { neighbours0[[i]][[j]]= integer(0) }
       }
   }
   neighbours0= do.call(c, neighbours0)
   return(neighbours0)
}

#Return neighbours of tensor terms (interactions) in multi-dimensional case (ncol(z) > 1)
neighbours_multid_tensor= function(w) {
   #Store name of interaction term (e.g. "x1:x2") into w0names.vname, and the basis id within each term (e.g. (5,1)) into w0names.ids
   nn= colnames(w)
   w0names.vname= character(ncol(w))
   w0names.ids= vector("list", ncol(w))
   for (i in 1:ncol(w)) {
       termsi= strsplit(nn[i], split=":")[[1]]
       termsi= strsplit(termsi, split="\\.")
       w0names.vname[i]= paste(sapply(termsi, "[[", 1), collapse=":")
       w0names.ids[[i]]= as.numeric(sapply(termsi, "[[", 2))
   }
   w0names.ids= do.call(rbind, w0names.ids)
   #For each term, find the neighbouring basis (differing at most 1 in all coordinates)
   nn= unique(w0names.vname)
   neighbours0= vector("list", length(nn))
   for (i in 1:length(nn)) {
       sel= (w0names.vname == nn[i]) #select variable
       selwhich= which(sel)
       neighbours0[[i]]= vector("list",sum(sel))
       #ids.sel= w0names.ids[sel,]
       if (!any(is.na(w0names.ids[sel]))) { #if main effect has some neighbours
           for (j in 1:length(selwhich)) {
               coord_dist= t(w0names.ids) - w0names.ids[selwhich[j],]
               ans= which(sel & (colSums(abs(coord_dist) <= 1) == ncol(w0names.ids)) & (colSums(coord_dist) != 0))
               neighbours0[[i]][[j]]= ans
           }
       } else {
           for (j in 1:length(selwhich)) { neighbours0[[i]][[j]]= integer(0) }
       }
   }
   neighbours0= do.call(c, neighbours0)
   return(neighbours0)
}


neighbours_multid_cut= function(w, ncolz) {
    #Extract numeric region identifiers for each coordinate in z
    regionids= strsplit(sub("R","",colnames(w)), split="\\.")
    ids= matrix(NA, nrow=length(regionids), ncol=ncolz)
    for (i in 1:nrow(ids)) {
        ids[i,]= as.numeric(regionids[[i]])[1:ncolz]
    }
    #Find neighbours
    neighbours1= vector("list",nrow(ids))
    for (j in 1:nrow(ids)) {
        coord_dist= t(ids) - ids[j,]
        ans= setdiff(which(colSums(abs(coord_dist) <= 1) == ncolz), j)
        #ans= which((colSums(abs(coord_dist) <= 1) == ncolz) & (colSums(coord_dist) != 0))
        neighbours1[[j]]= ans
    }
    return(neighbours1)
}


#Create design matrix with baseline effects of x and orthogonalized cut basis effects of x interacted with z
# Input
# - x: covariate values
# - z: coordinates
# - y: outcome (only used if useSigma is TRUE)
# - x.adjust: optional argument. Adjustment covariate values (no testing performed on these)
# - xnew: covariate values for new observations
# - znew: covariate values for new observations
# - function_id: function identifier (only used if useSigma is TRUE)
# - region: identifier of the region that each row in z belongs to
# - regionnew: identifier of the region that each row in znew belongs to
# - regionbounds: boundaries of the regions
# - basedegree: baseline spline basis degree
# - cutdegree: cut spline basis degree
# - knots: knots for baseline spline
# - dropzeroes: if TRUE, columns with <5 non-zero entries are dropped
# - usecutbasis: if FALSE, then the basis is not cut and a standard spline basis is returned
# - useSigma: if TRUE, ytilde= Sigma^{-1/2} y and Sigma^{-1/2} w are returned
# - Sigma: covariance matrix, as given to localnulltest
# - rowids.fullrank: ensure that design matrix is full-rank for these rows. If necessary, drop constant columns from design matrix
# Output
# - w: design matrix. Its first columns corresponds to baseline design matrix w0, the rest to the cut spline basis w1. The output satisfies cor(w0,w1)=0
# - wnew: design matrix for (xnew, znew). Note that cor(w0new, w1new) need not be zero
# - ncolw0, ncolw1: number of columns in w0 and w1
# - vargroups: variable groups to be used in modelSelection, indicating variables which should be added/dropped as a group. For example, if there are multiple columns in w1 coding for a local effect, these are assigned to the same group
# - vargroupsn: same as vargroups, using numerical instead of text values
# - w1varname: name of the covariate associated to each column in w1
# - ytilde: block-diag(Sigma)^{-1/2} y
# - wtilde: block-diag(Sigma)^{-1/2} w
# - wtilde.adjust: block-diag(Sigma)^{-1/2} x.adjust
# - logdetSinv: log-determinant of block-diag(Sigma)^{-1}
createDesignLocaltest= function(x, z, y, x.adjust, xnew, znew, function_id, region, regionnew, regionbounds, basedegree, cutdegree, knots, dropzeroes=TRUE, usecutbasis=TRUE, useSigma=FALSE, Sigma, rowids.fullrank) {
    if (!missing(xnew)) {
        xall= rbind(x, xnew)
        zall= rbind(z, znew)
        regionall= c(region, regionnew)
    } else {
        xall= x
        zall= z
        regionall= region
    }
    if (ncol(z)==1) {
       w0= bspline(zall, degree=basedegree, knots=knots[[1]])
       neighbours0= neighbours_1d(1:ncol(w0))
    } else {
       w0= tensorbspline(zall, degree=basedegree, knots=knots, maineffects=TRUE)
       neighbours0.main= neighbours_multid_main(w0$main)
       neighbours0.tensor= neighbours_multid_tensor(w0$tensor)
       w0= cbind(w0$main, w0$tensor)
       neighbours0= c(neighbours0.main, neighbours0.tensor)
    }
    #Remove columns with no observations
    sel= apply(w0,2,'sd')>0
    w0= w0[,sel]  
    neighbours0= neighbours0[sel]
    #Define cut basis
    wcut= cutbasis(zall, degree=cutdegree, region=regionall, knots=regionbounds, dropzeroes=dropzeroes, usecutbasis=usecutbasis)
    if (ncol(z) != 1) neigh_multid= neighbours_multid_cut(wcut$basis, ncolz=ncol(z))
    w1= vargroups= neighbours1= vector("list",ncol(x))
    for (j in 1:ncol(x)) {
        w1[[j]]= xall[,j] * wcut$basis
        vargroups[[j]]= paste('x',j,'.R',wcut$regionid,sep='')
        if (ncol(z)==1) {
            neighbours1[[j]]= neighbours_1d(1:ncol(w1[[j]]))
        } else {
            neighbours1[[j]]= neigh_multid
        }
    }
    w1varname= rep(paste('x',1:ncol(x),sep=''), sapply(w1, ncol))
    w1= do.call(cbind, w1)
    vargroups= do.call(c, vargroups)
    vargroupsn= c(1:ncol(w0), ncol(w0) + as.numeric(factor(vargroups)))
    vargroups= c(paste("baseline",1:ncol(w0),sep=''), vargroups)
    neighbours1= do.call(c, neighbours1)
    # Separate w from wnew
    if (!missing(xnew)) {
        sel= 1:nrow(x)
        w0new= w0[-sel,]; w1new= w1[-sel,]
        w0= w0[sel,]; w1= w1[sel,]
    } else {
        w0new= w1new= NULL
    }
    # Orthogonalize design matrix coding for local covariate effects
    w_decorrelated= decorrelate(w0=w0, w1=w1, region=region, w0new=w0new, w1new=w1new, regionnew=regionnew)
    # Remove columns that became constant after removing xnew
    w1o= w_decorrelated$w1o
    colsel0= (apply(w0, 2, 'sd') > 0); colsel1= (apply(w1o, 2, 'sd') > 0)
    w0= w0[,colsel0,drop=FALSE]; w1o= w1o[,colsel1,drop=FALSE]
    vargroups= vargroups[c(colsel0,colsel1)]
    vargroupsn= vargroupsn[c(colsel0,colsel1)]
    w1varname= w1varname[colsel1]
    w= cbind(w0, w1o)
    neighbours= c(neighbours0[colsel0], neighbours1[colsel1])
    if (!missing(xnew)) {
        wnew= cbind(w0new[,colsel0,drop=FALSE], w_decorrelated$w1newo[,colsel1,drop=FALSE])
    } else {
        wnew= NULL
    }
    #Ensure that design matrix is full rank
    ct= apply(w[rowids.fullrank,,drop=FALSE], 2, 'sd') == 0
    if (any(ct)) {
        w= w[,!ct]
        wnew= wnew[,!ct]
        ncolw0= ncol(w0) - sum(ct[1:ncol(w0)])
        ncolw1= ncol(w_decorrelated$w1o) - sum(ct[-1:-ncol(w0)])
        vargroups= vargroups[!ct]
        vargroupsn= vargroupsn[!ct]
        w1varname= w1varname[ !ct[-1:-ncol(w0)] ]
        neighbours= neighbours[!ct]
    } else {
        ncolw0= ncol(w0)
        ncolw1= ncol(w_decorrelated$w1o)
    }
    #If errors are dependent, transform the outcome and the design matrix
    if (useSigma) {
        if (!is.function(Sigma)) {
            S= estimateSigma_localtest(y=y, w=w, z=z, function_id=function_id, region=region, method=Sigma)
        } else {
            zregion= unique(cbind(as.numeric(region), z))
            S= evalSigma_localtest(Sigma, region=zregion[,1], z=zregion[,-1,drop=FALSE])
        }
        yt= uncorrelate_outcome(y=y, w=w, x.adjust=x.adjust, region=region, S=S, function_id=function_id) #return Sigma^{-1/2} y, Sigma^{-1/2} w, log |Sinv|
        ytilde= yt$ytilde; wtilde= yt$wtilde; logdetSinv= yt$logdetSinv
        wtilde.adjust= w_decorrelated$wtilde.adjust
    } else {
        ytilde= wtilde= NULL
        logdetSinv= 0
    }
    #Return output
    ans= list(w=w, wnew=wnew, ncolw0= ncolw0, ncolw1=ncolw1, vargroups=vargroups, vargroupsn=vargroupsn, w1varname=w1varname, neighbours=neighbours, ytilde=ytilde, wtilde=wtilde, logdetSinv=logdetSinv)
    return(ans)
}




cutbasis= function(z, degree, region, knots, dropzeroes=TRUE, usecutbasis=TRUE) {
    #For each region, create a cut B-spline basis for z that is zero when z is outside the region. Tensor B-splines are used when ncol(z)>1
    #Input
    # - z: matrix for which cut spline basis is to be created
    # - degree: spline degree
    # - region: vector of length == nrow(z), indicating the region of each row in z
    # - knots: list of length == ncol(z) indicating the knots for each dimension of z
    # - dropzeroes: if TRUE, columns that have <4 non-zero elements are dropped
    # - usecutbasis: if FALSE, then the basis is not cut and a standard spline basis is returned
    #Output
    # - basis: the cut basis
    # - regionid: indicates the region that each column in basis corresponds to
    if (length(region) != nrow(z)) stop("length(region) must be equal to nrow(z)")
    #Obtain standard B-splines
    if (ncol(z)==1) {
       baseline= bspline(z, degree=degree, knots=knots[[1]])
    } else {
       baseline= tensorbspline(z, degree=degree, knots=knots, maineffects=FALSE)$tensor
    }
    #Obtain cut splines by placing 0's in the standard B-splines
    # First, order regionds by region id. Like regionids= regionids[order(as.numeric(regionids))], but applicable when ncol(z)>=3
    regionids= unique(region)
    regionidschar= strsplit(regionids, split="\\.")
    maxchar= max(sapply(regionidschar, function(z) max(nchar(z))), na.rm=TRUE)
    regionidschar= sapply(regionidschar, function(z) {
        ans= paste(rep("0",maxchar-1), z, sep='')
        ans= substr(ans, start=nchar(ans)-maxchar+1, stop=nchar(ans))
        paste(ans, collapse=".")
    })
    regionidschar[is.na(regionids)]= 'NA'
    regionids= regionids[order(regionidschar)]
    nregions= length(regionids)
    basis= vector("list",nregions)
    for (i in 1:nregions) {
        if (!is.na(regionids[i])) {
          rowsel= (region == regionids[i])
          rowsel[is.na(rowsel)]= FALSE
          basissel= (colSums(baseline[rowsel,,drop=FALSE] != 0) >= ifelse(dropzeroes,5,1)) #remove columns that are essentially always 0
          if (usecutbasis) {
              basis[[i]]= matrix(0, nrow=nrow(z), ncol=sum(basissel))
              basis[[i]][rowsel,]= baseline[rowsel,basissel]
          } else {
              basis[[i]]= baseline[,basissel,drop=FALSE]
          }
          if (sum(basissel)==1) {
              colnames(basis[[i]])= paste("R",regionids[i],sep='')
          } else if (sum(basissel)>1) {
              colnames(basis[[i]])= paste("R",regionids[i],".",1:ncol(basis[[i]]),sep='')
          }
        } else { #regionids[i]==NA when some regions didn't have enough observations and hence were removed
          basis[[i]]= matrix(0, nrow=nrow(z), ncol=0)
        }
    }
    regionid= rep(regionids, sapply(basis,ncol))
    basis= do.call(cbind, basis)
    ans= list(basis=basis, regionid=regionid)
    return(ans)    
}



cutbasisuniv= function(z, degree, regionbounds) {
    #Same as cutbasis, when z is a vector (i.e. one-dimensional coordinates)
    if (!is.vector(z)) stop("z must be a vector")
    regioninterval= cbind(regionbounds[-length(regionbounds)], regionbounds[-1])
    regioninterval= regioninterval[(regioninterval[,2] > min(z)) & (regioninterval[,1] < max(z)),] #restrict regions to range of observed z's
    nregions= nrow(regioninterval)
    #Range of z values spanned by each standard B-spline
    basisbounds= matrix(NA, nrow=length(regionbounds)-degree-1, ncol=2)
    for (i in 1:nrow(basisbounds)) basisbounds[i,]= c(regionbounds[i],regionbounds[i+degree+1])
    #Obtain standard B-splines
    baseline= bspline(z, degree=degree, knots=regionbounds)
    #Obtain cut splines by placing 0's in the standard B-splines
    basis= matrix(0, nrow=length(z), ncol= nregions * (degree+1))
    subsetid= integer(ncol(basis))
    for (i in 1:nregions) {
        rowsel= ((z >= regioninterval[i,1]) & (z < regioninterval[i,2]))
        colsel= (1 + (i-1)*(degree+1)): (i*(degree+1))
        basissel= (basisbounds[,1] < regioninterval[i,2]) & (basisbounds[,2] > regioninterval[i,1])
        if (length(colsel) > sum(basissel)) colsel= colsel[1:sum(basissel)]
        basis[rowsel,colsel]= baseline[rowsel, basissel]
        subsetid[colsel]= i
    }
    nonzero= (colSums(basis != 0) >= 5) #remove columns that are essentially always 0
    basis= basis[,nonzero]
    subsetid= subsetid[nonzero]
    regionwithid= data.frame(1:nrow(regioninterval), regioninterval)
    names(regionwithid)= c('subsetid','regionstart','regionend')
    subsetid= merge(data.frame(subsetid=subsetid), regionwithid, by='subsetid')
    ans= list(basis=basis, subsetid=subsetid, regions=regioninterval)
    return(ans)    
}



decorrelate= function(w0, w1, region, w0new, w1new, regionnew) {
    # Decorrelate basis w1 coding for local covariate effects in each region from basis w0 coding for the baseline mean
    # Input
    # - w0: design matrix for baseline
    # - w1: design matrix for effect of covariates
    # - region: vector indicating the region that each row in (w0,w1) corresponds to
    # - w0new: design matrix for baseline for new observations
    # - w1new: design matrix for effect of covariates for new observations
    # - regionnew: region that each row in (w0new, w1new) corresponds to
    # Ouput:
    # - w1o: residuals from regressing w1 onto w0
    # - w1newo: residuals from predicting w1new from w0new, using the least-squares coefficient regressing w1 onto w0
    # Note: entries in cor(w0, w1o) are guaranteed to all be zero, but not so for (w0new, w1newo)
    if (nrow(w0) != nrow(w1)) stop("w0 and w1 must have the same number of rows")
    if (nrow(w0) != length(region)) stop("region must be a vector of length equal to nrow(w0)")
    w1o= matrix(0, nrow=nrow(w1), ncol=ncol(w1))
    if (!is.null(w0new)) {
        if (nrow(w0new) != nrow(w1new)) stop("w0new and w1new must have the same number of rows")
        if (nrow(w0new) != length(regionnew)) stop("regionnew must be a vector of length equal to nrow(w0new)")
        w1newo= matrix(0, nrow=nrow(w1new), ncol=ncol(w1new))
    } else {
        w1newo= NULL
    }
    regionids= unique(region)
    for (j in 1:length(regionids)) {
        rowsel= (region == regionids[j])     #rows in w0 corresponding to observations in region j
        rowsel[is.na(rowsel)]= FALSE
        colsel1= (colSums(w1[rowsel,,drop=FALSE]!=0) > 0) #columns in w1 coding for region j
        colsel0= (colSums(w0[rowsel,,drop=FALSE]!=0) > 0) #columns in w0 coding for region j
        if (any(colsel1)) {
            fit= lm(w1[rowsel,colsel1] ~ w0[rowsel,colsel0])
            w1o[rowsel,colsel1]= residuals(fit)
            if (!is.null(w0new)) {
                rowselnew= (regionnew == regionids[j])   #rows in w0new corresponding to observations in region j
                if (any(rowselnew)) {
                    b= coef(fit)
                    b[is.na(b)]= 0
                    w1newo[rowselnew,colsel1] = w1new[rowselnew,colsel1,drop=FALSE] - cbind(rep(1,sum(rowselnew)), w0new[rowselnew,colsel0,drop=FALSE]) %*% b
                }
            }
        }
    }
    ans= list(w1o=w1o, w1newo=w1newo)
    return(ans)
}


#For each local test, find the regions that have an overlap with the test
# Input
# - localgrid: list where each element corresponds to a coordinate, and contains the grid defining the local test boundaries in each coordinate
# - regioncoord: coordinates of the local test regions. A list where element j is a data.frame with a row for each test region that indicates the start/end in the j^th coordinate. For example, regioncoord[[1]][1,] is the start/end of testing region 1 in the first coordinate z[,1], regioncoord[[2]][1,] that for the 2nd coordinate
# Output
# - testIntervals: localgrid formatted as intervals
# - testxregion: list with an entry for each element in the grid, indicating the local test regions overlapping that element
testOverlaps= function(localgrid, regioncoord) {
    ndim= length(localgrid)
    testIntervals= overlaps= vector("list", length(localgrid))
    #For each coordinate, find overlaps between local tests & regions
    for (i in 1:ndim) {
        testIntervals[[i]]= intervals::Intervals(cbind(localgrid[[i]][-length(localgrid[[i]])], localgrid[[i]][-1]))
        regionIntervals= intervals::Intervals(regioncoord[[i]]) 
        overlaps[[i]]= intervals::interval_overlap(testIntervals[[i]], regionIntervals)
    }
    #For each grid element, select regions that overlap in all coordinates
    if (ndim == 1) {
      testxregion= overlaps[[1]]
    } else {
      tests= expand.grid(lapply(localgrid, function(l) 1:(length(l)-1)))
      names(tests)= paste('dim',1:ncol(tests),sep='')
      testxregion= vector("list", nrow(tests))
      for (j in 1:nrow(tests)) {
          testjoverlap= vector("list",ndim)
          for (i in 1:ndim) {
              testdimi= tests[j,i]
              testjoverlap[[i]]= overlaps[[i]][[testdimi]]  #regions overlapping j^th test in i^th coordinate
          }
          testxregion[[j]]= do.call(intersect, testjoverlap)
      }
    }
    #Convert region ids to names
    regionnames= rownames(regioncoord[[1]])
    testxregion= lapply(testxregion, function(zz) regionnames[zz])
    #Return output
    ans= list(testIntervals=testIntervals, testxregion=testxregion)
    return(ans)
}


#For each local test, compute its posterior probability as the prob of any region overlapping the test being active
# Input
# - regionids: binary matrix with 1 column per region
# - ppregions: posterior probability of each row in regionids
# - testxregion: list with an entry for each local test, indicating the regions overlapping that test. These region labels must match column names in regionids
postProblocaltest= function(regionids, ppregions, testxregion) {
    ntests= length(testxregion)
    ans= double(ntests)
    for (i in 1:ntests) {
        sel= colnames(regionids) %in% testxregion[[i]]         #regions overlapping with test i
        ans[i]= sum(ppregions[rowSums(regionids[,sel,drop=FALSE]) > 0])   #sum pp whenever any region is active
    }
    return(ans)
}

###########################################################################################
##
##  GRADIENT AND HESSIAN FOR LOG NON-LOCAL PRIOR (MOM, EMOM AND IMOM PRIOR) DENSITIES. ALSO FOR JOINT (NLP, INV. GAMMA)
##
##  NOTE: THESE ROUTINES ARE ALSO IMPLEMENTED IN C
##
###########################################################################################


###########################################################################################
## DERIVATIVES OF MOM PRIOR
###########################################################################################

#Gradient of log-pMOM(th;0,phi*tau) density wrt th
dmomgrad <- function(th, logphi, tau) { 2/th - th/(exp(logphi)*tau) }

#Hessian of log-pMOM(th;0,phi*tau) density wrt th
dmomhess <- function(th, logphi, tau) { ans <- diag(length(th)); diag(ans) <- -2/th^2 - 1/(exp(logphi)*tau); return(ans) }

#Gradient of log-pMOM(th;0,phi*tau) + log-IG(th;phi,alpha/2,lambda/2) wrt (th, logphi) where logphi=log(phi)
dmomiggrad <- function(th, logphi, tau, alpha, lambda) {
  if (length(th)>0) {
    ans <- c(dmomgrad(th, logphi=logphi, tau=tau), -1.5*length(th) - 0.5*alpha -1 + 0.5*(sum(th)^2/tau + lambda) * exp(-logphi))
  } else {
    ans <- - 0.5*alpha -1 + 0.5*lambda*exp(-logphi)
  }
  return(ans)
}

#Hessian of log-pMOM(0,phi*tau) + log-IG(phi,alpha,lambda) wrt (th, logphi) where logphi=log(phi)
dmomighess <- function(th, logphi, tau, alpha, lambda) {
  p <- length(th)
  ans <- matrix(NA,nrow=p+1,ncol=p+1)
  if (length(th)>0) {
    ans[1:p,1:p] <- dmomhess(th, logphi, tau)
    ans[1:p,p+1] <- ans[p+1,1:p] <- th/(exp(logphi)*tau)
  }
  ans[p+1,p+1] <- -0.5 * exp(-logphi) * (sum(th^2)/tau+lambda)
  return(ans)
}



###########################################################################################
## DERIVATIVES OF EMOM PRIOR
###########################################################################################

#Gradient of log-peMOM(0,phi*tau) density
demomgrad <- function(th, logphi, tau) { 2*tau*exp(logphi)/th^3 - th*exp(-logphi)/tau }

#Hessian of log-peMOM(0,phi*tau) density
demomhess <- function(th, logphi, tau) { ans <- diag(length(th)); diag(ans) <- -6*tau*exp(logphi)/th^4 - exp(-logphi)/tau; return(ans) }

#Gradient of log-peMOM(0,phi*tau) + log-IG(phi,alpha/2,lambda/2) under log(phi) parameterization
demomiggrad <- function(th, logphi, tau, alpha, lambda) {
  if (length(th)>0) {
    ans <- c(demomgrad(th, logphi=logphi, tau=tau), -0.5*length(th) - 0.5*alpha -1 + 0.5*(sum(th)^2/tau + lambda) * exp(-logphi) - exp(logphi)*tau*sum(1/th^2))
  } else {
    ans <- -0.5*alpha -1 + 0.5*lambda * exp(-logphi)
  }
  return(ans)
}

#Hessian of log-peMOM(0,phi*tau) + log-IG(phi,alpha,lambda) under log(phi) parameterization
demomighess <- function(th, logphi, tau, alpha, lambda) {
  p <- length(th)
  ans <- matrix(NA,nrow=p+1,ncol=p+1)
  if (p>0) {
    ans[1:p,1:p] <- demomhess(th, logphi, tau)
    ans[1:p,p+1] <- ans[p+1,1:p] <- th/(exp(logphi)*tau) + 2*tau*exp(logphi)/th^3
  }
  ans[p+1,p+1] <- -0.5 * exp(-logphi) * (sum(th^2)/tau+lambda) - tau * exp(logphi) * sum(1/th^2)
  return(ans)
}


###########################################################################################
## DERIVATIVES OF IMOM PRIOR
###########################################################################################

#Gradient of log-pimom(0,phi*tau) density
dimomgrad <- function(th, logphi, tau) { 2*tau*exp(logphi)/th^3 - 2/th }

#Hessian of log-pimom(0,phi*tau) density
dimomhess <- function(th, logphi, tau) { ans <- diag(length(th)); diag(ans) <- -6*tau*exp(logphi)/th^4 + 2/th^2; return(ans) }

#Gradient of log-pimom(0,phi*tau) + log-IG(phi,alpha/2,lambda/2) under log(phi) parameterization
dimomiggrad <- function(th, logphi, tau, alpha, lambda) {
  if (length(th)>0) {
    ans <- c(dimomgrad(th, logphi=logphi, tau=tau), 0.5*length(th) - 0.5*alpha -1 + 0.5*lambda*exp(-logphi) - exp(logphi)*tau*sum(1/th^2))
  } else {
    ans <- -0.5*alpha -1 + 0.5*lambda*exp(-logphi)
  }
  return(ans)
}

#Hessian of log-pimom(0,phi*tau) + log-IG(phi,alpha,lambda) under log(phi) parameterization
dimomighess <- function(th, logphi, tau, alpha, lambda) {
  p <- length(th)
  ans <- matrix(NA,nrow=p+1,ncol=p+1)
  if (p>0) {
    ans[1:p,1:p] <- dimomhess(th, logphi, tau)
    ans[1:p,p+1] <- ans[p+1,1:p] <- 2*tau*exp(logphi)/th^3
  }
  ans[p+1,p+1] <- -0.5*exp(-logphi)*lambda - tau * exp(logphi) * sum(1/th^2)
  return(ans)
}

context("Test msPriorSpec initializers")
library("modelSelection")

source(test_path("data-for-tests.R"))

patrick::with_parameters_test_that(
  "msPriorSpec can be initialized with tau:", {
    pr_default <- priorfun()
    pr_taustd <- priorfun(taustd=2)
    pr_tau <- priorfun(tau=.3)
    expect_equal(as.double(pr_default@priorPars['taustd']), 1)
    expect_equal(as.double(pr_taustd@priorPars['taustd']), 2)
    expect_equal(as.double(pr_tau@priorPars['tau']), .3)
  },
  .test_name=c("momprior", "zellnerprior", "normalidprior", "groupmomprior", "groupzellnerprior"),
  priorfun=c(momprior, zellnerprior, normalidprior, groupmomprior, groupzellnerprior)
)

\name{msfit-class}
\Rdversion{1.1}
\docType{class}
\alias{msfit-class}
\alias{msfit}
\alias{show,msfit-method}
\alias{msfit.coef}
\alias{msfit.plot}
\alias{msfit.predict}
\alias{coefByModel}
\alias{coefByModel-methods}
\alias{coefByModel,msfit-method}


\title{Class "msfit"}
\description{
Stores the output of Bayesian variable selection, as produced by
function \code{modelSelection}.
The class extends a list, so all usual methods for lists also work for
\code{msfit} objects, e.g. accessing elements, retrieving names etc.

Methods are provided to compute posterior probabilities,
obtaining regression coefficient estimates and posterior intervals
(both via Bayesian model averaging and for individual models),
and sampling from their posterior distribution, as indicated below.
}
\section{Objects from the Class}{
  Typically objects are automatically created by a call to \code{modelSelection}.
  Alternatively, objects can be created by calls of the form
  \code{new("msfit",x)} where \code{x} is a list with the adequate
  elements (see slots).
}
\section{Slots}{
  The class extends a list with elements:
  \describe{

    \item{postSample}{\code{matrix} with posterior samples for the model
      indicator. \code{postSample[i,j]==1}
      indicates that variable j was included in the model in the MCMC
      iteration i}
    \item{postOther}{\code{postOther}
      returns posterior samples for parameters other than the model
      indicator, i.e. basically hyper-parameters.
      If hyper-parameters were fixed in the model specification, \code{postOther} will be empty.}
    \item{margpp}{Marginal posterior probability for inclusion of each
      covariate. This is computed by averaging marginal post prob for
      inclusion in each Gibbs iteration, which is much more accurate than
      simply taking \code{colMeans(postSample)}}.
    \item{postMode}{Model with highest posterior probability amongst all those visited}
    \item{postModeProb}{Unnormalized posterior prob of posterior mode (log scale)}
    \item{postProb}{Unnormalized posterior prob of each visited model (log
      scale)}
    \item{family}{Residual distribution, i.e. argument \code{family}
      when calling \code{modelSelection}}
    \item{p}{Number of variables}
    \item{priors}{Priors specified when calling \code{modelSelection}}
    \item{ystd}{For internal use. Stores the response variable,
      standardized if \code{center} or \code{scale} were set to
      \code{TRUE}}
    \item{xstd}{For internal use. Stores the covariates,
      standardized if \code{center} or \code{scale} were set to
      \code{TRUE}}
    \item{stdconstants}{For internal use. If \code{center} or
      \code{scale} were set to \code{TRUE}, stores the sample mean
      and standard deviation of the outcome and covariates}
    \item{call}{Stores info about the call, the formula used (if any),
      splines used etc}
  }
}
\section{Methods}{
  \describe{
    \item{coef}{Obtains posterior means and intervals via Bayesian model
      averaging}
    \item{coefByModel}{Obtains posterior means and intervals for
      individual models}
    \item{plot}{Shows estimated posterior inclusion probability for each
      parameter vs. number of MCMC iterations}
    \item{predict}{Obtains posterior means and intervals for given
      covariate values. These are posterior intervals for the mean, not
      posterior predictive intervals for the outcome}
    \item{show}{\code{signature(object = "msfit")}: Displays general information about the object. }
    \item{postProb}{\code{signature(object = "msfit")}: Extracts
      posterior model probabilities. }
    \item{rnlp}{\code{signature(object = "msfit")}: Obtain posterior
      samples for regression coefficients. }
  }
}
\references{
Johnson VE, Rossell D. Non-Local Prior Densities for Default Bayesian Hypothesis Tests. Journal of the Royal Statistical Society B, 2010, 72, 143-170

Johnson VE, Rossell D. Bayesian model selection in high-dimensional
settings. Journal of the American Statistical Association, 107, 498:649-660.
}
\author{
David Rossell
}
\seealso{
See also \code{\link{modelSelection}} and \code{\link{rnlp}}.
}
\examples{
showClass("msfit")
}
\keyword{classes}

\name{modelSelection}
\alias{modelSelection}
\alias{modelSelection_eBayes}
\alias{modelsearchBlockDiag}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ Bayesian variable selection for generalized linear and
  generalized additive models.
}
\description{
  Bayesian model selection for linear, asymmetric linear,
  median and quantile regression, for GLMs and GAMs under
  non-local or Zellner priors. p>>n can be handled.

  modelSelection enumerates all models when feasible
  and uses a Gibbs scheme otherwise.
  See \code{coef} and \code{coefByModel} for estimates and posterior
  intervals of regression coefficients, and \code{rnlp} for posterior samples.

  modelSelection performs standard Bayesian model selection.
  
  modelSelection_eBayes uses empirical Bayes to set prior inclusion
  probabilities pi. pi can be regressed on meta-covariates Z via
  log(pi/(1-pi))= Z w. If Z contains only an intercept, a global prior
  inclusion probability is learn from data.

  modelsearchBlockDiag seeks the highest posterior
  probability model using an iterative block search.
}
\usage{
modelSelection(y, x, data, smoothterms, nknots=9,
groups, constraints, center=TRUE, scale=TRUE,
enumerate, includevars, models,
maxvars, niter=5000, thinning=1,
burnin=round(niter/10), family='normal', priorCoef,
priorGroup, priorModel=modelbbprior(1,1),
priorConstraints,
priorVar=igprior(.01,.01),
priorSkew=momprior(tau=0.348),
neighbours, phi, deltaini,
initSearch='CDA', method='auto', adj.overdisp='intercept',
hess='asymp', optimMethod, optim_maxit, initpar='none', B=10^5,
XtXprecomp, verbose=TRUE)

modelSelection_eBayes(Z, wini, niter.mcmc= 5000, niter.mstep= 1000,
niter.eBayes= 20, priorvar.w, verbose=TRUE, ...)

modelsearchBlockDiag(y, x, priorCoef=momprior(tau=0.348),
priorModel=modelbbprior(1,1), priorVar=igprior(0.01,0.01),
blocksize=10, maxiter=10, maxvars=100, maxlogmargdrop=20,
maxenum=10, verbose=TRUE)
}
\arguments{
\item{y}{Either a formula with the regression equation or a vector with
  observed responses. The response can be either continuous or of class
  \code{Surv} (survival outcome). If \code{y} is a formula then \code{x},
  \code{groups} and \code{constraints} are automatically created}
\item{x}{Design matrix with linear covariates for which we want to
  assess if they have a linear effect on the response. Ignored if
  \code{y} is a formula}
\item{data}{If \code{y} is a formula then \code{data} should be a data
  frame containing the variables in the model}
\item{smoothterms}{Formula for non-linear covariates (cubic splines),
  modelSelection assesses if the variable has no effect, linear or
  non-linear effect. \code{smoothterms} can also be a design matrix or
  data.frame containing linear terms, for each column modelSelection
  creates a spline basis and tests no/linear/non-linear effects}
\item{nknots}{Number of spline knots. For cubic splines the non-linear
  basis adds knots-4 coefficients for each linear term, we recommend
  setting \code{nknots} to a small/moderate value}
\item{groups}{If variables in \code{x} such be added/dropped in groups,
  \code{groups} indicates the group that each variable corresponds to.
  By default \code{groups} is \code{1:ncol(x)}, i.e. 
  each variable goes into a separate group}
\item{constraints}{Constraints on the model space. List with length
  equal to the number of groups;
  if group[[i]]=c(j,k) then group i can only be in the model if groups j and k are also in the model}
\item{center}{If \code{TRUE}, \code{y} and \code{x} are centered to have
  zero mean. Dummy variables corresponding to factors are NOT centered}
\item{scale}{If \code{TRUE}, \code{y} and columns in \code{x} are
  scaled to have variance=1. Dummy variables corresponding to factors are NOT scaled}
\item{enumerate}{Default is \code{TRUE} if there's less than 15 variable
  groups. If \code{TRUE} all models with up to \code{maxvars} are
  enumerated, else Gibbs sampling is used to explore the model space}
\item{includevars}{Logical vector of length ncol(x) indicating variables
that should always be included, i.e. variable selection is
not performed for these variables. By default \code{includevars} is \code{rep(FALSE,ncol(x))}}
\item{models}{Optional logical matrix indicating the models to be
  enumerated with rows equal to the number of desired models and columns
to the number of variables in \code{x}. }
\item{maxvars}{When \code{enumerate==TRUE} only models with up to maxvars variables
  enumerated (defaults to all variables). In \code{modelsearchBlockDiag} a sequence of models
  is defined from 1 up to \code{maxvars}}
\item{niter}{Number of Gibbs sampling iterations}
\item{thinning}{MCMC thinning factor, i.e. only one out of each \code{thinning} iterations are reported. Defaults to thinning=1, i.e. no thinning}
\item{burnin}{Number of burn-in MCMC iterations. Defaults to
  \code{.1*niter}. Set to 0 for no burn-in}
\item{family}{Family of parametric distribution. Use
  'normal' for Normal errors, 'binomial' for logistic regression,
  'poisson' for Poisson regression.
  'twopiecenormal' for two-piece Normal,
  'laplace' for Laplace errors and 'twopiecelaplace' for double
  exponential.
  For 'auto' the errors are assumed continuous and their distribution
  is inferred from the data among
  'normal', 'laplace', 'twopiecenormal' and 'twopiecelaplace'.
  'laplace' corresponds to median regression and 'twopiecelaplace'
  to quantile regression. See argument \code{priorSkew}}
\item{priorCoef}{Prior on coefficients, created
  by \code{momprior}, \code{imomprior}, \code{emomprior} or
  \code{zellnerprior}.
  Prior dispersion is on coefficients/sqrt(scale) for Normal and
  two-piece Normal, and on coefficients/sqrt(2*scale) for Laplace
  and two-piece Laplace.}
\item{priorGroup}{Prior on grouped coefficients (e.g. categorical
  predictors with >2 categories, splines). Created by
  \code{groupmomprior}, \code{groupemomprior},
  \code{groupimomprior} or \code{groupzellnerprior}}
\item{priorModel}{Prior on model space. Use \code{modelbbprior()}
  for Beta-Binomial prior, \code{modelbinomprior(p)} for Binomial
  prior with prior inclusion probability \code{p},
  \code{modelcomplexprior} for Complexity prior,
  or \code{modelunifprior()} for Uniform prior}
\item{priorConstraints}{Prior distribution on the number of terms
  subject to hierarchical constrains that are included in the model}
\item{priorVar}{Inverse gamma prior on scale parameter.
  For Normal outcomes variance=scale, for Laplace outcomes
  variance=2*scale}
\item{priorSkew}{Either a fixed value for tanh(alpha) where alpha is
  the asymmetry parameter or a prior on tanh(alpha).
  For \code{family=='twopiecelaplace'} setting alpha=a is equivalent
  to performing quantile regression for the quantile (1+a)/2.
  Ignored if \code{family} is 'normal' or 'laplace'.}
\item{neighbours}{Only used if priorCoef is an icarplus prior.
  neighbours is a list with the same length as the design matrix.
  Its entry j should be a vector indicating the neighbours of j,
  and have 0 length if j has no neighbours.}
\item{phi}{The error variance in Gaussian models, typically this is
  unknown and is left missing}
\item{deltaini}{Logical vector of length \code{ncol(x)} indicating which
  coefficients should be initialized to be non-zero.
  Defaults to all being zero, except those indicated by \code{includevars}}
\item{initSearch}{Algorithm to refine
  \code{deltaini}. \code{initSearch=='CDA'} uses a coordinate descent
  algorithm. \code{initSearch=='SCAD'} sets \code{deltaini} to the
  non-zero elements in a SCAD fit with cross-validated regularization
  parameter. \code{initSearch=='none'} leaves \code{deltaini} unmodified}
\item{method}{Method to compute marginal likelihood.
  \code{method=='Laplace'} for Laplace approx, \code{method=='ALA'}
  for approximate Laplace approximation.
  \code{method=='MC'} for Importance Sampling, \code{method=='Hybrid'}
  for Hybrid Laplace-IS (only available for piMOM prior). See Details.}
  \code{method=='auto'} attempts to use exact calculations when
  possible, otherwise ALA if available, otherwise Laplace approx.
\item{adj.overdisp}{Only used when method=='ALA'. Over-dispersion
  adjustment in models with fixed dispersion parameter, as in logistic
  and Poisson regression. adj.overdisp='none' for no adjustment (not
  recommended, particularly for Poisson
  models). adj.overdisp='intercept' to estimate over-dispersion from the
  intercept-only model, and adj.overdisp='residuals' from the Pearson
  residuals of each model}
\item{hess}{Method to estimat the hessian in the Laplace approximation to the integrated
  likelihood under Laplace or asymmetric Laplace errors. When
  hess=='asymp' the asymptotic hessian is used, hess=='asympDiagAdj' a
  diagonal adjustment is applied (see Rossell and Rubio for details).}
\item{optimMethod}{Algorithm to maximize objective function when
  method=='Laplace'. Leave unspecified or set optimMethod=='auto' for an
  automatic choice. optimMethod=='LMA' uses modified
  Newton-Raphson algorithm, 'CDA' coordinate descent algorithm}
\item{optim_maxit}{Maximum number of iterations when method=='Laplace'}
\item{initpar}{Initial regression parameter values when finding the
  posterior mode to approximate the integrated likelihood. 'none', 'MLE',
  'L1', or a numeric vector with initial
  values. 'auto': if p<n/2 MLE is used, else L1 (regularization parameter set
  via BIC)}
%\item{initpar}{Initial regression parameter values when finding the
%  posterior mode to approximate the integrated likelihood. 'none', 'MLE',
%  'MLE-aisgd', 'L1', 'L2-aisgd', or a numeric vector with initial
%  values. If p<n/2 MLE is used, else L1 (regularization parameter set
%  via BIC). 'auto': if n>10,000 or p>200, then MLE-aisgd or L2-aisgd are
%  used. aisgd stands for averaged intrinsic stochastic gradient descent
%  (see function sgd in package sgd)}
\item{B}{Number of samples to use in Importance Sampling scheme. Ignored
  if \code{method=='Laplace'}}
\item{XtXprecomp}{Set to \code{TRUE} to pre-compute the Gram matrix x'x
  upfront (saves time), to \code{FALSE} to compute and store elements
  only as needed (saves memory). By default
  \code{XtXprecomp <- ifelse(ncol(x)<10^4,TRUE,FALSE)}}

\item{verbose}{Set \code{verbose==TRUE} to print iteration progress}

\item{blocksize}{Maximum number of variables in a block. Careful, the
  cost of the algorithm is of order \code{2^blocksize}}
\item{maxiter}{Maximum number of iterations, each iteration includes a
  screening pass to add and subtract variables}
\item{maxlogmargdrop}{Stop the sequence of models when the drop in log
  p(y|model) is greater than \code{maxlogmargdrop}. This option avoids
  spending unnecessary time exploring overly large models}
\item{maxenum}{If the posterior mode found has less than \code{maxenum}
  variables then do a full enumeration of all its submodels}


\item{Z}{p x q matrix containing the q meta-covariates for the p
  covariates. An intercept is automatically added (including when
  \code{Z} is missing)}
\item{wini}{Optional. Initial value for the q-dimensional
  hyper-parameter w}
\item{niter.mcmc}{Number of iterations in the final MCMC,
  run once after hyper-parameter estimates have been obtained}
\item{niter.mstep}{Number of MCMC iterations in each M-step
  required to update hyper-parameter estimates}
\item{niter.eBayes}{Max number of iterations in the empirical
  Bayes optimization algorithm. The algorithm also stop when
  the objective function didn't improve in >=2 iterations}
\item{priorvar.w}{Hyper-parameters w follow a prior
  w ~ N(0, priorvar.w (Z^T Z/p)^(-1)) where priorvar.w is the prior
  variance. By default is set such that all prior inclusion
  probabilities lie in (0.001,0.999) with 0.95 prior probability}
\item{...}{Further parameters passed onto \code{modelSelection}}

}
\details{
  Let delta be the vector indicating inclusion/exclusion of each
  column of x in the model. The Gibbs algorithm sequentially samples from the
  posterior of each element in delta conditional on all the remaining
  elements in delta and the data.
  To do this it is necessary to evaluate the marginal likelihood for any
  given model. These have closed-form expression for the MOM prior, but
  for models with >15 variables these are expensive to compute and
  Laplace approximations are used instead (for the residual variance a
  log change of variables is used, which improves the approximation).
  For other priors closed forms
  are not available, so by default Laplace approximations are used.
  For the iMOM prior we also implement a Hybrid Laplace-IS
  which uses a Laplace approximation to evaluate the integral wrt beta
  and integrates wrt phi (residual variance) numerically.

  It should be noted that Laplace approximations tend to under-estimate
  the marginal densities when the MLE for some parameter is very close
  to 0. That is, it tends to be conservative in the sense of excluding
  more variables from the model than an exact calculation would.

  Finally, method=='plugin' provides a BIC-type approximation that is
  faster than exact or Laplace methods, at the expense of some
  accuracy. In non-sparse situations where models with many variables
  have large posterior probability method=='plugin' can be substantially
  faster.

  For more details on the methods used to compute marginal densities see
  Johnson & Rossell (2012).

  \code{modelsearchBlockDiag} uses the block search method described in
  Papaspiliopoulos & Rossell. Briefly, spectral clustering is run on
  X'X to cluster variables into blocks of \code{blocksize} and
  subsequently the Coolblock algorithm is used to define a sequence
  of models of increasing size. The exact integrated likelihood
  is evaluated for all models in this path, the best model chosen,
  and the scheme iteratively repeated to add and drop variables
  until convergence.
}
\value{
Object of class \code{msfit}, which extends a list with elements
\item{postSample}{\code{matrix} with posterior samples for the model
  indicator. \code{postSample[i,j]==1}
  indicates that variable j was included in the model in the MCMC
  iteration i}
\item{postOther}{\code{postOther}
  returns posterior samples for parameters other than the model
  indicator, i.e. basically hyper-parameters.
  If hyper-parameters were fixed in the model specification, \code{postOther} will be empty.}
\item{margpp}{Marginal posterior probability for inclusion of each
  covariate. This is computed by averaging marginal post prob for
  inclusion in each Gibbs iteration, which is much more accurate than
  simply taking \code{colMeans(postSample)}}.
\item{postMode}{Model with highest posterior probability amongst all those visited}
\item{postModeProb}{Unnormalized posterior prob of posterior mode (log scale)}
\item{postProb}{Unnormalized posterior prob of each visited model (log
  scale)}
\item{priors}{List with priors specified when calling \code{modelSelection}}
}
\references{
  Johnson V.E., Rossell D. Non-Local Prior Densities for Default
  Bayesian Hypothesis Tests. Journal of the Royal Statistical Society B,
  2010, 72, 143-170.

  Johnson V.E., Rossell D. Bayesian model selection in high-dimensional
  settings. Journal of the American Statistical Association, 2012, 107,
  649-660.

  Papaspiliopoulos O., Rossell, D. Scalable Bayesian variable selection
  and model averaging under block orthogonal design. 2016

  Rossell D., Rubio F.J. Tractable Bayesian variable selection: beyond
  normality. 2016
}
\author{ David Rossell }
\seealso{
  \code{\link{msfit-class}} for details on the output.
  \code{\link{postProb}} to obtain posterior model probabilities.
  \code{coef.msfit} for Bayesian model averaging estimates and
  intervals. \code{predict.msfit} for BMA estimates and intervals
  for user-supplied covariate values.
  \code{plot.msfit} for an MCMC diagnostic plot showing estimated
  marginal posterior inclusion probabilities vs. iteration number.
 \code{\link{rnlp}} to obtain posterior samples for the coefficients.
 \code{\link{marginalLikelihood}} to compute marginal densities for a given model.


}
\examples{
#Simulate data
x <- matrix(rnorm(100*3),nrow=100,ncol=3)
theta <- matrix(c(1,1,0),ncol=1)
y <- x \%*\% theta + rnorm(100)
df <- data.frame(y, x)

#Specify prior parameters
priorCoef <- momprior()
priorModel <- modelunifprior()

#Alternative model space prior: 0.5 prior prob for including any covariate
priorModel <- modelbinomprior(p=0.5)

#Alternative: Beta-Binomial prior for model space
priorModel <- modelbbprior()

#Model selection
fit1 <- modelSelection(y ~ ., data=df, 
priorCoef=priorCoef, priorModel=priorModel)

coef(fit1) #BMA estimates, 95% intervals, marginal post prob

postProb(fit1) #posterior model probabilities

}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ models }
\keyword{ htest }

\name{localnulltest}
\alias{localnulltest}
\alias{localnulltest_fda}
\alias{localnulltest_givenknots}
\alias{localnulltest_fda_givenknots}

%- Also NEED an '\alias' for EACH other topic documented here.
\title{ Local variable selection }
\description{
  Learn whether covariate effects are zero at given coordinates using
  Bayesian model selection or information criteria.

  Use \code{coef} to extract estimates and posterior
  probabilities for local effects.
}
\usage{

localnulltest(y, x, z, x.adjust, localgridsize, localgrid,
nbaseknots=20, nlocalknots=c(5,10,15), localknots,
basedegree=3, cutdegree=0,
usecutbasis=TRUE, priorCoef=normalidprior(),
priorGroup=priorCoef, priorModel=modelbbprior(),
mc.cores=min(4,length(nlocalknots)), return.mcmc=FALSE, verbose=FALSE,
...)

localnulltest_fda(y, x, z, x.adjust, function_id,
Sigma='AR/MA', localgridsize, localgrid, nbaseknots=20,
nlocalknots=c(5,10,15), localknots,
basedegree=3, cutdegree=0, usecutbasis=TRUE,
priorCoef=momprior(), priorGroup=groupmomprior(),
priorModel=modelbbprior(), mc.cores=min(4,length(nlocalknots)),
return.mcmc=FALSE, verbose=FALSE, ...)

localnulltest_givenknots(y, x, z, x.adjust, localgridsize,
localgrid, nbaseknots=20, nlocalknots=10, localknots,
basedegree=3, cutdegree=0,
usecutbasis=TRUE, priorCoef=normalidprior(),
priorGroup=priorCoef, priorModel=modelbbprior(),
verbose=FALSE, ...)

localnulltest_fda_givenknots(y, x, z, x.adjust, function_id,
Sigma='AR/MA', localgridsize, localgrid, nbaseknots=20,
nlocalknots=10, localknots,
basedegree=3, cutdegree=0, usecutbasis=TRUE,
priorCoef=momprior(), priorGroup=groupmomprior(),
priorModel=modelbbprior(), verbose=FALSE, ...)

}
\arguments{
\item{y}{Vector with the outcome variable}
\item{x}{Numerical matrix with covariate values}
\item{z}{Matrix with d-dimensional coordinates (d>=1$ for each entry in \code{y}, and d columns)}
\item{x.adjust}{Optionally, further adjustment covariates to be included
  in the model with no testing being performed}
\item{function_id}{Function identifier. It is assumed that one observes
  multiple functions over z, this is the identifier of each individual
  function}
\item{Sigma}{Error covariance. By default 'identity', other options are
  'MA', 'AR' or 'AR/MA' (meaning that BIC is used to choose between MA
  and AR). Alternatively the user can supply a function such that
  \code{Sigma(z[i,],z[j,])} returns the within-function \code{cov(y[i,], y[j,])}}
\item{localgridsize}{Local test probabilities will be returned for a
  grid of \code{z} values of size \code{localgridsize} for each
  dimension. If \code{ncol(z)==1} then \code{localgridsize} defaults to
  100, else to 10}
\item{localgrid}{Regions at which tests will be performed. Defaults to
  dividing each \code{[min(z[,i]),  max(z[,i])]} into 10 equal
  intervals. If provided, \code{localgrid} must be a list with one entry
  for each \code{z[,i]}, containing a vector with the desired grid for that \code{z[,i]}}
\item{nbaseknots}{Number of knots for the spline approximation to the
  baseline effect of \code{x} on \code{y}}
\item{nlocalknots}{Number of knots for the basis capturing the local effects. 
Ignored if \code{localknots} is specified}
\item{localknots}{Knots to be used for the local tests. The same knots are used for all 
columns in \code{z}, so make sure that all columns span the same range of values.
For a multi-resolution analysis, \code{localknots} should be a list where each entry
specifyes the knots for one resolution}
\item{basedegree}{Degree of the spline approximation to the baseline}
\item{cutdegree}{Degree of the cut spline basis used for testing}
\item{usecutbasis}{If \code{FALSE}, then the basis is not cut and a
  standard spline basis is returned (not recommended unless you know
  what you're doing)}
\item{priorCoef}{Prior on the coefficients, passed on to
  \code{modelSelection}}
\item{priorGroup}{Prior on grouped coefficients, passed on to
  \code{modelSelection}}
\item{priorModel}{Prior on the models, passed on to
  \code{modelSelection}}
\item{mc.cores}{If package parallel is available on your system and
  \code{nlocalknots} has several entries defining several resolution
  levels, they will be run in parallel on \code{mc.cores}}
\item{return.mcmc}{Set to \code{TRUE} to return the MCMC output from \code{modelSelection}}
\item{verbose}{If \code{TRUE} some progress information is printed}
\item{...}{Other arguments to be passed on to \code{modelSelection},
  e.g. \code{family='binomial'} for logistic regression}
}
\details{
  Local variable selection considers the model

  \deqn{y_i= \beta_0(z_i) + sum_{j=1}^p \beta_j(z_i, x_i) + e_i}

  \eqn{\beta_0(z_i)} is the baseline mean
  
  \eqn{\beta_j(z_i,x_i)} is local effect of covariate j at coordinate z_i
  
  \eqn{e_i} a Gaussian error term assumed either independent or with a
  covariance structure given by Sigma. If assuming independence it is
  possible to consider alternatives to Gaussianity,
  e.g. set \code{family='binomial'} for logistic regression
  or \code{family='poisson'} for Poisson regression

  Note: a sum-to-zero type constraint is set on \eqn{\beta_1(z_i,x_i)} so
  that it defines a deviation from the baseline mean \eqn{\beta_0(z_i)}

  We model \eqn{\beta_0} using B-splines of degree \code{basedegree} with
  \code{nbaseknots} knots.
  We model \eqn{\beta_j} using B-splines of degree \code{cutdegree} with
  \code{nlocalknots}. Using \code{cutdegree=0} runs fastest is usually
  gives similar inference than higher degrees, and is hence recommended
  by default.
  
}
\value{
  Object of class \code{localtest}, which extends a list with elements

\item{covareffects}{Estimated local covariate effects at different
  \code{z} values, 0.95 posterior intervals and posterior probability
  for the existence of an effect}
\item{pplocalgrid}{Posterior probabilities for the existence of an
  effect for regions of \code{z} values. Do not use these unless you
  know what you're doing}
\item{covareffects.mcmc}{MCMC output used to build covareffects. Only
  returned if \code{return.mcmc=TRUE}}
\item{ms}{Objects of class \code{msfit} returned by \code{modelSelection}}
\item{pp_localknots}{Posterior probability for each resolution level
  (value of \code{nlocalknots})}
\item{Sigma}{Input parameter}
\item{nlocalknots}{Input parameter}
\item{basedegree}{Input parameter}
\item{cutdegree}{Input parameter}
\item{knots}{Input parameters}
\item{regionbounds}{List with region bounds defined by the local testing
  knots at each resolution level}

}
\author{ David Rossell }
\examples{

#Simulate outcome and 2 covariates
#Covariate 1 has local effect for z>0
#Covariate 2 has no effect for any z

truemean= function(x,z) {
    ans= double(nrow(x))
    group1= (x[,1]==1)
    ans[group1]= ifelse(z[group1] <=0, cos(z[group1]), 1)
    ans[!group1]= ifelse(z[!group1]<=0, cos(z[!group1]), 1/(z[!group1]+1)^2)
    return(ans)
}

n= 1000
x1= rep(0:1,c(n/2,n/2))
x2= x1 + rnorm(n)
x= cbind(x1,x2)
z= runif(n,-3,3)
m= truemean(x,z)
y= truemean(x,z) + rnorm(n, 0, .5)

#Run localnulltest with 10 knots
fit0= localnulltest(y, x=x, z=z, nlocalknots=10, niter=1000)

#Estimated covariate effects and posterior probabilities
b= coef(fit0)
b

}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ models }
\keyword{ htest }

\name{cil}
\alias{cil}
%\alias{modelsearchBlockDiag}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Treatment effect estimation for linear models via Confounder Importance
  Learning using non-local priors.}
\description{
  Treatment effect estimation for linear models in the presence of
  multiple treatments and a potentially high-dimensional number of controls,
  i.e. \eqn{p \gg n} can be handled.

  Confounder Importance Learning (CIL) proposes an estimation framework where
  the importance of the relationship between treatments and controls is
  factored in into the establishment of prior inclusion probabilities for each
  of these controls on the response model. This is combined with the use of
  non-local priors to obtain BMA estimates and posterior model probabilities.

  \code{cil} is built on \code{modelSelection} and produces objects of type
  \code{cilfit}. Use \code{coef} and \code{postProb} to obtain treatment effect
  point estimates and posterior model probabilities, respectively, on this
  object class.
}
\usage{
cil(y, D, X, I = NULL, family = 'normal', familyD = 'normal',
  R = 1e4, Rinit = 500, th.search = 'EB', mod1 = 'lasso_bic',
  th.prior = 'unif', priorCoef = momprior(taustd=1),
  rho.min = NULL, rho.max = 0.95,
  th.range = NULL, max.mod = 2^20, lpen = 'lambda.1se',
  eps = 1e-10, bvs.fit0 = NULL, th.EP = NULL, center = TRUE, scale =
  TRUE, includevars, verbose = TRUE)
}
\arguments{
\item{y}{one-column matrix containing the observed responses. The response must
  be continuous (currently the only type supported)}
\item{D}{treatment matrix with numeric columns, continuous or discrete. Any finite
  number of treatments are supported. If only one treatment is provided, supply
  this object in the same format used for \code{y}}
\item{X}{matrix of controls with numeric columns, continuous or discrete. If only
  one treatment is provided, supply this object in the same format used for \code{y}}
\item{I}{matrix with the desired interaction terms between \code{D} and
  \code{X}. If not informed, i.e. supplied as the default \code{NULL}, this term
  will not be included into the response model}
\item{family}{Distribution of the outcome, e.g. 'normal', 'binomial' or
  'poisson'. See \code{help}(modelSelection) for a full list of options}
\item{familyD}{Distribution of the treatment(s). Only 'normal' or
  'binomial' currently allowed}
\item{R}{Number of MCMC iterations to be
  run by \code{modelSelection} on each stage of CIL (see argument \code{niter}
  therein)}
\item{Rinit}{MCMC iterations to estimate marginal posterior
  inclusion probabilities under a uniform model prior, needed for EP}
\item{th.search}{method to estimate theta values in the marginal prior inclusion
  probabilities of the CIL model. Options are: \code{EB} (Empirical Bayes, based
  on maximum marginal likelihood) and \code{EP} (Expectation propagation
  approximation)}
\item{mod1}{method to estimate the feature parameters corresponding to the
  influence of the controls on the treatments. Supported values for this
  argument are 'ginv' (generalised pseudo-inverse), \code{lasso} (see
  argument \code{lpen}), \code{lasso_bic} (default), and \code{ridge})}
\item{th.prior}{prior associated to the thetas for the Empirical Bayes
  estimation. Currently only \code{unif} (Uniform prior) is supported,
  effectively making the EB approach the maximisation of the marginal likelihood}
\item{priorCoef}{Prior on the response model parameters, see \code{modelSelection}}
\item{rho.min}{Lower bound on the covariate's prior inclusion probability. 
  By default, it is set to \eqn{1/p}, where p is the number of covariates}
\item{rho.max}{Upper bound on the covariate's prior inclusion probability}
\item{th.range}{sequence of values to be considered in the grid when searching
  for points to initialise the search for the optimal theta parameters. If left
  uninformed, the function will determine a computationally suitable grid
  depending on the number of parameters to be estimated}
\item{max.mod}{Maximum number of models considered when computing the marginal
  likelihood required by empirical Bayes.
  If set to \code{Inf} all visited models by the enumeration/MCMC
  are considered, but it might be computationally desirable to restrict this
  number when the dimension of \code{D} and/or \code{X} is large}
\item{lpen}{penalty type supplied to \code{glmnet} if \code{mod1} is set to
  \code{lasso}. Default is \code{lambda.1se} (see documentation corresponding to
  \code{glmnet} for options on how to set this parameter)}
\item{eps}{small scalar used to avoid round-offs to absolute zeroes or ones in
  marginal prior inclusion probabilities.}
\item{bvs.fit0}{object returned by \code{modelSelection} under \eqn{\theta = 0},
  used as a model exploration tool to compute EB approximation on the thetas.
  This argument is only supposed to be used in case of a second computation the
  model on the same data where \code{th.search} has ben changed to \code{EB},
  in order to avoid repeating the computation of the initial
  \code{modelSelection} fit. To use this argument, supply the object residing
  in the slot \code{init.msfit} of a \code{cilfit}-class object.}
\item{th.EP}{Optimal theta values under the EP approximation, obtained in a
  previous CIL run. This argument is only supposed to be used in case of
  a second computation the model on the same data where \code{th.search}
  has ben changed to \code{EB}, in order to save the cost of the EP search
  to initialise the optimisation algorithm. To use this argument, supply the
  object residing int the slot \code{th.hat} of a \code{cilfit}-class
  object.}
\item{center}{If \code{TRUE}, \code{y} and \code{x} are centered to have
  zero mean. Dummy variables corresponding to factors are NOT centered}
\item{scale}{If \code{TRUE}, \code{y} and columns in \code{x} are
  scaled to have variance=1. Dummy variables corresponding to factors are NOT scaled}
\item{includevars}{Logical vector of length ncol(x) indicating variables
that should always be included in the model, i.e. variable selection is
not performed for these variables}
\item{verbose}{Set \code{verbose==TRUE} to print iteration progress}
}
\details{
  We estimate treatment effects for the features present in the treatment
  matrix \code{D}. Features in \code{X}, which may or may not be causal
  factors of the treatments of interest, only act as controls and, therefore,
  are not used as inferential subjects.

  Confounder importance learning learns from data the amount of
  confounding in the data, based on a so-called confounding
  coefficient, and uses this to set covariate prior inclusion
  probabilities.
  Roughly, the coefficient measures to what extend covariates that are
  truly related to the outcome are also truly related (high confounding)
  or not related (no confounding) to the treatment(s).

  In high confounding, covariates in \code{X} that appear to be related
  to \code{D} are assigned high inclusion probability in the regression
  for \code{y}. In low confounding, they're assigned low prior inclusion
  probability. See function \code{plotprior} to visualize these prior
  probabilities. 
  Prior probabilities are regulated through a hyper-parameter theta that
  is set according to the method supplied to \code{th.search}.
  While the \code{EB} option obtains a more precise estimate,
  particularly when there are many covariates in \code{X}, the
  \code{EP} is much faster computationally and typically gives very
  similar results.

  See references for further details on implementation and computation.
}
\value{
  Object of class \code{cilfit}, which extends a list with elements
\item{cil.teff}{BMA estimates, 0.95 intervals and posterior inclusion
  probabilities for treatment effects in \code{D}}
\item{coef}{BMA inference for treatment effects and all other covariates}
\item{model.postprobs}{\code{matrix} returning the posterior model probabilities
  computed in the CIL model}
\item{margpp}{\code{numeric} vector containing the estimated marginal
  posterior inclusion probabilities of the featured treatments and controls}
\item{margprior}{Marginal prior inclusion probabilities, as estimated by
  CIL}
\item{margpp.unif}{Marginal posterior inclusion probabilities that would
  be obtained under a uniform model prior}
\item{theta.hat}{Values used for the hyper-parameter theta, estimated according
  to the argument \code{th.search} specified}
\item{treat.coefs}{Estimated weights of the effect of the control variables
  on each of the treatments, as estimated with the method specified in argument
  \code{mod1}}
\item{msfit}{Object returned by \code{modelSelection} (of class \code{msfit})
  of the final model estimated by CIL.}
\item{theta.EP}{Estimated values of theta using the EP algorithm. It coincides
  with \code{theta.hat} if the argument \code{th.search} is set to \code{EB}}
\item{init.msfit}{Initial \code{msfit} object used to estimate the inital model
  where all elements in theta are set to zero (used in the optimisation process
  of this hyper-parameter)}
}
\references{
  Torrens i Dinares M., Papaspiliopoulos O., Rossell D. Confounder
  importance learning for treatment effect inference.
  https://arxiv.org/abs/2110.00314, 2021, 1--48.
}
\author{
  Miquel Torrens
}
\seealso{
  \code{\link{postProb}} to obtain posterior model probabilities.
  
  \code{coef} for inference on the treatment parameters.

  \code{plotprior} to plot the estimated prior probabilities, as a
  function of 
  %\code{postProb} to obtain posterior model probabilities.
  %\code{coef.cilfit} for Bayesian model averaging estimates for the
  %treatment parameters.
}
\examples{
# Simulate data
set.seed(1)
X <- matrix(rnorm(100 * 50), nrow = 100, ncol = 50)
beta_y <- matrix(c(rep(1, 6), rep(0, 44)), ncol = 1)
beta_d <- matrix(c(rep(1, 6), rep(0, 44)), ncol = 1)
alpha <- 1
d <- X \%*\% beta_d + rnorm(100)
y <- d * alpha + X \%*\% beta_y + rnorm(100)

# Confounder Importance Learning
fit1 <- cil(y = y, D = d, X = X, th.search = 'EP')

# BMA for treatment effects
coef(fit1)

# BMA for all covariates
head(fit1$coef)

# Estimated prior inclusion prob
# vs. treatment regression coefficients
plotprior(fit1)

}

% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ models }
\keyword{ cil }

\name{postSamples}
\alias{postSamples}
\alias{postSamples-methods}
\alias{postSamples,mixturebf-method}

\title{
Extract posterior samples from an object
}
\description{
  Obtain posterior model probabilities after running Bayesian model selection
}
\usage{
  postSamples(object)
}
\arguments{
  \item{object}{Object containing posterior samples, e.g. of class
    mixture bf as returned by \code{bfnormmix}}
}
\value{
  For objects of class \code{mixturebf}, a list with one element for
  each considered number of mixture components.

  Each element in the list
  contains posterior samples on the mixture weights (eta) and other
  component-specific parameters such as means (mu) and Cholesky
  decomposition of the inverse covariance matrix (cholSigmainv)
}
\author{
David Rossell
}
\examples{
#See help(bfnormmix)
}
\keyword{ models }
\keyword{ distribution }

\name{dalapl}
\alias{dalapl}
\alias{palapl}
\alias{ralapl}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ Density and random draws from the asymmetric Laplace distribution }
\description{
  \code{dalapl} evaluates the probability density function,
  \code{palapl} the cumulative probability function
  and \code{ralapl} generates random draws.
}
\usage{
dalapl(x, th=0, scale=1, alpha=0, logscale=FALSE)

palapl(x, th=0, scale=1, alpha=0)

ralapl(n, th=0, scale=1, alpha=0)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{Vector of values at which to evaluate the pdf/cdf}
  \item{n}{ Number of random draws}
  \item{th}{ Location parameter (mode) }
  \item{scale}{ Scale parameter (proportional to variance)}
  \item{alpha}{ Asymmetry parameter, must be between -1 and 1}
  \item{logscale}{If TRUE the log-pdf is returned}
}
\details{
  For x<=th the asymmetric Laplace pdf is

  0.5*exp(-abs(th-x)/(sqrt(scale)*(1+alpha)))/sqrt(scale)

  and for x>th it is

  0.5*exp(-abs(th-x)/(sqrt(scale)*(1-alpha)))/sqrt(scale)
  }
\value{
  \code{dalapl} returns the density function,
  \code{palapl} the cumulative probability,
  \code{ralapl} random draws.
}
\author{ David Rossell }
\examples{
e <- ralapl(n=10^4, th=1, scale=2, alpha=0.5)
thseq <- seq(min(e),max(e),length=1000)
hist(e, main='', breaks=30, prob=TRUE)
lines(thseq, dalapl(thseq, th=1, scale=2, alpha=0.5), col=2)
}
\keyword{ distribution }

\name{modelSelectionGGM}
\alias{modelSelectionGGM}
\alias{modelSelectionGGM_eBayes}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ Bayesian variable selection for Gaussian graphical models }
\description{

Bayesian model selection and averaging for Gaussian graphical models.

modelSelectionGGM performs standard BMS/BMA under the specified priors

modelSelectionGGM_eBayes uses empirical Bayes to set edge prior inclusion
  probabilities pi. By default a common prior inclusion probability is set
  on all edges. If Z is specified, the edge inclusion probabilities can 
  be regressed on meta-covariates Z via log(pi/(1-pi))= Z w.
  
}
\usage{

modelSelectionGGM(y, priorCoef=normalidprior(tau=1), 
priorModel=modelbinomprior(1/ncol(y)), 
priorDiag=exponentialprior(lambda=1), center=TRUE, scale=TRUE, 
global_proposal= 'none', prob_global=0.5,
tempering= 0.5, truncratio= 100,
save_proposal= FALSE, niter=10^3, burnin= round(niter/10), 
updates_per_iter= ceiling(sqrt(ncol(y))), updates_per_column= 10,
sampler='Gibbs', pbirth=0.75, pdeath=0.5*(1-pbirth),
bounds_LIT, Omegaini='glasso-ebic', verbose=TRUE)

modelSelectionGGM_eBayes(Z, wini, niter.mcmc= 5000, 
niter.mstep= 1000, niter.eBayes= 20, priorvar.w, 
verbose=TRUE, ...)

}
\arguments{
  
  \item{y}{Data matrix}
  
  \item{priorCoef}{Prior on off-diagonal entries of the precision
    matrix, conditional on their not being zero (slab). See details}
  
  \item{priorModel}{Prior probabilities on having non-zero diagonal
    entries. Possible priors are modelbinomprior() and modelbbprior().
    See details. 
    See also \code{modelSelectionGGM_eBayes} for an empirical Bayes 
    model prior}
  
  \item{priorDiag}{Prior on diagonal entries of the precision matrix.
  See details.}
  
  \item{center}{If \code{TRUE}, the columns of \code{y} will be centered
    to zero mean}
  
  \item{scale}{If \code{TRUE}, the columns of \code{y} will be scaled to
    unit sample variance}

  \item{global_proposal}{Either \code{'none'}, \code{'regression'} or 
  \code{'in-sample'}. 
  If \code{'none'}, serial MCMC is used as specified by \code{sampler}. 
  If \code{'regression'}, 
  MCMC uses Metropolis-Hastings where models are proposed for each column
  using regression posterior model probabilities for that column.
  If \code{'in-sample'}, each column's proposal use posterior model
  probabilities given a precision matrix estimate for the other columns}

  \item{prob_global}{Probability of proposing a sample from a
  global proposal, see details. This argument is ignored if
  \code{global_proposal == "none"}.
  }

  \item{tempering}{If \code{global_proposal != 'none'}, the posterior model
  probabilities of the proposal distribution are raised to the power
  indicated by \code{tempering} (set to 1 for no tempering)}

  \item{truncratio}{In the global proposal, any model's proposal probability 
  is >= prob(top model) / truncratio. This ensures bounded weight ratios
  in the MH step, to improve poor mixing when the current state has low
  proposal probability, often at the cost of decreasing the acceptance rate.
  If \code{truncratio <= 0}, no truncation is done}

  \item{save_proposal}{If \code{TRUE}, the global proposals are saved
  in \code{proposal} (a list with p entries, one per column) and the
  corresponding proposal densities in \code{proposaldensity}. Neither
  are typically needed, as they were already used to produce the
  posterior samples in \code{postSample}}
  
  \item{sampler}{Posterior sampler used when \code{global=="none"},
  and also to run the parallel proposals when \code{global!="none"}. 
  Options are \code{"Gibbs"} for Gibbs sampling,\code{"birthdeath"} for
  birth-death-swap, and \code{"LIT"} for the locally-informed truncated 
  algorithm of Zhou et al (2022)}
  
  \item{niter}{Number of posterior samples to be obtained. Each iteration
  consists of selecting a column of the precision matrix at random and 
  making \code{updates_per_column} updates to its entries}

  \item{burnin}{The first burnin samples will be discarded}

  \item{updates_per_iter}{An iteration consists of selecting
  \code{updates_per_iter} columns at random, and proposing
  \code{updates_per_column} edge updates within each column}

  \item{updates_per_column}{See \code{updates_per_iter}}

  \item{pbirth}{Probability of a birth move. The probability of a swap move 
  is \code{1-pbirth-pdeath}. Ignored unless \code{sampler=="birthdeath"}}

  \item{pdeath}{Probability of a death move. Ignored unless
    \code{sampler=="birthdeath"}}  

  \item{bounds_LIT}{log-proposal density bound for the locally-informed LIT 
  algorithm of Zhou et al. These bound the proposal density for a death move
  and for a birth move. By  default, \code{bounds_LIT} is
  \code{log(c("lbound_death"=1/p, "ubound_death"= 1, "lbound_birth"=1/p, "ubound_birth"=p))}
  }
 
  \item{Omegaini}{Initial value of the precision matrix Omega. "null"
  sets all off-diagonal entries to 0. "glasso-bic" and "glasso-ebic" use
  GLASSO with regularization parameter set via BIC/EBIC,
  respectively. Alternatively, \code{Omegaini} can be a matrix}

  \item{verbose}{Set \code{verbose==TRUE} to print iteration progress}

\item{Z}{p x q matrix containing the q meta-covariates for the p
  covariates. An intercept is automatically added (including when
  \code{Z} is missing)}

\item{wini}{Optional. Initial value for the q-dimensional
  hyper-parameter w}

\item{niter.mcmc}{Number of iterations in the final MCMC,
  run once after hyper-parameter estimates have been obtained}

\item{niter.mstep}{Number of MCMC iterations in each M-step
  required to update hyper-parameter estimates}

\item{niter.eBayes}{Max number of iterations in the empirical
  Bayes optimization algorithm. The algorithm also stop when
  the objective function didn't improve in >=2 iterations}

\item{priorvar.w}{Hyper-parameters w follow a prior
  w ~ N(0, priorvar.w (Z^T Z/p)^(-1)) where priorvar.w is the prior
  variance. By default is set such that all prior inclusion
  probabilities lie in (0.001,0.999) with 0.95 prior probability}

\item{...}{Further parameters passed onto \code{modelSelection}}

}

\details{

  Let Omega be the inverse covariance matrix.
  A spike-and-slab prior is used. Specifically,
  independent priors are set on all Omega[j,k], and then a
  positive-definiteness truncation is added.
  
  The prior on diagonal entries Omega[j,j] is given by \code{priorDiag}.
  Specificaly, if priorDiag = exponentialprior(lambda) then 
  the prior is Omega[j,j] ~ Exp(lambda).

  Off-diagonal Omega[j,k] are equal to zero with probability given by
  \code{modelPrior} and otherwise
  arises from the distribution indicated in \code{priorCoef} (slab).
  If priorCoef == normalidprior(tau) then Omega[j,k] ~ N(0, var=tau).

  If modelPrior == modelbinomprior(p) then Omega[j,k] = 0 
  with probability p independently across j < k.
  If modelPrior == modelbbprior(a,b) then p ~ Beta(a,b).

  Inference is based on MCMC posterior sampling. All sampling algorithms
  proceed by updating Omega[,k] given y and Omega[,-k]
  (of course, Omega[k,] is also set to Omega[,k]).
  Omega[,k] is updated by first updating the set of non-zero entries
  (i.e. edges in the graphical model) using either Gibbs sampling
  or a proposal distribution (see below), and then the non-zero
  entries of Omega[,k] are updated from their exact posterior given
  the current set of edges.

  An MCMC iteration consists of iterating over \code{updates_per_iter} columns 
  chosen at random and, for each column, do \code{updates_per_column} 
  proposals.

  If \code{global_proposal == "none"}, a local MCMC proposal is used to 
  update what entries in Omega[,k] are zero. 
  Local means that the proposed model is a neighbour of the current model,
  e.g. only one edge is added / killed.
  Available local kernels are Gibbs, birth-death-swap and LIT (Zhou et al 2022).

  If \code{global_proposal == "regression"}, a global proposal is used.
  A new model (set of non-zero entries in Omega[,k]) is proposed based 
  on the posterior distribution of a linear regression of y[,k] on the other
  covariates. 
  \code{prob_global} indicates the probability of using the global proposal,
  otherwise a local proposal is used.
  Proposal probabilities are tempered by raising them to the power
  \code{tempering}. Further, any model with probability 
  below prob(top model) / truncratio is assigned proposal probability
  prob(top model) / truncratio.
 
}
\value{
  Posterior inference on the inverse covariance of \code{y}.
  Object of class \code{msfit_ggm}, which extends a list with elements
  
\item{postSample}{Posterior samples for the upper-diagonal entries of
  the precision matrix. Stored as a sparse matrix, see package Matrix
  to utilities to work with such matrices}

\item{prop_accept}{If \code{almost_parallel} is \code{TRUE}, a vector
  with the proportion of accepted edge proposals. Note that Omega[,k]
  is always updated from its exact conditional posterior, regardless
  of the edge proposal being accepted or rejected.}

\item{proposal}{If \code{almost_parallel} and \code{save_proposal} are
  \code{TRUE}, this is a list with one entry per column of Omega,
  containing the proposed values of each column}

\item{proposaldensity}{log-proposal density for the samples in
  \code{proposal}. Entry (i,j) stores the log-proposal density for
  proposed sample i of column j}

\item{margpp}{Rao-Blackwellized estimates of posterior marginal
  inclusion probabilities. Only valid when using the Gibbs algorithm}

\item{priors}{List storing the priors specified when calling
  \code{modelSelectionGGM}}

\item{p}{Number of columns in \code{y}}

\item{indexes}{Indicates what row/column of Omega is stored in each
  column of \code{postSample}}

\item{samplerPars}{MCMC sampling parameters}

\item{almost_parallel}{Stores the input argument \code{almost_parallel}}

}
\author{ David Rossell }
\seealso{
  \code{\link{msfit_ggm-class}} for further details on the output.
  \code{icov} for the estimated precision (inverse covariance) matrix.
  \code{coef.msfit_ggm} for Bayesian model averaging estimates and
  intervals. 
}
\references{
  Zhou, Quan, et al. Dimension-free mixing for high-dimensional Bayesian 
  variable selection. JRSS-B 2022, 84, 1751-1784
}
\examples{

#Simulate data with p=3
Th= diag(3); Th[1,2]= Th[2,1]= 0.5
sigma= solve(Th)

z= matrix(rnorm(1000*3), ncol=3)
y= z %*% chol(sigma)

#Obtain posterior samples
#y has few columns, initialize the MCMC at the sample precision matrix
#Else, leave the argument Omegaini in modelSelectionGGM empty
Omegaini= solve(cov(y))
fit= modelSelectionGGM(y, scale=FALSE, Omegaini=Omegaini)

#Parameter estimates, intervals, prob of non-zero
coef(fit)

#Estimated inverse covariance
icov(fit)

#Estimated inverse covariance, entries set to 0
icov(fit, threshold=0.95)

#Shows first posterior samples
head(fit$postSample)

}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ models }
\keyword{ htest }

\name{mixturebf-class}
\Rdversion{1.1}
\docType{class}
\alias{mixturebf-class}
\alias{mixturebf}
\alias{show,mixturebf-method}
\alias{coef.mixturebf}


\title{Class "mixturebf"}
\description{
Stores the output of Bayesian model selection for mixture models,
e.g. as produced by function \code{bfnormmix}.

Methods are provided for retrieving the posterior probability of a given
number of mixture components, posterior means and posterior samples of
the mixture model parameters.
}
\section{Objects from the Class}{
  Typically objects are automatically created by a call to \code{bfnormmix}.
}
\section{Slots}{
  The class has the following slots:
  \describe{

    \item{postprob}{data.frame containing posterior probabilities for
      different numbers of components (k) and log-posterior probability of
      a component being empty (contain no individuals)}

    \item{p}{Number of variables in the data to which the model was fit}

    \item{n}{Number of observations in the data to which the model was fit}

    \item{priorpars}{Prior parameters used when fitting the model}
    
    \item{postpars}{Posterior parameters for a 1-component mixture,
    e.g. for a Normal mixture the posterior is N(mu1,Sigma/prec)
    IW(nu1,S1)}
  
    \item{mcmc}{For each considered value of k, posterior samples for
    the parameters of the k-component model are stored}
  }
}
%  \S3method{coef}{mixturebf}(object, ...)
\section{Methods}{
  \describe{
    \item{coef}{Computes posterior means for all parameters}
    
    \item{show}{\code{signature(object = "mixturebf")}: Displays general
      information about the object. }
    
    \item{postProb}{\code{signature(object = "mixturebf")}: Extracts
      posterior model probabilities, Bayes factors and posterior
      probability of a cluster being empty}

    \item{postSamples}{\code{signature(object = "mixturebf")}: Extracts
      posterior samples}
  }
}
\references{
Fuquene J., Steel M.F.J., Rossell D. On choosing mixture components via
  non-local priors. 2018. arXiv
}
\author{
David Rossell
}
\seealso{
See also \code{\link{bfnormmix}}
}
\examples{
showClass("mixturebf")
}
\keyword{classes}

\name{bbPrior}
\alias{bbPrior}
\alias{unifPrior}
\alias{binomPrior}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ Priors on model space for variable selection problems }
\description{
  \code{unifPrior}
  implements a uniform prior (equal a priori probability for all
  models). \code{binomPrior} implements a Binomial prior.
  \code{bbPrior} implements a Beta-Binomial prior. 
}
\usage{
unifPrior(sel, logscale=TRUE, groups=1:length(sel),
constraints=lapply(1:length(unique(groups)), function(z) integer(0)))

binomPrior(sel, prob=.5, logscale=TRUE, probconstr=prob, groups=1:length(sel),
constraints=lapply(1:length(unique(groups)), function(z) integer(0)))

bbPrior(sel, alpha=1, beta=1, logscale=TRUE, alphaconstr=alpha,
betaconstr=beta, groups=1:length(sel),
constraints=lapply(1:length(unique(groups)), function(z) integer(0)))
}
\arguments{
  \item{sel}{ Logical vector indicating which variables are included in
    the model}
  \item{logscale}{ Set to \code{TRUE} to return the log-prior
    probability.}
  \item{groups}{Group that each variable belongs to (e.g. dummy
    indicators for categorical variables with >2 categories). The idea
    is that all variables in a group are jointly added/removed from the
    model. By default all variables are assumed to be in separate groups}
  \item{constraints}{List with length equal to the number of groups
    (distinct elements in \code{groups}). Element j in the list should
    indicate any hierarchical constraints on the group, for instance
    constraints[[3]]==c(1,2) indicates that group 3 can only be included
  in the model if groups 1 and 2 are also in the model. This can be used
  to enforce that an interaction can only be in the model if the main
  effects are also in the model.
  }

  \item{prob}{ Success probability for the Binomial prior}
  \item{probconstr}{Success probability for the Binomial prior for
  groups that are subject to constraints}
  
  \item{alpha}{ First parameter of the Beta-Binomial prior, which is equivalent
    to specifying a Beta(alpha,beta) prior on \code{prob}.}
  \item{beta}{ First parameter of the Beta-Binomial prior, which is equivalent
    to specifying a Beta(alpha,beta) prior on \code{prob}.}

  \item{alphaconstr}{Same as alpha for the groups that are subject to constraints}
  \item{betaconstr}{Same as beta for the groups that are subject to constraints}
}
\value{
  Prior probability of the specified model
}
\author{ David Rossell }
\examples{
sel <- c(TRUE,TRUE,FALSE,FALSE)
unifPrior(sel,logscale=FALSE)
binomPrior(sel,prob=.5,logscale=FALSE)
bbPrior(sel,alpha=1,beta=1,logscale=FALSE)
}
\keyword{ distribution }

\name{bfnormmix}
\alias{bfnormmix}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ Number of Normal mixture components under Normal-IW and
  Non-local priors}
\description{
  Posterior sampling and Bayesian model selection to choose the number
  of components k in multivariate Normal mixtures.

  \code{bfnormmix} computes posterior probabilities under non-local
  MOM-IW-Dir(q) priors, and also for local Normal-IW-Dir(q.niw) priors.
  It also computes posterior probabilities on cluster occupancy
  and posterior samples on the model parameters for several k.
}
\usage{
bfnormmix(x, k=1:2, mu0=rep(0,ncol(x)), g, nu0, S0, q=3, q.niw=1,
B=10^4, burnin= round(B/10), logscale=TRUE, returndraws=TRUE, verbose=TRUE)
}
\arguments{
\item{x}{n x p input data matrix}
\item{k}{Number of components}
\item{mu0}{Prior on mu[j] is N(mu0,g Sigma[j])}
\item{g}{Prior on mu[j] is N(mu0,g Sigma[j]). This is a critical MOM-IW prior
  parameter that specifies the separation between
  components deemed practically relevant. It defaults to assigning 0.95
  prior probability to any pair of mu's giving a bimodal mixture, see details}
\item{S0}{Prior on Sigma[j] is IW(Sigma_j; nu0, S0)}
\item{nu0}{Prior on Sigma[j] is IW(Sigma_j; nu0, S0)}
\item{q}{Prior parameter in MOM-IW-Dir(q) prior}
\item{q.niw}{Prior parameter in Normal-IW-Dir(q.niw) prior}
\item{B}{Number of MCMC iterations}
\item{burnin}{Number of burn-in iterations}
\item{logscale}{If set to TRUE then log-Bayes factors are returned}
\item{returndraws}{If set to \code{TRUE} the MCMC posterior draws under
  the Normal-IW-Dir prior are returned for all \code{k}}
\item{verbose}{Set to \code{TRUE} to print iteration progress}
}
\details{
  The likelihood is

  p(x[i,] | mu,Sigma,eta)= sum_j eta_j N(x[i,]; mu_j,Sigma_j)

  The Normal-IW-Dir prior is

  Dir(eta; q.niw) prod_j N(mu_j; mu0, g Sigma) IW(Sigma_j; nu0, S0)

  The MOM-IW-Dir prior is

  \deqn{d(\mu,A) Dir(\eta; q) \prod_j N(\mu_j; \mu0, g \Sigma_j) IW(\Sigma_j; \nu_0, S0)}

  where

  \deqn{d(\mu,A)= [\prod_{j<l} (\mu_j-\mu_l)' A (\mu_j-\mu_l)]}

  and A is the average of \eqn{\Sigma_1^{-1},...,\Sigma_k^{-1}}. Note that
  one must have q>1 for the MOM-IW-Dir to define a non-local prior.

  By default the prior parameter g is set such that

  P( (mu[j]-mu[l])' A (mu[j]-mu[l]) < 4)= 0.05.

  The reasonale when Sigma[j]=Sigma[l] and eta[j]=eta[l]
  then (mu[j]-mu[l])' A (mu[j]-mu[l])>4 corresponds to a bimodal
  density. That is, the default g focuses 0.95 prior prob on a degree of
  separation between components giving rise to a bimodal mixture density.

  \code{bfnormmix} computes posterior model probabilities under the
  MOM-IW-Dir and Normal-IW-Dir priors using MCMC output. As described in
  Fuquene, Steel and Rossell (2018) the estimate is based on the
  posterior probability that one cluster is empty under each possible k.
}
\value{
  A list with elements
\item{k}{Number of components}
\item{pp.momiw}{Posterior probability of k components under a
  MOM-IW-Dir(q) prior}
\item{pp.niw}{Posterior probability of k components under a
  Normal-IW-Dir(q.niw) prior}
\item{probempty}{Posterior probability that any one cluster is empty under a
  MOM-IW-Dir(q.niw) prior}
\item{bf.momiw}{Bayes factor comparing 1 vs k components under a
  MOM-IW-Dir(q) prior}
\item{logpen}{log of the posterior mean of the MOM-IW-Dir(q) penalty term}
\item{logbf.niw}{Bayes factor comparing 1 vs k components under a
  Normal-IW-Dir(q.niw) prior}
}
\references{
Fuquene J., Steel M.F.J., Rossell D. On choosing mixture components via
  non-local priors. 2018. arXiv
}
\author{ David Rossell }
\examples{
x <- matrix(rnorm(100*2),ncol=2)

bfnormmix(x=x,k=1:3)
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ models }
\keyword{ htest }

\name{dmom}
\alias{dmom}
\alias{dimom}
\alias{pmom}
\alias{pimom}
\alias{qmom}
\alias{qimom}
\alias{demom}
\alias{demom-methods}
\alias{demom,data.frame-method}
\alias{demom,matrix-method}
\alias{demom,vector-method}
\alias{pemom}
\alias{dmomigmarg}
\alias{pmomigmarg}
\alias{demomigmarg}
\alias{pemomigmarg}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ Non-local prior density, cdf and quantile functions. }
\description{
  \code{dmom}, \code{dimom} and \code{demom} return the density for the
  moment, inverse moment and exponential moment priors.
  \code{pmom}, \code{pimom} and \code{pemom} return the distribution function for the univariate
  moment, inverse moment and exponential moment priors (respectively).
  \code{qmom} and \code{qimom} return the quantiles for the univariate
  moment and inverse moment priors.
  \code{dmomigmarg} returns the marginal density implied by a
  MOM(x;tau*phi)*Invgamma(phi;a/2,b/2), \code{pmomigmarg} its cdf.
  Analogously \code{demomigmarg} and \code{demomigmarg} for
  eMOM(x;tau*phi)*Invgamma(phi;a/2,b/2)
}
\usage{
dmom(x, tau, a.tau, b.tau, phi=1, r=1, V1, baseDensity='normal', nu=3,
logscale=FALSE, penalty='product')
dimom(x, tau=1, phi=1, V1, logscale=FALSE, penalty='product')
demom(x, tau, a.tau, b.tau, phi=1, logscale=FALSE)

pmom(q, V1 = 1, tau = 1)
pimom(q, V1 = 1, tau = 1, nu = 1)
pemom(q, tau, a.tau, b.tau)

qmom(p, V1 = 1, tau = 1)
qimom(p, V1 = 1, tau = 1, nu = 1)

dmomigmarg(x,tau,a,b,logscale=FALSE)
pmomigmarg(x,tau,a,b)

demomigmarg(x,tau,a,b,logscale=FALSE)
pemomigmarg(x,tau,a,b)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{ In the univariate setting, \code{x} is a vector with the
    values at which to evaluate the density. In the multivariate setting
    it is a matrix with an observation in each row.}
  \item{q}{ Vector of quantiles.}
  \item{p}{ Vector of probabilities.}
  \item{V1}{ Scale matrix (ignored if \code{penalty=='product'}). Defaults to 1 in univariate setting and
    the identity matrix in the multivariate setting. }
  \item{tau}{ Prior dispersion parameter is \code{tau*phi}. See
    details. }
  \item{a.tau}{If \code{tau} is left missing, an Inverse Gamma(a.tau/2,b.tau/2)
  is placed on \code{tau}. In this case \code{dmom} and \code{demom} return the density
  marginalized with respect to \code{tau}.}
  \item{b.tau}{See \code{a.tau}.}
  \item{phi}{ Prior dispersion parameter is \code{tau*phi}. See
    details. }
  \item{r}{Prior power parameter for MOM prior is \code{2*r}}
  \item{baseDensity}{For \code{baseDensity=='normal'} a Normal MOM prior
    is used, for \code{baseDensity=='laplace'} a Laplace MOM prior,
    for \code{baseDensity=='t'} a T MOM prior with \code{nu}
  degrees of freedom is used.}
  \item{nu}{ Prior parameter indicating the degrees of freedom for the
    quadratic T MOM and iMOM prior densities. The
    tails of the inverse moment prior are proportional to the tails of a
    multivariate T with \code{nu} degrees of freedom.}
  \item{penalty}{\code{penalty=='product'} indicates that product MOM/iMOM should
    be used. \code{penalty=='quadratic'} indicates quadratic iMOM. See Details.}
  \item{logscale}{ For \code{logscale==TRUE}, \code{dimom} returns the
    natural log of the prior density.}
  \item{a}{The marginal prior on phi is IG(a/2,b/2)}
  \item{b}{The marginal prior on phi is IG(a/2,b/2)}
}
\details{
 For \code{type=='quadratic'} the density is as follows.
 Define the quadratic form q(theta)= (theta-theta0)' *
   solve(V1) * (theta-theta0) / (tau*phi).
   The normal moment prior density is proportional to
   q(theta)*dmvnorm(theta,theta0,tau*phi*V1).
   The T moment prior is proportional to
   q(theta)*dmvt(theta,theta0,tau*phi*V1,df=nu).
   The inverse moment prior density is proportional to
   \code{q(theta)^(-(nu+d)/2) * exp(-1/q(theta))}.

   pmom, pimom and qimom use closed-form expressions, while qmom uses
   nlminb to find quantiles numerically.
   Only the univariate version is implemented. In this case the product
   MOM is equivalent to the quadratic MOM. The same happens for the
   iMOM.

   \code{dmomigmarg} returns the marginal density

   p(x)= int MOM(x;0,tau*phi) IG(phi;a/2,b/2) dphi


}
\value{
  Prior density, cumulative distribution function or quantile.
}
\references{
    Johnson V.E., Rossell D. Non-Local Prior Densities for Default
  Bayesian Hypothesis Tests. Journal of the Royal Statistical Society B,
  2010, 72, 143-170.

  Johnson V.E., Rossell D. Bayesian model selection in high-dimensional
  settings. Journal of the American Statistical Assocation, 2012, 107,
  649-660

See http://rosselldavid.googlepages.com for technical
  reports. }
\author{ David Rossell }
\examples{
#evaluate and plot moment and inverse moment prior densities
tau <- 1
thseq <- seq(-3, 3, length=500)
plot(thseq, dmom(thseq,tau=tau), type='l', ylab='Prior density')
lines(thseq, dimom(thseq,tau=tau), lty=2, col=2)
}
\keyword{ distribution }
\keyword{ htest }

\name{postProb}
\alias{postProb}
\alias{postProb-methods}
\alias{postProb,msfit-method}
\alias{postProb,msfit_ggm-method}
\alias{postProb,cilfit-method}
\alias{postProb,localtest-method}
\alias{postProb,mixturebf-method}

\title{
Obtain posterior model probabilities
}
\description{
  Obtain posterior model probabilities after running Bayesian model selection
}
\usage{
  postProb(object, nmax, method='norm')
}
\arguments{
  \item{object}{Object of class msfit returned by \code{modelSelection},
    class mixturebf returned by \code{bfnormmix},
    class cilfit returned by \code{cil}
    or class localtest returned by \code{localnulltest}}
  \item{nmax}{Maximum number of models to report (defaults to no max)}
  \item{method}{Only when \code{class(object)} is msfit.
    For 'norm' probabilities are obtained by renormalizing the
  stored integrated likelihoods, for 'exact' they are given by the proportion
  of MCMC visits to each model. 'norm' has less variability but can be biased
  if the chain has not converged.}
}
\value{
  A \code{data.frame} with posterior model probabilities in column pp.
  Column modelid indicates the indexes of the selected covariates (empty
  for the null model with no covariates)

  For \code{msfit_ggm} objects, a list with posterior probabilities
  (element "pp") and the corresponding model (element "modelid")
  indicating what edges are non-zero

  For \code{mixturebf} objects, posterior probabilities for the
  specified number of components

  For \code{localtest} objects, posterior probabilities of a local covariate
  effect at various regions
}
\author{
David Rossell
}
\seealso{
\code{\link{modelSelection}} to perform model selection
}
\examples{
#See help(modelSelection)
}
\keyword{ models }
\keyword{ distribution }

\name{priorp2g}
\alias{priorp2g}
\title{ Moment and inverse moment prior elicitation }
\description{
  \code{priorp2g} finds the \code{g} value giving \code{priorp} prior
  probability to the interval (\code{-q},\code{q}).
}
\usage{
priorp2g(priorp, q, nu=1, prior=c("iMom", "normalMom", "tMom"))
}
\arguments{

  \item{prior}{\code{prior=='normalMom'} does computations for the normal
    moment prior, \code{prior=='tMom'} for the T moment prior,
    \code{prior=='iMom'} does computations for the inverse moment prior.
    Currently \code{prior=='tMom'} is not implemented in \code{priorp2g}.}
  \item{q}{ \code{priorp2g} returns g giving \code{priorp} prior
    probability to the interval \code{(-q,q)}.}
  \item{nu}{ Prior degrees of freedom for the T moment prior or the iMom
    prior (ignored if \code{prior=='normalMom'}).}
  \item{priorp}{ \code{priorp2g} returns g giving \code{priorp} prior
    probability to the interval \code{(-q,q)}}

}
\details{
  See \code{pmom} and \code{pimom} for the MOM/iMOM cumulative
  distribution functions.
}
\value{
  \code{priorp2g} returns g giving \code{priorp} prior probability to the
  interval \code{(-q,q)}.
}
\references{
 See http://rosselldavid.googlepages.com for technical reports.
}
\author{
  David Rossell \email{rosselldavid@gmail.com}
}
\seealso{
  \code{\link{pmom}},
  \code{\link{pimom}}
}
\examples{

#find g value giving 0.05 probability to interval (-.2,.2)
priorp <- .05; q <- .2
gmom <- priorp2g(priorp=priorp, q=q, prior='normalMom')
gimom <- priorp2g(priorp=priorp, q=q, prior='iMom')
gmom
gimom

}
\keyword{distribution}
\keyword{htest}

\name{plotprior}
\alias{plotprior}
\alias{plotprior-methods}
\alias{plotprior,cilfit-method}

\title{
Plot estimated marginal prior inclusion probabilities
}
\description{
  Plot marginal prior inclusion probabilities as estimated by cil versus
  regression coefficients for the treatment(s) equation(s)
}
\usage{
  plotprior(object, xlab, ylab, ylim=c(0,1), ...)
}
\arguments{
  \item{object}{Object of class cilfit returned by \code{cil}}
  \item{xlab}{x-axis label}
  \item{ylab}{y-axis label}
  \item{ylim}{y-axis limits}
  \item{...}{Other arguments passed on to \code{plot}}
}
\value{
  A plot of prior inclusion probabilities vs treatment regression
  coefficients (dots). The line shows the (empirical Bayes) fit
}
\author{
David Rossell
}
\seealso{
\code{\link{cil}} 
}
\examples{
#See help(cil)
}
\keyword{ models }
\keyword{ hplot }

\name{icov}
\alias{icov}
\title{ Extract estimated inverse covariance }
\description{
  
  Extract the estimated inverse covariance from an \code{msfit_ggm}
  object.

  Bayesian model averaging is used, optionally entries with posterior
  probability of being non-zero below a threshold are set to 0.

}
\usage{

icov(fit, threshold) 

}
\arguments{
  
  \item{fit}{Object of class \code{msfit_ggm}, returned by
    \code{modelSelectionGGM}}
  
  \item{threshold}{Entries with posterior probability of being non-zero
  below threshold are set to 0. If missing this argument is ignored and 
  no entries are set to exact zeroes. When the goal is to identify
  zeroes, a sensible default is \code{threshold=0.95}}

  
}
\details{

  The inverse covariance is obtained via Bayesian model averaging, using
  posterior samples of Omega. When \code{threshold} is specified,
  entries in the BMA estimate are set to zero, which may result in a non
  positive-definite matrix.
  
}
\value{

  Estimated inverse covariance matrix.

}
\author{ David Rossell }
\seealso{
  \code{\link{modelSelectionGGM}}, 
  \code{coef.msfit_ggm} for Bayesian model averaging estimates and
  intervals. 
}
\examples{
## See modelSelectionGGM
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ models }
\keyword{ htest }

\name{marginalNIW}
\alias{marginalNIW}
\alias{marginalNIW-methods}
\alias{marginalNIW,missing,ANY,matrix,numeric,missing-method}
\alias{marginalNIW,matrix,missing,missing,missing,missing-method}
\alias{marginalNIW,matrix,missing,missing,missing,vector-method}
\alias{marginalNIW,missing,list,list,numeric,missing-method}

\title{
Marginal likelihood under a multivariate Normal likelihood and a conjugate
Normal-inverse Wishart prior.
}
\description{
The argument \code{z} can be used to specify cluster allocations. If
left missing then the usual marginal likelihood is computed, else it is
computed conditional on the clusters (this is equivalent to the product
of marginal likelihoods across clusters)
}
\usage{
marginalNIW(x, xbar, samplecov, n, z, g,  mu0=rep(0,ncol(x)),
nu0=ncol(x)+4, S0, logscale=TRUE)
}
\arguments{
  \item{x}{Data matrix (individuals in rows, variables in
    columns). Alternatively you can leave missing and specify
    \code{xbar}, \code{samplecov} and \code{n} instead}
  \item{xbar}{Either a vector with column means of \code{x} or a list
    where each element corresponds to the column means for each cluster}
  \item{samplecov}{Either the sample covariance matrix \code{cov(x)} or
    a list where each element contains the covariance for each clsuter}
  \item{n}{Either an integer indicating the sample size \code{nrow(x)}
    or a vector indicating the cluster counts \code{table(z)}}
  \item{z}{Optional argument specifying cluster allocations}
  \item{g}{Prior dispersion parameter for mu}
  \item{mu0}{Prior mean for mu}
  \item{nu0}{Prior degrees of freedom for Sigma}
  \item{S0}{Prior scale matrix for Sigma, by default set to I/nu0}
  \item{logscale}{set to TRUE to get the log-posterior density}
}
\value{
If \code{z} is missing the integrated likelihood under a Normal-IW
prior. If \code{z} was specified then the product of integrated
likelihoods across clusters
}
\details{
  The function computes

  p(x)= int p(x | mu,Sigma) p(mu,Sigma) dmu dSigma

  where p(x[i])= N(x[i]; mu,Sigma) iid i=1,...,n

  p(mu | Sigma)= N(mu; mu0, g Sigma)
  p(Sigma)=     IW(Sigma; nu0, S0)
}
\author{
David Rossell
}
\seealso{
\code{\link{dpostNIW}} for the posterior Normal-IW density.
}
\examples{
#Simulate data
x= matrix(rnorm(100),ncol=2)

#Integrated likelihood under correct model
marginalNIW(x,g=1,nu0=4,log=FALSE)

#Integrated likelihood under random cluster allocations
z= rep(1:2,each=25)
marginalNIW(x,z=z,g=1,nu0=4,log=FALSE)
}
\keyword{ models }
\keyword{ htest }

\name{eprod}
\alias{eprod}
\title{ Expectation of a product of powers of Normal or T random
  variables }
\description{
  Compute the mean of prod(x)^power when x follows T_dof(mu,sigma)
  distribution (dof= -1 for multivariate Normal).

}
\usage{
eprod(m, S, power = 1, dof = -1) 
}
\arguments{
  \item{m}{Location parameter}
  \item{S}{Scale matrix. For multivariate T with dof>2 the covariance is
    S*dof/(dof-2). For the multivariate Normal the covariance is S.}
  \item{power}{Power that the product is raised to}
  \item{dof}{Degrees of freedom of the multivariate T. Set to -1 for the
  multivariate Normal.}
}
\details{
  The calculation is based on the computationally efficient approach by Kan (2008).
}
\value{
  Expectation of the above-mentioned product
}
\references{
Kan R. From moments of sum to moments of product. Journal of
  Multivariate Analysis 99 (2008), 542-554.
}
\author{ John Cook }
\examples{
#Check easy independence case
m <- c(0,3); S <- matrix(c(2,0,0,1),ncol=2)

eprod(m, S, power=2)

(m[1]^2+S[1][1])*(m[2]^2+S[2][2])
}
\keyword{ models }
\keyword{ distrib }

\name{dpostNIW}
\alias{dpostNIW}
\alias{rpostNIW}
\title{Posterior Normal-IWishart density}
\description{
  dpostNIW evalutes the posterior Normal-IWishart density at (mu,Sigma).
  rpostNIW draws independent samples.
  This posterior corresponds to a Normal model for the data

  x[i,] ~ N(mu,Sigma) iid i=1,...,n

  under conjugate priors

  mu | Sigma ~ N(mu0, g Sigma)
  Sigma      ~ IW(nu0, S0)
}
\usage{
dpostNIW(mu, Sigma, x, g=1, mu0=rep(0,length(mu)), nu0=nrow(Sigma)+1, S0,
  logscale=FALSE)

rpostNIW(n, x, g=1, mu0=0, nu0, S0, precision=FALSE)
}
\arguments{
  \item{mu}{Vector of length p}
  \item{Sigma}{p x p positive-definite covariance matrix}
  \item{x}{n x p data matrix (individuals in rows, variables in columns)}
  \item{g}{Prior dispersion parameter for mu}
  \item{mu0}{Prior mean for mu}
  \item{nu0}{Prior degrees of freedom for Sigma}
  \item{S0}{Prior scale matrix for Sigma, by default set to I/nu0}
  \item{logscale}{set to TRUE to get the log-posterior density}
  \item{n}{Number of samples to draw}
  \item{precision}{If set to \code{TRUE}, samples from the precision
    matrix (inverse of Sigma) are returned instead}
}
\value{
\code{dpostNIW} returns the Normal-IW posterior density evaluated at
(mu,Sigma).

\code{rpostNIW} returns a list with two elements. The first element are
posterior draws for the mean. The second element are posterior draws for
the covariance (or its inverse if \code{precision==TRUE}). Only
lower-diagonal elements are returned (\code{Sigma[lower.tri(Sigma,diag=TRUE)]}).
}
\author{ David Rossell }
\seealso{ \code{\link{diwish}} for the inverse Wishart prior density,
\code{\link{marginalNIW}} for the integrated likelihood under a
  Normal-IW prior}
\examples{
#Simulate data
x= matrix(rnorm(100),ncol=2)
#Evaluate posterior at data-generating truth
mu= c(0,0)
Sigma= diag(2)
dpostNIW(mu,Sigma,x=x,g=1,nu0=4,log=FALSE)
}
\keyword{ distribution }

\name{icfit-class}
\Rdversion{1.1}
\docType{class}
\alias{icfit-class}
\alias{icfit}
\alias{show,icfit-method}
\alias{icfit.coef}
\alias{icfit.predict}
\alias{icfit.summary}



\title{Class "icfit"}
\description{
Stores the output of the search for the model with best information
criterion value, e.g. produced by \code{bestBIC}, \code{bestBIC},
\code{bestAIC} or \code{bestIC}.
The class extends a list, so all usual methods for lists also work for
\code{icfit} objects, e.g. accessing elements, retrieving names etc.

Methods are provided to extract coefficients, predictions, confidence
intervals and summary information about the best model.
}
\section{Objects from the Class}{
  icfit objects are automatically created by a call to
  \code{bestBIC} or similar.
}
\section{Slots}{
  The class extends a list with elements:
  \describe{

  \item{topmodel}{names of the variables in the top model}

  \item{topmodel.fit}{top model as fitted by glm}

  \item{models}{data frame with the information criterion for all models
  (when enumeration is feasible) or those visited by an MCMC model
  search in modelSelection (when enumeration is not feasible)}

  \item{varnames}{the names of all variables in the design matrix}

  \item{msfit}{Output of modelSelection (used to search the top model)}

  }
}
\section{Methods}{
  \describe{
    \item{coef}{glm fit for the top model}
    \item{confint}{Confidence intervals under the top model}
    \item{predict}{Predictions for the top model (predict.glm)}
    \item{show}{\code{signature(object = "icfit")}: Displays general information about the object. }
  }
}
\author{
David Rossell
}
\seealso{
See also \code{\link{bestBIC}}.
}
\examples{
showClass("icfit")
}
\keyword{classes}

\name{bestBIC}
\alias{bestAIC}
\alias{bestBIC}
\alias{bestEBIC}
\alias{bestIC}
\alias{bestAIC_fast}
\alias{bestBIC_fast}
\alias{bestEBIC_fast}
\alias{bestIC_fast}

\title{
Model with best AIC, BIC, EBIC or other general information criteria (getIC)
}
\description{
  Search for the regression model attaining the best value of the
  specified information criterion
}
\usage{
  bestAIC(...)

  bestAIC_fast(..., fastmethod="all")

  bestBIC(...)

  bestBIC_fast(..., fastmethod="all")

  bestEBIC(...)

  bestEBIC_fast(..., fastmethod="all")

  bestIC(..., penalty)

  bestIC_fast(..., penalty, fastmethod="all")
}
\arguments{
  \item{...}{Arguments passed on to \code{modelSelection}. The first and
    main argument is a model formula, see the examples}
  \item{penalty}{General information penalty. For example, since the AIC
    penalty is 2, bestIC(...,penalty=2) is the same as bestAIC(...)}
  \item{fastmethod}{Method used for fast model search. Set "L0Learn" to
    use the L0Learn package, "L1" to use LASSO (glmnet package), 
    "adaptiveL1" for adaptive LASSO, and
    "CDA" for coordinate descent. Not all these options may be
    available for some GLM families (e.g. L0Learn only supports
    \code{family="normal"} and \code{"binomial"}). The default "all"
    uses all available methods}
}
\value{Object of class \code{icfit}. Use (coef, summary,
  confint, predict) to get inference for the top model,
  and \code{help(icfit-class)} for more details on the returned object.
}
\details{

  \code{bestAIC}, \code{bestBIC}, \code{bestEBIC}, \code{bestIC} perform
  full model enumeration when possible and otherwise resort to MCMC to
  explore the models, as discussed in function \code{modelSelection}.
  
  \code{bestAIC_fast}, \code{bestBIC_fast}, \code{bestEBIC_fast},
  \code{bestIC_fast} use a faster algorithm. It first identifies
  a subset of promising models, and then computes the specified
  criterion for each of them to find the best one within the subset.
  For Gaussian and binary outcomes it uses function \code{L0Learn.fit} from
  package \code{L0Learn} (Hazimeh et al, 2023), which combines
  coordinate descent with local combinatorial search to find good models
  of each size.
  L1 returns all the models found in the LASSO regularization path.
  CDA returns a single model found by coordinate descent,
  i.e. adding/dropping one covariate at a time to improve the
  specified criterion (BIC, AIC, ...). 
  
  \code{bestBIC} and the other functions documented here take similar
  arguments to those of \code{modelSelection}, but here no priors
  on models or parameters are needed.
  
  Let p be the total number of parameters and n the sample size. The BIC of a model k with p_k
  parameters is

  - 2 L_k + p_k log(n)

  the AIC is
  
  - 2 L_k + p_k 2
  
  the EBIC is

  - 2 L_k + p_k log(n) + 2 log(p choose p_k)

  and a general information criterion with a given model size penalty

  - 2 L_k + p_k penalty


  The MCMC model search is based on assigning a probability to each
  model, and then using MCMC to sample models from this
  distribution. The probability of model k is

  exp(- IC_k / 2) / sum_l exp(- IC_l / 2)

  where IC_k is the value of the information criterion (BIC, EBIC...)

  Hence the model with best (lowest) IC_k has highest probability, which
  means that it is likely to be sampled by the MCMC algorithm.
 
}
\author{
David Rossell
}
\seealso{
\code{\link{modelSelection}} to perform Bayesian model selection, and
for a description of the arguments that should be passed in \code{...}
}
\references{
H. Hazimeh, R. Mazumder, T. Nonet. L0learn: A scalable package for sparse learning using l0 regularization. Journal of Machine Learning Research 24.205 (2023): 1-8.
}
\examples{
x <- matrix(rnorm(100*3),nrow=100,ncol=3)
theta <- matrix(c(1,1,0),ncol=1)
y <- x \%*\% theta + rnorm(100)
ybin <- y>0
df <- data.frame(y, ybin, x)

#BIC for all models (the intercept is also selected in/out)
fit= bestBIC(y ~ X1 + X2, data=df)
fit

#Same, but setting the BIC's log(n) penalty manually
#change the penalty for other General Info Criteria
#n= nrow(x)
#fit= bestIC(y ~ X1 + X2, data=df, penalty=log(n))

summary(fit) #usual GLM summary

coef(fit) #MLE under top model

#confint(fit) #conf int under top model (requires MASS package)


#Binary outcome
fit2= bestBIC(ybin ~ X1 + X2, data=df, family='binomial')
fit2


}
\keyword{ models }
\keyword{ distribution }

\name{msPriorSpec-class}
\Rdversion{1.1}
\docType{class}
\alias{msPriorSpec}
\alias{msPriorSpec-class}
\alias{aic}
\alias{bic}
\alias{bicprior}
\alias{ic}
\alias{emomprior}
\alias{exponentialprior}
\alias{icarplusprior}
\alias{imomprior}
\alias{momprior}
\alias{zellnerprior}
\alias{normalidprior}
\alias{groupmomprior}
\alias{groupemomprior}
\alias{groupimomprior}
\alias{groupzellnerprior}
\alias{modelunifprior}
\alias{modelbinomprior}
\alias{modelbbprior}
\alias{modelcomplexprior}
\alias{igprior}

\title{Class "msPriorSpec"}
\description{
Stores the prior distributions to be used for Bayesian variable selection
in normal regression models.
This class can be used to specify the prior on non-zero regression coefficients,
the model indicator or the nuisance parameters.
}
\usage{
aic()
bic()
bicprior()
ic(penalty)

momprior(taustd=1, tau, tau.adj=10^6, r=1)
imomprior(tau, tau.adj=10^6)
emomprior(tau, tau.adj=10^6)
zellnerprior(taustd=1, tau, tau.adj=10^6)
normalidprior(taustd=1, tau, tau.adj=10^6)
icarplusprior(a=0.5, taustd=1, tau.adj=10^6)

exponentialprior(lambda = 1)

groupmomprior(taustd=1, tau, tau.adj=10^6)
groupimomprior(tau, tau.adj=10^6)
groupemomprior(tau, tau.adj=10^6)
groupzellnerprior(taustd=1, tau, tau.adj=10^6)

modelunifprior()
modelbinomprior(p=0.5)
modelbbprior(alpha.p=1, beta.p=1)
modelcomplexprior(c=1)

igprior(alpha=.01, lambda=.01)
}
\arguments{
  \item{penalty}{Penalty on model dimension, i.e. for the AIC penalty=2}
  \item{tau}{Prior dispersion parameter for covariates undergoing selection}
  \item{taustd}{Prior dispersion parameter for covariates undergoing selection.
    It is calibrated so that `taustd=1` equals the unit information prior.}
  \item{tau.adj}{Prior variance in Normal prior for covariates not undergoing selection}
  \item{r}{MOM prior parameter is \code{2*r}}
  \item{a}{The icarplus prior precision matrix is a P + (1-a) tau I, where P is
  an ICAR precision matrix and tau I a normalidprior precision matrix}
  \item{p}{Prior inclusion probability for binomial prior on model space}
  \item{alpha.p}{Beta-binomial prior on model space has parameters alpha.p, beta.p}
  \item{beta.p}{Beta-binomial prior on model space has parameters
    alpha.p, beta.p}
  \item{c}{Under the Complexity prior the prior
    probability of having k variables in the model is proportional to \code{1/p^(ck)}}
  \item{alpha}{Inverse gamma prior has parameters alpha/2, lambda/2}
  \item{lambda}{\code{igprior} defines an inverse gamma prior with
    parameters alpha/2, lambda/2. \code{exponentialprior} defines an
    exponential prior with rate parameter \code{lambda}}
}
\section{Objects from the Class}{
Objects can be created by calls of the form \code{new("msPriorSpec",
  ...)}, but it is easier to use creator functions.

For priors on regression coefficients use \code{momprior},
\code{imomprior} or \code{emomprior}.
For prior on model space \code{modelunifprior}, \code{modelbinomprior}
\code{modelbbprior}, or \code{modelcomplexprior}.
For prior on residual variance use \code{igprior}.
}
\section{Slots}{
  \describe{
    \item{\code{priorType}:}{Object of class \code{"character"}. \code{"coefficients"} indicates
that the prior is for the non-zero regression coefficients.
\code{"modelIndicator"} that it is for the model indicator,
and \code{"nuisancePars"} that it is for the nuisance parameteres.
Several prior distributions are available for each choice of \code{priorType},
and these can be speicified in the slot \code{priorDist}. }

    \item{\code{priorDistr}:}{Object of class \code{"character"}.
If \code{priorType=="coefficients"}, \code{priorDistr} can be equal to
"pMOM", "piMOM", "peMOM", "zellner", "normalid", "groupMOM" or "groupzellner"
(product moment, product inverse moment, product exponential moment, Zellner prior, normal prior with \eqn{\Sigma=\mathbf{I}}{V=I}, respectively).
If \code{priorType=="modelIndicator"}, \code{priorDistr} can be equal to "uniform" or "binomial"
to specify a uniform prior (all models equaly likely a priori) or a
binomial prior, or to "complexity" for the Complexity prior of Castillo
et al 2015. For a binomial prior,
the prior inclusion probability for any single variable must be
specified in slot \code{priorPars['p']}. For a beta-binomial prior, the
Beta hyper-prior parameters must be in \code{priorPars['alpha.p']} and
\code{priorPars['beta.p']}.
For the Complexity prior, the prior parameter must be in the slot
\code{priorPars['c']}.
If \code{priorType=="nuisancePars"}, \code{priorDistr} must be equal to "invgamma". This corresponds to an
inverse gamma distribution for the residual variance, with parameters
specified in the slot \code{priorPars}.}

    \item{\code{priorPars}:}{Object of class \code{"vector"}, where each element must be named.
For \code{priorDistr=='pMOM'}, there must be an element "r" (MOM power
is 2r).
For any \code{priorDistr} there must be either an element "tau" indicating
the prior dispersion or elements "a.tau" and "b.tau" specifying an
inverse gamma hyper-prior for "tau".
Optionally, there may be an element "tau.adj" indicating the prior
dispersion for the adjustment variables (i.e. not undergoing variable
selection). If not defined, "tau.adj" is set to 0.001 by default.
For \code{priorDistr=='binomial'}, there must be either an element "p" specifying the prior inclusion probability
for any single covariate, or a vector with elements "alpha.p" and
"beta.p" specifying a Beta(alpha.p,beta.p) hyper-prior on p.
For \code{priorDistr=='invgamma'} there must be elements "alpha" and "lambda". The prior for the residual variance
is an inverse gamma with parameteres \code{.5*alpha} and \code{.5*lambda}.
  }
  }
}
\section{Methods}{
No methods defined with class "msPriorSpec" in the signature.
}
\details{
  DISCUSSION OF PRIOR ON PARAMETERS

  Let beta=(beta_1,...,beta_p) be the regression coefficients for
  individual variables and delta=(delta_1,...,delta_q) those for grouped
  variables (e.g. factors or smooth terms in modelSelection).

  momprior, emomprior, imomprior, zellnerprior, normalid and icarplus
  can be priors on both beta or delta. For further information see the vignette.

  groupzellnerprior is the prior density on delta

  \deqn{p_z(\delta; \tau)= \prod_j N(\delta_j; 0, (\tau/p_j)) (X_j'X_j)^{-1}}

  where \eqn{X_j} are the design matrix columns associated to \eqn{delta_j} and p_j=ncol(X_j)
  is the number of covariates in the group (for groupmomprior, the term in the
  denominator is (p_j +2) instead of p_j). A default
  tau=n=nrow(X_j) mimics the unit information prior and implies that the
  ratio of variance explained by X_j / residual variance is expected to be
  1 a priori. To set the dispersion in terms of unit information prior, taustd
  is also available.

  groupmomprior adds a quadratic MOM penalty

  p_m(delta; tau)= p_z(delta; tau * n) prod_j delta_j'X_j'X_jdelta_j ncol(X_j)/(tau * n * p_j / (p_j + 2))

  and analogously for eMOM and iMOM. Note that unlike groupzellnerprior, the nrow(X_j)
  factor is already included in the code. This is done to give user introduced tau values
  a roughly similar meaning between momprior and groupmomprior.


  DISCUSSION OF PRIOR ON MODELS

Under the uniform prior, the prior probability of any model is
1 / number of models.

Under the Binomial, Beta-Binomial and Complexity priors a model with k
out of K active variables has prior probability
P(Z=k) / (K choose k), where
where Z ~ Binomial(K,p),
Z ~ BetaBinomial(K,alpha.p,beta.p)
or for the Complexity prior P(Z=k) proportional to \code{1/K^(c*k)}.
}
\references{
Johnson VE, Rossell D. Non-Local Prior Densities for Default Bayesian Hypothesis Tests. Journal of the Royal Statistical Society B, 2010, 72, 143-170

Johnson VE, Rossell D. Bayesian model selection in high-dimensional
settings. Journal of the American Statistical Association, 107,
498:649-660.

Rossell D, Abril O, Bhattacharya A. Approximate Laplace approximations for scalable model selection (2021). Journal of the Royal Statistical Society B, 83, 4, 853-879
}
\author{
David Rossell
}
\note{
When new instances of the class are created a series of check are performed to ensure that a valid prior
specification is produced.
}
\seealso{
See also \code{\link{modelSelection}} for an example of defining an instance of the class
and perform Bayesian model selection.
}
\examples{
showClass("msPriorSpec")
}
\keyword{classes}

\name{rnlp}
\alias{rnlp}
\alias{rnlp-methods}
\alias{rnlp,missing,missing,missing,missing,msfit,missing,missing-method}
\alias{rnlp,ANY,matrix,missing,missing,missing,missing,missing-method}
\alias{rnlp,ANY,matrix,missing,missing,msfit,missing,missing-method}
\alias{rnlp,missing,missing,numeric,matrix,missing,missing,missing-method}
\alias{rnlp,ANY,matrix,missing,missing,missing,character,character-method}
\title{
Posterior sampling for regression parameters
}
\description{
Gibbs sampler for linear, generalized linear and survival models
under product non-local priors, Zellner's prior and a Normal
approximation to the posterior.
Both sampling conditional on a model and Bayesian model averaging are
implemented (see Details).

If x and y not specified samples from non-local priors/posteriors with
density proportional to d(theta) N(theta; m, V) are produced,
where d(theta) is the non-local penalty term.
}
\usage{
rnlp(y, x, m, V, msfit, outcometype, family, priorCoef, priorGroup,
priorVar, priorprec, isgroup,
niter=10^3, burnin=round(niter/10), thinning=1, pp='norm')
}
\arguments{
  \item{y}{Vector with observed responses. When \code{class(y)=='Surv'}
    sampling is based on the Cox partial likelihood, else a linear model
  is assumed.}
  \item{x}{Design matrix with all potential predictors}
  \item{m}{Mean for the Normal kernel}
  \item{V}{Covariance for the Normal kernel}
  \item{msfit}{Object of class \code{msfit} returned by
    \code{modelSelection}. If specified Bayesian model averaging
    posterior samples are returned, according to posterior model
    probabilities in \code{msfit}, and then arguments \code{y},
    \code{x}, \code{m}, \code{V} etc. If \code{msfit} is missing then
    posterior samples under the full model \code{y ~ x} are returned}
  \item{outcometype}{Type of outcome. Possible values are "Continuous",
    "glm" or "Survival"}
  \item{family}{Assumed family for the family. Some possible values are
    "normal", "binomial logit" and "Cox"}
  \item{priorCoef}{Prior distribution for the coefficients. Ignored if
    \code{msfit} is supplied. Must be object of class
    \code{msPriorSpec}, e.g. created by \code{momprior},
    \code{emomprior}, \code{imomprior}, \code{zellnerprior}}
  \item{priorGroup}{Prior on grouped coefficients (e.g. categorical
  predictors with >2 categories, splines), as passed to \code{modelSelection}}
  \item{priorVar}{Prior on residual variance. Ignored if \code{msfit}
    supplied. Must be object of class \code{msPriorSpec}, e.g. created
    with \code{igprior}}
  \item{priorprec}{Prior precision matrix (inverse
    covariance). Currently, this is only needed for the ICAR+ prior}
  \item{isgroup}{Logical vector where \code{TRUE} indicates that the
    variable is part of a group, e.g. one of several dummy indicators
    for a discrete covariate}
  \item{niter}{Number of MCMC iterations}
  \item{burnin}{Number of burn-in MCMC iterations. Defaults to \code{.1*niter}. Set to 0 for no burn-in}
  \item{thinning}{MCMC thinning factor, i.e. only one out of each \code{thinning} iterations are reported. Defaults to no thinning}
  \item{pp}{When \code{msfit} is provided this is the method to compute posterior model probabilities,
  which determine the sampled models. Can be 'norm' or 'exact', see \code{postProb} for details. }
}
\details{
  The algorithm is implemented for product MOM (pMOM), product iMOM
  (piMOM) and product eMOM (peMOM) priors.
  The algorithm combines an
  orthogonalization that provides low serial correlation
  with a latent truncation representation that allows fast sampling.

  When \code{y} and \code{x} are specified sampling is for the linear
  regression posterior.
  When argument \code{msfit} is left missing, posterior sampling is for
  the full model regressing \code{y} on all covariates in \code{x}.
  When \code{msfit} is specified each model is drawn with
  probability given by \code{postProb(msfit)}. In this case, a Bayesian
  Model Averaging estimate of the regression coefficients can be
  obtained by applying \code{colMeans} to the \code{rnlp} ouput matrix.

  When \code{y} and \code{x} are left missing, sampling is from a
  density proportional to d(theta) N(theta; m,V), where d(theta) is the
  non-local penalty (e.g. d(theta)=prod(theta^(2r)) for the pMOM prior).
}
\value{
Matrix with posterior samples
}
\references{
  D. Rossell and D. Telesca. Non-local priors for high-dimensional
  estimation, 2014. http://arxiv.org/pdf/1402.5107v2.pdf
}
\author{
David Rossell
}
\seealso{
\code{\link{modelSelection}} to perform model selection and compute
posterior model probabilities.
For more details on prior specification see \code{\link{msPriorSpec-class}}.
}
\examples{
#Simulate data
x <- matrix(rnorm(100*3),nrow=100,ncol=3)
theta <- matrix(c(1,1,0),ncol=1)
y <- x \%*\% theta + rnorm(100)
fit1 <- modelSelection(y=y, x=x, center=FALSE, scale=FALSE)

th <- rnlp(msfit=fit1, niter=100)
colMeans(th)
}
\keyword{ models }
\keyword{ distribution }

\name{diwish}
\alias{diwish}
\title{Density for Inverse Wishart distribution}
\description{
  \code{diwish} returns the density for the inverse Wishart(nu,S)
  evaluated at Sigma.
}
\usage{
diwish(Sigma, nu, S, logscale=FALSE)
}
\arguments{
  \item{Sigma}{Positive-definite matrix}
  \item{nu}{ Degrees of freedom of the inverse Wishart}
  \item{S}{ Scale matrix of the inverse Wishart}
  \item{logscale}{ If \code{logscale==TRUE} the log-density is returned}
}
\value{
  Inverse Wishart(nu,S) density evaluated at Sigma
}
\author{ David Rossell }
\seealso{ \code{\link{dpostNIW}} for the Normal-IW posterior density}
\examples{
Sigma= matrix(c(2,1,1,2),nrow=2)
diwish(Sigma,nu=4,S=diag(2))
}
\keyword{ distribution }

\name{ddir}
\alias{ddir}
\title{ Dirichlet density }
\description{
  Evaluate the density of a Dirichlet distribution
}
\usage{
ddir(x, q, logscale=TRUE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{ Vector or matrix containing the value at which to evaluate
    the density. If a matrix, the density is evaluated for each
    row. Rows are renormalized to ensure they add up to 1}
  \item{q}{ Dirichlet parameters. Must have the same length as
    \code{ncol(x)}, or length 1 (in which case a symmetric Dirichlet
    density is valuated)}
  \item{logscale}{ For \code{logscale==TRUE}, \code{dimom} returns the
    natural log of the prior density}
}
\value{
Density of a Dirichlet(q) distribution evaluated at each row of \code{x}
}
\author{ David Rossell }
\examples{
x= matrix(c(1/3,2/3,.5,.5),nrow=2,byrow=TRUE)
ddir(x,q=2)
}
\keyword{ distribution }

\name{marginalLikelihood}
\alias{marginalLikelihood}
\title{ Marginal (or integrated) likelihood density of the observed data
  for an individual model handled by modelSelection (regression, GLM,
  GAM, accelerated failure time, regression with Normal, two-piece
  Normal, Laplace or two-piece Laplace residuals }
\description{
  The marginal density of the data, i.e. the likelihood
  integrated with respect to the prior distribution on the
  regression coefficients of the variables included in the model.
}
\usage{

marginalLikelihood(sel, y, x, data, smoothterms, nknots=9, groups=1:ncol(x),
family="normal", priorCoef, priorGroup,
priorVar=igprior(alpha=0.01,lambda=0.01), priorSkew=momprior(tau=0.348),
neighbours, 
phi, method='auto', adj.overdisp='intercept', hess='asymp', optimMethod,
optim_maxit, initpar='none', B=10^5, logscale=TRUE, XtX, ytX)

}
%- maybe also 'usage' for other objects documented here.
\arguments{
\item{sel}{Vector with indexes of columns in x to be included in the model.
  Ignored if \code{y} is a formula}
\item{y}{Either a formula with the regression equation or a vector with
  observed responses. The response can be either continuous or of class
  \code{Surv} (survival outcome). If \code{y} is a formula then \code{x},
  \code{groups} and \code{constraints} are automatically created}
\item{x}{Design matrix with linear covariates for which we want to
  assess if they have a linear effect on the response. Ignored if
  \code{y} is a formula}
\item{data}{If \code{y} is a formula then \code{data} should be a data
  frame containing the variables in the model}
\item{smoothterms}{Formula for non-linear covariates (cubic splines),
  modelSelection assesses if the variable has no effect, linear or
  non-linear effect. \code{smoothterms} can also be a design matrix or
  data.frame containing linear terms, for each column modelSelection
  creates a spline basis and tests no/linear/non-linear effects}
\item{nknots}{Number of spline knots. For cubic splines the non-linear
  basis adds knots-4 coefficients for each linear term, we recommend
  setting \code{nknots} to a small/moderate value}
\item{groups}{If variables in \code{x} such be added/dropped in groups,
  \code{groups} indicates the group that each variable corresponds to
  (by default each variable goes in a separate group)}
\item{family}{Parametric distribution family. Possible values include
  'normal','laplace', 'binomial', 'poisson', see function 
\code{modelSelection} for the full list}
\item{priorCoef}{Prior on coefficients, created
  by \code{momprior}, \code{imomprior}, \code{emomprior} or
  \code{zellnerprior}.
  Prior dispersion is on coefficients/sqrt(scale) for Normal and
  two-piece Normal, and on coefficients/sqrt(2*scale) for Laplace
  and two-piece Laplace.}
\item{priorGroup}{Prior on grouped coefficients (e.g. categorical
  predictors with >2 categories, splines). Created by
  \code{groupmomprior}, \code{groupemomprior},
  \code{groupimomprior} or \code{groupzellnerprior}}
\item{priorVar}{Inverse gamma prior on scale parameter, created by
  \code{igprior(). For Normal variance=scale, for Laplace variance=2*scale.}}
\item{priorSkew}{Either a number fixing tanh(alpha) where alpha is the
  asymmetry parameter or a prior on residual skewness parameter,
  assumed to be of
  the same family as priorCoef. Ignored if \code{family} is 'normal' or
  'laplace'.}
\item{neighbours}{Only used if priorCoef is an icarplus prior.
  neighbours is a list with the same length as the design matrix.
  Its entry j should be a vector indicating the neighbours of j,
  and have 0 length if j has no neighbours.}
\item{method}{Method to approximate the integral. See
  \code{help(modelSelection)}.}
\item{adj.overdisp}{Only used for method=='ALA'. Over-dispersion adjustment for models with fixed
  dispersion parameter such as logistic and Poisson regression}
\item{hess}{Method to estimat the hessian in the Laplace approximation to the integrated
  likelihood under Laplace or asymmetric Laplace errors. When
  hess=='asymp' the asymptotic hessian is used, hess=='asympDiagAdj' a
  diagonal adjustment is applied (see Rossell and Rubio for details).}
\item{optimMethod}{Algorithm to maximize objective function when
  method=='Laplace'. Leave unspecified or set optimMethod=='auto' for an
  automatic choice. optimMethod=='LMA' uses modified
  Newton-Raphson algorithm, 'CDA' coordinate descent algorithm}
\item{optim_maxit}{Maximum number of iterations when method=='Laplace'}
\item{initpar}{Initial regression parameter values when finding the
  posterior mode to approximate the integrated likelihood. See
  help(modelSelection)}
\item{B}{Number of Monte Carlo samples to use (ignored unless
  \code{method=='MC'})}
\item{logscale}{If \code{logscale==TRUE} the log marginal density is returned.}
\item{XtX}{Optionally, specify the matrix X'X.
  Useful when the function must be called a large number of times.}
\item{ytX}{Optionally, specify the vector y'X.
  Useful when the function must be called a large number of times.}
\item{phi}{If the disperson parameter (e.g. error variance) is known, it
  can be specified here. Leave blank unless you know what you're doing}
}
\details{
  The marginal density of the data y under a given model is

  p(y | model) int p(y | theta) d P(theta | model)

  where P(theta | model) is the prior distribution on the parameters
  included by the model.   

}
\value{
  Marginal (or integrated) likelihood of the data under the specified prior.
}
\author{ David Rossell }
\seealso{ \code{\link{modelSelection}} to perform model selection based
  on product non-local priors. }
\examples{
x <- matrix(rnorm(100*2),ncol=2)
y <- x \%*\% matrix(c(.5,1),ncol=1) + rnorm(nrow(x))

#Marginal likelihood for 2 models under pMOM prior
marginalLikelihood(c(TRUE,FALSE), y=y, x=x, priorCoef=momprior())
marginalLikelihood(c(TRUE, TRUE), y=y, x=x, priorCoef=momprior())

#Same, under Normal prior with diagonal covariance
marginalLikelihood(c(TRUE,FALSE), y=y, x=x, priorCoef=normalidprior())
marginalLikelihood(c(TRUE, TRUE), y=y, x=x, priorCoef=normalidprior())

}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ models }
\keyword{ htest }

\name{msfit_ggm-class}
\Rdversion{1.1}
\docType{class}
\alias{msfit_ggm-class}
\alias{msfit_ggm}
\alias{show,msfit_ggm-method}
\alias{msfit_ggm.coef}


\title{Class "msfit_ggm"}
\description{
Stores the output of Bayesian Gaussian graphical model selection and
averaging, as produced by function \code{modelSelectionGGM}.
The class extends a list, so all usual methods for lists also work for
\code{msfit_ggm} objects, e.g. accessing elements, retrieving names etc.

Methods are provided to obtain parameter estimates, posterior intervals
(Bayesian model averaging), and posterior probabilities of parameters
being non-zero

}
\section{Objects from the Class}{
  Objects are created by a call to \code{modelSelectionGGM}.
}
\section{Slots}{
  The class extends a list with elements:
  \describe{

    \item{postSample}{Sparse matrix (\code{dgCMatrix}) with posterior
      samples for the Gaussian precision (inverse covariance)
      parameters. Each row is a posterior sample. Within each row, only
      the upper-diagonal of the precision matrix is stored in a flat
      manner. The row and column indexes are stored in indexes}

    \item{indexes}{For each column in postSample, it indicates the row
    and column of the precision matrix}

    \item{p}{Number of variables}
    
    \item{priors}{Priors specified when calling \code{modelSelection}}

  }
}
\section{Methods}{
  \describe{
    \item{coef}{Obtain BMA posterior means, intervals and posterior
      probability of non-zeroes}

    \item{plot}{Shows estimated posterior inclusion probability for each
      parameter vs. number of MCMC iterations. Only up to the first 5000
      parameters are shown}

    \item{show}{\code{signature(object = "msfit_ggm")}:
      Displays general information about the object. }
  }
}
\author{
David Rossell
}
\seealso{
\code{\link{modelSelectionGGM}}
}
\examples{
showClass("msfit_ggm")
}
\keyword{classes}
